package com.sw.mes.itemclean.clean.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sw.mes.core.enums.EnumValuesFactory;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.DynmaicQueryWrapper;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.core.utils.StringUtils;
import com.sw.mes.itemclean.clean.domain.CleaningProcessFlow;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.itemclean.clean.dto.CleaningProcessFlowDto;
import com.sw.mes.itemclean.clean.service.CleaningProcessFlowService;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/CleaningProcessFlow")
@Slf4j
@Api(tags = "料理清洗工艺路线接口")
public class CleaningProcessFlowController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private CleaningProcessFlowService cleaningProcessFlowService;

	@ApiOperation(value = "分页查询料理清洗工艺路线列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<CleaningProcessFlowDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		//分页查询
		QueryWrapper<CleaningProcessFlow> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, CleaningProcessFlow.class);
		Page<CleaningProcessFlow> listPage = cleaningProcessFlowService.page(new Page<>(requestPage.getPage(), requestPage.getRows()), queryWrapper);
		List<CleaningProcessFlowDto> dtos = listPage.getRecords().stream()
				.map(entity -> toLazyDto(entity))
				.collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, listPage.getTotal());
		return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有料理清洗工艺路线")
	@GetMapping("/listAll")
	public Result<List<CleaningProcessFlowDto>> listAll(ServletRequest request) {
		QueryWrapper<CleaningProcessFlow> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, CleaningProcessFlow.class);
		List<CleaningProcessFlow> materialBrokens = cleaningProcessFlowService.list(queryWrapper);
		List<CleaningProcessFlowDto> materialBrokenDtos = materialBrokens.stream()
				.map(materialBroken -> toLazyDto(materialBroken))
				.collect(Collectors.toList());
		return ResultGenerator.success(materialBrokenDtos);
	}

	@ApiOperation(value = "料理清洗工艺路线详细信息")
	@GetMapping("/{id}")
	public Result<CleaningProcessFlowDto> get(@PathVariable @ApiParam(value="料理清洗工艺路线id") Integer id) {
		CleaningProcessFlow materialBroken = cleaningProcessFlowService.getById(id);
		if (null == materialBroken) {
			return ResultGenerator.fail("该对象不存在");
		}
		return ResultGenerator.success(toEagerDto(materialBroken));
	}

	@ApiOperation(value = "创建料理清洗工艺路线")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "料理清洗工艺路线对象") CleaningProcessFlowDto cleaningProcessFlowDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		CleaningProcessFlow cleaningProcessFlow = fromDto(cleaningProcessFlowDto);
		cleaningProcessFlowService.saveOrUpdate(cleaningProcessFlow);
		return ResultGenerator.success();
	}

    /**
     *
     * @param workOrder 工单/出站工单号(批次号)
     * @param masterWorkOrder 母工单/出站工单号
     * @param weight 重量
     * @param unusedWeight 未使用重量
     * @param usedWeight 已使用重量
     * @param deviceNo 设备号
     * @param wipStorageCode 站点代码
     * @param flowLineLable 站点路径
     * @param boxNo 载具号
     * @param lastSite 上一站点
     * @param type 0：工序进站，1：工序出站，2：工序待进站
     * @return
     */
    @ApiOperation("料理-出站")
    @GetMapping("/outStation")
    public Result outStation(@RequestParam String workOrder, @RequestParam String masterWorkOrder, @RequestParam BigDecimal weight, @RequestParam BigDecimal unusedWeight, @RequestParam BigDecimal usedWeight, @RequestParam String deviceNo, @RequestParam String wipStorageCode, @RequestParam String flowLineLable, @RequestParam String boxNo, @RequestParam String lastSite, @RequestParam Integer type) {
        Result result = cleaningProcessFlowService.inStation(workOrder, masterWorkOrder, weight, unusedWeight, usedWeight, deviceNo, wipStorageCode, flowLineLable, boxNo, lastSite, type);
        return result;
    }

    @ApiOperation("料理-进站")
    @GetMapping("/inStation")
    public Result inStation(@RequestParam String workOrder, @RequestParam String masterWorkOrder, @RequestParam BigDecimal weight, @RequestParam BigDecimal unusedWeight, @RequestParam BigDecimal usedWeight, @RequestParam String deviceNo, @RequestParam String wipStorageCode, @RequestParam String flowLineLable, @RequestParam String boxNo, @RequestParam String lastSite, @RequestParam Integer type) {
        Result result = cleaningProcessFlowService.outStation(workOrder, masterWorkOrder, weight, unusedWeight, usedWeight, deviceNo, wipStorageCode, flowLineLable, boxNo, lastSite, type);
        return result;
    }

	public CleaningProcessFlowDto toLazyDto(CleaningProcessFlow entity) {
		return modelMapper.map(entity, CleaningProcessFlowDto.class);
	}

	public CleaningProcessFlowDto toEagerDto(CleaningProcessFlow entity) {
		CleaningProcessFlowDto dto = modelMapper.map(entity, CleaningProcessFlowDto.class);
		return dto;
	}

	public CleaningProcessFlow fromDto(CleaningProcessFlowDto dto) {
		return modelMapper.map(dto, CleaningProcessFlow.class);
	}
}
