package com.sw.mes.itemclean.clean.service.impl;

import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.sw.mes.core.annotation.Jinko;
import com.sw.mes.core.api.system.StoreFeignClients;
import com.sw.mes.core.api.system.YbFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.CarrierConfigDto;
import com.sw.mes.core.dto.DictionaryItemDto;
import com.sw.mes.core.dto.MaterialModelDto;
import com.sw.mes.core.dto.ProductionOrderDto;
import com.sw.mes.core.dto.materialCleanDto.MaterialCleanModelDto;
import com.sw.mes.core.dto.sap.ReportSapChargItemDto;
import com.sw.mes.core.dto.sap.ReportSapGoodsmvtDto;
import com.sw.mes.core.dto.store.*;
import com.sw.mes.core.enums.EnumValuesFactory;
import com.sw.mes.core.enums.ItemCleanEnums;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.ThreadManager;
import com.sw.mes.itemclean.clean.domain.*;
import com.sw.mes.itemclean.clean.dto.*;
import com.sw.mes.itemclean.clean.dto.boxNoOperate.BoxNoOperate;
import com.sw.mes.itemclean.clean.dto.boxNoOperate.MaterialNo;
import com.sw.mes.itemclean.clean.mapper.*;
import com.sw.mes.itemclean.clean.mapping.ScanBoxNoMapping;
import com.sw.mes.itemclean.clean.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sw.mes.itemclean.clean.vo.MaterialUnpackingVo;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 分选拆箱 服务实现类
 * </p>
 *
 * @author liuzemin
 * @since 2022-11-10
 */
@Service
public class MaterialUnpackingServiceImpl extends ServiceImpl<MaterialUnpackingMapper, MaterialUnpacking> implements MaterialUnpackingService, ScanBoxNoService {

    @Autowired
    GenerateBusinessNoService generateBusinessNoService;

    @Autowired
    MaterialApplyOrderService materialApplyOrderService;

    @Autowired
    ModelMapper modelMapper;

    @Autowired
    MaterialUnpackingMapper materialUnpackingMapper;

    @Autowired
    MaterialUnpackingDetailMapper materialUnpackingDetailMapper;

    @Autowired
    MaterialUnpackingDetailService materialUnpackingDetailService;

    @Autowired
    MaterialApplyOrderMapper materialApplyOrderMapper;

    @Autowired
    MaterialApplyOrderRelationMapper materialApplyOrderRelationMapper;

    @Autowired
    MaterialStoreSignBoxMapper materialStoreSignBoxMapper;

    @Autowired
    private MaterialStoreSignBoxService materialStoreSignBoxService;

    @Autowired
    private CleaningProcessFlowService cleaningProcessFlowService;

    @Autowired
    YbFeignClients ybFeignClients;

    @Autowired
    StoreFeignClients storeFeignClients;

    private final static String FX_MARK = "PP";

    @Override
    public BoxPropertyDto scanBoxNo(GeneralScanBoxNoParam param) {

        LambdaQueryWrapper<MaterialStoreSignBox> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MaterialStoreSignBox::getMaterialBoxNo, param.getBoxNo());
        queryWrapper.eq(MaterialStoreSignBox::getPickingStatus, 1);
        queryWrapper.ne(MaterialStoreSignBox::getIsEmptyBox, 1);

//        queryWrapper.eq(MaterialStoreSignBox::getIsPack, 1);
//        queryWrapper.eq(MaterialStoreSignBox::getDelFlag, 0);
//        queryWrapper.in(MaterialStoreSignBox::getMaterialType, Arrays.asList(ItemCleanEnums.MaterialType.原生多晶.getKey()
//                ,ItemCleanEnums.MaterialType.回收料.getKey()
//                ,ItemCleanEnums.MaterialType.返工料.getKey()));
        queryWrapper.last("limit 1");
        MaterialStoreSignBox box = materialStoreSignBoxService.getOne(queryWrapper);
        if (box == null) {
            throw new RuntimeException(param.getBoxNo() + "该箱号不存在！");
        }
        if (box.getIsEmptyBox() == 1) {
            throw new RuntimeException(param.getBoxNo() + "该箱号为空箱！");
        }
        if (box.getMaterialType() == ItemCleanEnums.MaterialType.原生多晶.getKey()
                || box.getMaterialType() == ItemCleanEnums.MaterialType.回收料.getKey()
                || box.getMaterialType() == ItemCleanEnums.MaterialType.返工料.getKey()
        ){
            return ScanBoxNoMapping.toDto(box);
        }else {
            throw new RuntimeException(param.getScanName()+":当前箱号"+param.getBoxNo()+"属于"+ItemCleanEnums.MaterialType.getText(box.getMaterialType()));
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int createStorageSign(MaterialUnpackingVo materialUnpackingVo) {
        MaterialUnpacking materialUnpacking=fromVo(materialUnpackingVo);

        //新增
        if(materialUnpackingVo.getMaterialBoxNo()==null){
            //主表信息
            String materialBoxNo= generateBusinessNoService.generateBoxNo(new BoxNoOperate(ItemCleanEnums.BoxNoType.分选成箱,new MaterialNo(materialUnpackingVo.getMaterialCode())));
            materialUnpacking.setMaterialBoxNo(materialBoxNo);
            materialUnpacking.setCreateUser(BaseContextHandler.getUsername());
            materialUnpacking.setCreateTime(LocalDateTime.now());
            materialUnpacking.setStagingFlag(1);
            materialUnpacking.setDelFlag(0);
            this.saveOrUpdate(materialUnpacking);

            //附表信息
            List<MaterialUnpackingDetail> materialUnpackingDetails = materialUnpackingVo.getMaterialUnpackingDetails();
            for(MaterialUnpackingDetail materialUnpackingDetail:materialUnpackingDetails){
                materialUnpackingDetail.setMaterialBoxNo(materialBoxNo);
                materialUnpackingDetail.setCreateUser(BaseContextHandler.getUsername());
                materialUnpackingDetail.setCreateTime(LocalDateTime.now());
                materialUnpackingDetail.setDelFlag(0);
                materialUnpackingDetail.setMaterialType(materialUnpackingDetail.getMaterialType());
                // 更新原箱子的“是否空箱”字段
                MaterialStoreSignBox box = materialStoreSignBoxService.getByBoxNo(materialUnpackingDetail.getUnpackingBoxNo());
                if (box!=null && !box.getIsEmptyBox().equals(materialUnpackingDetail.getIsEmptyBox())){
                    box.setIsEmptyBox(materialUnpackingDetail.getIsEmptyBox());
                    materialStoreSignBoxService.saveOrUpdate(box);
                }
            }
            //批量插入附表信息
            materialUnpackingDetailService.insertUnpackingDetails(materialUnpackingDetails);
        }else{
            //修改
            materialUnpacking.setUpdateUser(BaseContextHandler.getUsername());
            materialUnpacking.setUpdateTime(LocalDateTime.now());
            materialUnpacking.setStagingFlag(1);
            materialUnpacking.setDelFlag(0);
            LambdaQueryWrapper<MaterialUnpacking> updateWrapper = new LambdaQueryWrapper<>();
            updateWrapper.eq(MaterialUnpacking::getMaterialBoxNo, materialUnpacking.getMaterialBoxNo());
            materialUnpackingMapper.update(materialUnpacking,updateWrapper);

            //删除附表相关子数据
            LambdaQueryWrapper<MaterialUnpackingDetail> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(MaterialUnpackingDetail::getMaterialBoxNo, materialUnpacking.getMaterialBoxNo());
            materialUnpackingDetailMapper.delete(queryWrapper);

            //插入新的附表数据
            List<MaterialUnpackingDetail> materialUnpackingDetails = materialUnpackingVo.getMaterialUnpackingDetails();

            for(MaterialUnpackingDetail materialUnpackingDetail:materialUnpackingDetails){
                materialUnpackingDetail.setCreateUser(BaseContextHandler.getUsername());
                materialUnpackingDetail.setCreateTime(LocalDateTime.now());
                materialUnpackingDetail.setUpdateUser(BaseContextHandler.getUsername());
                materialUnpackingDetail.setUpdateTime(LocalDateTime.now());
                materialUnpackingDetail.setDelFlag(0);
                materialUnpackingDetail.setMaterialBoxNo(materialUnpacking.getMaterialBoxNo());
                materialUnpackingDetail.setMaterialType(materialUnpackingDetail.getMaterialType());
                // 更新原箱子的“是否空箱”字段
                MaterialStoreSignBox box = materialStoreSignBoxService.getByBoxNo(materialUnpackingDetail.getUnpackingBoxNo());
                if (box!=null && !box.getIsEmptyBox().equals(materialUnpackingDetail.getIsEmptyBox())){
                    box.setIsEmptyBox(materialUnpackingDetail.getIsEmptyBox());
                    materialStoreSignBoxService.saveOrUpdate(box);
                }
            }
            //批量插入附表信息
            materialUnpackingDetailService.insertUnpackingDetails(materialUnpackingDetails);

        }
        return 1;
    }

    @Override
    public MaterialUnpackingVo selectStorageSign(String materialBoxNo) {

        LambdaQueryWrapper<MaterialUnpacking> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MaterialUnpacking::getMaterialBoxNo, materialBoxNo);
        MaterialUnpacking MaterialUnpacking=materialUnpackingMapper.selectOne(queryWrapper);
        if(MaterialUnpacking==null) {
            return null;
        }
        MaterialUnpackingVo materialUnpackingVo=toLazyDto(MaterialUnpacking);

        LambdaQueryWrapper<MaterialUnpackingDetail> queryDetailWrapper = new LambdaQueryWrapper<>();
        queryDetailWrapper.eq(MaterialUnpackingDetail::getMaterialBoxNo, materialBoxNo);
        List<MaterialUnpackingDetail> materialUnpackingDetails=materialUnpackingDetailMapper.selectList(queryDetailWrapper);
        materialUnpackingVo.setMaterialUnpackingDetails(materialUnpackingDetails);

        return materialUnpackingVo;
    }


  /*  @Override
    @Transactional(rollbackFor = Exception.class)
    public String createFinishStorageSign(MaterialUnpackingVo materialUnpackingVo) {
        MaterialUnpacking materialUnpacking=fromVo(materialUnpackingVo);

//        List<MaterialApplyOrder> materialApplyOrderOrds=new ArrayList<>();

        List<MaterialApplyOrder> materialApplyOrderCxs=new ArrayList<>();
        List<MaterialApplyOrder> materialApplyOrderBcxs=new ArrayList<>();
        MaterialApplyOrder materialApplyOrderFxcx=new MaterialApplyOrder();

        //新增
        if(materialUnpackingVo.getMaterialBoxNo()==null){
            //新增主表信息
            String materialBoxNo= generateBusinessNoService.generateBoxNo(new BoxNoOperate(ItemCleanEnums.BoxNoType.分选成箱,new MaterialNo(materialUnpackingVo.getMaterialCode())));
            materialUnpacking.setMaterialBoxNo(materialBoxNo);
            materialUnpacking.setCreateUser(BaseContextHandler.getUsername());
            materialUnpacking.setCreateTime(LocalDateTime.now());
            materialUnpacking.setStagingFlag(0);
            materialUnpacking.setDelFlag(0);
            this.saveOrUpdate(materialUnpacking);

            //附表信息
            List<MaterialUnpackingDetail> materialUnpackingDetails = materialUnpackingVo.getMaterialUnpackingDetails();
            for(MaterialUnpackingDetail materialUnpackingDetail:materialUnpackingDetails){
                //１封装附表的数据行对象
                materialUnpackingDetail.setMaterialBoxNo(materialBoxNo);
                materialUnpackingDetail.setCreateUser(BaseContextHandler.getUsername());
                materialUnpackingDetail.setCreateTime(LocalDateTime.now());
                materialUnpackingDetail.setDelFlag(0);
                materialUnpackingDetail.setMaterialType(materialUnpackingDetail.getMaterialType());

                //２封装领料单新增拆箱记录的对象
                MaterialApplyOrder materialApplyOrder = materialApplyOrderService.selectMaterialOrder(materialUnpackingDetail);
                //新增拆箱记录的对象
                MaterialApplyOrder materialApplyOrderCx=new MaterialApplyOrder();
                BeanUtil.copyProperties(materialApplyOrder,materialApplyOrderCx,"id");
                materialApplyOrderCx.setMaterialProp(ItemCleanEnums.MaterialProp.分选拆箱.getKey());
                materialApplyOrderCx.setDelFlag(0);
                materialApplyOrderCx.setCreateUser(BaseContextHandler.getUsername());
                materialApplyOrderCx.setCreateTime(LocalDateTime.now());
                materialApplyOrderCxs.add(materialApplyOrderCx);


                //3封装领料单修改被拆箱记录　逻辑删除的　对象
                MaterialApplyOrder materialApplyOrdeBcx=new MaterialApplyOrder();
                BeanUtil.copyProperties(materialApplyOrdeBcx,materialApplyOrder);
                materialApplyOrdeBcx.setDelFlag(1);
                materialApplyOrdeBcx.setUpdateUser(BaseContextHandler.getUsername());
                materialApplyOrdeBcx.setUpdateTime(LocalDateTime.now());
                materialApplyOrderBcxs.add(materialApplyOrdeBcx);

                //4封装领料单分选成箱的记录对象
                BeanUtil.copyProperties(materialApplyOrder,materialApplyOrderFxcx,"id");
                materialApplyOrderFxcx.setMaterialProp(ItemCleanEnums.MaterialProp.分选成箱.getKey());
                materialApplyOrderFxcx.setMaterialBoxNo(materialUnpackingDetail.getMaterialBoxNo());
                materialApplyOrderFxcx.setDelFlag(0);
                materialApplyOrderFxcx.setWeight(materialUnpacking.getWeight());
                materialApplyOrderFxcx.setCreateUser(BaseContextHandler.getUsername());
                materialApplyOrderFxcx.setCreateTime(LocalDateTime.now());
                materialApplyOrderFxcx.setApplyTime(LocalDateTime.now());
                materialApplyOrderFxcx.setApplyName(BaseContextHandler.getUsername());

            }
            //批量插入附表信息
            materialUnpackingDetailService.insertUnpackingDetails(materialUnpackingDetails);
        }else{
            //修改主表信息
            materialUnpacking.setUpdateUser(BaseContextHandler.getUsername());
            materialUnpacking.setUpdateTime(LocalDateTime.now());
            materialUnpacking.setStagingFlag(0);

            LambdaQueryWrapper<MaterialUnpacking> updateWrapper = new LambdaQueryWrapper<>();
            updateWrapper.eq(MaterialUnpacking::getMaterialBoxNo, materialUnpacking.getMaterialBoxNo());
            materialUnpackingMapper.update(materialUnpacking,updateWrapper);

            //删除附表相关子数据
            LambdaQueryWrapper<MaterialUnpackingDetail> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(MaterialUnpackingDetail::getMaterialBoxNo, materialUnpacking.getMaterialBoxNo());
            materialUnpackingDetailMapper.delete(queryWrapper);

            //插入新的附表数据
            List<MaterialUnpackingDetail> materialUnpackingDetails = materialUnpackingVo.getMaterialUnpackingDetails();
            for(MaterialUnpackingDetail materialUnpackingDetail:materialUnpackingDetails){
                materialUnpackingDetail.setCreateUser(BaseContextHandler.getUsername());
                materialUnpackingDetail.setCreateTime(LocalDateTime.now());
                materialUnpackingDetail.setUpdateUser(BaseContextHandler.getUsername());
                materialUnpackingDetail.setUpdateTime(LocalDateTime.now());
                materialUnpackingDetail.setDelFlag(0);
                materialUnpackingDetail.setMaterialType(materialUnpackingDetail.getMaterialType());

                //２封装领料单新增拆箱记录的对象
                MaterialApplyOrder materialApplyOrder = materialApplyOrderService.selectMaterialOrder(materialUnpackingDetail);

                //新增拆箱记录的对象
                MaterialApplyOrder materialApplyOrderCx=new MaterialApplyOrder();
                BeanUtil.copyProperties(materialApplyOrder,materialApplyOrderCx,"id");
                materialApplyOrderCx.setMaterialProp(ItemCleanEnums.MaterialProp.分选拆箱.getKey());
                materialApplyOrderCx.setDelFlag(0);
                materialApplyOrderCx.setCreateUser(BaseContextHandler.getUsername());
                materialApplyOrderCx.setCreateTime(LocalDateTime.now());
                materialApplyOrderCxs.add(materialApplyOrderCx);


                //3封装领料单修改被拆箱记录　逻辑删除的　对象
                MaterialApplyOrder materialApplyOrdeBcx=new MaterialApplyOrder();
                BeanUtil.copyProperties(materialApplyOrdeBcx,materialApplyOrder);
                materialApplyOrdeBcx.setDelFlag(1);
                materialApplyOrdeBcx.setUpdateUser(BaseContextHandler.getUsername());
                materialApplyOrdeBcx.setUpdateTime(LocalDateTime.now());
                materialApplyOrderBcxs.add(materialApplyOrdeBcx);

                //4封装领料单分选成箱的记录对象
                BeanUtil.copyProperties(materialApplyOrder,materialApplyOrderFxcx,"id");
                materialApplyOrderFxcx.setMaterialProp(ItemCleanEnums.MaterialProp.分选成箱.getKey());
                materialApplyOrderFxcx.setMaterialBoxNo(materialUnpackingDetail.getMaterialBoxNo());
                materialApplyOrderFxcx.setDelFlag(0);
                materialApplyOrderFxcx.setWeight(materialUnpacking.getWeight());
                materialApplyOrderFxcx.setCreateUser(BaseContextHandler.getUsername());
                materialApplyOrderFxcx.setCreateTime(LocalDateTime.now());
                materialApplyOrderFxcx.setApplyTime(LocalDateTime.now());
                materialApplyOrderFxcx.setApplyName(BaseContextHandler.getUsername());
            }
            //批量插入附表信息
            materialUnpackingDetailService.insertUnpackingDetails(materialUnpackingDetails);
        }

        //一、处理领料单
        //1插入成箱数据行---增加一条6分选成箱的数据
        materialApplyOrderMapper.insert(materialApplyOrderFxcx);

        //2修改被拆箱的逻辑删除标志；把delFlag　从0修改为1
        for(MaterialApplyOrder materialApplyOrderBcx:materialApplyOrderBcxs){
            materialApplyOrderMapper.updateById(materialApplyOrderBcx);
        }

        //3插入被拆箱的数据行，其中　delFlag为0,materialProp　物料类型修改为　5分选拆箱
        for(MaterialApplyOrder materialApplyOrderCx:materialApplyOrderCxs){
            materialApplyOrderMapper.insert(materialApplyOrderCx);
        }
        //4生成拆箱和成箱之间的关联关系表
        for(MaterialApplyOrder materialApplyOrderCx:materialApplyOrderCxs){
            MaterialApplyOrderRelation materialApplyOrderRelation=new MaterialApplyOrderRelation();
            materialApplyOrderRelation.setMaterialBoxNoParent(materialApplyOrderCx.getMaterialBoxNo());
            materialApplyOrderRelation.setParentId(materialApplyOrderCx.getId());
            materialApplyOrderRelation.setSonId(materialApplyOrderFxcx.getId());
            materialApplyOrderRelation.setMaterialBoxNoSon(materialApplyOrderFxcx.getMaterialBoxNo());
            materialApplyOrderRelationMapper.insert(materialApplyOrderRelation);
        }

        // 二、打签箱暂存表：删除原拆箱箱号删除，新的箱号进行插入
        MaterialStoreSignBox materialStoreSignBox=new MaterialStoreSignBox();
        BeanUtil.copyProperties(materialUnpacking,materialStoreSignBox,"id");
        materialStoreSignBox.setPickingStatus(1);
        materialStoreSignBox.setDelFlag(0);
        materialStoreSignBox.setIsPack(1);
        materialStoreSignBox.setIsEmptyBox(0);

        materialStoreSignBoxMapper.insert(materialStoreSignBox);

        for(MaterialUnpackingDetail materialUnpackingDetail : materialUnpackingVo.getMaterialUnpackingDetails()){
            LambdaQueryWrapper<MaterialStoreSignBox> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(MaterialStoreSignBox::getMaterialBoxNo,materialUnpackingDetail.getUnpackingBoxNo());
            MaterialStoreSignBox ordMaterialStoreSignBox= materialStoreSignBoxMapper.selectOne(queryWrapper);
            ordMaterialStoreSignBox.setDelFlag(1);
            materialStoreSignBoxMapper.updateById(ordMaterialStoreSignBox);
        }

      return materialUnpacking.getMaterialBoxNo();
    }
*/

    /**
     * 升级后分选成箱-成箱打签功能
     *
     * @param materialUnpackingVo
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public FxToBoxInfoPrintDto createFinishStorageSign(MaterialUnpackingVo materialUnpackingVo) {
        MaterialUnpacking materialUnpacking = fromVo(materialUnpackingVo);
        MaterialStoreSignBox materialStoreSignBox = materialStoreSignBoxService.useInfoByMaterialProductBatchNo(materialUnpackingVo.getProductBatchNo());

        //新增主表信息
        String ppProductBatch = generateBusinessNoService.generateBoxNoForUnpack(FX_MARK);
        materialUnpacking.setMaterialBoxNo(ppProductBatch);
        materialUnpacking.setFxProductBatchNo(ppProductBatch);
        materialUnpacking.setUnpackingBoxNo(materialStoreSignBox.getBoxNo());
        materialUnpacking.setCreateUser(BaseContextHandler.getUsername());
        materialUnpacking.setCreateTime(LocalDateTime.now());
        materialUnpacking.setStagingFlag(1);
        materialUnpacking.setOtherWeight(materialUnpackingVo.getOtherWeight());
        materialUnpacking.setDelFlag(ItemCleanEnums.MaterialDelFlag.未删除.getKey());
        materialUnpacking.setHasGone(materialUnpackingVo.getHasGone());


        //附表信息
        List<MaterialUnpackingDetail> materialUnpackingDetails = materialUnpackingVo.getMaterialUnpackingDetails();
        if (CollectionUtils.isNotEmpty(materialUnpackingDetails)) {
            for (MaterialUnpackingDetail materialUnpackingDetail : materialUnpackingDetails) {
                //１封装附表的数据行对象
                materialUnpackingDetail.setMaterialBoxNo(ppProductBatch);
                materialUnpackingDetail.setCreateUser(BaseContextHandler.getUsername());
                materialUnpackingDetail.setCreateTime(LocalDateTime.now());
                materialUnpackingDetail.setDelFlag(0);
                materialUnpackingDetail.setMaterialType(materialUnpackingDetail.getMaterialType());
            }
        }

        //1、处理领料单
        BigDecimal usedWeightTotal = materialStoreSignBox.getUsedWeight().add(materialUnpackingVo.getWeight());
        BigDecimal unusedWeightTotal = materialStoreSignBox.getWeight().subtract(usedWeightTotal);
        if (unusedWeightTotal.compareTo(BigDecimal.ZERO) < 0) {
            throw new BusinessException("该箱:" + materialStoreSignBox.getBoxNo() + "物料重量不足!");
        }
        materialStoreSignBox.setUseStatus(ItemCleanEnums.MaterialUseStatus.已分选.getKey());
        if (unusedWeightTotal.compareTo(BigDecimal.ZERO) == 0) {
            materialStoreSignBox.setUseStatus(ItemCleanEnums.MaterialUseStatus.已清空.getKey());
        }
        materialStoreSignBox.setUpdateUser(BaseContextHandler.getUsername());
        materialStoreSignBox.setUpdateTime(LocalDateTime.now());
        materialStoreSignBox.setUsedWeight(usedWeightTotal);
        materialStoreSignBox.setUnusedWeight(unusedWeightTotal);
        materialStoreSignBox.setHasGone(materialUnpackingVo.getHasGone());


        //5 分选完成,硅料原材料仓出库(要求: 维护的一定和余料工单的下个工序的副产品材料仓一致)+向sap报工原料消耗
        try {
            String warehouseCode = getWareHouseCodeByWorkOrderNo(materialUnpacking.getAufnr());
            String recycledMaterialCode = materialStoreSignBox.getMaterialCode();
            String recycledProductBatchNo = materialStoreSignBox.getProductBatchNo();
            WarehouseOutputBillBo warehouseOutputBillBo = newOutputBill(materialUnpacking, recycledMaterialCode, recycledProductBatchNo, warehouseCode);
            Result<String> resultStore = storeFeignClients.outputApply(warehouseOutputBillBo);
            if (ResultCode.SUCCESS.getCode() != resultStore.getCode()) {
                log.error("线边仓出库失败:" + resultStore.getMessage());
                throw new BusinessException("线边仓出库失败:" + resultStore.getMessage());
            } else {
                //保存分选信息
                this.saveOrUpdate(materialUnpacking);
                //批量插入附表信息
                materialUnpackingDetailService.insertUnpackingDetails(materialUnpackingDetails);
                //处理领料单
                materialStoreSignBoxService.updateById(materialStoreSignBox);

                //3 过站,分选出站,清洗待进站(老批次,箱号出站;新批次,箱号进入下一站点)
                cleaningProcessFlowService.outStation(materialStoreSignBox.getProductBatchNo(), materialStoreSignBox.getProductBatchNo(), materialUnpacking.getWeight(), materialUnpacking.getUnusedWeight(), materialUnpacking.getUsedWeight(), "", EnumValuesFactory.CleaningWipStorageCode.分选.getValue(), "", materialUnpackingVo.getUnpackingBoxNo(), EnumValuesFactory.CleaningWipStorageCode.领料.getValue(), EnumValuesFactory.CleaningWipStorageStatus.工序出站.getKey());

                cleaningProcessFlowService.inStation(materialUnpacking.getFxProductBatchNo(), materialUnpackingVo.getProductBatchNo(), materialUnpacking.getWeight(), materialUnpacking.getUnusedWeight(), materialUnpacking.getUsedWeight(), "", EnumValuesFactory.CleaningWipStorageCode.清洗.getValue(), "", materialUnpacking.getBoxNo(), EnumValuesFactory.CleaningWipStorageCode.分选.getValue(), EnumValuesFactory.CleaningWipStorageStatus.工序待进站.getKey());

                this.handleReportSapGoodsmvtDto(materialUnpacking.getAufnr(), recycledMaterialCode,recycledProductBatchNo, materialUnpacking.getWeight(), "KG", warehouseCode);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        //4 返回打印对象
        FxToBoxInfoPrintDto fxToBoxInfoPrintDto = buildFxToBoxInfoPrintDto(materialUnpacking);
        return fxToBoxInfoPrintDto;
    }

    /**
     * 通过工单号获取 材料仓库位
     *
     * @param workOrderNo 计划工单
     * @return
     */
    @Jinko(author = "yyl", value = "通过工单号获取材料仓库位")
    private String getWareHouseCodeByWorkOrderNo(String workOrderNo) {
        //通过计划工单中的片区field4找到字典中的材料仓
        Result<ProductionOrderDto> productionOrder = ybFeignClients.getListByAufnr(workOrderNo);
        String field4 = "";
        String warehouseCode = "";
        if (productionOrder.getCode() != ResultCode.SUCCESS.getCode() || null == productionOrder.getData()) {
            throw new RuntimeException("工单查询异常：" + productionOrder.getMessage());
        }
        ProductionOrderDto orderData = productionOrder.getData();
        if (null != orderData) {
            field4 = orderData.getField4();
        }
        if (null != field4) {
            Result<DictionaryItemDto> dictionaryItemDtoResult = ybFeignClients.findNameByThemeValueAndItemValue("ByProductWarehouseLocation", field4);
            if (null != dictionaryItemDtoResult && null != dictionaryItemDtoResult.getData() && ResultCode.SUCCESS.getCode() == dictionaryItemDtoResult.getCode()) {
                warehouseCode = dictionaryItemDtoResult.getData().getExtendValue();
            } else {
                throw new BusinessException(dictionaryItemDtoResult.getMessage() + "详情: 关键字[ByProductWarehouseLocation]中的[工作中心编码]: " + field4 + " 未配置");
            }
        }
        return warehouseCode;
    }

    @Override
    public MaterialUnpacking getByBoxNo(String boxNo) {
        LambdaQueryWrapper<MaterialUnpacking> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MaterialUnpacking::getBoxNo, boxNo);
        queryWrapper.eq(MaterialUnpacking::getDelFlag, ItemCleanEnums.MaterialDelFlag.未删除.getKey());
        queryWrapper.orderByDesc(MaterialUnpacking::getCreateTime);
        queryWrapper.last("limit 1");
        return this.getOne(queryWrapper);
    }

    @Override
    public MaterialUnpacking getFxProductBatchNo(String fxProductBatchNo) {
        LambdaQueryWrapper<MaterialUnpacking> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MaterialUnpacking::getFxProductBatchNo, fxProductBatchNo);
        queryWrapper.last("limit 1");
        return this.getOne(queryWrapper);
    }

    @Override
    public MaterialUnpacking getByPpProductBatchNo(String fxProductBatchNo) {
        LambdaQueryWrapper<MaterialUnpacking> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MaterialUnpacking::getFxProductBatchNo, fxProductBatchNo);
        queryWrapper.eq(MaterialUnpacking::getDelFlag, ItemCleanEnums.MaterialDelFlag.未删除);
        queryWrapper.eq(MaterialUnpacking::getUseStatus,  ItemCleanEnums.MaterialFxUseStatus.未使用.getKey());
        queryWrapper.last("limit 1");
        return this.getOne(queryWrapper);
    }

    @Override
    public void updateBoxStatus(String unpackingBoxNo, BigDecimal unusedWeightTotal, String boxNo) {
        BigDecimal allowUnusedWeight = BigDecimal.ZERO;
        Result<List<DictionaryItemDto>> result = ybFeignClients.findByThemeValue("allow_unusedWeight");
        if (result.getCode() == ResultCode.SUCCESS.getCode()) {
            List<DictionaryItemDto> dictionaryItemDtoList = result.getData();
            for (DictionaryItemDto dto : dictionaryItemDtoList) {
                allowUnusedWeight = new BigDecimal(dto.getValue());
            }
        }

        //2.1 被领料载具使用状态可能变更,通过剩余重量和字典配置比较判断是否释放箱子
        if (unusedWeightTotal.compareTo(allowUnusedWeight) <= 0) {
            CarrierConfigDto carrierConfigDto = new CarrierConfigDto();
            carrierConfigDto.setCarrierNo(unpackingBoxNo);
            carrierConfigDto.setStatus(0);
            ybFeignClients.updateCarrierConfigStatus(carrierConfigDto);
        }
        //2.2 领料载具使用状态变更为使用中
        CarrierConfigDto carrierConfigDto = new CarrierConfigDto();
        if(null != boxNo){
            carrierConfigDto.setCarrierNo(boxNo);
            carrierConfigDto.setStatus(1);
            ybFeignClients.updateCarrierConfigStatus(carrierConfigDto);
        }
    }

    /**
     * 供清洗上料使用
     * @param currentProductBatchNo
     * @return
     */
    @Override
    public MaterialUnpacking getUseInfoByPpProductBatchNo(String currentProductBatchNo) {
        LambdaQueryWrapper<MaterialUnpacking> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MaterialUnpacking::getFxProductBatchNo, currentProductBatchNo);
        queryWrapper.eq(MaterialUnpacking::getDelFlag, ItemCleanEnums.MaterialDelFlag.未删除);
        queryWrapper.in(MaterialUnpacking::getUseStatus, ItemCleanEnums.MaterialFxUseStatus.未使用.getKey(), ItemCleanEnums.MaterialFxUseStatus.已使用.getKey());
        queryWrapper.last("limit 1");
        return this.getOne(queryWrapper);
    }

    /**
     *
     * 供破碎上料使用
     * @param boxNo
     * @return
     */
    @Override
    public MaterialUnpacking getUnUseInfoByBoxNo(String boxNo) {
        LambdaQueryWrapper<MaterialUnpacking> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MaterialUnpacking::getBoxNo, boxNo);
        queryWrapper.eq(MaterialUnpacking::getDelFlag, ItemCleanEnums.MaterialDelFlag.未删除.getKey());
        queryWrapper.eq(MaterialUnpacking::getHasGone, ItemCleanEnums.MaterialHasGone.破碎.getKey());
        queryWrapper.eq(MaterialUnpacking::getUseStatus, ItemCleanEnums.MaterialFxUseStatus.未使用.getKey());
        queryWrapper.last("limit 1");
        return this.getOne(queryWrapper);
    }

    @Override
    public IPage<MaterialUnpackingJoinPageDto> queryList(IPage<MaterialUnpackingJoinPageDto> iPage, MaterialUnpackingParamQuery query) {
        return materialUnpackingMapper.queryList(iPage, query);
    }


    public MaterialUnpackingVo toLazyDto(MaterialUnpacking entity) {
        return modelMapper.map(entity, MaterialUnpackingVo.class);
    }

    public MaterialUnpacking fromVo(MaterialUnpackingVo vo) {
        return modelMapper.map(vo, MaterialUnpacking.class);
    }

    /**
     * 构建分选成箱打签数据
     * @param materialUnpacking
     * @return
     */
    public FxToBoxInfoPrintDto buildFxToBoxInfoPrintDto(MaterialUnpacking materialUnpacking) {
        // 查询物料编码所属厂家
      /*  Result<MaterialCleanModelDto> result = ybFeignClients.findMaterialCleanByCode(materialUnpacking.getMaterialCode());
        if (!result.getCode().equals(ResultCode.SUCCESS.code)) {
            throw new RuntimeException("查询原料清洗产品模型发生错误");
        }
        MaterialCleanModelDto dto = result.getData();
        if (dto == null) {
            throw new RuntimeException("不存在" + materialUnpacking.getMaterialCode() + "原料清洗产品模型！");
        }*/
        FxToBoxInfoPrintDto fxToBoxInfoPrintDto = new FxToBoxInfoPrintDto();
        fxToBoxInfoPrintDto.setBoxNo(materialUnpacking.getBoxNo())
                .setFxProductBatchNo(materialUnpacking.getFxProductBatchNo())
                .setProductBatchNo(materialUnpacking.getProductBatchNo())
               // .setFromMark(dto.getFactoryCode())
                .setMaterialName(materialUnpacking.getMaterialName())
                .setMaterialCode(materialUnpacking.getMaterialCode())
                .setCreateTime(new Date())
                .setWeight(String.valueOf(materialUnpacking.getWeight()))
                .setMaterialModel(materialUnpacking.getMaterialModel())
                .setMaterialGear(materialUnpacking.getMaterialGear())
                .setMaterialLevel(materialUnpacking.getMaterialLevel())
                .setClassNo(BaseContextHandler.getUserClassNo())
                .setRemark("");
        return fxToBoxInfoPrintDto;
    }

//    public MaterialApplyOrder toMaterialApplyOrder(MaterialUnpacking materialUnpacking) {
//        MaterialApplyOrder materialApplyOrder = modelMapper.map(materialUnpacking, MaterialApplyOrder.class);
//        materialApplyOrder.setMaterialProp(ItemCleanEnums.MaterialProp.分选成箱.getKey());
//        return materialApplyOrder;
//    }



    public MaterialApplyOrder detailToMaterialApplyOrder(MaterialUnpackingDetail materialUnpackingDetail) {
        MaterialApplyOrder materialApplyOrder = modelMapper.map(materialUnpackingDetail, MaterialApplyOrder.class);
        materialApplyOrder.setMaterialProp(ItemCleanEnums.MaterialProp.分选成箱.getKey());
        return materialApplyOrder;
    }

    /**
     * 消耗副产品报给内部系统,供上报sap使用
     *
     * @param workOrderNo    工单
     * @param materialCode   物料编码
     * @param produceBatchNo 产品批次
     * @param weight         数量(重量)
     * @param unitName       单位(KG,PC)
     * @param warehouseCode       库存地点编码
     */
    @Jinko(author = "yyl", value = "消耗副产品报给内部系统,供上报sap使用")
    private void handleReportSapGoodsmvtDto(String workOrderNo, String materialCode, String produceBatchNo, BigDecimal weight, String unitName,String warehouseCode) {
        ReportSapGoodsmvtDto reportSapGoodsmvtDto = new ReportSapGoodsmvtDto();
        List<ReportSapChargItemDto> reportSapChargItemDtos = new ArrayList<>();
        ReportSapChargItemDto reportSapChargItemDto = new ReportSapChargItemDto();
//        reportSapChargItemDto.setAtnam("P01");
//        reportSapChargItemDto.setAtwrt("JK1");
        reportSapChargItemDtos.add(reportSapChargItemDto);
        reportSapGoodsmvtDto.setChargItems(reportSapChargItemDtos);
        reportSapGoodsmvtDto.setAufnr(workOrderNo);
        reportSapGoodsmvtDto.setCharg(produceBatchNo);
        reportSapGoodsmvtDto.setBwart("531");
        reportSapGoodsmvtDto.setWerks("5041");
        reportSapGoodsmvtDto.setMatnr(materialCode);
        reportSapGoodsmvtDto.setMenge(weight == null ? "0" : weight.toString());
        reportSapGoodsmvtDto.setMeins(unitName);
        reportSapGoodsmvtDto.setZfcpFlag("X");
        reportSapGoodsmvtDto.setLgort(warehouseCode);
        reportSapGoodsmvtDto.setProcName("硅料清洗分选");
        ybFeignClients.sendSapGoodsmvt(reportSapGoodsmvtDto);
    }


    /**
     * 清洗硅料副产品从材料仓出库,被消耗
     *
     * @param entity
     * @param recycledMaterialCode
     * @param recycledProductBatchNo
     * @param warehouseCode
     * @return
     */
    private WarehouseOutputBillBo newOutputBill(MaterialUnpacking entity, String recycledMaterialCode, String recycledProductBatchNo, String warehouseCode) {
        WarehouseOutputBillBo outputBillBo = new WarehouseOutputBillBo();
        outputBillBo.setType("生产领用");
        outputBillBo.setAutoAudit(true);
        outputBillBo.setDestination("车间");
        List<WarehouseOutputBillDetailBo> boList = new ArrayList<>();

        Result<MaterialModelDto> result = ybFeignClients.getMaterialByCode(recycledMaterialCode);
        MaterialModelDto model = result.getData();
        if (model == null) {
            throw new BusinessException("【" +recycledMaterialCode + "】物料主数据不存在，请检查数据！");
        }
        WarehouseOutputBillDetailBo bo = new WarehouseOutputBillDetailBo();
        bo.setMaterialModelId(model.getId().longValue());
        bo.setMaterialModelCode(model.getCode());
        bo.setMaterialModelName(model.getName());
        bo.setBatchNo(recycledProductBatchNo);
        bo.setMaterialTypeName(model.getMaterialTypeDto().getName());
        bo.setSupplier(model.getSupplier());
        bo.setSpecification(model.getSpecification());
        bo.setMaterialTh(model.getNumberOfDrawings());
        bo.setRequireCount(entity.getWeight());
        bo.setUnitName(model.getUnitDto().getName());
        bo.setWarehouseName(warehouseCode);
        boList.add(bo);
        outputBillBo.setOutputBillDetails(boList);
        return outputBillBo;
    }
}
