package com.sw.mes.fb.checkout.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.mes.pc.fb.FbCheckOutDataDto;
import com.sw.mes.core.entity.FbCheckOutData;
import com.sw.mes.core.enums.SectionIndexRangeEnum;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.*;
import com.sw.mes.fb.checkout.dto.ExportFbCheckOutDataDto;
import com.sw.mes.fb.checkout.dto.ImportGrade;
import com.sw.mes.fb.checkout.dto.MatchOrder;
import com.sw.mes.fb.checkout.entity.OperateCheckOutDataLog;
import com.sw.mes.fb.checkout.service.FbCheckOutDataService;
import io.swagger.annotations.*;
import jodd.time.TimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/ZhxxOperateCheckOutData")
@Slf4j
@Api(description = "方棒检验数据库接口")
public class FbCheckOutDataController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private FbCheckOutDataService fbCheckOutDataService;

    @ApiOperation(value = "分页查询方棒检验数据库列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"), @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"), @ApiImplicitParam(name = "search_LIKE_ybSectionNo", value = "圆棒编号", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_LIKE_fbSectionNo", value = "方棒编号", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_EQ_checkUserName", value = "检验人员", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_LIKE_sectionNo", value = "晶棒编号", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_GTE_checkTime", value = "检验开始时间", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_LTE_checkTime", value = "检验结束时间", dataType = "String", paramType = "query"),})
    @GetMapping("/listByPage")
    public Result<ResponsePage<FbCheckOutDataDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<FbCheckOutData> specification = DynamicSpecifications.bySearchFilter(request, FbCheckOutData.class);
        Page<FbCheckOutData> page = fbCheckOutDataService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<FbCheckOutDataDto> dtos = page.getContent().stream().map(entity -> toLazyDto(entity)).collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "创建方棒检验数据库")
    @PostMapping("/matchOrder")
    public Result matchOrder(@RequestBody @Valid @ApiParam(value = "方棒匹配订单接口") MatchOrder matchOrder, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (matchOrder.getIds().isEmpty()) {
            return ResultGenerator.success("至少选择一个编号");
        }
        if (matchOrder.getIds().size() > 100) {
            return ResultGenerator.success("每次最多只能选择100个编号");
        }
        for (Integer id : matchOrder.getIds()) {

        }
        return ResultGenerator.success();
    }

    @ApiOperation(value = "导出方棒检验数据库")
    @ApiImplicitParams({@ApiImplicitParam(name = "search_LIKE_ybSectionNo", value = "圆棒编号", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_LIKE_fbSectionNo", value = "方棒编号", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_EQ_checkUserName", value = "检验人员", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_LIKE_sectionNo", value = "晶棒编号", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_GTE_checkTime", value = "检验开始时间", dataType = "String", paramType = "query"), @ApiImplicitParam(name = "search_LTE_checkTime", value = "检验结束时间", dataType = "String", paramType = "query"),})
    @GetMapping("/export")
    public void export(ServletRequest request, @ApiIgnore RequestPage requestPage, HttpServletResponse response) {
        try {
            Specification<FbCheckOutData> specification = DynamicSpecifications.bySearchFilter(request, FbCheckOutData.class);
            List<FbCheckOutData> list = fbCheckOutDataService.findByExampleSort(specification, null);
            List<ExportFbCheckOutDataDto> dtos = list.stream().map(entity -> {
                ExportFbCheckOutDataDto exportFbCheckOutDataDto = toLazyExportDto(entity);
                String date = DateUtil.formatDate(DateUtil.addMinute(exportFbCheckOutDataDto.getCheckTime(), -510), DateUtil.shortFormat);
                exportFbCheckOutDataDto.setTime(date);
                return exportFbCheckOutDataDto;
            }
            ).collect(Collectors.toList());
            EasyExcelUtils.write(response, ExportFbCheckOutDataDto.class, "导出方棒检验数据库", "导出方棒检验数据库", dtos);
        } catch (Exception e) {
            throw new RuntimeException("方棒拼棒记录导出失败，错误信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "获取所有方棒检验数据库")
    @GetMapping("/listAll")
    public Result<List<FbCheckOutDataDto>> listAll(ServletRequest request) {
        Specification<FbCheckOutData> specification = DynamicSpecifications.bySearchFilter(request, FbCheckOutData.class);
        List<FbCheckOutData> fbCheckOutData = fbCheckOutDataService.findByExampleSort(specification, null);
        List<FbCheckOutDataDto> fbCheckOutDataDtos = fbCheckOutData.stream().map(fbCheckOutData2 -> toLazyDto(fbCheckOutData2)).collect(Collectors.toList());
        return ResultGenerator.success(fbCheckOutDataDtos);
    }

    @ApiOperation(value = "方棒检验数据库详细信息")
    @GetMapping("/{id}")
    public Result<FbCheckOutDataDto> get(@PathVariable @ApiParam(value = "ZhxxOperateCheckOutDataid") Long id) {
        FbCheckOutData fbCheckOutData = fbCheckOutDataService.get(id);
        if (null == fbCheckOutData) return ResultGenerator.fail("该方棒检验信息不存在");
        return ResultGenerator.success(toEagerDto(fbCheckOutData));
    }

    @ApiOperation(value = "接收MCS的编号查询检验数据")
    @PostMapping("/bySectionList")
    public Result<List<FbCheckOutData>> getData(@RequestBody List<String> sectionNos) {
        List<FbCheckOutData> checkOutDataByFbSectionNoIns = new ArrayList<>();
        if(null != sectionNos && sectionNos.size() > 0){
            checkOutDataByFbSectionNoIns = fbCheckOutDataService.findCheckOutDataByFbSectionNoIns(sectionNos);
            if (null == checkOutDataByFbSectionNoIns) {
                return ResultGenerator.fail("检验数据不存在");
            }
        }
        return ResultGenerator.success(checkOutDataByFbSectionNoIns);
    }

    @ApiOperation(value = "创建方棒检验数据库")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "ZhxxOperateCheckOutData对象") FbCheckOutDataDto fbCheckOutDataDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        FbCheckOutData fbCheckOutData = fromDto(fbCheckOutDataDto);
        fbCheckOutDataService.saveOrUpdate(fbCheckOutData);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新方棒检验数据库")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "ZhxxOperateCheckOutData") FbCheckOutDataDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null) return ResultGenerator.fail("id不能为空");
        FbCheckOutData entity = fbCheckOutDataService.get(dto.getId());
        if (entity == null) return ResultGenerator.fail("原信息不存在");
        try {
            FbCheckOutData entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            fbCheckOutDataService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除方棒检验数据库")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
        for (Long id : ids) {
            fbCheckOutDataService.delete(id);
        }
        return ResultGenerator.success();
    }


    @ApiOperation(value = "根据单晶编号查询")
    @GetMapping("/findDataByFbSectionNo")
    Result findDataByFbSectionNo(@RequestParam String fbSectionNo) {
        return ResultGenerator.success(fbCheckOutDataService.findDataByFbSectionNo(fbSectionNo));
    }

    @ApiOperation(value = "根据单晶编号集合查询")
    @GetMapping("/findDataByFbSectionNoList")
    Result findDataByFbSectionNoList(@RequestParam List<String> fbSectionNos) {
        return ResultGenerator.success(fbCheckOutDataService.findCheckOutDataByFbSectionNoIns(fbSectionNos));
    }

    @ApiOperation(value = "根据单晶编号查询拼棒组合编号")
    @GetMapping("/getSectionComposeBySectionNo")
    Result getSectionComposeBySectionNo(@RequestParam(value = "方棒编号") String fbSectionNo) {
        String sectionCompose = fbCheckOutDataService.getSectionComposeBySectionNo(fbSectionNo);
        return ResultGenerator.success(sectionCompose);
    }

    /**
     * 晶段编号、规格、产品等级、实验等级
     *
     * @param file
     */
    @ApiOperation(value = "导入")
    @PostMapping("/import")
    public Result importGrade(MultipartFile file) {
        Object objList = EasyExcelUtils.readExcel(file, new ImportGrade(), 1, 1);
        if (objList != null) {
            List<ImportGrade> importGrades = (List<ImportGrade>) objList;
            StringBuilder failMessage = new StringBuilder();
            int successCount = 0, failCount = 0;
            for (ImportGrade importGrade : importGrades) {
                FbCheckOutData fbCheckOutData = fbCheckOutDataService.findDataByFbSectionNo(importGrade.getFbSectionNo());
                try {
                    if (fbCheckOutData != null) {
                        OperateCheckOutDataLog operateCheckOutDataLog = new OperateCheckOutDataLog();
                        BeanUtils.copyProperties(fbCheckOutData, operateCheckOutDataLog, BeanUtil.getIgnoreProperties(fbCheckOutData, "id", null));
                        operateCheckOutDataLog.setOperateTime(new Date());
                        operateCheckOutDataLog.setOperator(BaseContextHandler.getUsername());
                        operateCheckOutDataLog.setOperateType(1);
                        operateCheckOutDataLog.setRemarks(BaseContextHandler.getUsername() + "导入评级，数据变动");
                        BeanUtils.copyProperties(importGrade, fbCheckOutData);
                        fbCheckOutDataService.saveAndLog(fbCheckOutData, operateCheckOutDataLog);
                        successCount++;
                    } else {
                        failMessage.append(importGrade).append("导入失败【 检验信息不存在 】\\n");
                        failCount++;
                    }
                } catch (Exception e) {
                    failCount++;
                    failMessage.append(importGrade.getFbSectionNo()).append("导入失败【").append(e.getMessage()).append("】；\\n");
                }
                successCount++;
            }
            if (successCount > 0) {
                return ResultGenerator.success("共【" + successCount + failCount + "】条，成功【" + successCount + "】条；\\n"
                        + "失败【" + failCount + "】：" + failMessage);
            } else {
                return ResultGenerator.fail("共【" + successCount + failCount + "】条，全部失败：" + failMessage);
            }
        } else {
            return ResultGenerator.fail("导入数据为空");
        }
    }

    public FbCheckOutDataDto toLazyDto(FbCheckOutData entity) {
        FbCheckOutDataDto dto = modelMapper.map(entity, FbCheckOutDataDto.class);
        String fromMark = dto.getYbSectionNo().substring(SectionIndexRangeEnum.基地追溯产品类型.getBeginIndex(), SectionIndexRangeEnum.基地追溯产品类型.getEndIndex());
        if (fromMark.matches("[A-Z]+")) {
            dto.setCrystalNo(dto.getYbSectionNo().substring(SectionIndexRangeEnum.毛棒范围.getBeginIndex(), SectionIndexRangeEnum.毛棒范围.getEndIndex()));
        } else {
            dto.setCrystalNo(dto.getYbSectionNo().substring(0, 12));
        }
        return dto;
    }
    public ExportFbCheckOutDataDto toLazyExportDto(FbCheckOutData entity) {
        ExportFbCheckOutDataDto dto = modelMapper.map(entity, ExportFbCheckOutDataDto.class);
        String fromMark = dto.getYbSectionNo().substring(SectionIndexRangeEnum.基地追溯产品类型.getBeginIndex(), SectionIndexRangeEnum.基地追溯产品类型.getEndIndex());
        if (fromMark.matches("[A-Z]+")) {
            dto.setCrystalNo(dto.getYbSectionNo().substring(SectionIndexRangeEnum.毛棒范围.getBeginIndex(), SectionIndexRangeEnum.毛棒范围.getEndIndex()));
        } else {
            dto.setCrystalNo(dto.getYbSectionNo().substring(0, 12));
        }
        return dto;
    }

    public FbCheckOutDataDto toEagerDto(FbCheckOutData entity) {
        return modelMapper.map(entity, FbCheckOutDataDto.class);
    }

    public FbCheckOutData fromDto(FbCheckOutDataDto dto) {
        return modelMapper.map(dto, FbCheckOutData.class);
    }


}
