package com.sw.mes.fb.turnstore.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sw.mes.base.factory.inspect.dto.ProductRuleValidateDto;
import com.sw.mes.core.annotation.TokenNotValidation;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.entity.FbCheckOutData;
import com.sw.mes.core.enums.EnumValues;
import com.sw.mes.core.enums.EnumValuesFactory;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.*;
import com.sw.mes.base.factory.inspect.dto.ProductRuleDto;
import com.sw.mes.base.factory.inspect.entity.ProductAssemblyRule;
import com.sw.mes.base.factory.inspect.service.ProductAssemblyRuleService;
import com.sw.mes.fb.checkout.service.FbCheckOutDataService;
import com.sw.mes.fb.turnstore.dto.*;
import com.sw.mes.fb.turnstore.entity.FbTurnstoreTray;
import com.sw.mes.fb.turnstore.entity.FbTurnstoreTrayItem;
import com.sw.mes.fb.turnstore.service.FbTurnstoreTrayItemService;
import com.sw.mes.fb.turnstore.service.FbTurnstoreTrayService;
import com.sw.mes.fb.turnstore.service.RawMaterialReceivingService;
import com.sw.mes.base.workinprocess.entity.PlanProcessingOrder;
import com.sw.mes.base.workinprocess.service.PlanProcessingOrderService;
import com.sw.mes.base.workinprocess.service.WipStorageService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/FbTurnstoreTray")
@Slf4j
@Api(description = "方棒组托接口")
public class FbTurnstoreTrayController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private FbTurnstoreTrayService fbTurnstoreTrayService;

    @Autowired
    private FbTurnstoreTrayItemService fbTurnstoreTrayItemService;

    @Autowired
    private RawMaterialReceivingService receivingService;

    @Autowired
    private FbCheckOutDataService checkOutService;

    @Autowired
    private WipStorageService wipStorageService;

    @Autowired
    RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private ProductAssemblyRuleService productAssemblyRuleService;

    @Autowired
    private PlanProcessingOrderService planProcessingOrderService;

    @Autowired
    private ObjectMapper objectMapper;

    @ApiOperation(value = "分页查询方棒组托列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_trayNo", value = "托号", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_status", value = "打托状态", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_inputType", value = "入库状态", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_time", value = "组托时间", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_time", value = "组托时间", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_confirmTime", value = "确认时间", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_confirmTime", value = "确认时间", dataType = "String", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<FbTurnstoreTrayDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<FbTurnstoreTray> specification = DynamicSpecifications.bySearchFilter(request, FbTurnstoreTray.class);
        Page<FbTurnstoreTray> page = fbTurnstoreTrayService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<FbTurnstoreTrayDto> dtos = page.getContent().stream()
                .map(this::toLazyDto)
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有方棒组托")
    @GetMapping("/listAll")
    public Result<List<FbTurnstoreTrayDto>> listAll(ServletRequest request) {
        Specification<FbTurnstoreTray> specification = DynamicSpecifications.bySearchFilter(request, FbTurnstoreTray.class);
        List<FbTurnstoreTray> fbTurnstoreTrays = fbTurnstoreTrayService.findByExampleSort(specification, null);
        List<FbTurnstoreTrayDto> fbTurnstoreTrayDtos = fbTurnstoreTrays.stream()
                .map(this::toLazyDto)
                .collect(Collectors.toList());
        return ResultGenerator.success(fbTurnstoreTrayDtos);
    }

    @ApiOperation(value = "方棒组托详细信息")
    @GetMapping("/{id}")
    public Result<FbTurnstoreTrayPostDto> get(@PathVariable @ApiParam(value = "方棒组托id") Integer id) {
        FbTurnstoreTray fbTurnstoreTray = fbTurnstoreTrayService.get(id);
        if (null == fbTurnstoreTray) {
            return ResultGenerator.fail("该方棒组托不存在");
        }
        FbTurnstoreTrayPostDto postDto = new FbTurnstoreTrayPostDto();
        postDto.setFbTray(fbTurnstoreTray);
        //查询详情列表
        List<FbTurnstoreTrayItem> fbTurnstoreTrayItemList = fbTurnstoreTrayItemService.getItemsByTrayId(id);
        postDto.setTrayItems(fbTurnstoreTrayItemList);
        //查询组托规则
        if (null != fbTurnstoreTray.getStation()) {
            ProductAssemblyRule ruleByStation = productAssemblyRuleService.findRuleByStation(fbTurnstoreTray.getStation());
            postDto.setProductRule(ruleByStation);
        }
        //更新打印时间
        fbTurnstoreTray.setPrintTime(new Date());
        fbTurnstoreTrayService.saveOrUpdate(fbTurnstoreTray);
        return ResultGenerator.success(postDto);
    }

    @ApiOperation(value = "创建组拖主表")
    @PostMapping("/create")
//    public Result create(@RequestBody @Valid @ApiParam(value = "组拖主表对象") FbTurnstoreTrayPostDto postDto, BindingResult bindingResult) {
    public Result create(@RequestBody @Valid @ApiParam(value = "组拖主表对象") FbTurnstoreTrayDto postDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (null == postDto) {
            return ResultGenerator.fail("组拖申请错误");
        }
        if (null == postDto.getTrayItems() || postDto.getTrayItems().isEmpty()) {
            return ResultGenerator.fail("组拖明细错误");
        }
        //FbTurnstoreTray turnstoreTray = postDto.getFbTray();
        FbTurnstoreTray turnstoreTray = fromDto(postDto);
        List<FbTurnstoreTrayItem> items = postDto.getTrayItems();
//        ProductAssemblyRule productRule = postDto.getProductRule();
        ProductAssemblyRule productRule = null;
        if (null != postDto.getRuleId()) {
            productRule = productAssemblyRuleService.get(postDto.getRuleId());
        }
        turnstoreTray.setUser(BaseContextHandler.getUsername());
        fbTurnstoreTrayService.createTray(productRule, turnstoreTray, items);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新组拖主表")
    @PostMapping("/update")
//    public Result update(@RequestBody @Valid @ApiParam(value = "组拖主表") FbTurnstoreTrayPostDto postDto, BindingResult bindingResult) {
    public Result update(@RequestBody @Valid @ApiParam(value = "组拖主表") FbTurnstoreTrayDto postDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
//        if (postDto.getFbTray().getId() == null) {
        if (postDto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
//        FbTurnstoreTray entity = fbTurnstoreTrayService.get(postDto.getFbTray().getId());
        FbTurnstoreTray entity = fbTurnstoreTrayService.get(postDto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        //查询托号是否重复
//        FbTurnstoreTray fbTurnstoreTray = fbTurnstoreTrayService.findByTrayNo(postDto.getFbTray().getTrayNo());
        FbTurnstoreTray fbTurnstoreTray = fbTurnstoreTrayService.findByTrayNo(postDto.getTrayNo());
//        if (null != fbTurnstoreTray && !postDto.getFbTray().getId().equals(fbTurnstoreTray.getId())) {
        if (null != fbTurnstoreTray && !postDto.getId().equals(fbTurnstoreTray.getId())) {
            return ResultGenerator.fail("新提交的托号已存在,请重新编辑");
        }
        if (postDto.getTrayItems() == null || postDto.getTrayItems().isEmpty()) {
            return ResultGenerator.fail("组拖明细错误");
        }

        //组拖规则
//        ProductAssemblyRule productRule = postDto.getProductRule();
        ProductAssemblyRule productRule = null;
        if (null != postDto.getRuleId()) {
            productRule = productAssemblyRuleService.get(postDto.getRuleId());
        }

      /*  //取消
        if (null != postDto.getFbTray().getState() && 4 == postDto.getFbTray().getState()) {
            if (postDto.getFbTray().getHasGone().contains("WMS")) {
                return ResultGenerator.fail("去向为WMS的,暂不能进行取消操作,请联系管理员处理");
            }
            if (0 != entity.getState()) {
                return ResultGenerator.fail("当前不处于待确认状态,不能进行取消操作,请刷新后再试");
            }
            //更新主表
            FbTurnstoreTray turnstoreTray = postDto.getFbTray();
            //通知下游取消
            Result<String> receive = receivingService.cancelState(turnstoreTray.getTrayNo(), turnstoreTray.getUpdateUser());
            if (receive.getCode() != ResultCode.SUCCESS.getCode()) {
                return ResultGenerator.fail(receive.getMessage());
            }
            //更新详情
            List<FbTurnstoreTrayItem> turnstoreTrayItemList = postDto.getTrayItems();
            for (FbTurnstoreTrayItem trayItem : turnstoreTrayItemList) {
                //解锁组托状态
                PlanProcessingOrder planProcessingOrder = planProcessingOrderService.findByCode(trayItem.getSectionNo());
                if (null == planProcessingOrder) {
                    throw new BusinessException(trayItem.getSectionNo() + "工单号不存在");
                }
                planProcessingOrder.setStatus(EnumValuesFactory.PlanProcessingOrderStatus.加工中.getKey());
                planProcessingOrderService.saveOrUpdate(planProcessingOrder);

                trayItem.setState(turnstoreTray.getState());
                trayItem.setOperationTime(new Date());
                //退站操作
                wipStorageService.exitStation(trayItem.getSectionNo());
                fbTurnstoreTrayItemService.saveOrUpdate(trayItem);
            }
            //更新主表
            turnstoreTray.setUpdateUser(BaseContextHandler.getUsername());
            Integer tornieNum = productRule.getTornieNum();
            if(tornieNum == turnstoreTrayItemList.size() && null == turnstoreTray.getReportingTime()){
                turnstoreTray.setReportingTime(new Date());
            }
            fbTurnstoreTrayService.saveOrUpdate(turnstoreTray);
        } else {
            try {
                if (9 != entity.getState()) {
                    return ResultGenerator.fail("当前不处于待出库状态,不能进行编辑操作,请刷新后再试");
                }
                FbTurnstoreTray turnstoreTray = postDto.getFbTray();
                if (!turnstoreTray.getTrayNo().equals(entity.getTrayNo())) {
                    turnstoreTray.setOrderNum(Integer.valueOf(turnstoreTray.getTrayNo().substring(11, 14)));
                }
                List<FbTurnstoreTrayItem> items = postDto.getTrayItems();
                turnstoreTray.setUpdateUser(BaseContextHandler.getUsername());
                Integer tornieNum = productRule.getTornieNum();
                if(tornieNum == items.size() &&  null == turnstoreTray.getReportingTime()){
                    turnstoreTray.setReportingTime(new Date());
                }
                fbTurnstoreTrayService.updateTray(turnstoreTray, items);
            } catch (Exception e) {
                return ResultGenerator.fail(e.getMessage());
            }
        }*/

        try {
            if (9 != entity.getState()) {
                return ResultGenerator.fail("当前不处于待出库状态,不能进行编辑操作,请刷新后再试");
            }
            FbTurnstoreTray turnstoreTray = fromDto(postDto);
            if (!turnstoreTray.getTrayNo().equals(entity.getTrayNo())) {
                turnstoreTray.setOrderNum(Integer.valueOf(turnstoreTray.getTrayNo().substring(11, 14)));
            }
            List<FbTurnstoreTrayItem> items = postDto.getTrayItems();
            turnstoreTray.setUpdateUser(BaseContextHandler.getUsername());
            Integer tornieNum = productRule.getTornieNum();
            if (tornieNum == items.size() && null == turnstoreTray.getReportingTime()) {
                turnstoreTray.setReportingTime(new Date());
            }
            fbTurnstoreTrayService.updateTray(turnstoreTray, items);
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
        return ResultGenerator.success();
    }

    @ApiOperation(value = "方棒组托退库")
    @GetMapping("/cancel/{id}")
    public Result cancel(@PathVariable @ApiParam(value = "方棒组托id") Integer id) {
        FbTurnstoreTrayPostDto fbTurnstoreTrayPostDto = fbTurnstoreTrayItemService.cancelStock(id);
        return ResultGenerator.success("退库成功", fbTurnstoreTrayPostDto);
    }

    @ApiOperation(value = "单包组拖长度校验")
    @PostMapping("/lengthCheck")
    public Result lengthCheck(@RequestBody @Valid @ApiParam(value = "成品组拖规则") ProductRuleDto ruleDto) {
        Result result = fbTurnstoreTrayService.lengthCheck(ruleDto);
        return result;
    }

    @ApiOperation(value = "根据托号查询信息")
    @GetMapping("/getTray")
    public Result<ResponsePage<FbTurnstoreTrayDto>> getFbTray(@ApiParam(value = "托号", type = "query") @RequestParam(value = "trayNo") String trayNo) {
        FbTurnstoreTray result = fbTurnstoreTrayService.getFbTray(trayNo);
        if (null == result) {
            return ResultGenerator.fail("未查询到组拖信息");
        }
        List<FbTurnstoreTrayDto> list = new ArrayList<>();
        list.add(toLazyDto(result));
        ResponsePage responsePage = new ResponsePage(list, 1L);
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "校验晶段信息是否符合规则")
    @PostMapping("/validateRuleNew")
    public Result validateRuleNew(@RequestBody @Valid @ApiParam(value = "组托规则校验Dto") ProductRuleValidateDto ruleDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        try {
            return fbTurnstoreTrayService.validateRuleNew(ruleDto);
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "校验晶段信息是否符合规则")
    @PostMapping("/validateRule")
    public Result validateRule(@RequestBody @Valid @ApiParam(value = "成品组拖规则") ProductRuleDto ruleDto) {
        Result result;
        //查询晶段信息
        FbCheckOutData checkOut = checkOutService.findDataByFbSectionNo(ruleDto.getSectionNo());
        if (null == checkOut || null == checkOut.getYbLevel()) {
            return ResultGenerator.fail("当前晶段信息不存在或晶段暂未评级");
        }
        if (null == checkOut.getMaterialNumber()) {
            return ResultGenerator.fail("当前晶段信息缺失物料号, 请在方棒评级测试，是否为“正常”，氧含量是否合格");
        }
        //晶棒重复扫描卡控
        FbTurnstoreTrayItem bySectionNo = fbTurnstoreTrayItemService.findBySectionNo2(ruleDto.getSectionNo());
        if (null != bySectionNo) {
            return ResultGenerator.fail("当前晶段已经执行过组托！请勿重复组托");
        }
        //如果是拼棒,则查询获取所有
        String split = ",";
        //判断入库类型只要满足一根即可
        String state = "";
        List<FbTurnstoreTrayItem> fbTurnstoreTrayItemList = new ArrayList<>();
        if (null != checkOut.getSectionCompose() && checkOut.getSectionCompose().contains(split)) {
            String[] sections = checkOut.getSectionCompose().split(split);
            List<String> sectionNoList = Arrays.asList(sections);
            //for (String sectionNo : sections) {
            for (int i = 0; i < sectionNoList.size(); i++) {
                FbCheckOutData checkOutData = checkOutService.findDataByFbSectionNo(sectionNoList.get(i));

                if (null != checkOutData) {
                    result = fbTurnstoreTrayService.validateRuleAndData(ruleDto, checkOutData, state);
                    if (result.getCode() != ResultCode.SUCCESS.getCode()) {
                        return ResultGenerator.fail(result.getMessage());
                    }
                    state = result.getMessage();
                    fbTurnstoreTrayItemList.add((FbTurnstoreTrayItem) result.getData());
                }
                String productLevelFb = checkOutData.getProductLevelFb();
                boolean productLevelFbFlag = false;
                if (i > 0) {
                    //大于一条数据时候,校验两颗棒子的方棒评级/入库类别
                    FbCheckOutData previousCheckOutData = checkOutService.findDataByFbSectionNo(sectionNoList.get(i - 1));
                    String previousProductLevelFb = previousCheckOutData.getProductLevelFb();
                    productLevelFbFlag = judgmentProductLevelFb(previousProductLevelFb, productLevelFb);
                    if (!productLevelFbFlag) {
                        return ResultGenerator.fail(sectionNoList.get(i) + "入库类别:[" + productLevelFb + "] 与单晶" + sectionNoList.get(i - 1) + "入库类别:[" + previousProductLevelFb + "]不同!");
                    }
                }

            }
            if ("综计WMS".equals(ruleDto.getHasGone()) || "综计返修".equals(ruleDto.getHasGone())) {
                if (StringUtils.isBlank(state)) {
                    return ResultGenerator.fail("没有符合入库类别为：" + ruleDto.getInputType() + "的晶段");
                }
            }
        } else {
            result = fbTurnstoreTrayService.validateRuleAndData(ruleDto, checkOut, state);
            if (result.getCode() != ResultCode.SUCCESS.getCode()) {
                return ResultGenerator.fail(result.getMessage());
            }
            state = result.getMessage();
            fbTurnstoreTrayItemList.add((FbTurnstoreTrayItem) result.getData());
        }
        if ("综计WMS".equals(ruleDto.getHasGone()) || "综计返修".equals(ruleDto.getHasGone())) {
            if (StringUtils.isBlank(state)) {
                return ResultGenerator.fail("没有符合入库类别为：" + ruleDto.getInputType() + "的晶段");
            }
        }
        if ("综计WMS".equals(ruleDto.getHasGone())) {
            if (checkOut.getEnableCheck() != null && checkOut.getEnableCheck() == 0) {
                return ResultGenerator.fail("综计WMS：该晶棒在抽检中不合格,无法组托到WMS");
            }
        }
        return ResultGenerator.success(fbTurnstoreTrayItemList);
    }

    @ApiOperation(value = "取消入库接口")
    @PostMapping("/cancelWarehousing")
    @TokenNotValidation
    public Result cancelWarehousing(@RequestBody WmsCancelWarehousingDto wmsCancelWarehousingDto) {
        String trayNo = wmsCancelWarehousingDto.getTrayNo();
        Integer flag = wmsCancelWarehousingDto.getFlag();
        FbTurnstoreTray byTrayNo = fbTurnstoreTrayService.findByTrayNo(trayNo);
        if (null == byTrayNo) {
            return ResultGenerator.fail(0, "未查询到此托盘号");
        }
        byTrayNo.setFlag(flag);
        fbTurnstoreTrayService.saveOrUpdate(byTrayNo);
        return ResultGenerator.success(1, "SUCCESS", trayNo);
    }

    @ApiOperation(value = "发起入库")
    @PostMapping("/initiateReceipt")
    public Result initiateReceipt(@RequestBody @Valid @ApiParam(value = "组拖主表对象") FbTurnstoreTray turnstoreTray) {
        //校验状态
        FbTurnstoreTray entity = fbTurnstoreTrayService.get(turnstoreTray.getId());
        if (null == entity) {
            return ResultGenerator.fail("原信息不存在");
        }
        if (9 != entity.getState()) {
            return ResultGenerator.fail("当前不处于待出库状态,不能进行发起入库操作,请刷新后再试");
        }
        List<FbTurnstoreTrayItem> turnstoreTrayItemList = fbTurnstoreTrayItemService.getItemsByTrayId(turnstoreTray.getId());
        //获取详情列表
        if (null == turnstoreTrayItemList || turnstoreTrayItemList.isEmpty()) {
            return ResultGenerator.fail("组托详情不能为空");
        }
        if (null == turnstoreTray.getBorderWidth()) {
            return ResultGenerator.fail("边距不能为空");
        }
        //发起入库
        return fbTurnstoreTrayService.initiateReceipt(turnstoreTray, turnstoreTrayItemList);
    }

    @ApiOperation(value = "方棒组托上报ERP")
    @PostMapping("/reportedErp")
    public Result reportedErp(@RequestBody @Valid @ApiParam(value = "组拖主表对象") FbTurnstoreTray turnstoreTray) {
        //校验状态
        FbTurnstoreTray entity = fbTurnstoreTrayService.get(turnstoreTray.getId());
        if (null == entity) {
            return ResultGenerator.fail("原信息不存在");
        }
        // 已入库  未上报ERP
        if (1 == entity.getState() && EnumValues.ReportedErp.未上报.getKey() == entity.getReportedState()) {
            List<FbTurnstoreTrayItem> turnstoreTrayItemList = fbTurnstoreTrayItemService.getItemsByTrayId(turnstoreTray.getId());
            //获取详情列表
            if (null == turnstoreTrayItemList || turnstoreTrayItemList.isEmpty()) {
                return ResultGenerator.fail("组托详情不能为空");
            }
            for (FbTurnstoreTrayItem fbTurnstoreTrayItem : turnstoreTrayItemList) {
                if (EnumValues.ReportedErp.未上报.getKey() != fbTurnstoreTrayItem.getReportedState()) {
                    return ResultGenerator.fail("组托详情当前不处于未上报状态,不能进行上报ERP操作：" + fbTurnstoreTrayItem.getSectionNo());
                }
            }
            //组托上报ERP
            return fbTurnstoreTrayService.reportedErp(entity, turnstoreTrayItemList);
        } else {
            return ResultGenerator.fail("当前不处于入库或未上报状态,不能进行上报ERP操作,请刷新后再试");
        }
    }

    @ApiOperation(value = "方棒WMS接收接口")
    @PostMapping("/fbWmsReceive")
    public Result fbWmsReceive(@RequestBody List<WmsToFbReceiveDto> receiveDto) {

        for (WmsToFbReceiveDto wmsToFbReceiveDto : receiveDto) {
            //查询组托信息
            FbTurnstoreTray turnstoreTray = fbTurnstoreTrayService.findByTrayNo(wmsToFbReceiveDto.getBoxCode());
            if (null == turnstoreTray) {
                return ResultGenerator.fail("该:" + wmsToFbReceiveDto.getBoxCode() + " 托号信息不存在");
            }
            //拒收时增加组托状态判断---只有待确认的可以拒收
            if ((1 == wmsToFbReceiveDto.getType() || 2 == wmsToFbReceiveDto.getType()) && turnstoreTray.getState() != 0) {
                return ResultGenerator.fail("该:" + wmsToFbReceiveDto.getBoxCode() + " 托号不是待确认状态,不能进行接收拒收操作");
            }
            //冲销时增加组托状态判断---只有已确认的可以冲销
            if (3 == wmsToFbReceiveDto.getType() && turnstoreTray.getState() != 1) {
                return ResultGenerator.fail("该:" + wmsToFbReceiveDto.getBoxCode() + " 托号不是已确认状态,不能冲销");
            }
            //查询组托明细
            List<FbTurnstoreTrayItem> trayItemList = fbTurnstoreTrayItemService.getItemsByTrayId(turnstoreTray.getId());
            if (null == trayItemList || trayItemList.isEmpty()) {
                return ResultGenerator.fail("该:" + wmsToFbReceiveDto.getBoxCode() + " 组拖明细错误");
            }
            //构建FbTurnstoreTray更新信息
            buildEntityProp(wmsToFbReceiveDto, turnstoreTray);
            //更新组托表头信息
            fbTurnstoreTrayService.updateTurnstoreTrayStateById(wmsToFbReceiveDto, turnstoreTray);
            //更新组托明细信息
            fbTurnstoreTrayService.updateTrayItemState(wmsToFbReceiveDto, turnstoreTray, trayItemList);
        }
        return ResultGenerator.success();
    }

    private void buildEntityProp(WmsToFbReceiveDto wmsToFbReceiveDto, FbTurnstoreTray turnstoreTray) {
        //更新信息
        turnstoreTray.setConfirmUser(wmsToFbReceiveDto.getRejectionUser());
        turnstoreTray.setConfirmTime(wmsToFbReceiveDto.getRejectionDate());
        //根据状态进行操作
        if (1 == wmsToFbReceiveDto.getType()) {
            //确认接收
            turnstoreTray.setState(1);
        } else if (2 == wmsToFbReceiveDto.getType()) {
            //拒收
            turnstoreTray.setState(3);
            turnstoreTray.setRemark(wmsToFbReceiveDto.getRejectionReason());
        } else {
            //冲销==取消
            turnstoreTray.setState(5);
            if (null != wmsToFbReceiveDto.getRejectionReason()) {
                turnstoreTray.setRemark(wmsToFbReceiveDto.getRejectionReason());
            } else {
                turnstoreTray.setRemark("冲销");
            }
        }
    }

    @ApiOperation(value = "导出Excel")
    @GetMapping("/exportExcel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "search_EQ_trayNo", value = "托号", dataType = "String", paramType = "query"),
    })
    public void exportExcel(HttpServletRequest request, HttpServletResponse resp) {
        fbTurnstoreTrayItemService.exportExcel(request, resp);
    }

    @ApiOperation(value = "导出方棒组拖Excel")
    @GetMapping("/exportExcel1")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "search_GTE_time", value = "开始时间", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "search_LT_time", value = "结束时间", dataType = "String", paramType = "query"),
    })
    public void exportExcel1(HttpServletRequest request, HttpServletResponse resp) {
        fbTurnstoreTrayItemService.exportExcel1(request, resp);
    }

    @ApiOperation(value = "删除方棒组托")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            //校验状态
            FbTurnstoreTray entity = fbTurnstoreTrayService.get(id);
            if (null == entity) {
                return ResultGenerator.fail("原信息不存在");
            }
            if (9 != entity.getState()) {
                return ResultGenerator.fail("当前不处于待出库状态,不能进行删除操作,请刷新后再试");
            }
            //删除详情
            fbTurnstoreTrayItemService.deleteByTrayId(id);
            fbTurnstoreTrayService.delete(id);
        }
        return ResultGenerator.success();
    }


    @ApiOperation(value = "组托前根据方棒编号获取晶棒信息")
    @GetMapping("/getInfoByFbSectionNo")
    public Result<FbInfoDto> getInfoByFbSectionNo(@ApiParam(value = "方棒编号", type = "query") @RequestParam(value = "fbSectionNo") String fbSectionNo) {

        if (null == fbSectionNo || StringUtils.isEmpty(fbSectionNo)) {
            return ResultGenerator.fail("方棒编号不能为空！");
        }
        return fbTurnstoreTrayService.getInfoByFbSectionNo(fbSectionNo);

    }

    @ApiOperation(value = "修改方棒组托状态")
    @PostMapping("/updateStateByTrayNo")
    public Result updateStateByTrayNo(@RequestBody UpdateStateByTrayNoDto dto) {
        try {
            log.info("调用修改方棒组托状态请求参数:{}", objectMapper.writeValueAsString(dto));
            fbTurnstoreTrayService.updateStateByTrayNo(dto);
        } catch (JsonProcessingException e) {
            return ResultGenerator.fail("修改方棒组托状态失败");
        }
        return ResultGenerator.success();
    }
    public FbTurnstoreTrayDto toLazyDto(FbTurnstoreTray entity) {
        return modelMapper.map(entity, FbTurnstoreTrayDto.class);
    }

    public FbTurnstoreTrayDto toEagerDto(FbTurnstoreTray entity) {
        return modelMapper.map(entity, FbTurnstoreTrayDto.class);
    }

    public FbTurnstoreTray fromDto(FbTurnstoreTrayDto dto) {
        return modelMapper.map(dto, FbTurnstoreTray.class);
    }

    /**
     *
     * 判断不同晶棒的方棒等级/入库类型 是否相同
     * @param previousProductLevelFb
     * @param productLevelFb
     * @return
     */
    private boolean judgmentProductLevelFb(String previousProductLevelFb, String productLevelFb) {
        boolean flag = false;
        //判断入库类别
        if (org.apache.commons.lang3.StringUtils.isNotBlank(previousProductLevelFb) && org.apache.commons.lang3.StringUtils.isNotBlank(productLevelFb) && previousProductLevelFb.equals(productLevelFb)) {
            flag = true;
        }

        return flag;
    }

}
