package com.sw.mes.fb.turnstore.controller;


import com.sw.mes.base.workinprocess.entity.PlanProcessingOrder;
import com.sw.mes.base.workinprocess.service.PlanProcessingOrderService;
import com.sw.mes.core.dto.FbTurnstoreTrayItemDto;
import com.sw.mes.core.enums.EnumValuesFactory;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.fb.turnstore.entity.FbTurnstoreTrayItem;
import com.sw.mes.fb.turnstore.service.FbTurnstoreTrayItemService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/FbTurnstoreTrayItem")
@Slf4j
@Api(description = "方棒组托详情接口")
public class FbTurnstoreTrayItemController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private FbTurnstoreTrayItemService fbTurnstoreTrayItemService;

    @Autowired
    private PlanProcessingOrderService planProcessingOrderService;

    @ApiOperation(value = "分页查询方棒组托详情列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_trayId", value = "主表id", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_sectionNo", value = "晶段编号", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_trayNo", value = "托号", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_state", value = "入库状态", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_time", value = "组托时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_time", value = "组托时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_toWmsTime", value = "入综计时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_toWmsTime", value = "入综计时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_operationTime", value = "操作/确认时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_operationTime", value = "操作/确认时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_confirmTime", value = "确认时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_confirmTime", value = "确认时间", dataType = "string", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<FbTurnstoreTrayItemDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<FbTurnstoreTrayItem> specification = DynamicSpecifications.bySearchFilter(request, FbTurnstoreTrayItem.class);
        Page<FbTurnstoreTrayItem> page = fbTurnstoreTrayItemService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<FbTurnstoreTrayItemDto> dtos = page.getContent().stream()
                .map(this::toLazyDto)
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有方棒组托详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "search_EQ_trayId", value = "主表id", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listAll")
    public Result<List<FbTurnstoreTrayItemDto>> listAll(ServletRequest request) {
        Specification<FbTurnstoreTrayItem> specification = DynamicSpecifications.bySearchFilter(request, FbTurnstoreTrayItem.class);
        List<FbTurnstoreTrayItem> fbTurnstoreTrayItems = fbTurnstoreTrayItemService.findByExampleSort(specification, null);
        List<FbTurnstoreTrayItemDto> fbTurnstoreTrayItemDtos = fbTurnstoreTrayItems.stream()
                .map(this::toLazyDto)
                .collect(Collectors.toList());
        return ResultGenerator.success(fbTurnstoreTrayItemDtos);
    }

    @ApiOperation(value = "方棒组托详情详细信息")
    @GetMapping("/{id}")
    public Result<FbTurnstoreTrayItemDto> get(@PathVariable @ApiParam(value = "方棒组托详情id") Integer id) {
        FbTurnstoreTrayItem fbTurnstoreTrayItem = fbTurnstoreTrayItemService.get(id);
        if (null == fbTurnstoreTrayItem) {
            return ResultGenerator.fail("该方棒组托详情不存在");
        }
        return ResultGenerator.success(toEagerDto(fbTurnstoreTrayItem));
    }

    @ApiOperation(value = "创建方棒组托详情")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "方棒组托详情对象") FbTurnstoreTrayItemDto fbTurnstoreTrayItemDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        FbTurnstoreTrayItem fbTurnstoreTrayItem = fromDto(fbTurnstoreTrayItemDto);
        fbTurnstoreTrayItemService.saveOrUpdate(fbTurnstoreTrayItem);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新方棒组托详情")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "方棒组托详情") FbTurnstoreTrayItemDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
        FbTurnstoreTrayItem entity = fbTurnstoreTrayItemService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            FbTurnstoreTrayItem entityFromFto = fromDto(dto);
            BeanUtils.copyProperties(entityFromFto, entity, BeanUtil.getIgnoreProperties(entityFromFto, null, null));
            fbTurnstoreTrayItemService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除方棒组托详情")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            FbTurnstoreTrayItem trayItem = fbTurnstoreTrayItemService.get(id);
            if (null == trayItem) {
                return ResultGenerator.fail("原信息不存在");
            }
            if (9 != trayItem.getState()) {
                return ResultGenerator.fail("当前不处于待出库状态,不能进行删除,请刷新后再试");
            }
            //解锁组托状态
            PlanProcessingOrder planProcessingOrder = planProcessingOrderService.findByCode(trayItem.getSectionNo());
            planProcessingOrder.setStatus(EnumValuesFactory.PlanProcessingOrderStatus.加工中.getKey());
            planProcessingOrderService.saveOrUpdate(planProcessingOrder);

            fbTurnstoreTrayItemService.delete(id);
        }
        return ResultGenerator.success();
    }

    @ApiOperation(value = "导出Excel")
    @GetMapping("/exportExcel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "search_EQ_sectionNo", value = "晶段编号", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_trayNo", value = "托号", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_EQ_state", value = "入库状态", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_time", value = "组托时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_time", value = "组托时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_toWmsTime", value = "入综计时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_toWmsTime", value = "入综计时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_operationTime", value = "操作/确认时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_operationTime", value = "操作/确认时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_GTE_confirmTime", value = "确认时间", dataType = "string", paramType = "query"),
            @ApiImplicitParam(name = "search_LTE_confirmTime", value = "确认时间", dataType = "string", paramType = "query"),
    })
    public void exportExcel(HttpServletRequest request, HttpServletResponse resp) {
        fbTurnstoreTrayItemService.exportExcel(request, resp);
    }

    public FbTurnstoreTrayItemDto toLazyDto(FbTurnstoreTrayItem entity) {
        return modelMapper.map(entity, FbTurnstoreTrayItemDto.class);
    }

    public FbTurnstoreTrayItemDto toEagerDto(FbTurnstoreTrayItem entity) {
        return modelMapper.map(entity, FbTurnstoreTrayItemDto.class);
    }

    public FbTurnstoreTrayItem fromDto(FbTurnstoreTrayItemDto dto) {
        return modelMapper.map(dto, FbTurnstoreTrayItem.class);
    }


}
