package com.sw.mes.fb.turnstore.service.impl;


import cn.hutool.core.exceptions.ExceptionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sw.mes.base.erp.entity.ErpOrder;
import com.sw.mes.base.erp.service.ErpOrderService;
import com.sw.mes.base.factory.inspect.dto.ProductRuleDto;
import com.sw.mes.base.factory.inspect.dto.ProductRuleValidateDto;
import com.sw.mes.base.factory.inspect.entity.ProductAssemblyRule;
import com.sw.mes.base.factory.inspect.entity.SquareSpellRule;
import com.sw.mes.base.factory.inspect.service.LevelCodeMaintenanceService;
import com.sw.mes.base.factory.inspect.service.ProductAssemblyRuleService;
import com.sw.mes.base.factory.inspect.service.SquareSpellRuleService;
import com.sw.mes.base.factory.material.entity.MaterialModel;
import com.sw.mes.base.factory.material.service.MaterialModelService;
import com.sw.mes.base.system.entity.DictionaryItem;
import com.sw.mes.base.system.entity.SysLog;
import com.sw.mes.base.system.service.DictionaryItemService;
import com.sw.mes.base.workinprocess.entity.PlanProcessingOrder;
import com.sw.mes.base.workinprocess.service.PlanProcessingOrderService;
import com.sw.mes.base.workinprocess.service.WipStorageService;
import com.sw.mes.core.api.system.YbFeignClients;
import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.CrucibleInfoDto;
import com.sw.mes.core.dto.DataInspectLogDto;
import com.sw.mes.core.dto.ItemDto;
import com.sw.mes.core.dto.erp.ErpAddResultDto;
import com.sw.mes.core.dto.erp.ErpRequestDto;
import com.sw.mes.core.dto.mes.pc.yb.YbInspectDataDto;
import com.sw.mes.core.dto.wms.MesasnDto;
import com.sw.mes.core.dto.wms.MesasnOrderDto;
import com.sw.mes.core.entity.FbCheckOutData;
import com.sw.mes.core.entity.MesToErpCcpEntity;
import com.sw.mes.core.entity.TurnstoreTrayItem;
import com.sw.mes.core.enums.EnumValues;
import com.sw.mes.core.enums.EnumValuesFactory;
import com.sw.mes.core.enums.ErpPypeEnum;
import com.sw.mes.core.enums.SectionIndexRangeEnum;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.*;
import com.sw.mes.fb.checkout.dao.FbCheckOutDataDao;
import com.sw.mes.fb.checkout.service.FbCheckOutDataService;
import com.sw.mes.fb.turnstore.dao.FbTurnstoreTrayDao;
import com.sw.mes.fb.turnstore.dto.*;
import com.sw.mes.fb.turnstore.entity.FbToMkWmsDetail;
import com.sw.mes.fb.turnstore.entity.FbTurnstoreTray;
import com.sw.mes.fb.turnstore.entity.FbTurnstoreTrayItem;
import com.sw.mes.fb.turnstore.entity.RawMaterialReceiving;
import com.sw.mes.fb.turnstore.service.FbTurnstoreTrayItemService;
import com.sw.mes.fb.turnstore.service.FbTurnstoreTrayService;
import com.sw.mes.fb.turnstore.service.RawMaterialReceivingService;
import com.sw.mes.plan.dto.MatchOrderInfo;
import com.sw.mes.plan.entity.FbProductionOrder;
import com.sw.mes.plan.service.FbProductionOrderService;
import com.sw.mes.rabbitmq.constants.RoutingKey;
import com.sw.mes.rabbitmq.enums.ExchangeEnum;
import com.sw.mes.rabbitmq.enums.QueueEnum;
import com.sw.mes.rabbitmq.service.RabbitMqService;
import com.sw.mes.sap.SapServer;
import com.sw.mes.third.utils.ErpRequestUtil;
import jodd.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.hibernate.hql.internal.CollectionSubqueryFactory;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.web.client.RestTemplate;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@Slf4j
@Service
@Transactional
public class FbTurnstoreTrayServiceImpl extends BaseServiceImpl<FbTurnstoreTray> implements FbTurnstoreTrayService {
    FbTurnstoreTrayDao dao;

    @Value("${spring.profiles.active}")
    private String active;

    @Autowired
    public void setDao(FbTurnstoreTrayDao dao) {
        super.setBaseDao(dao);
        this.dao = dao;
    }

    @Autowired
    private DictionaryItemService dictionaryItemService;

    @Autowired
    private FbTurnstoreTrayItemService fbTurnstoreTrayItemService;

    @Autowired
    RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private FbCheckOutDataService checkOutService;

    @Autowired
    private WipStorageService wipStorageService;

    @Autowired
    private RawMaterialReceivingService receivingService;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private PlanProcessingOrderService planProcessingOrderService;

    @Value("${wms.url}")
    private String wmsUrl;

    @Autowired
    private FbProductionOrderService productionOrderService;

    @Autowired
    private SapServer sapServer;

    @Autowired
    private RabbitMqService rabbitMqService;
    @Autowired
    private SquareSpellRuleService squareSpellRuleService;
    @Autowired
    private ProductAssemblyRuleService productAssemblyRuleService;
    @Autowired
    private FbCheckOutDataService fbCheckOutDataService;
    @Autowired
    private YbFeignClients ybFeignClients;

    @Autowired
    private ErpRequestUtil erpRequestUtil;
    @Value("${spring.erpServiceUrl}")
    private String erpServiceUrl;
    @Autowired
    private ErpOrderService erpOrderService;
    @Autowired
    private MaterialModelService materialModelService;
    @Autowired
    private FbCheckOutDataDao checkOutDao;

    @Autowired
    private LevelCodeMaintenanceService levelCodeMaintenanceService;

    @Autowired
    private ModelMapper modelMapper;

    @Autowired
    private ObjectMapper objectMapper;

    @Override
    public Result<FbInfoDto> getInfoByFbSectionNo(String fbSectionNo) {
        FbInfoDto fbInfoDto = new FbInfoDto();
        fbInfoDto.setFbSectionNo(fbSectionNo);
        //方棒检验数据
        FbCheckOutData dataByFbSectionNo = fbCheckOutDataService.findDataByFbSectionNo(fbSectionNo);
        if (null == dataByFbSectionNo) {
            return ResultGenerator.fail("方棒检验数据为空！");
        }
        if (null == dataByFbSectionNo.getYbSectionNo() || StringUtils.isEmpty(dataByFbSectionNo.getYbSectionNo())) {
            return ResultGenerator.fail("方棒检验数据中圆棒编号为空，无法获取圆棒相关信息");
        }

        //根据圆棒编号获取圆棒检验信息
        Result<YbInspectDataDto> inspectDataBaseBySectionNo = ybFeignClients.getInspectDataBaseBySectionNo(dataByFbSectionNo.getYbSectionNo());
        if (inspectDataBaseBySectionNo.getCode() == ResultCode.SUCCESS.getCode()) {
            if (null != inspectDataBaseBySectionNo.getData()) {
                YbInspectDataDto data = inspectDataBaseBySectionNo.getData();
                fbInfoDto.setYbSectionNo(dataByFbSectionNo.getYbSectionNo());
                fbInfoDto.setDopant(data.getDopant());
                fbInfoDto.setProductTypeSecond(data.getProductTypeSecond());
                fbInfoDto.setOxygenContentHead(data.getOxygenContentHead());
                fbInfoDto.setOxygenContentTail(data.getOxygenContentTail());
                fbInfoDto.setCarbonContentHead(data.getCarbonContentHead());
                fbInfoDto.setCarbonContentTail(data.getCarbonContentTail());
                fbInfoDto.setWCentreLife(data.getWCentreLife());
                fbInfoDto.setTCentreLife(data.getTCentreLife());
                fbInfoDto.setProductType(data.getProductType());
                fbInfoDto.setClosureIdentification(data.getClosureIdentification());
                fbInfoDto.setProductLevel(data.getProductLevel());
                if (null == dataByFbSectionNo.getCrystalNo() || StringUtils.isEmpty(dataByFbSectionNo.getCrystalNo())) {
                    fbInfoDto.setCrystalNo(data.getCrystalNo());
                    Result<YbInspectDataDto> inspectDatasByCrystalNo = ybFeignClients.getInspectDatasByCrystalNo(data.getCrystalNo());
                    if (inspectDatasByCrystalNo.getCode() == ResultCode.SUCCESS.getCode()) {
                        if (null != inspectDatasByCrystalNo.getData()) {
                            fbInfoDto.setWYCentreLife(inspectDatasByCrystalNo.getData().getWCentreLife());
                        }
                    }
                }
            }
        }
        return ResultGenerator.success(fbInfoDto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createTray(ProductAssemblyRule productRule, FbTurnstoreTray fbTurnstoreTray, List<FbTurnstoreTrayItem> items) {
        //生成托号和班次
        createTrayNoMk(fbTurnstoreTray, productRule);
        //保存主表
        fbTurnstoreTray.setUser(BaseContextHandler.getUsername());
        Integer tornieNum = productRule.getTornieNum();
        Date date = new Date();
        if (tornieNum == items.size()) {
            fbTurnstoreTray.setReportingTime(date);
        }
        fbTurnstoreTray.setTime(date);
        fbTurnstoreTray.setOperationTime(date);
        if (null == fbTurnstoreTray.getBorderWidth()){
            fbTurnstoreTray.setBorderWidth("0");
        }
        FbTurnstoreTray turnstoreTray = dao.save(fbTurnstoreTray);
        //保存明细字段
        saveItemList(turnstoreTray, items);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateTray(FbTurnstoreTray turnstoreTray, List<FbTurnstoreTrayItem> items) {
        //判断去向及是否满拖状态
        saveOrUpdate(turnstoreTray);
        //SET明细字段
        saveItemList(turnstoreTray, items);
    }

    @Override
    public Result lengthCheck(ProductRuleDto ruleDto) {
        ProductAssemblyRule ruleByStation = productAssemblyRuleService.findRuleByStation(ruleDto.getStation());
        if (null == ruleByStation) {
            return ResultGenerator.fail("该成品组托规则不存在");
        }
        String length = ruleByStation.getLength();
        if (length.equals("无限制")) {
            return ResultGenerator.success();
        }
        String[] str1 = length.split("-");
        List<String> list = Arrays.asList(str1);

        FbCheckOutData dataByFbSectionNo = fbCheckOutDataService.findDataByFbSectionNo(ruleDto.getSectionNo());
        BigDecimal actualLength = new BigDecimal(0);
        String sectionCompose = dataByFbSectionNo.getSectionCompose();
        if (null == sectionCompose) {
            actualLength = dataByFbSectionNo.getLength();
        } else {
            String[] split = sectionCompose.split(",");
            List<String> list1 = Arrays.asList(split);
            List<FbCheckOutData> checkOutList = checkOutDao.findCheckOutDataByFbSectionNoIns(list1);
            if (checkOutList == null || checkOutList.size() == 0) {
                return ResultGenerator.fail("错误,根据单晶编号获取方棒检验信息失败");
            }
            //计算拼棒总长
            actualLength = checkOutList.stream().map(e -> e.getLength() == null ? new BigDecimal(0) :
                    e.getLength()).reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        if (null == actualLength) {
            return ResultGenerator.fail(dataByFbSectionNo.getFbSectionNo() + "晶段长度为空");
        }
        if (actualLength.floatValue() >= Integer.parseInt(list.get(0)) && actualLength.floatValue() <= Integer.parseInt(list.get(1))) {
            return ResultGenerator.success();
        }

        return ResultGenerator.fail(dataByFbSectionNo.getFbSectionNo() + "晶段长度为" + actualLength.toString() + "不符合长度范围" + length);
    }

    @Override
    public String validateRules(ProductRuleDto ruleDto, FbCheckOutData checkOut) {
        String hun = "混类别";
        String sectionNo = checkOut.getFbSectionNo();
        //判断规格是否符合
        if (null != ruleDto.getSpecs() && !"".equals(ruleDto.getSpecs())) {

            if (null == checkOut.getSpecification() || !ruleDto.getSpecs().contains(checkOut.getSpecification())) {
                return sectionNo + "晶段的规格不符合规则，只能为：" + ruleDto.getSpecs();
            }
        }
        //判断导电型号是否符合
        if (null != ruleDto.getConductiveType() && !"".equals(ruleDto.getConductiveType())) {
            if (null == checkOut.getConductionType() || !ruleDto.getConductiveType().contains(checkOut.getConductionType())) {
                return sectionNo + "晶段的导电型号不符合规则，只能为：" + ruleDto.getConductiveType();
            }
        }
        //判断产品类别是否符合
        if (null != ruleDto.getProductType() && !"".equals(ruleDto.getProductType())) {
            if (null == checkOut.getProductType()) {
                return sectionNo + "晶段的产品类别不符合规则，只能为：" + ruleDto.getProductType();
            }
            String productType = ruleDto.getProductType();
            String[] split = productType.split(",");
            int count = 0;
            for (String s : split) {
                if (s.equals(checkOut.getProductType())) {
                    count++;
                }
            }
            if (count == 0) {
                return sectionNo + "晶段的产品类别不符合规则，只能为：" + ruleDto.getProductType();
            }
        }
        //判断产品等级是否符合
        if (null != ruleDto.getProductGrade() && !"".equals(ruleDto.getProductGrade()) && !hun.equals(ruleDto.getProductGrade())) {
            if (null == checkOut.getYbLevel() || !ruleDto.getProductGrade().contains(checkOut.getYbLevel())) {
                return sectionNo + "晶段的产品等级不符合规则，只能为：" + ruleDto.getProductGrade();
            }
        }


        //判断入库类别是否符合
        if (null != ruleDto.getInputType() && !"".equals(ruleDto.getInputType())) {
            if (null == checkOut.getProductLevelFb() || !ruleDto.getInputType().contains(checkOut.getProductLevelFb())) {
                return sectionNo + "晶段的入库类别不符合规则，只能为：" + ruleDto.getInputType();
            }
        }
        //判断边长KEY是否符合
        if (null != ruleDto.getSideLengthRange() && !"".equals(ruleDto.getSideLengthRange())) {
            if (null != checkOut.getSideLengthRange() && !"".equals(checkOut.getSideLengthRange()) && !ruleDto.getSideLengthRange().equals(checkOut.getSideLengthRange())) {

                return sectionNo + "晶段的边长不符合规则，只能为：" + ruleDto.getSideLengthRange();
            }
        }
        //判断长度范围是否符合
       /* if (null != ruleDto.getLength() && !"".equals(ruleDto.getLength()) && !"非标".equals(ruleDto.getLength())) {
            String length = ruleDto.getLength();
            String lengthType = "不区分棒长";
            if (!lengthType.equals(length) && null != checkOut.getLength()) {
                boolean flag = false;
                String[] lengths = length.split(",");
                for (String s : lengths) {
                    String[] split = s.split("-");
                    if (split.length != 2) {
                        return sectionNo + "晶段的长度范围规则不正确:" + length;
                    }
                    double minLength = Double.parseDouble(split[0]);
                    double maxLength = Double.parseDouble(split[1]);
                    double actualLength;
                    if (null != checkOut.getSpellBarLength()) {
                        actualLength = checkOut.getSpellBarLength().doubleValue();
                    } else {
                        actualLength = checkOut.getLength().doubleValue();
                    }
                    if (minLength <= actualLength && actualLength <= maxLength) {
                        flag = true;
                    }
                }
                if (!flag) {
                    return sectionNo + "晶段的长度范围不符合规则，只能为：" + ruleDto.getLength();
                }
            }
        }*/
        return "符合";
    }

    @Override
    public FbTurnstoreTray findByTrayNo(String trayNo) {
        return dao.findByTrayNo(trayNo);
    }

    @Override
    public List<FbTurnstoreTray> findByTime(String startTime, String endTime) {
        return dao.findByTime(startTime, endTime);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateItem(FbTurnstoreTray turnstoreTray, List<FbTurnstoreTrayItem> turnstoreTrayItemList) {
        for (FbTurnstoreTrayItem trayItem : turnstoreTrayItemList) {
            trayItem.setState(turnstoreTray.getState());
            trayItem.setOperationTime(turnstoreTray.getOperationTime());
            trayItem.setUser(turnstoreTray.getUser());
            trayItem.setIsToWms(turnstoreTray.getIsToWms());
            trayItem.setToWmsTime(turnstoreTray.getToWmsTime());
            //更新站点信息
            if ("方棒返修".equals(turnstoreTray.getHasGone())) {
                wipStorageService.outStation(trayItem.getSectionNo(), "D", null);
            } else if ("粘方棒".equals(turnstoreTray.getHasGone())) {
                wipStorageService.outStation(trayItem.getSectionNo(), "C", null);
            } else if ("综计WMS".equals(turnstoreTray.getHasGone()) || "综计返修".equals(turnstoreTray.getHasGone())) {
                //同步站点信息---WMS综计待进站
                wipStorageService.outStation(trayItem.getSectionNo(), "A", null);
            }
            fbTurnstoreTrayItemService.saveOrUpdate(trayItem);
        }
    }

    @Override
    public FbTurnstoreTray getFbTray(String trayNo) {
        FbTurnstoreTray byTrayNo = dao.findByTrayNo(trayNo);
        if (null != byTrayNo) {
            return byTrayNo;
        }
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result initiateReceipt(FbTurnstoreTray turnstoreTray, List<FbTurnstoreTrayItem> turnstoreTrayItemList) {
//        List<FbToMkWmsDetail> fbToMkWmsDetails = new ArrayList<>();
//        //查询是否处于当前站点
//        for (FbTurnstoreTrayItem trayItem : turnstoreTrayItemList) {
//            //重新匹配订单
//            if (!"退料".equals(turnstoreTray.getHasGone())) {
//                wipStorageService.selectWipStorage(trayItem.getSectionNo(), EnumValuesFactory.WipStorageCode.方棒检验.getKey(), true, EnumValuesFactory.PlanProcessingOrderStatus.组托.getKey());
//                // todo:匹配订单:根据料号重新匹配订单
//                MatchOrderInfo matchOrderInfo = productionOrderService.matchOrder(trayItem.getSectionNo(), trayItem.getMaterialNumber(), trayItem.getOrderType());
//                if (null != matchOrderInfo) {
//                    trayItem.setOrderNo(matchOrderInfo.getOrderNo());
//                    trayItem.setResistanceType(matchOrderInfo.getResistanceType());
//                    checkOutService.updateOrderNoBySectionNo(trayItem.getSectionNo(), matchOrderInfo.getOrderNo());
//                }
//            }
//            //组装传递对象
//            if ("综计WMS".equals(turnstoreTray.getHasGone()) || "综计返修".equals(turnstoreTray.getHasGone())) {
//                if (null == trayItem.getOrderNo()) {
//                    return ResultGenerator.fail(trayItem.getSectionNo() + "该晶段未匹配到订单号,请联系管理员处理");
//                }
//
//                FbToMkWmsDetail fbToMkWmsDetail = assembleData(trayItem, turnstoreTray, turnstoreTrayItemList);
//                fbToMkWmsDetails.add(fbToMkWmsDetail);
//            }
//        }
//
//        turnstoreTray.setOperationTime(new Date());
//        //根据去向调用不同接口
//        try {
//            if ("综计WMS".equals(turnstoreTray.getHasGone()) || "综计返修".equals(turnstoreTray.getHasGone())) {
//                //发送至WMS　　TODO 修改新的WMS接口　qe
//                turnstoreTray.setIsToWms(1);
//                turnstoreTray.setToWmsTime(new Date());
//                turnstoreTray.setState(0);
//
//
//                FbToMkWmsReceiptDto fbToMkWmsReceiptDto = new FbToMkWmsReceiptDto();
//                fbToMkWmsReceiptDto.setPlanNo(turnstoreTray.getId() + "");
//
//                fbToMkWmsReceiptDto.setDeliveryDT(new DateUtil().formatDateNotStatic(new Date()));
//                fbToMkWmsReceiptDto.setTrayNo(turnstoreTray.getTrayNo());
//                fbToMkWmsReceiptDto.setShift(turnstoreTray.getClassNo());
//                fbToMkWmsReceiptDto.setRule(turnstoreTray.getStation());
//                fbToMkWmsReceiptDto.setTest(turnstoreTray.getTestZt());
//                fbToMkWmsReceiptDto.setOperator(turnstoreTray.getUser());
//                fbToMkWmsReceiptDto.setOperateDT(new DateUtil().formatDateNotStatic(turnstoreTray.getOperationTime()));
//                fbToMkWmsReceiptDto.setRemark(turnstoreTray.getRemark());
//                //设置厂区
//                List<DictionaryItem> factoryCate = dictionaryItemService.findByThemeValueAndEnable("FactoryCate", true);
//                Map<String, DictionaryItem> factoryCateMap = factoryCate.stream().collect(Collectors.toMap(DictionaryItem::getValue, Function.identity(), (t1, t2) -> t1));
//                String factoryName = turnstoreTray.getFactoryName();
//                if (factoryCateMap.containsKey(factoryName)) {
//                    fbToMkWmsReceiptDto.setWorkShop(factoryCateMap.get(factoryName).getName());
//                }
//
//                fbToMkWmsReceiptDto.setDetail(fbToMkWmsDetails);
//                // 新增字段赋值
//                //对照边距
//                fbToMkWmsReceiptDto.setBorderWidth(turnstoreTray.getBorderWidth());
//                //配棒规则识别码
//                fbToMkWmsReceiptDto.setIdentifyingCode(turnstoreTray.getIdentifyingCode());
//                //配棒规则识别名称
//                fbToMkWmsReceiptDto.setIdentifyingName(turnstoreTray.getIdentifyingName());

//                //头尾部电阻率
//                FbTurnstoreTrayItem trayItem = turnstoreTrayItemList.get(0);
//                if (null != trayItem && null != trayItem.getTRadialCentre()) {
//                    fbToMkWmsReceiptDto.setHeadResistance(trayItem.getTRadialCentre().setScale(3));
//                }
//                if (null != trayItem && null != trayItem.getWRadialCentre()) {
//                    fbToMkWmsReceiptDto.setTailResistance(trayItem.getWRadialCentre().setScale(3));
//                }
                //调用接口
//                String jsonString = JSON.toJSONString(fbToMkWmsReceiptDto, SerializerFeature.WriteNullStringAsEmpty);
//                log.info("成品组托综计wms -- " + jsonString);
//                String url = wmsUrl + "api/OfflineProduct";
//                String send = null;
//                if (active.equals("prod")) {
//                    send = send(jsonString, url);
//                }
//
//                String requestName = "api/OfflineProduct";
//                String requestId = UUID.randomUUID().toString();
//                try {
//                    SysLog syslog = new SysLog();
//                    syslog.setCreateTime(new Date());
//                    syslog.setType("info");
//                    syslog.setCreateUserId(BaseContextHandler.getUserID());
//                    syslog.setCreateUserName("SYSTEM");
//                    syslog.setContent(jsonString);
//                    syslog.setResult(send);
//                    syslog.setMethodName(requestName);
//                    syslog.setStatus(1);
//                    syslog.setName("成品组托综计wms-成品发货出库");
//                    syslog.setServerAddr(url);
//                    syslog.setReqType("HTTP");
//                    syslog.setRequestId(requestId);
//                    syslog.setReqIp(IpUtils.getLocalIP());
//                    rabbitMqService.sendMessage(JSONObject.toJSONString(syslog), ExchangeEnum.SYS_LOG_EXCHANGE, QueueEnum.SYS_LOG_QUEUE, RoutingKey.SYS_LOG_KEY);
//                } catch (Exception e) {
//                    log.error(ExceptionUtil.stacktraceToString(e));
//                    e.printStackTrace();
//                }
//
//                if (null != send && send.contains("\"code\":0")) {
//                    return ResultGenerator.fail(send);
//                }
//
//                //跟新主表信息
//                saveOrUpdate(turnstoreTray);
//                //更新详情数据
//                updateItem(turnstoreTray, turnstoreTrayItemList);
//
//                //直接入库操作
//                //更新信息 更新组托表头信息
//                turnstoreTray.setConfirmUser(turnstoreTray.getUser());
//                turnstoreTray.setConfirmTime(new Date());
//                //根据状态进行操作
//                turnstoreTray.setState(1);
//                updateTurnstoreTrayStateById(null, turnstoreTray);
//
//                //更新组托明细信息
//                updateTrayItemList(turnstoreTray, turnstoreTrayItemList);
//
//
//                return ResultGenerator.success();
//            } else if ("方棒返修".equals(turnstoreTray.getHasGone()) || "粘方棒".equals(turnstoreTray.getHasGone())) {
//                FbTurnstoreTrayPostDto trayPostDto = new FbTurnstoreTrayPostDto();
//                trayPostDto.setFbTray(turnstoreTray);
//                trayPostDto.setTrayItems(turnstoreTrayItemList);
//                //发送至方棒
//                RawMaterialReceiving rawMaterialReceiving = receivingService.findByTrayNoAndState(turnstoreTray.getTrayNo(), 0);
//                if (null != rawMaterialReceiving) {
//                    return ResultGenerator.fail("已存在未确认状态的托号,不能重复入库");
//                }
//                //发送至方棒接收
//                receivingService.saveFbRawMaterial(trayPostDto);
//                turnstoreTray.setState(0);
//            } else {
//                //退料  修改状态并更新退料记录
//                for (FbTurnstoreTrayItem trayItem : turnstoreTrayItemList) {
//                    FbTurnstoreTrayItem turnstoreTrayItem = fbTurnstoreTrayItemService.findBySectionNoAndState(trayItem.getSectionNo(), 1);
//                    if (null != turnstoreTrayItem && "退料".equals(turnstoreTrayItem.getHasGone())) {
//                        return ResultGenerator.fail("该晶段已发起退料，不能重复退料,所在托号为: " + turnstoreTrayItem.getTrayNo());
//                    }
//                    //修改订单状态
//                    PlanProcessingOrder planProcessingOrder = planProcessingOrderService.findByCode(trayItem.getSectionNo());
//                    if (null == planProcessingOrder) {
//                        throw new BusinessException(trayItem.getSectionNo() + "工单号不存在");
//                    }
//                    planProcessingOrder.setStatus(EnumValuesFactory.PlanProcessingOrderStatus.已作废.getKey());
//                    planProcessingOrderService.saveOrUpdate(planProcessingOrder);
//                }
//                //修改状态
//                turnstoreTray.setState(1);
//            }
//            saveOrUpdate(turnstoreTray);
//            //更新详情数据
//            updateItem(turnstoreTray, turnstoreTrayItemList);
//        } catch (
//                Exception e) {
//            e.printStackTrace();
//            log.error(ExceptionUtil.stacktraceToString(e));
//            return ResultGenerator.fail(e.getMessage());
//        }
        BigDecimal totalWeight = turnstoreTrayItemList.stream()
                .filter(item -> item != null && item.getWeight() != null)
                .map(FbTurnstoreTrayItem::getWeight)
                .reduce(BigDecimal.ZERO, BigDecimal::add);


        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Date currentTime = new Date();
        String currentTimeString = formatter.format(currentTime);
        long warehouseEntryNumber = System.currentTimeMillis();
        FbTurnstoreTrayItem fbTurnstoreTrayItem = turnstoreTrayItemList.get(0);
        List<MesasnDto> mesasnDtoList = new ArrayList<>();
        MesasnDto dto = new MesasnDto();
        dto.setOrganizationid("JINKO");
        dto.setWarehouseid("5041");
        dto.setCustomerid("5041");
        dto.setDocno(String.valueOf(warehouseEntryNumber));//入库单号
        dto.setAsntype("OK");//入库类型
        dto.setAsnstatus("00");//订单状态
        dto.setAsnreferencea(turnstoreTray.getId().toString());
        dto.setAsnreferenceb("5041");//车间编号
        dto.setAsnreferencec("P501");//线边库存地
        dto.setAsncreationtime(currentTimeString);//MES创建时间
        dto.setTraceid(turnstoreTray.getTrayNo());//托盘号
        dto.setSerialno(turnstoreTray.getTrayNo());//箱号
        dto.setSku(fbTurnstoreTrayItem.getMaterialNumber());//物料编码
        dto.setExpectedqty(String.valueOf(totalWeight));//组件数量todo
        dto.setLotatt01(currentTimeString);//生产日期
        dto.setLotatt04(fbTurnstoreTrayItem.getBatchNo());//入库批次
        dto.setLotatt08("N");//质量状态
        dto.setLotatt09("5041");//厂家
        dto.setLotatt12(fbTurnstoreTrayItem.getBatchNo());//生产批次
        dto.setLotatt19(fbTurnstoreTrayItem.getPlanResistivity());//电阻率
        dto.setLotatt20(fbTurnstoreTrayItem.getProductLevel());//等级
        dto.setCreatesource("MES");//创建来源
        ArrayList<MesasnOrderDto> orderList = new ArrayList<>();
        for (FbTurnstoreTrayItem fbTurnstoreTrayItem1 : turnstoreTrayItemList){
            MesasnOrderDto mesasnOrderDto = new MesasnOrderDto();
            mesasnOrderDto.setSku(fbTurnstoreTrayItem1.getMaterialNumber());//物料编码
            mesasnOrderDto.setTraceid(turnstoreTray.getTrayNo());//托号
            mesasnOrderDto.setSerialno(fbTurnstoreTrayItem1.getSectionNo());//箱号
            mesasnOrderDto.setExpectedqty(String.valueOf(totalWeight));//数量
            mesasnOrderDto.setLotatt01(currentTimeString);//生产日期
            mesasnOrderDto.setLotatt03(currentTimeString);//生产日期
            mesasnOrderDto.setLotatt04(fbTurnstoreTrayItem1.getBatchNo());//入库批次
            mesasnOrderDto.setLotatt08("N");//质量状态
            mesasnOrderDto.setLotatt09("5041");//厂家
            mesasnOrderDto.setLotatt12(fbTurnstoreTrayItem1.getBatchNo());//生产批次
            mesasnOrderDto.setLotatt19(fbTurnstoreTrayItem1.getPlanResistivity());//电阻率
            mesasnOrderDto.setLotatt20(fbTurnstoreTrayItem1.getProductLevel());//等级
            mesasnOrderDto.setDedi07(fbTurnstoreTrayItem1.getTotalLength().toString());//整箱长度
            mesasnOrderDto.setDedi08(fbTurnstoreTrayItem1.getBatchNo());//MES下线批
            orderList.add(mesasnOrderDto);
        }
        dto.setOrderList(orderList);
        mesasnDtoList.add(dto);
        log.info("====:{}",JSONObject.toJSONString(mesasnDtoList));
        Result mesasn = ybFeignClients.mesasn(mesasnDtoList);
        if (!mesasn.getCode().equals(200)) {
            return ResultGenerator.fail("调用成品MES入库接口失败");
        }
        turnstoreTray.setState(1);
        turnstoreTray.setWarehouseEntryNumber(String.valueOf(warehouseEntryNumber));
        saveOrUpdate(turnstoreTray);
        return ResultGenerator.success();
    }

    private FbToMkWmsDetail assembleData(FbTurnstoreTrayItem trayItem, FbTurnstoreTray turnstoreTray, List<FbTurnstoreTrayItem> turnstoreTrayItemList) {
        FbToMkWmsDetail fbToMkWmsDetail = new FbToMkWmsDetail();
        fbToMkWmsDetail.setMaterialNo(trayItem.getMaterialNumber());
//头尾部电阻率
//        if (null != trayItem && null != trayItem.getTRadialCentre()){
//            fbToMkWmsDetail.setHeadResistance(trayItem.getTRadialCentre().setScale(3));
//        }
//        if (null != trayItem && null != trayItem.getWRadialCentre()){
//            fbToMkWmsDetail.setTailResistance(trayItem.getWRadialCentre().setScale(3));
//        }
        //头尾部电阻率,从圆棒检验数据库中取
        fbToMkWmsDetail.setValue11(Optional.of(trayItem).map(FbTurnstoreTrayItem::getLife).map(Objects::toString).orElse(""));
        fbToMkWmsDetail.setValue12(Optional.of(trayItem).map(FbTurnstoreTrayItem::getWLifeMin).map(Objects::toString).orElse(""));
        FbCheckOutData checkOutData = checkOutService.findDataByFbSectionNo(trayItem.getSectionNo());
        if (null != checkOutData) {
            if (null == checkOutData.getDiameterMax() || checkOutData.getDiameterMax().compareTo(BigDecimal.ZERO) == 0) {
                fbToMkWmsDetail.setMaxDiameter(checkOutData.getEapDiameterMax() == null ? BigDecimal.ZERO : checkOutData.getEapDiameterMax());
            } else {
                fbToMkWmsDetail.setMaxDiameter(checkOutData.getDiameterMax());
            }
        }

        Result<YbInspectDataDto> inspectDataBaseBySectionNo = ybFeignClients.getInspectDataBaseBySectionNo(trayItem.getSectionNo());
        YbInspectDataDto inspectDataDto = inspectDataBaseBySectionNo.getData();
        if (inspectDataBaseBySectionNo.getCode() == ResultCode.SUCCESS.getCode()) {
            if (null == inspectDataDto) {
                // FbCheckOutData checkOutData = checkOutService.findDataByFbSectionNo(trayItem.getSectionNo());
                if (null != checkOutData && !StringUtils.isEmpty(checkOutData.getYbSectionNo())) {
                    //用方棒检验数据中的圆棒单晶编号重新查询圆棒检验数据库
                    Result<YbInspectDataDto> inspectDataBaseBySectionNo2 = ybFeignClients.getInspectDataBaseBySectionNo(checkOutData.getYbSectionNo());
                    inspectDataDto = inspectDataBaseBySectionNo2.getData();
                }
            }
        } else {
            // FbCheckOutData checkOutData = checkOutService.findDataByFbSectionNo(trayItem.getSectionNo());
            if (null != checkOutData && !StringUtils.isEmpty(checkOutData.getYbSectionNo())) {
                //用方棒检验数据中的圆棒单晶编号重新查询圆棒检验数据库
                Result<YbInspectDataDto> inspectDataBaseBySectionNo2 = ybFeignClients.getInspectDataBaseBySectionNo(checkOutData.getYbSectionNo());
                inspectDataDto = inspectDataBaseBySectionNo2.getData();
            }
        }
        if (null != inspectDataDto) {
            if (null != inspectDataDto.getTRadialCentre()) {
                fbToMkWmsDetail.setHeadResistance(inspectDataDto.getTRadialCentre().setScale(3));
            }
            if (null != inspectDataDto.getWRadialCentre()) {
                fbToMkWmsDetail.setTailResistance(inspectDataDto.getWRadialCentre().setScale(3));
            }
            if (trayItem.getSectionNo().startsWith("B") && StringUtils.isEmpty(fbToMkWmsDetail.getValue11())) {
                fbToMkWmsDetail.setValue11(String.valueOf(inspectDataDto.getTCentreLife()));
            }
            if (trayItem.getSectionNo().startsWith("B") && StringUtils.isEmpty(fbToMkWmsDetail.getValue12())) {
                fbToMkWmsDetail.setValue12(String.valueOf(inspectDataDto.getWCentreLife()));
            }
        }
//        fbToMkWmsDetail.setProductDT(trayItem.getCheckTime());
        Result<ItemDto> itemDtoResult = ybFeignClients.queryItemByCode(trayItem.getMaterialNumber());
        ItemDto itemDto = itemDtoResult.getData();
        if (itemDtoResult.getCode() == ResultCode.SUCCESS.getCode()) {
            if (itemDto != null) {
                fbToMkWmsDetail.setSpecification(itemDto.getSpecification());
                fbToMkWmsDetail.setUnit(itemDto.getUnitName());
            }
        }
//        } else {
//            throw new BusinessException("物料主数据获取失败，请重试");
//        }

        fbToMkWmsDetail.setBatchNo(trayItem.getSectionNo());
        fbToMkWmsDetail.setValue1(Optional.of(trayItem).map(FbTurnstoreTrayItem::getSideLengthMax).map(Objects::toString).orElse(""));
        fbToMkWmsDetail.setValue2(Optional.of(trayItem).map(FbTurnstoreTrayItem::getSideLengthMin).map(Objects::toString).orElse("") + "");
        fbToMkWmsDetail.setValue3(Optional.of(trayItem).map(FbTurnstoreTrayItem::getDiameterMax).map(Objects::toString).orElse(""));
        fbToMkWmsDetail.setValue4(Optional.of(trayItem).map(FbTurnstoreTrayItem::getDiameterMin).map(Objects::toString).orElse(""));
        fbToMkWmsDetail.setValue5(Optional.of(trayItem).map(FbTurnstoreTrayItem::getLength).map(Objects::toString).orElse(""));
        fbToMkWmsDetail.setValue6(Optional.of(trayItem).map(FbTurnstoreTrayItem::getCollapseLossSh).map(Objects::toString).orElse("0"));
        fbToMkWmsDetail.setValue7(Optional.of(trayItem).map(FbTurnstoreTrayItem::getCollapseLossZh).map(Objects::toString).orElse("0"));
        fbToMkWmsDetail.setValue8(Optional.of(trayItem).map(FbTurnstoreTrayItem::getCollapseLossXia).map(Objects::toString).orElse("0"));
        //扣减崩损长度
        fbToMkWmsDetail.setValue9(Optional.of(trayItem).map(FbTurnstoreTrayItem::getCollapseLoss).map(Objects::toString).orElse("0"));
        //有效切片长度
        fbToMkWmsDetail.setValue10(Optional.of(trayItem).map(FbTurnstoreTrayItem::getEffectiveLength).map(Objects::toString).orElse(""));
        //  fbToMkWmsDetail.setValue11(Optional.of(trayItem).map(FbTurnstoreTrayItem::getLife).map(Objects::toString).orElse(""));
        //fbToMkWmsDetail.setValue12(Optional.of(trayItem).map(FbTurnstoreTrayItem::getWLifeMin).map(Objects::toString).orElse(""));

        Result<List<DataInspectLogDto>> result;
        DataInspectLogDto dto;
        //获取圆棒检验记录
//        result = ybFeignClients.getDataInspectLogBySectionNo(trayItem.getSectionNo());
//        if(ResultCode.SUCCESS.getCode() == result.getCode() && CollectionUtils.isNotEmpty(result.getData())){
//            dto = getDataInspectLogDto(result);
//        } else {
        String fromMark = trayItem.getSectionNo().substring(SectionIndexRangeEnum.基地追溯产品类型.getBeginIndex(), SectionIndexRangeEnum.基地追溯产品类型.getEndIndex());
        if (fromMark.matches("[A-Z]+")) {
            result = ybFeignClients.getDataInspectLog(trayItem.getSectionNo().substring(SectionIndexRangeEnum.毛棒到截断位数.getBeginIndex(), SectionIndexRangeEnum.毛棒到截断位数.getEndIndex()));
        } else {
            result = ybFeignClients.getDataInspectLog(trayItem.getSectionNo().substring(0, 13));
        }
        dto = getDataInspectLogDto(result);
//        }

        //获取反切圆棒检验记录
        DataInspectLogDto recutDto = getRecutDataInspectLogDto(result, trayItem.getSectionNo());
        if (trayItem.getSectionNo().startsWith("B")) {
            fbToMkWmsDetail.setValue13(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getOxygenContentHead).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue14(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getCarbonContentTail).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue15(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getOxygenContentHead).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue16(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getCarbonContentHead).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue17(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getOxygenContentTail).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue18(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getCarbonContentTail).map(Objects::toString).orElse("0"));
            fbToMkWmsDetail.setValue19(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getOxygenContentTail).map(Objects::toString).orElse("0"));
            fbToMkWmsDetail.setValue20(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getCarbonContentTail).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue21(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getTCentreLife).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue22(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getWCentreLife).map(Objects::toString).orElse(""));
        } else {
            fbToMkWmsDetail.setValue13(Optional.ofNullable(recutDto).map(DataInspectLogDto::getOxygenContentHead).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue14(Optional.ofNullable(recutDto).map(DataInspectLogDto::getCarbonContentTail).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue15(Optional.ofNullable(dto).map(DataInspectLogDto::getOxygenContentHead).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue16(Optional.ofNullable(dto).map(DataInspectLogDto::getCarbonContentHead).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue17(Optional.ofNullable(dto).map(DataInspectLogDto::getOxygenContentTail).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue18(Optional.ofNullable(dto).map(DataInspectLogDto::getCarbonContentTail).map(Objects::toString).orElse("0"));
            fbToMkWmsDetail.setValue19(Optional.ofNullable(recutDto).map(DataInspectLogDto::getOxygenContentTail).map(Objects::toString).orElse("0"));
            fbToMkWmsDetail.setValue20(Optional.ofNullable(recutDto).map(DataInspectLogDto::getCarbonContentTail).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue21(Optional.ofNullable(recutDto).map(DataInspectLogDto::getTCentreLife).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setValue22(Optional.ofNullable(dto).map(DataInspectLogDto::getWCentreLife).map(Objects::toString).orElse(""));
            fbToMkWmsDetail.setDeliveryNo(Optional.ofNullable(inspectDataDto).map(YbInspectDataDto::getLotNo).map(Objects::toString).orElse(""));
            //通过炉次号(发料编号)查询配料任务表的id,关联出 装料(配料)明细表 的坩埚信息
            if (!StringUtils.isEmpty(fbToMkWmsDetail.getDeliveryNo())) {
                Result<Map<String, Object>> crucibleInfoDtoResult = ybFeignClients.findCrucibleInfoByLotNo(fbToMkWmsDetail.getDeliveryNo());

                if (crucibleInfoDtoResult.getCode() == ResultCode.SUCCESS.getCode()) {
                    if (null != crucibleInfoDtoResult.getData() && !crucibleInfoDtoResult.getData().isEmpty()) {
                        fbToMkWmsDetail.setCrucibleNo(String.valueOf(crucibleInfoDtoResult.getData().get("batchNo")));
                        fbToMkWmsDetail.setCrucibleSupplier(String.valueOf(crucibleInfoDtoResult.getData().get("factoryBrand")));
                    }
                }
            }
        }
        fbToMkWmsDetail.setBatchNoGroup(trayItem.getSectionCompose());
        //查询数据字典拼接规格  D-10吋、B-9吋
        DictionaryItem byNameAndThemeId = dictionaryItemService.findByNameAndThemeId(trayItem.getSpecification(), 7);
        fbToMkWmsDetail.setSize(byNameAndThemeId.getExtendValue() + "-" + trayItem.getSpecification());
        //查询数据字典拼接掺杂剂S-镓、N-磷
        DictionaryItem byNameAndThemeId1 = dictionaryItemService.findByNameAndThemeId(trayItem.getDopant(), 9);
        fbToMkWmsDetail.setDopingClass(trayItem.getDopant() + "-" + byNameAndThemeId1.getExtendValue());
        //查询数据字典拼接产品类型，T-自有、H-H型、A-P型实验、B-H型实验、G-颗粒硅实验
        DictionaryItem byValueAndThemeId = dictionaryItemService.findByValueAndThemeIdAndEnable(trayItem.getProductType(), 17, true);
        //实测重量
        fbToMkWmsDetail.setWeight(trayItem.getTheoryWeightFb().toString());
        if (null == trayItem.getProductType()) {
            throw new BusinessException(trayItem.getSectionNo() + "产品类型为空");
        }
        //type一位直接取，两位取第二位，类型名称截取（）前面的
        String type;
        String name1 = "";
        if (trayItem.getProductType().length() <= 1) {
            type = trayItem.getProductType();
        } else {
            type = trayItem.getProductType().substring(1, 2);
        }
        if (byValueAndThemeId != null && StringUtil.isNotBlank(byValueAndThemeId.getName())) {
            String[] arr = byValueAndThemeId.getName().split("\\(|（");
            name1 = arr[0];
        }
//        fbToMkWmsDetail.setType(trayItem.getProductType() + "-" + byValueAndThemeId.getName());
        fbToMkWmsDetail.setType(type + "-" + name1);
        fbToMkWmsDetail.setResistanceClass(trayItem.getResistanceType());
        fbToMkWmsDetail.setLevelID(trayItem.getProductLevel());
        fbToMkWmsDetail.setProductDT(new DateUtil().formatDateNotStatic(trayItem.getCheckTime()));
        fbToMkWmsDetail.setCollaboratingRules(getSpellRuleName(trayItem, turnstoreTrayItemList));

        //设置长度系数
        //List<DictionaryItem> weightRatio = dictionaryItemService.findByThemeValueAndEnable("TheoreticalWeightCoefficientFb", true);
        //List<DictionaryItem> weightRatio = dictionaryItemService.findByThemeValueAndEnable("customer", true);
        //Map<String, DictionaryItem> weightMap = weightRatio.stream().collect(Collectors.toMap(DictionaryItem::getName, Function.identity(), (t1, t2) -> t1));
        ////String specification = trayItem.getSpecification();
        ////试验,长度折重系数新的逻辑，根据组托主表的customer，然后去主题customer字典map匹配，获取customer的对应值
        //String customer = turnstoreTray.getCustomer();
        //if (weightMap.containsKey(customer)){
        //    fbToMkWmsDetail.setLengthFactor(weightMap.get(customer).getValue());
        //}

        //长度系数重新赋值,根据字典multipleSideLogic来查找，0701
        List<DictionaryItem> multipleSideLogicItems = dictionaryItemService.findByThemeValueAndEnable("multipleSideLogic", true);
        Map<String, DictionaryItem> multipleSideLogicMap = multipleSideLogicItems.stream().collect(Collectors.toMap(DictionaryItem::getName, Function.identity(), (t1, t2) -> t1));
        String borderWidthKey = turnstoreTray.getBorderWidthKey();
//        if (multipleSideLogicMap.containsKey(borderWidthKey)) {
//            fbToMkWmsDetail.setLengthFactor(multipleSideLogicMap.get(borderWidthKey).getValue());
//        }
        String productType = trayItem.getProductType();
        for (DictionaryItem dictionaryItem : multipleSideLogicItems) {
            String name = dictionaryItem.getName();
            String extendValue2 = dictionaryItem.getExtendValue2();
            // 通过方棒边长和产品类别确定方棒折重系数
            if (name.equalsIgnoreCase(borderWidthKey) &&
                    !org.apache.commons.lang3.StringUtils.isEmpty(extendValue2) &&
                    extendValue2.equalsIgnoreCase(productType)) {
                fbToMkWmsDetail.setLengthFactor(dictionaryItem.getValue());
                break;
            }
        }
        if (StringUtils.isEmpty(fbToMkWmsDetail.getLengthFactor())) {
            throw new BusinessException(trayItem.getSectionNo() + " 没有匹配到方棒折重系数");
        }
        // 电阻类型
        if (!org.apache.commons.lang3.StringUtils.isEmpty(trayItem.getResistanceType()) && trayItem.getResistanceType().equals("/")) {
            fbToMkWmsDetail.setResistanceClass(trayItem.getResistanceType().replace("/", ""));
        } else {
            fbToMkWmsDetail.setResistanceClass(trayItem.getResistanceType());
        }

        //设置拼棒类型
        String sticksCount = turnstoreTray.getSticksCount();
        if (StringUtils.equals(sticksCount, "多拼")) {
            fbToMkWmsDetail.setCollarType("多拼");
        } else {
            int sticksNo = Integer.parseInt(sticksCount);
            if (sticksNo == 1) {
                fbToMkWmsDetail.setCollarType("单拼");
                fbToMkWmsDetail.setCollaboratingRules("单拼规则");
            } else if (sticksNo == 2) {
                fbToMkWmsDetail.setCollarType("双拼");
            } else if (sticksNo == 3) {
                fbToMkWmsDetail.setCollarType("三拼");
            } else {
                fbToMkWmsDetail.setCollarType("多拼");
            }
        }
        //设置拼棒组号
//        fbToMkWmsDetail.setBatchNoGroup(trayItem.getGroupNo()+"-"+trayItem.getFloorNo());
        fbToMkWmsDetail.setBatchNoGroup(trayItem.getBatchNoGroup());
        String no;
        if (trayItem.getGroupNo().length() == 1) {
            no = "0" + trayItem.getGroupNo();
        } else if (trayItem.getGroupNo().contains("-")) {
            String[] split = trayItem.getGroupNo().split("-");
            if (split[0].length() == 1) {
                no = "0" + split[0];
            } else {
                no = split[0];
            }
        } else {
            no = trayItem.getGroupNo();
        }
        fbToMkWmsDetail.setCollarGroupNo(turnstoreTray.getTrayNo() + no);
        return fbToMkWmsDetail;
    }

    private String getSpellRuleName(FbTurnstoreTrayItem trayItem, List<FbTurnstoreTrayItem> turnstoreTrayItemList) {
        String specification = trayItem.getSpecification();
        String productLevel = trayItem.getProductLevel();
        String productType = trayItem.getProductType();
        String conductionType = trayItem.getConductionType();


        Map<String, List<FbTurnstoreTrayItem>> collect = turnstoreTrayItemList.stream().collect(Collectors.groupingBy(FbTurnstoreTrayItem::getGroupNo));
        Set<Map.Entry<String, List<FbTurnstoreTrayItem>>> entries = collect.entrySet();
        BigDecimal lengthMin = new BigDecimal(0);
        BigDecimal lengthMax = new BigDecimal(0);
        int count = 1;
        for (Map.Entry<String, List<FbTurnstoreTrayItem>> entry : entries) {
            List<FbTurnstoreTrayItem> value = entry.getValue();
            BigDecimal totalLength = value.stream().map(e -> e.getLength() == null ? new BigDecimal(0) :
                    e.getLength()).reduce(BigDecimal.ZERO, BigDecimal::add);
            if (count == 1) {
                lengthMin = totalLength;
                lengthMax = totalLength;
            } else {
                if (totalLength.floatValue() < lengthMin.floatValue()) {
                    lengthMin = totalLength;
                }
                if (totalLength.floatValue() > lengthMax.floatValue()) {
                    lengthMax = totalLength;
                }
            }
            count++;
        }
        //最大直径
        BigDecimal diameterMax = turnstoreTrayItemList.stream().max(Comparator.comparing(FbTurnstoreTrayItem::getDiameterMax)).get().getDiameterMax();
        //最小直径
        BigDecimal diameterMin = turnstoreTrayItemList.stream().min(Comparator.comparing(FbTurnstoreTrayItem::getDiameterMin)).get().getDiameterMin();
        //计算直径差
        BigDecimal diameter = diameterMax.subtract(diameterMin);
        String ruleName = "";
        List<SquareSpellRule> squareSpellRules = squareSpellRuleService.querySpellRuleName(specification, productLevel, conductionType, productType);
        if (CollectionUtils.isNotEmpty(squareSpellRules)) {
            for (int j = 0; j < squareSpellRules.size(); j++) {
                SquareSpellRule squareSpellRule = squareSpellRules.get(j);
                String length = squareSpellRule.getLength();
                //判断长度范围
                if (length != null && !length.equals("")) {
                    String[] str = length.split(",");
                    for (String s : str) {
                        List<String> list;
                        String[] str1 = s.split("-");
                        list = Arrays.asList(str1);
                        if (lengthMin.floatValue() >= Integer.parseInt(list.get(0)) && lengthMax.floatValue() <= Integer.parseInt(list.get(1))) {
                            ruleName = squareSpellRule.getRuleName();
                            break;
                        }
                    }
                }
            }
        }

        return ruleName;
    }

    private DataInspectLogDto getRecutDataInspectLogDto(Result<List<DataInspectLogDto>> result, String sectionNo) {
        if (ResultCode.SUCCESS.getCode() == result.getCode()) {
            List<DataInspectLogDto> list = result.getData();
            Map<String, List<DataInspectLogDto>> collect1 = list.stream().collect(Collectors.groupingBy(DataInspectLogDto::getSectionNo));
//            List<DataInspectLogDto> collect = list.stream().filter(item -> item.getOperateType() == 0).collect(Collectors.toList());
            if (collect1.size() >= 2) { //如果检验记录有两个及以上的新增操作，说明有反切
//                return list.get(list.size()-1); //返回最后一条
                return list.get(0); //返回最后一条
            }
            if (collect1.size() == 1) {
                Set<Map.Entry<String, List<DataInspectLogDto>>> entries = collect1.entrySet();
                for (Map.Entry<String, List<DataInspectLogDto>> entry : entries) {
                    List<DataInspectLogDto> value = entry.getValue();
                    if (CollectionUtils.isNotEmpty(value)) {
                        if (value.get(0).getSectionNo().endsWith("Y") && !sectionNo.endsWith("Y")) {
                            return list.get(0);
                        }
                    }
                }
            }
        }
        return null;
    }

    private DataInspectLogDto getDataInspectLogDto(Result<List<DataInspectLogDto>> result) {
        if (ResultCode.SUCCESS.getCode() == result.getCode()) {
            List<DataInspectLogDto> list = result.getData();
            if (CollectionUtils.isNotEmpty(list)) {
                return list.get(0); //获取第一条
            }
        }
        return null;
    }

    @Override
    public void updateTurnstoreTrayStateById(WmsToFbReceiveDto receiveDto, FbTurnstoreTray turnstoreTray) {
        try {
            //更新主表
            saveOrUpdate(turnstoreTray);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public Result reportedErp(FbTurnstoreTray turnstoreTray, List<FbTurnstoreTrayItem> turnstoreTrayItemList) {

        Date date = new Date();
        boolean flag = false;
        FbProductionOrder fbProductionOrder = null;
        double sumWeight = 0D;
        double sumLength = 0D;
        String productType = "";
        for (FbTurnstoreTrayItem fbTurnstoreTrayItem : turnstoreTrayItemList) {
            // 上报成功修改ERP上报状态
            fbTurnstoreTrayItem.setReportedState(EnumValues.ReportedErp.已上报.getKey());
            /**
             * 物料名称、物料编码、工单编号、领用数量、仓库编码、领用类别、领料部门、ERP入库批次号、领用状态、上报时间
             */

            if (!flag) {
                fbProductionOrder = productionOrderService.findByAufnr(fbTurnstoreTrayItem.getOrderNo());
                flag = true;
            }
//            sumWeight += fbTurnstoreTrayItem.getWeight().doubleValue();
            sumLength += fbTurnstoreTrayItem.getLength().doubleValue();
            productType = fbTurnstoreTrayItem.getProductType();
        }
        MesToErpCcpEntity mesToErpCcpEntity = new MesToErpCcpEntity();
        // "cwhcode": "315", --仓库编码	从订单，常规：315，研发：316
        mesToErpCcpEntity.setCwhcode("018");
        // "crdcode": "10216", --出库类别/收发类别	常规：10216，研发：10201
        mesToErpCcpEntity.setCrdcode("10206");
        // "cdepcode": "1413", --部门编码	从订单获取
        mesToErpCcpEntity.setCdepcode("1202");
        // "cwhname": "回收原料库", --仓库名称
        mesToErpCcpEntity.setCwhname("开方产成品库（三期）");
        // "cdepname": "开方五车间",--部门名称	从订单获取
        mesToErpCcpEntity.setCdepname("开方一车间");
        if (null != fbProductionOrder) {
            // "iproorderid": "1000009317", --生产订单id	从订单获取
            mesToErpCcpEntity.setIproorderid(fbProductionOrder.getTid());
            // "cinvcode": "100100180103", --材料编码	从订单获取
            mesToErpCcpEntity.setCinvcode(fbProductionOrder.getMatnr());
            // "cmpocode": "2308011408-12吋(BNE01A1)(正泰045)", --订单号     合并汇总用的订单：erpOrderNo
            mesToErpCcpEntity.setCmpocode(fbProductionOrder.getAufnr());

            List<MaterialModel> materialModelByCode = materialModelService.findMaterialModelByCode(fbProductionOrder.getMatnr());
            if (!materialModelByCode.isEmpty()) {
                MaterialModel materialModel = materialModelByCode.get(0);
                // 材料名称  产成品使用母建，回收料使用子健 新增  cinvname
                mesToErpCcpEntity.setCinvname(materialModel.getName());
                //  单位 产成品使用母建，回收料使用子健 新增   cinvm_unit
                mesToErpCcpEntity.setCinvm_unit(materialModel.getUnit().getCode());
            }
            List<ErpOrder> byModIdAndInvCode1 = erpOrderService.findByModIdAndInvCode1(fbProductionOrder.getModid(), fbProductionOrder.getAufnr());
            if (!byModIdAndInvCode1.isEmpty()) {
                ErpOrder erpOrder = byModIdAndInvCode1.get(0);
                // 生产订单行号,目前填写1 产成品使用母建，回收料使用子健 新增  imoseq
                mesToErpCcpEntity.setImoseq(NumberUtil.object2String(erpOrder.getImoseq(), null));
                // 子建行号,目前填写1 产成品使用母建，回收料使用子健 新增    irowno
                mesToErpCcpEntity.setIrowno(NumberUtil.object2String(erpOrder.getIrowno(), null));
            }
            // 规格型号 产成品使用母建，回收料使用子健 新增 cinsted
//            mesToErpCcpEntity.setCinsted(fbProductionOrder.get);

            // 应入库数量，取订单数量 产成品使用母建，回收料使用子健 新增 inquantity todo 不确定
            mesToErpCcpEntity.setInquantity(NumberUtil.object2String(fbProductionOrder.getMenge(), null));

            // 入库记录ID 先不进行填写进行调试 产成品使用母建，回收料使用子健 新增 impoids todo
//            mesToErpCcpEntity.setImpoids();

            // 是否关联副产品 先不进行填写进行调试 产成品使用母建，回收料使用子健 新增    brelated
//            mesToErpCcpEntity.setBrelated();

            // 生产订单号 产成品使用母建，回收料使用子健 新增     cmocode
            mesToErpCcpEntity.setCmocode(fbProductionOrder.getAufnr());


        }
        // "cbatch": "(2)2023-8-16/129BP(A)正泰045", --批号
        mesToErpCcpEntity.setCbatch(turnstoreTray.getTrayNo());
        // "iquantity": "1.0000000000", --数量	同种订单产品合并汇总后的数量
        String borderWidthKey = turnstoreTray.getBorderWidthKey();
        List<DictionaryItem> multipleSideLogicItems = dictionaryItemService.findByThemeValueAndEnable("multipleSideLogic", true);
        Double v = null;
        if (!multipleSideLogicItems.isEmpty() && sumLength > 0) {
            for (DictionaryItem dictionaryItem : multipleSideLogicItems) {
                String name = dictionaryItem.getName();
                String extendValue2 = dictionaryItem.getExtendValue2();
                // 通过方棒边长和产品类别确定方棒折重系数
                if (name.equalsIgnoreCase(borderWidthKey) &&
                        !org.apache.commons.lang3.StringUtils.isEmpty(extendValue2) &&
                        extendValue2.contains(productType)) {
                    String[] split = extendValue2.split(",");
                    if (split.length > 0) {
                        for (int i = 0; i < split.length; i++) {
                            if (split[i].equalsIgnoreCase(productType)) {
                                v = MathUtils.keepDecimalDigits(sumLength / Double.parseDouble(dictionaryItem.getValue()));
                                break;
                            }
                        }
                    }
                    if (null != v) {
                        break;
                    }
                }
            }
        } else {
            return ResultGenerator.fail("数据字典中没有找到方棒系数");
        }
//                mesToErpCcpEntity.setIquantity(String.valueOf(sunWeight));
        if (null != v) {
            mesToErpCcpEntity.setIquantity(String.valueOf(v));
        } else {
            return ResultGenerator.fail("上报数据中，没有匹配到方棒系数");
        }
        // 备注
        mesToErpCcpEntity.setCmemo(turnstoreTray.getStation());
        mesToErpCcpEntity.setErpType(ErpPypeEnum.方棒产出.getKey());
        mesToErpCcpEntity.setDocTime(turnstoreTray.getToWmsTime());
        ErpRequestDto erpRequestDto = new ErpRequestDto();
        erpRequestDto.setMesToErpCcpEntity(mesToErpCcpEntity);
        erpRequestDto.setUrl(erpServiceUrl);
        ErpAddResultDto erpAddResultDto = erpRequestUtil.postToErp(erpRequestDto);
        if (null == erpAddResultDto || !erpAddResultDto.getErrcode().equals(String.valueOf(ResultCode.SUCCESS.getCode()))) {
            throw new RuntimeException("方棒产出上报失败：" + JSONObject.toJSONString(erpAddResultDto.getErrmsg()));
        }
        turnstoreTray.setReportedState(EnumValues.ReportedErp.已上报.getKey());
        // 回写ERP订单
        for (FbTurnstoreTrayItem trayItem : turnstoreTrayItemList) {
            trayItem.setErpOrder(erpAddResultDto.getCcode());
            trayItem.setErpTime(date);
        }
        fbTurnstoreTrayItemService.saveAll(turnstoreTrayItemList);
        turnstoreTray.setErpOrder(erpAddResultDto.getCcode());
        turnstoreTray.setErpTime(date);
        saveOrUpdate(turnstoreTray);

        return ResultGenerator.success();
    }

    @Override
    public Result validateRuleNew(ProductRuleValidateDto ruleDto) {
        if (null == ruleDto.getRuleId()) {
            return ResultGenerator.fail("组托规则未选择！");
        }
        if (StringUtils.isBlank(ruleDto.getSectionNo())) {
            return ResultGenerator.fail("单晶传入为空！");
        }
        //查询晶段信息
        FbCheckOutData checkOut = checkOutService.findDataByFbSectionNo(ruleDto.getSectionNo());
        if (null == checkOut || StringUtils.isBlank(checkOut.getYbLevel())) {
            return ResultGenerator.fail("当前晶段信息不存在或晶段暂未评级!");
        }
        if (StringUtils.isBlank(checkOut.getMaterialNumber())) {
            return ResultGenerator.fail("当前晶段信息缺失物料号, 请在方棒评级测试，是否为“正常”，氧含量是否合格!");
        }
        //晶棒重复扫描卡控
        FbTurnstoreTrayItem bySectionNo = fbTurnstoreTrayItemService.findBySectionNo2(ruleDto.getSectionNo());
        if (null != bySectionNo) {
            return ResultGenerator.fail("当前晶段已经执行过组托！请勿重复组托!");
        }
        ProductAssemblyRule productAssemblyRule = productAssemblyRuleService.get(ruleDto.getRuleId());
        if (null == productAssemblyRule) {
            return ResultGenerator.fail("组托规则为空!");
        }
        //获取等级码中的物料号
        List<String> materialCodeList = levelCodeMaintenanceService.findByLevelCodeAndModelAndSpecificationAndProductLevel(productAssemblyRule.getLevelCode(),
                productAssemblyRule.getConductiveType(), productAssemblyRule.getSpecs(), productAssemblyRule.getProductGrade());
        if (CollectionUtils.isEmpty(materialCodeList)) {
            return ResultGenerator.fail("等级码维护的维护的物料号为空!");
        }
        if (!materialCodeList.contains(checkOut.getMaterialNumber())) {
            return ResultGenerator.fail("单晶料号跟组托规则物料号不一致!");
        }
        if (!StringUtils.equals(checkOut.getProductLevel(), productAssemblyRule.getProductGrade())) {
            return ResultGenerator.fail("单晶检验产品等级与规则不一致!");
        }
        List<FbTurnstoreTrayItem> result = new ArrayList<>();
        FbTurnstoreTrayItem fbTurnstoreTrayItem = modelMapper.map(checkOut, FbTurnstoreTrayItem.class);
        if (StringUtils.isBlank(fbTurnstoreTrayItem.getSectionNo())) {
            fbTurnstoreTrayItem.setSectionNo(checkOut.getFbSectionNo());
        }
        result.add(fbTurnstoreTrayItem);
        return ResultGenerator.success(result);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateStateByTrayNo(UpdateStateByTrayNoDto dto) {
        FbTurnstoreTray fbTurnstoreTrayDb = dao.findByWarehouseEntryNumber(dto.getZinbd());
        Assert.notNull(fbTurnstoreTrayDb, "方棒组拖信息不存在");
        String wmsAuditResult;
        try {
            wmsAuditResult = objectMapper.writeValueAsString(dto);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
        if ("S".equals(dto.getZshjg())) {
            fbTurnstoreTrayDb.setState(1);
            fbTurnstoreTrayDb.setWmsAuditResult(wmsAuditResult);
        } else if ("C".equals(dto.getZshjg())) {
            fbTurnstoreTrayDb.setState(2);
            fbTurnstoreTrayDb.setWmsAuditResult(wmsAuditResult);
        }
        saveOrUpdate(fbTurnstoreTrayDb);
        List<FbTurnstoreTrayItem> fbTurnstoreTrayItemList = fbTurnstoreTrayItemService.findByTrayId(fbTurnstoreTrayDb.getId());
        if (CollectionUtils.isNotEmpty(fbTurnstoreTrayItemList)) {
            for (FbTurnstoreTrayItem fbTurnstoreTrayItem : fbTurnstoreTrayItemList) {
                fbTurnstoreTrayItem.setState(fbTurnstoreTrayDb.getState());
                fbTurnstoreTrayItemService.saveOrUpdate(fbTurnstoreTrayItem);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateTrayItemState(WmsToFbReceiveDto receiveDto, FbTurnstoreTray turnstoreTray, List<FbTurnstoreTrayItem> trayItemList) {
        //更新明细表
        ThreadManager.getThreadPollProxy().execute(() -> {
            try {
                updateTrayItemList(turnstoreTray, trayItemList);
            } catch (Exception e) {
                e.printStackTrace();
                //回滚表头信息
                turnstoreTray.setState(0);
                //todo 数据库字段只有50
                turnstoreTray.setRemark("失败原因：" + e.getMessage().substring(0, 40));
                saveOrUpdate(turnstoreTray);
            }
        });
    }

    /**
     * 更新组托明细相关内容
     *
     * @param turnstoreTray 组托主表
     * @param trayItemList  组表明细列表
     */
    public void updateTrayItemList(FbTurnstoreTray turnstoreTray, List<FbTurnstoreTrayItem> trayItemList) {
        List<FbTurnstoreTrayItem> turnstoreTrayItemGrs = new ArrayList<>();
        List<FbTurnstoreTrayItem> turnstoreTrayItemGrcs = new ArrayList<>();
        try {
            for (FbTurnstoreTrayItem trayItem : trayItemList) {
                //解锁组托状态
                PlanProcessingOrder planProcessingOrder = updatePlanOrder(trayItem.getSectionNo());
                if (1 == turnstoreTray.getState()) {
                    //成功--执行进站操作
                    wipStorageService.inStation(trayItem.getSectionNo(), null, false);
                    //完成组托状态
                    overPlanOrder(planProcessingOrder);
                    turnstoreTrayItemGrs.add(trayItem);
                } else if (3 == turnstoreTray.getState()) {
                    //拒收---执行退站操作
                    wipStorageService.exitStation(trayItem.getSectionNo());
                } else {
                    //冲销---出站到方棒检验进站状态
                    wipStorageService.outStation(trayItem.getSectionNo(), "A", EnumValuesFactory.WipStorageStatus.已进站.getKey());
                    turnstoreTrayItemGrcs.add(trayItem);
                }
                //将信息更新至明细
                trayItem.setState(turnstoreTray.getState());
                trayItem.setConfirmTime(turnstoreTray.getConfirmTime());
                trayItem.setConfirmUser(turnstoreTray.getConfirmUser());
                if (null != turnstoreTray.getRemark()) {
                    trayItem.setReturnRemark(turnstoreTray.getRemark());
                }
                fbTurnstoreTrayItemService.saveOrUpdate(trayItem);
            }
//            //发送GR信息
//            if (turnstoreTrayItemGrs.size() > 0) {
//                sapServer.sendGr(turnstoreTrayItemGrs);
//            }
//            //发送GR信息(冲销)
//            if (turnstoreTrayItemGrcs.size() > 0) {
//                sapServer.sendGrc(turnstoreTrayItemGrcs);
//            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 解锁组托状态
     *
     * @param sectionNo 单晶编号
     */
    public PlanProcessingOrder updatePlanOrder(String sectionNo) {
        //解锁组托状态
        PlanProcessingOrder planProcessingOrder = planProcessingOrderService.findByCode(sectionNo);
        if (null == planProcessingOrder) {
            throw new BusinessException(sectionNo + "工单号不存在");
        }
        planProcessingOrder.setStatus(EnumValuesFactory.PlanProcessingOrderStatus.加工中.getKey());
        planProcessingOrderService.saveOrUpdate(planProcessingOrder);
        return planProcessingOrder;
    }

    /**
     * 完成组托状态
     *
     * @param planProcessingOrder 单晶编号
     */
    public void overPlanOrder(PlanProcessingOrder planProcessingOrder) {
        //解锁组托状态
        planProcessingOrder.setStatus(EnumValuesFactory.PlanProcessingOrderStatus.入库完成.getKey());
        planProcessingOrderService.saveOrUpdate(planProcessingOrder);
    }

    @Override
    public List<FbTurnstoreTrayItem> groupItemList(FbCheckOutData checkOut) {
        List<FbTurnstoreTrayItem> trayItemList = new ArrayList<>();
        //如果是拼棒则带出所有晶段信息
        String split = ",";
        if (null != checkOut.getSectionCompose() && checkOut.getSectionCompose().contains(split)) {
            String[] sections = checkOut.getSectionCompose().split(split);
            if (sections.length > 0) {
                for (String sectionNo : sections) {
                    FbCheckOutData checkOutData = checkOutService.findDataByFbSectionNo(sectionNo);
                    if (null != checkOutData) {
                        trayItemList.add(packageItemList(checkOutData));
                    }
                }
            }
        } else {
            //组装当前晶段信息
            trayItemList.add(packageItemList(checkOut));
        }
        return trayItemList;
    }

    @Override
    public FbTurnstoreTrayItem groupItem(FbCheckOutData checkOutData) {
        return packageItemList(checkOutData);
    }


    private Result valudate(FbCheckOutData checkOut) {
        String sectionNo = checkOut.getFbSectionNo();
        if (null == checkOut.getMaterialNumber()) {
            return ResultGenerator.fail(sectionNo + ": 料号不能为空");
        } else if (null == checkOut.getWeight()) {
            return ResultGenerator.fail(sectionNo + ": 实际称重不能为空");
        } else if (null == checkOut.getLength()) {
            return ResultGenerator.fail(sectionNo + ": 硅棒总长不能为空");
        } else if (null == checkOut.getOrderNo()) {
            return ResultGenerator.fail(sectionNo + ": 订单号不能为空");
        } else if (null == checkOut.getProductLevel()) {
            return ResultGenerator.fail(sectionNo + ": 产品等级不能为空");
        } else if (null == checkOut.getProductLevelFb()) {
            return ResultGenerator.fail(sectionNo + ": 入库类别不能为空");
        } else if (null == checkOut.getProductType()) {
            return ResultGenerator.fail(sectionNo + ": 产品类别不能为空");
        }
        return ResultGenerator.success();
    }

    @Override
    public Result validateRuleAndData(ProductRuleDto ruleDto, FbCheckOutData checkOutData, String state) {
        //校验是否符合组托规则
        if ("综计WMS".equals(ruleDto.getHasGone()) || "综计返修".equals(ruleDto.getHasGone())) {
            String message = validateRules(ruleDto, checkOutData);
            if (!"符合".equals(message) && !message.contains("晶段的入库类别不符合规则")) {
                return ResultGenerator.fail(message);
            }
            if (!message.contains("晶段的入库类别不符合规则")) {
                state = "true";
            }
            //校验检验数据是否完整
            Result result = valudate(checkOutData);
            if (result.getCode() != ResultCode.SUCCESS.getCode()) {
                return ResultGenerator.fail(result.getMessage());
            }
        }
        //查询是否处于当前站点
        wipStorageService.selectWipStorage(checkOutData.getFbSectionNo(), EnumValuesFactory.WipStorageCode.方棒检验.getKey(), true);
        //查询是否存在
        List<FbTurnstoreTrayItem> trayItemList = fbTurnstoreTrayItemService.findBySectionNoValidate(checkOutData.getFbSectionNo());
        if (null != trayItemList && !trayItemList.isEmpty()) {
            for (FbTurnstoreTrayItem trayItem : trayItemList) {
                if (9 == trayItem.getState()) {
                    return ResultGenerator.fail("该晶段已进行组托，不能重复组托,所在托号为: " + trayItem.getTrayNo());
                } else if ("退料".equals(trayItem.getHasGone()) && 1 == trayItem.getState()) {
                    return ResultGenerator.fail("该晶段已发起退料且接收，不能重复组托,所在托号为: " + trayItem.getTrayNo());
                }
            }
        }
        //组装组托详情
        return ResultGenerator.success(state, groupItem(checkOutData));
    }

    /**
     * 组装详情列表
     *
     * @param checkOut
     */
    private FbTurnstoreTrayItem packageItemList(FbCheckOutData checkOut) { //TODO 打托信息上传接口
        FbTurnstoreTrayItem turnstoreTrayItem = new FbTurnstoreTrayItem();
        BeanUtil.copyProperties(checkOut, turnstoreTrayItem, BeanUtil.getIgnoreProperties(checkOut, null, null));

        //单晶组合
//        if (null == checkOut.getSectionCompose()) {
//            turnstoreTrayItem.setSectionCompose(sectionNo + ";");
//        } else {
//            //替换
//            String replaceAll = checkOut.getSectionCompose().replaceAll(",", ";");
//            turnstoreTrayItem.setSectionCompose(replaceAll + ";");
//        }
        //组托  产品等级  取 方棒的产品等级
        turnstoreTrayItem.setProductLevel(checkOut.getProductLevel());

//        //组托  生产日期  取 方棒检验时间
//        turnstoreTrayItem.setCheckTime(checkOut.getCheckTime());
//
//        //组托  物料编码  取 方棒物料编码
//        turnstoreTrayItem.setPlanMaterialCode(checkOut.getPlanMaterialCode());

//        turnstoreTrayItem.setSpecification(checkOut.getSpecification());

        if (checkOut.getLength() != null && checkOut.getCollapseLossSh() != null && checkOut.getCollapseLossZh() != null && checkOut.getCollapseLossXia() != null) {
            turnstoreTrayItem.setEffectiveLength(checkOut.getLength().subtract(checkOut.getCollapseLossSh()).subtract(checkOut.getCollapseLossZh()).subtract(checkOut.getCollapseLossXia()));
        }
        turnstoreTrayItem.setFbExperiLevel(checkOut.getExperiLevel());
        turnstoreTrayItem.setCustomerType(checkOut.getYbLevel());
        turnstoreTrayItem.setBevel(checkOut.getLengthSlope());
        turnstoreTrayItem.setOrderNo(checkOut.getOrderNo());
        turnstoreTrayItem.setDopant(checkOut.getDopant());
        //组托  单晶编号  取 方棒的单晶编号
        String sectionNo = checkOut.getFbSectionNo();
        turnstoreTrayItem.setSectionNo(sectionNo);

        //边距max
        if (null != checkOut.getSideLengthMax() && checkOut.getSideLengthMax().compareTo(BigDecimal.ZERO) > 0) {
            turnstoreTrayItem.setSideLengthMax(checkOut.getSideLengthMax());
        } else {
            turnstoreTrayItem.setSideLengthMax(checkOut.getEapSideMax());
        }
        //边距min
        if (null != checkOut.getSideLengthMin() && checkOut.getSideLengthMin().compareTo(BigDecimal.ZERO) > 0) {
            turnstoreTrayItem.setSideLengthMin(checkOut.getSideLengthMin());
        } else {
            turnstoreTrayItem.setSideLengthMin(checkOut.getEapSideMin());
        }
        //直径max
        if (null != checkOut.getDiameterMax() && checkOut.getDiameterMax().compareTo(BigDecimal.ZERO) > 0) {
            turnstoreTrayItem.setDiameterMax(checkOut.getDiameterMax());
        } else {
            turnstoreTrayItem.setDiameterMax(checkOut.getEapDiameterMax());
        }
        //直径min
        if (null != checkOut.getDiameterMin() && checkOut.getDiameterMin().compareTo(BigDecimal.ZERO) > 0) {
            turnstoreTrayItem.setDiameterMin(checkOut.getDiameterMin());
        } else {
            turnstoreTrayItem.setDiameterMin(checkOut.getEapDiameterMin());
        }

//        //工厂
//        String factory = sectionNo.substring(1, 2);
//        DictionaryItem factoryCate = dictionaryItemService.findByThemeValueAndValueAndEnable("FactoryCate", factory, true);
//        if (null != factoryCate) {
//            factory = factoryCate.getName();
//        }
//        turnstoreTrayItem.setFactoryName(factory);
//        //库存地点
//        turnstoreTrayItem.setStockPlace("Q003");
//        if (checkOut.getSticksCount() > 1 && null != checkOut.getSpellBarLength()) {
//            turnstoreTrayItem.setTotalLength(checkOut.getSpellBarLength());
//        }


        turnstoreTrayItem.setSquareRodRemark(checkOut.getSquareRodRemarks());
        //获取圆棒检验数据
        Result<YbInspectDataDto> ybInspectDataDtoResult;
        YbInspectDataDto ybInspectDataDto;
        ybInspectDataDtoResult = ybFeignClients.getInspectDataBaseBySectionNo(checkOut.getYbSectionNo());
        ybInspectDataDto = ybInspectDataDtoResult.getData();
        if (null == ybInspectDataDto) {
            String fromMark = checkOut.getYbSectionNo().substring(SectionIndexRangeEnum.基地追溯产品类型.getBeginIndex(), SectionIndexRangeEnum.基地追溯产品类型.getEndIndex());
            if (fromMark.matches("[A-Z]+")) {
                ybInspectDataDtoResult = ybFeignClients.accordSquareGetInspect(checkOut.getYbSectionNo().substring(SectionIndexRangeEnum.毛棒到截断位数.getBeginIndex(), SectionIndexRangeEnum.毛棒到截断位数.getEndIndex()));
            } else {
                ybInspectDataDtoResult = ybFeignClients.accordSquareGetInspect(checkOut.getYbSectionNo().substring(0, 13));
            }
            ybInspectDataDto = ybInspectDataDtoResult.getData();
        }
        if (ybInspectDataDtoResult.getCode() == ResultCode.SUCCESS.getCode()) {
            if (null != ybInspectDataDto) {
//                turnstoreTrayItem.setAgainNum(ybInspectDataDto.getAgainNum());
//                turnstoreTrayItem.setCrystalWorkshop(ybInspectDataDto.getCrystalNo().substring(1, 2));
//                turnstoreTrayItem.setHeadDefectGrade(ybInspectDataDto.getHeadDefectGrade());
//                turnstoreTrayItem.setHeadTailRecognition(ybInspectDataDto.getHeadTailRecognition());
//                turnstoreTrayItem.setTLifeMin(ybInspectDataDto.getTCentreLife());
//                turnstoreTrayItem.setWLifeMin(ybInspectDataDto.getWCentreLife());
//                turnstoreTrayItem.setTCentreLife(ybInspectDataDto.getTCentreLife());
                turnstoreTrayItem.setWCentreLife(ybInspectDataDto.getWCentreLife());
//                turnstoreTrayItem.setOxygenContentHead(ybInspectDataDto.getOxygenContentHead());
                turnstoreTrayItem.setOxygenContentHead(checkOut.getHeadOxygenContents());
                turnstoreTrayItem.setCarbonContentTail(ybInspectDataDto.getCarbonContentTail());
                //尾中心少子寿命
                turnstoreTrayItem.setWLifeMin(ybInspectDataDto.getWCentreLife());
                //头中心少子寿命
                turnstoreTrayItem.setLife(ybInspectDataDto.getTCentreLife());

//                turnstoreTrayItem.setCarbonContentHead(ybInspectDataDto.getCarbonContentHead());
//                turnstoreTrayItem.setCarbonContentTail(ybInspectDataDto.getCarbonContentTail());
//                turnstoreTrayItem.setCheckTime(ybInspectDataDto.getCreateTime());
//                turnstoreTrayItem.setTRadialCentre(ybInspectDataDto.getTRadialCentre());
//                turnstoreTrayItem.setTRadialEdge(ybInspectDataDto.getTRadialEdge1());
//                turnstoreTrayItem.setWRadialCentre(ybInspectDataDto.getWRadialCentre());
//                turnstoreTrayItem.setWRadialEdge(ybInspectDataDto.getWRadialEdge1());
            }
        } else {
            throw new BusinessException("圆棒检验数据获取失败，请重试");
        }
        return turnstoreTrayItem;
    }

    /**
     * 生成托号
     *
     * @param productRule     组托规则
     * @param fbTurnstoreTray 组托信息
     */
    public void createTrayNo(ProductAssemblyRule productRule, FbTurnstoreTray fbTurnstoreTray) {
        //规则匹配托号
        StringBuilder trayNo = new StringBuilder();
        String order3Num = null;
        String order5Num = null;
        String num3Value;
        String num5Value;
        String key5 = "FBOrderNum5";
        String key3 = "FBOrderNum3";

        //日期获取
        long time = new DateUtil().getChangeShiftsTime().getTime();
        Calendar cal = Calendar.getInstance();
        if (System.currentTimeMillis() < time) {
            cal.add(Calendar.DATE, -1);
        } else {
            cal.add(Calendar.DATE, 0);
        }
        String date = new SimpleDateFormat("yyMMdd").format(cal.getTime());
        if ("综计WMS".equals(fbTurnstoreTray.getHasGone()) || "综计返修".equals(fbTurnstoreTray.getHasGone())) {
            String spacer = ",";
            String workShop = "六期方棒";
            //第一位:厂别:Y
            DictionaryItem shopValue;
            shopValue = dictionaryItemService.findByThemeValueAndNameAndEnable("workShop", workShop, true);
            if (null != shopValue) {
                trayNo.append(shopValue.getValue());
            } else {
                trayNo.append("L");
            }
            //第二位:整拼类型
            DictionaryItem pinValue;
            if (null == productRule.getSticksCount()) {
                String pin = "混拼";
                pinValue = dictionaryItemService.findByThemeValueAndNameAndEnable("spell", pin, true);
            } else {
                String type = productRule.getSticksCount() + "拼";
                pinValue = dictionaryItemService.findByThemeValueAndNameAndEnable("spell", type, true);
                fbTurnstoreTray.setSticksCount(productRule.getSticksCount());
            }
            if (null != pinValue) {
                trayNo.append(pinValue.getValue());
            } else {
                trayNo.append("Z");
            }
            //第三位:棒长
            String lengthType = "不区分棒长";
            String dtoLength = productRule.getLength();
            DictionaryItem lengthValue;
            if (lengthType.equals(dtoLength)) {
                lengthValue = dictionaryItemService.findByThemeValueAndNameAndEnable("rodLength", lengthType, true);
            } else if (dtoLength.contains(spacer)) {
                lengthValue = dictionaryItemService.findByThemeValueAndNameAndEnable("rodLength", "800-900", true);
            } else {
                lengthValue = dictionaryItemService.findByThemeValueAndNameAndEnable("rodLength", dtoLength, true);
            }
            if (null != lengthValue) {
                trayNo.append(lengthValue.getValue());
            } else {
                trayNo.append("0");
            }
            //第四位:规格型号
            String spec = "转规格";
            String specs = productRule.getConductiveType() + "型" + productRule.getSpecs().substring(0, 3);
            DictionaryItem specValue;
            if (specs.contains("转")) {
                specValue = dictionaryItemService.findByThemeValueAndNameAndEnable("specMod", spec, true);
            } else {
                specValue = dictionaryItemService.findByThemeValueAndNameAndEnable("specMod", specs, true);
            }
            if (null != specValue) {
                trayNo.append(specValue.getValue());
            } else {
                trayNo.append("Z");
            }
            //第五位:产品等级
            String grade = "混类别";
            String productGrade = productRule.getProductGrade();
            DictionaryItem gradeValue;
            if (productGrade.contains(spacer) || productGrade.contains(grade)) {
                gradeValue = dictionaryItemService.findByThemeValueAndNameAndEnable("productRat", grade, true);
            } else {
                gradeValue = dictionaryItemService.findByThemeValueAndNameAndEnable("productRat", productGrade, true);
            }
            if (null != gradeValue) {
                trayNo.append(gradeValue.getValue());
            } else {
                trayNo.append("Z");
            }
            //第六-十一位:日期
            trayNo.append(date);
            //第十二-十四位:顺序号
            if (redisTemplate.hasKey(key3)) {
                //先获取redis中的顺序号
                num3Value = (String) redisTemplate.opsForValue().get(key3);
                int num3 = Integer.parseInt(num3Value);
                //如果数据库最大值-缓存=1,则+2,否则+1
                Integer num = dao.findOrderNumByWMS();
                if (null != num && num - num3 == 1) {
                    order3Num = String.valueOf(num3 + 2);
                } else {
                    order3Num = String.valueOf(num3 + 1);
                }
                if (order3Num.length() == 1) {
                    order3Num = "00" + order3Num;
                } else if (order3Num.length() == 2) {
                    order3Num = "0" + order3Num;
                }
            } else {
                order3Num = "001";
            }
            trayNo.append(order3Num);
            /*//第十五位:实验等级
            DictionaryItem experValue;
            if (null == productRule.getExperGrade() || productRule.getExperGrade().contains(spacer)) {
                experValue = dictionaryItemService.findByThemeValueAndNameAndEnable("tempSign", grade, true);
            } else {
                experValue = dictionaryItemService.findByThemeValueAndNameAndEnable("tempSign", productRule.getExperGrade(), true);
            }
            if (null != experValue) {
                trayNo.append(experValue.getValue());
            } else {
                trayNo.append("Z");
            }
            //第十六位:入库类别
            DictionaryItem categoryValue;
            if (null == productRule.getInputType() || productRule.getInputType().contains(spacer)) {
                trayNo.append("Z");
            } else {
                categoryValue = dictionaryItemService.findByThemeValueAndNameAndEnable("storType", productRule.getInputType(), true);
                if (null != categoryValue) {
                    trayNo.append(categoryValue.getValue());
                } else {
                    trayNo.append("Z");
                }
            }*/
            if ("六期".equals(fbTurnstoreTray.getFactoryName())) {
                fbTurnstoreTray.setFactoryName("1031");
            }
        } else {
            trayNo.append("F").append(date);
            //其他去向的托号按照流水号生成（年月日+五位流水）
            //Integer num = dao.findOrderNumBy();
            if (redisTemplate.hasKey(key5)) {
                //先获取redis中的顺序号
                num5Value = (String) redisTemplate.opsForValue().get(key5);
                order5Num = String.valueOf(Integer.parseInt(num5Value) + 1);
                if (order5Num.length() == 1) {
                    order5Num = "0000" + order5Num;
                } else if (order5Num.length() == 2) {
                    order5Num = "000" + order5Num;
                } else if (order5Num.length() == 3) {
                    order5Num = "00" + order5Num;
                } else if (order5Num.length() == 4) {
                    order5Num = "0" + order5Num;
                }
            } else {
                order5Num = "00001";
            }
            trayNo.append(order5Num);
        }
        //判断拖号是否重复
        if (null != dao.checkTrayNo(trayNo.toString())) {
            throw new RuntimeException("错误，生成的拖号已存在");
        }
        fbTurnstoreTray.setTrayNo(trayNo.toString());
        //更新缓存
        //每天08:59:59秒重置
        long timeLong;
        if (System.currentTimeMillis() < time) {
            //小于当天08:59:59,计算至当天08:59:59
            timeLong = time - System.currentTimeMillis();
        } else {
            //大于当天08:59:59,计算至第二天08:59:59点
            timeLong = new DateUtil().getCurrentDayEndTime().getTime() + 32400000 - System.currentTimeMillis();
        }
        if (timeLong > 1000) {
            if (null != order3Num) {
                fbTurnstoreTray.setOrderNum(Integer.parseInt(order3Num));
                redisTemplate.opsForValue().set(key3, order3Num, timeLong / 1000, TimeUnit.SECONDS);
            } else {
                fbTurnstoreTray.setOrderNum(Integer.parseInt(order5Num));
                redisTemplate.opsForValue().set(key5, order5Num, timeLong / 1000, TimeUnit.SECONDS);
            }
        }
    }

    /**
     * 生成托号
     *
     * @param productRule     组托规则
     * @param fbTurnstoreTray 组托信息
     */
    public void createTrayNoMk(FbTurnstoreTray fbTurnstoreTray, ProductAssemblyRule productRule) {
        //规则匹配托号
        //P222080707   P写死 1，2，3是从工厂数据过来，时间就是当前的年月，流水号：就是当前的组托顺序，保留4位，最高是9999，查询最近一次的组托流水码

        //根据月日构建流水号的key
        String key4 = "FBOrderNum4";
        long nyTime = System.currentTimeMillis() - 30600000;
        String nyTimeStr = new SimpleDateFormat("MM").format(nyTime);
        key4 = key4 + nyTimeStr;

        //关于当天的流水号标志
        String keyDt = "FBOrderNum4DT";

        StringBuilder trayNo = new StringBuilder();
        StringBuilder classNo = new StringBuilder();
        //白班标志
        String bbbz = "白班";
        trayNo.append("Y");

        //工厂名称/dictionaryItem
        String factoryName = fbTurnstoreTray.getFactoryName();
        trayNo.append(factoryName);

        //日期获取
        long time = new DateUtil().getChangeShiftsTime().getTime();
        Calendar cal = Calendar.getInstance();
        if (System.currentTimeMillis() < time) {
            cal.add(Calendar.DATE, -1);
            bbbz = "夜班";
        } else {
            cal.add(Calendar.DATE, 0);
        }
        String date = new SimpleDateFormat("yyMM").format(cal.getTime());
        trayNo.append(date);
        String classDate = new SimpleDateFormat("MMdd").format(cal.getTime());
        String classDateTime = new SimpleDateFormat("HH:mm").format(cal.getTime());
        classNo.append(classDate);
        classNo.append(bbbz);
        classNo.append("第");

        //流水号
        String order4Num = null;
        String num4Value;
        //当天流水号
        String order4NumDt = null;
        String num4ValueDt;

        String num = null;
        if (redisTemplate.hasKey(key4)) {
            //先获取redis中的顺序号
            num4Value = (String) redisTemplate.opsForValue().get(key4);
            order4Num = String.valueOf(Integer.parseInt(num4Value) + 1);

            if (order4Num.length() == 1) {
                order4Num = "000" + order4Num;
            } else if (order4Num.length() == 2) {
                order4Num = "00" + order4Num;
            } else if (order4Num.length() == 3) {
                order4Num = "0" + order4Num;
            }
        } else {
            order4Num = "0001";
            //月初进行刷新
            String ordKey = "FBOrderNum4" + new DateUtil().getLastMonth();
            redisTemplate.delete(ordKey);
        }

        //当天流水号叠加
        if (redisTemplate.hasKey(keyDt)) {
            num4ValueDt = (String) redisTemplate.opsForValue().get(keyDt);
            order4NumDt = String.valueOf(Integer.parseInt(num4ValueDt) + 1);
            num = order4NumDt;
        } else {
            num = "1";
        }

        trayNo.append(order4Num);
        classNo.append(num);
        classNo.append("托");
        classNo.append(classDateTime);
        //判断拖号是否重复
        if (null != dao.checkTrayNo(trayNo.toString())) {
            throw new RuntimeException("错误，生成的拖号已存在");
        }

        //判断当天流水号是否重复
        if (null != dao.checkClassNo(classNo.toString())) {
            throw new RuntimeException("错误，生成的拖号已存在");
        }

        fbTurnstoreTray.setTrayNo(trayNo.toString());
        fbTurnstoreTray.setClassNo(classNo.toString());

        if (productRule.getStation() != null) {
            String[] radeArr = productRule.getSpecs().split("[*]");
            if (radeArr.length > 1) {
                String spc = radeArr[1];
                fbTurnstoreTray.setPrintBz(spc + "-" + num);
            }
        }

        redisTemplate.opsForValue().set(key4, order4Num);

        long timeLong;
        if (System.currentTimeMillis() < time) {
            //小于当天08:59:59,计算至当天08:59:59
            timeLong = time - System.currentTimeMillis();
        } else {
            //大于当天08:59:59,计算至第二天08:59:59点
            timeLong = new DateUtil().getCurrentDayEndTime().getTime() + 30600000 - System.currentTimeMillis();
        }
        if (timeLong > 1000) {
            fbTurnstoreTray.setOrderNum(Integer.parseInt(num));
            redisTemplate.opsForValue().set(keyDt, num, timeLong / 1000, TimeUnit.SECONDS);
        }
    }

    private void saveItemList(FbTurnstoreTray fbTurnstoreTray, List<FbTurnstoreTrayItem> items) {
        if (null == fbTurnstoreTray.getState()) {
            fbTurnstoreTray.setState(0);
        }
        for (int i = 0; i < items.size(); i++) {
            /*if (9 == fbTurnstoreTray.getState()) {
                items.get(i).setState(fbTurnstoreTray.getState());
            }*/
            FbTurnstoreTrayItem fbTurnstoreTrayItem = items.get(i);
            //序号
            if (null == fbTurnstoreTrayItem.getOrderNum()) {
                fbTurnstoreTrayItem.setOrderNum(i + 1);
            }
            //空操作人赋值
            if (null == fbTurnstoreTrayItem.getUser()) {
                if (null != fbTurnstoreTray.getUpdateUser()) {
                    fbTurnstoreTrayItem.setUser(fbTurnstoreTray.getUpdateUser());
                } else {
                    fbTurnstoreTrayItem.setUser(fbTurnstoreTray.getUser());
                }
            }
            fbTurnstoreTrayItem.setTrayId(fbTurnstoreTray.getId());
            fbTurnstoreTrayItem.setTrayNo(fbTurnstoreTray.getTrayNo());
            if (null != fbTurnstoreTray.getToWmsTime()) {
                fbTurnstoreTrayItem.setToWmsTime(fbTurnstoreTray.getToWmsTime());
            }
            if (null != fbTurnstoreTray.getTime()) {
                fbTurnstoreTrayItem.setTime(fbTurnstoreTray.getTime());
            }
            if (null != fbTurnstoreTray.getOperationTime()) {
                fbTurnstoreTrayItem.setOperationTime(fbTurnstoreTray.getOperationTime());
            }
            fbTurnstoreTrayItem.setFactoryName(fbTurnstoreTray.getFactoryName());
            fbTurnstoreTrayItem.setSectionNo(fbTurnstoreTrayItem.getSectionNo().toUpperCase());
            if (null == fbTurnstoreTrayItem.getTotalLength()){
                fbTurnstoreTrayItem.setTotalLength(fbTurnstoreTrayItem.getLength());
            }
            //锁定为组托状态
            PlanProcessingOrder planProcessingOrder = planProcessingOrderService.findByCode(fbTurnstoreTrayItem.getSectionNo());
            if (null != planProcessingOrder) {
                if ("退料".equals(fbTurnstoreTray.getHasGone())) {
                    planProcessingOrder.setStatus(EnumValuesFactory.PlanProcessingOrderStatus.已作废.getKey());
                } else {
                    planProcessingOrder.setStatus(EnumValuesFactory.PlanProcessingOrderStatus.组托.getKey());
                }
                planProcessingOrderService.saveOrUpdate(planProcessingOrder);
            }

            fbTurnstoreTrayItemService.saveOrUpdate(fbTurnstoreTrayItem);
        }
    }

    private String send(String data, String url) {
        HttpHeaders headers = new HttpHeaders();
        MediaType mediaType = MediaType.parseMediaType("application/json; charset=UTF-8");
        headers.setContentType(mediaType);
        headers.add("Accept", MediaType.APPLICATION_JSON.toString());
        HttpEntity<String> httpEntity = new HttpEntity<>(data, headers);
        //使用exchange请求接口
        ResponseEntity<String> exchange = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);
        return exchange.getBody();
    }
}
