package com.sw.mes.production.service.impl;


import com.alibaba.fastjson.JSON;
import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.production.dao.FormulaProductSchemeDao;
import com.sw.mes.production.entity.FormulaProductScheme;
import com.sw.mes.production.service.FormulaProductSchemeService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@Service
@Transactional
public class FormulaProductSchemeServiceImpl extends BaseServiceImpl<FormulaProductScheme> implements FormulaProductSchemeService {
    FormulaProductSchemeDao dao;

    @Autowired
    public void setDao(FormulaProductSchemeDao dao) {
        super.setBaseDao(dao);
        this.dao = dao;
    }


    @Override
    public FormulaProductScheme findByEquipmentCodeAndProductModeCode(String equipmentCode, String productModeCode) {
        return dao.findByEquipmentCodeAndProductModeCode(equipmentCode, productModeCode);
    }

    @Override
    public List<FormulaProductScheme> creates(List<FormulaProductScheme> formulaProductSchemes) {
        Map<String, List<FormulaProductScheme>> formulaProductSchemeMap = formulaProductSchemes.stream().collect(Collectors.groupingBy(formulaProductScheme -> formulaProductScheme.getEquipmentCode() + formulaProductScheme.getProductModeCode(), Collectors.toList()));
        formulaProductSchemeMap.values().forEach(formulaProductSchemeList -> {
            Assert.isTrue(formulaProductSchemeList.size() == 1, "【" + formulaProductSchemeList.get(0).getEquipmentCode() + "/" + formulaProductSchemeList.get(0).getProductModeCode() + "】同一设备同一产品，只能有一条配方");
        });
        List<FormulaProductScheme> formulaProductSchemeList = new ArrayList<>();
        formulaProductSchemes.forEach(formulaProductScheme -> {
            FormulaProductScheme formulaProductSchemeOld = findByEquipmentCodeAndProductModeCode(formulaProductScheme.getEquipmentCode(),
                    formulaProductScheme.getProductModeCode());
            if (formulaProductSchemeOld != null) {
                if (formulaProductScheme.getStatus() == null) {
                    formulaProductSchemeList.add(formulaProductScheme);
                } else {
                    if (formulaProductScheme.getStatus() == 0) {
                        formulaProductScheme.setId(formulaProductSchemeOld.getId());
                        BeanUtils.copyProperties(formulaProductScheme, formulaProductSchemeOld, BeanUtil.getIgnoreProperties(formulaProductScheme, null, null));
                        formulaProductScheme = formulaProductSchemeOld;
                    } else {
                        FormulaProductScheme formulaProductSchemeNew = new FormulaProductScheme();
                        BeanUtils.copyProperties(formulaProductScheme, formulaProductSchemeNew);
                        formulaProductScheme = formulaProductSchemeOld;
                        formulaProductScheme.setNextFurnaceEffective(JSON.toJSONString(formulaProductSchemeNew));
                    }
                    formulaProductScheme.setGmtUpdate(new Date());
                    formulaProductScheme.setUserUpdate(BaseContextHandler.getUsername());
                }
            } else {
                formulaProductScheme.setGmtCreate(new Date());
                formulaProductScheme.setUserCreate(BaseContextHandler.getUsername());
                formulaProductScheme.setStatus(0);
            }
        });

        if (formulaProductSchemeList.size() > 0) {
            return formulaProductSchemeList;
        } else {
            saveAll(formulaProductSchemes);
            return new ArrayList<>();
        }
    }
}
