package com.sw.mes.sap.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.sap.dto.ReportErpInputRecordDto;
import com.sw.mes.sap.entity.LogSapMesInteractive;
import com.sw.mes.sap.dto.LogSapMesInteractiveDto;
import com.sw.mes.sap.service.LogSapMesInteractiveService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;
import com.sw.mes.core.annotation.Jinko;
import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.stereotype.Controller;

/**
 * 晶科SAP下发生产工单通信记录Controller
 * 
 * @author liru
 * @date 2023-12-29
 */
@Controller
@Jinko(value="晶科SAP下发生产工单通信记录Controller",author = "liru")
@RequestMapping("/mes/interactive")
public class LogSapMesInteractiveController
{
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private LogSapMesInteractiveService logSapMesInteractiveService;

    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<LogSapMesInteractiveDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
        Specification<LogSapMesInteractive> specification = DynamicSpecifications.bySearchFilter(request, LogSapMesInteractive.class);
        Page<LogSapMesInteractive> page = logSapMesInteractiveService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<LogSapMesInteractiveDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有数据")
    @GetMapping("/listAll")
    public Result<List<LogSapMesInteractiveDto>> listAll(ServletRequest request) {
        Specification<LogSapMesInteractive> specification = DynamicSpecifications.bySearchFilter(request, LogSapMesInteractive.class);
        List<LogSapMesInteractive> LogSapMesInteractives = logSapMesInteractiveService.findByExampleSort(specification, null);
        List<LogSapMesInteractiveDto> LogSapMesInteractiveDtos = LogSapMesInteractives.stream()
                .map(LogSapMesInteractive -> toLazyDto(LogSapMesInteractive))
                .collect(Collectors.toList());
        return ResultGenerator.success(LogSapMesInteractiveDtos);
    }

    @ApiOperation(value = "根据id获取详细信息")
    @GetMapping("/{id}")
    public Result<LogSapMesInteractiveDto> get(@PathVariable @ApiParam(value="LogSapMesInteractiveid") Long id) {
        LogSapMesInteractive LogSapMesInteractive = logSapMesInteractiveService.get(id);
        if (null == LogSapMesInteractive)
            return ResultGenerator.fail("该记录不存在");
        return ResultGenerator.success(toEagerDto(LogSapMesInteractive));
    }

    @ApiOperation(value = "创建数据")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "LogSapMesInteractive对象") LogSapMesInteractiveDto LogSapMesInteractiveDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        LogSapMesInteractive LogSapMesInteractive = fromDto(LogSapMesInteractiveDto);
        logSapMesInteractiveService.saveOrUpdate(LogSapMesInteractive);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新数据")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "LogSapMesInteractive") LogSapMesInteractiveDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        LogSapMesInteractive entity = logSapMesInteractiveService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            LogSapMesInteractive entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            logSapMesInteractiveService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除记录")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            logSapMesInteractiveService.delete(id);
        }
        return ResultGenerator.success();
    }

    public LogSapMesInteractiveDto toLazyDto(LogSapMesInteractive entity) {
        return modelMapper.map(entity, LogSapMesInteractiveDto.class);
    }

    public LogSapMesInteractiveDto toEagerDto(LogSapMesInteractive entity) {
        LogSapMesInteractiveDto dto = modelMapper.map(entity, LogSapMesInteractiveDto.class);
        return dto;
    }

    public LogSapMesInteractive fromDto(LogSapMesInteractiveDto dto) {
        return modelMapper.map(dto, LogSapMesInteractive.class);
    }
}
