package com.sw.mes.sap.controller;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.sw.mes.core.annotation.Jinko;
import com.sw.mes.core.dto.sap.ReportSapChargItemDto;
import com.sw.mes.core.dto.sap.ReportSapGoodsmvtDto;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.sap.entity.ReportSapGoodsmvt;
import com.sw.mes.sap.service.ReportSapGoodsmvtService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * MES报工SAP货物移动接口Controller
 *
 * @author liru
 * @date 2024-01-06
 */
@RestController
@Jinko(value="MES报工SAP货物移动接口Controller",author = "liru")
@RequestMapping("/sap/goodsmvt")
public class ReportSapGoodsmvtController
{
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private ReportSapGoodsmvtService reportSapGoodsmvtService;

    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<ReportSapGoodsmvtDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
        Specification<ReportSapGoodsmvt> specification = DynamicSpecifications.bySearchFilter(request, ReportSapGoodsmvt.class);
        Page<ReportSapGoodsmvt> page = reportSapGoodsmvtService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<ReportSapGoodsmvtDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有数据")
    @GetMapping("/listAll")
    public Result<List<ReportSapGoodsmvtDto>> listAll(ServletRequest request) {
        Specification<ReportSapGoodsmvt> specification = DynamicSpecifications.bySearchFilter(request, ReportSapGoodsmvt.class);
        List<ReportSapGoodsmvt> reportSapGoodsmvts = reportSapGoodsmvtService.findByExampleSort(specification, null);
        List<ReportSapGoodsmvtDto> reportSapGoodsmvtDtos = reportSapGoodsmvts.stream()
                .map(reportSapGoodsmvt -> toLazyDto(reportSapGoodsmvt))
                .collect(Collectors.toList());
        return ResultGenerator.success(reportSapGoodsmvtDtos);
    }

    
    
    @ApiOperation(value = "根据id获取详细信息")
    @GetMapping("/{id}")
    public Result<ReportSapGoodsmvtDto> get(@PathVariable @ApiParam(value="ReportSapGoodsmvtid") Long id) {
        ReportSapGoodsmvt reportSapGoodsmvt = reportSapGoodsmvtService.get(id);
        if (null == reportSapGoodsmvt)
            return ResultGenerator.fail("该记录不存在");
        return ResultGenerator.success(toEagerDto(reportSapGoodsmvt));
    }

    @ApiOperation(value = "创建数据")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "ReportSapGoodsmvt对象") ReportSapGoodsmvtDto reportSapGoodsmvtDto) {
        try {
	    	ReportSapGoodsmvt reportSapGoodsmvt = fromDto(reportSapGoodsmvtDto);
	        reportSapGoodsmvt.setStatus(false);
	        reportSapGoodsmvtService.saveOrUpdate(reportSapGoodsmvt);
        }catch (Exception e) {
        	ResultGenerator.fail(e.getMessage());
		}
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新数据")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "ReportSapGoodsmvt") ReportSapGoodsmvtDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        try {
            ReportSapGoodsmvt entityfromFto = fromDto(dto);
            reportSapGoodsmvtService.saveOrUpdate(entityfromFto);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除记录")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
        for (Long id : ids) {
                reportSapGoodsmvtService.delete(id);
        }
        return ResultGenerator.success();
    }

    @ApiOperation(value = "发送报工")
    @PostMapping("/send")
    public Result send(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
        try {
            reportSapGoodsmvtService.sendReportWord(ids);
        }catch (Exception e){
            return ResultGenerator.fail(e.getMessage());
        }
        return ResultGenerator.success();
    }

    @ApiOperation(value = "根据物料号和移动类型获取批次号")
    @GetMapping("/findByMatnrAndBwart")
    public Result<ReportSapGoodsmvtDto> findByMatnrAndBwart(@RequestParam String matnr,@RequestParam String bwart) {
        try {
            ReportSapGoodsmvt byMatnrAndBwart = reportSapGoodsmvtService.findByCrystalNoAndBwart(matnr, bwart);
            if (null == byMatnrAndBwart){
                return ResultGenerator.fail("未获取批次号!");
            }
            return ResultGenerator.success(toLazyDto(byMatnrAndBwart));
        }catch (Exception e){
            return ResultGenerator.fail(e.getMessage());
        }
    }

    public ReportSapGoodsmvtDto toLazyDto(ReportSapGoodsmvt entity) {
        ReportSapGoodsmvtDto dto = modelMapper.map(entity, ReportSapGoodsmvtDto.class);
        dto.setChargItems(new Gson().fromJson(entity.getChargItem(),new TypeToken<List<ReportSapChargItemDto>>(){}.getType()));
        return dto;
    }

    public ReportSapGoodsmvtDto toEagerDto(ReportSapGoodsmvt entity) {
        ReportSapGoodsmvtDto dto = modelMapper.map(entity, ReportSapGoodsmvtDto.class);
        dto.setChargItems(new Gson().fromJson(entity.getChargItem(),new TypeToken<List<ReportSapChargItemDto>>(){}.getType()));
        return dto;
    }

    public ReportSapGoodsmvt fromDto(ReportSapGoodsmvtDto dto) {
        ReportSapGoodsmvt reportSapGoodsmvt = modelMapper.map(dto, ReportSapGoodsmvt.class);
        reportSapGoodsmvt.setChargItem(new Gson().toJson(dto.getChargItems()));
        return reportSapGoodsmvt;
    }
}
