package com.sw.mes.sap.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.sw.mes.base.system.entity.DictionaryItem;
import com.sw.mes.base.system.service.DictionaryItemService;
import com.sw.mes.core.annotation.Jinko;
import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.sap.ReportSapChargItemDto;
import com.sw.mes.core.dto.sap.ReportSapGoodsmvtDto;
import com.sw.mes.core.utils.DateUtil;
import com.sw.mes.sap.dao.ReportSapGoodsmvtDao;
import com.sw.mes.sap.entity.ReportSapGoodsmvt;
import com.sw.mes.sap.service.ReportSapGoodsmvtService;
import com.sw.mes.sap.service.SAPOutputServiceService;
import com.sw.mes.utils.CodeUtil;
import com.sw.mes.utils.StringUtils;
import com.sw.mes.yb.plan.entity.ProductionOrder;
import com.sw.mes.yb.plan.service.ProductionOrderService;

import lombok.extern.slf4j.Slf4j;

/**
 * MES报工SAP货物移动接口Service业务层处理
 *
 * @author liru
 * @date 2024-01-06
 */
@Slf4j
@Service
@Transactional
@Jinko(value="MES报工SAP货物移动接口Service业务层处理",author = "liru")
public class ReportSapGoodsmvtServiceImpl extends BaseServiceImpl<ReportSapGoodsmvt> implements ReportSapGoodsmvtService
{
    private ReportSapGoodsmvtDao dao;

    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private SAPOutputServiceService sAPOutputServiceService;
    @Autowired
    private ProductionOrderService productionOrderService;
    @Autowired
    private DictionaryItemService dictionaryItemService;


    @Autowired
    public void setDao(ReportSapGoodsmvtDao dao) {
        super.setBaseDao(dao);
        this.dao=dao;
    }
    
    
    
    /**
    *
    * @param bwart 移动类型
    * @param lgort 库存地点 对应数据字典中库存点 WarehouseLocation
    * @param kostl 成本中心
    * @param aufnr SAP生产订单
    * @param matnr 物料编码
    * @param charg  批次号
    * @param umcha 收货批次号
    * @param zfcpFlag 副产品标识  如过是副产品传值   字符串: X
    * @param menge  数量
    * @param meins 单位
    */
//   public void sendReportSapGoodsmvt(String bwart,String lgort,String kostl,String aufnr,String matnr,String umcha,String charg,String zfcpFlag,String menge,String meins){
//	   ProductionOrder order = productionOrderService.findByAufnr(aufnr);
//	   
//       ReportSapGoodsmvt reportSap = new ReportSapGoodsmvt();
//       reportSap.setBwart(bwart);
//       reportSap.setKostl(kostl);
//       reportSap.setAufnr(aufnr);//生产订单号
//       reportSap.setMatnr(matnr);
//       reportSap.setCharg(charg);
//       reportSap.setMenge(menge);
//       reportSap.setMeins(meins);
//       reportSap.setLgort(lgort);
//       
//       
//       
//       reportSap.setWerks("5041");
//      // reportSap.setRspos(rspos);
//       
//       
//       
//       saveOrUpdate(reportSap);
//   }
    
    

    @Override
    public void saveOrUpdate(ReportSapGoodsmvt reportSapGoodsmvt) {
    	ProductionOrder order = productionOrderService.findByAufnr(reportSapGoodsmvt.getAufnr());
    	if(order!=null) {
	    	reportSapGoodsmvt.setVbeln(order.getSalesorder());//销售订单
	    	reportSapGoodsmvt.setPosnr(order.getKdpos());//销售订单行
	    	reportSapGoodsmvt.setEbeln(order.getZCollectNo());//采购订单 - 组合号   待确认 
	    	reportSapGoodsmvt.setEbelp(order.getZCollectItm());//采购订单行 - 组合行号   待确认
	    	//reportSapGoodsmvt.setRsnum(order.getComment()); //SAP预留  -  RSNUM - Comment
	    	reportSapGoodsmvt.setWerks(order.getWerks());
    	}
        //初始化默认数据
        reportSapGoodsmvt.setBldat(new Date());
        super.saveOrUpdate(reportSapGoodsmvt);
    }

    @Override
    public void sendReportWord(Long[] ids) {
        List<ReportSapGoodsmvt> list = new ArrayList<>();
        List<ReportSapGoodsmvtDto> dtolist = new ArrayList<>();
        for (Long id : ids){
            ReportSapGoodsmvt reportSapGoodsmvt = get(id);
            list.add(reportSapGoodsmvt);

            ReportSapGoodsmvtDto dto = toEagerDto(reportSapGoodsmvt);
            dtolist.add(dto);
        }

        String zuuid = CodeUtil.getUniqueCode();
        Date cDate = new Date();
        JSONArray array = new JSONArray();
        JSONArray items = JSONArray.parseArray(JSONArray.toJSONString(dtolist));
        JSONObject IT_DATA = new JSONObject();
        IT_DATA.put("ZUUID", zuuid);
        IT_DATA.put("BUDAT", DateUtil.formatDate(cDate,DateUtil.shortFormat2));
        IT_DATA.put("BLDAT", DateUtil.formatDate(cDate,DateUtil.shortFormat2));
        IT_DATA.put("ERNAM", BaseContextHandler.getUsername());
        IT_DATA.put("GOODS_ITEMS", items);
        array.add(IT_DATA);

        String MBLNR = "";
        
        try {
            JSONObject jsonObject = sAPOutputServiceService.sendWorkGoodsMvtToSap(array);
            JSONObject ES_BC_INFO = jsonObject.getJSONObject("ES_BC_INFO");
            String resultState = ES_BC_INFO.getString("TYPE");
            if(StringUtils.isNotBlank(resultState) && resultState.equals("E")){
            	String msg = ES_BC_INFO.getString("MSG");
            	
            	try {
	            	JSONArray ET_RETURN = jsonObject.getJSONArray("ET_RETURN");
	            	for (int i = 0; i < ET_RETURN.size(); i++) {
	            		JSONObject returnObj = ET_RETURN.getJSONObject(i);
	            		msg = returnObj.getString("MSGTX");
					}
            	}catch (Exception e) {
				}
            	
                throw new RuntimeException(msg);
            }else {
            	JSONArray ET_RETURN = jsonObject.getJSONArray("ET_RETURN");
            	for (int i = 0; i < ET_RETURN.size(); i++) {
            		JSONObject returnObj = ET_RETURN.getJSONObject(i);
            		MBLNR = returnObj.getString("MBLNR");
				}
            }
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e.getMessage());
        }

        //构造发送数据格式
        for (ReportSapGoodsmvt reportSapGoodsmvt : list){
            reportSapGoodsmvt.setZuuid(MBLNR);
            reportSapGoodsmvt.setBudat(cDate);
            reportSapGoodsmvt.setBldat(cDate);
            reportSapGoodsmvt.setErnam(BaseContextHandler.getUsername());
            reportSapGoodsmvt.setStatus(true);
            saveOrUpdate(reportSapGoodsmvt);
            
            String switchValue = dictionaryItemService.findValueByThemeValueAndItemName("AutoReportSwitch", reportSapGoodsmvt.getBwart());
            if(StringUtils.isNotBlank(switchValue) && switchValue.equalsIgnoreCase("true")) {
            	sendReportWord(new Long[] {reportSapGoodsmvt.getId()});
            }
        }
    }

    @Override
    public ReportSapGoodsmvt findByCrystalNoAndBwart(String crystalNo, String bwart) {
        if (StringUtils.isNotBlank(crystalNo) && StringUtils.isNotBlank(bwart)){
            return dao.findByCrystalNoAndBwart(crystalNo,bwart);
        }
        return null;
    }


    public ReportSapGoodsmvtDto toEagerDto(ReportSapGoodsmvt entity) {
        ReportSapGoodsmvtDto dto = modelMapper.map(entity, ReportSapGoodsmvtDto.class);
        dto.setChargItems(JSONArray.parseArray(entity.getChargItem(), ReportSapChargItemDto.class));
        return dto;
    }
}
