package com.sw.mes.sap.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.sw.mes.base.system.service.DictionaryItemService;
import com.sw.mes.core.annotation.Jinko;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.sap.ReportSapChargItemDto;
import com.sw.mes.core.dto.sap.ReportSapGoodsmvtDto;
import com.sw.mes.core.dto.wms.ReportSapWorkinghoursDto;
import com.sw.mes.core.utils.DateUtil;
import com.sw.mes.sap.entity.ReportSapGoodsmvt;
import com.sw.mes.sap.entity.ReportSapWorkinghours;
import com.sw.mes.sap.service.SAPOutputServiceService;
import com.sw.mes.utils.CodeUtil;
import com.sw.mes.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import com.sw.mes.core.base.BaseServiceImpl;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.sw.mes.sap.dao.ReportSapWorkinghoursDao;
import com.sw.mes.sap.entity.ReportSapWorkinghours;
import com.sw.mes.sap.service.ReportSapWorkinghoursService;

/**
 * MES报工SAP工时接口Service业务层处理
 *
 * @author liru
 * @date 2024-01-07
 */
@Slf4j
@Service
@Transactional
@Jinko(value="MES报工SAP工时接口Service业务层处理",author = "liru")
public class ReportSapWorkinghoursServiceImpl extends BaseServiceImpl<ReportSapWorkinghours> implements ReportSapWorkinghoursService
{
    

	private ReportSapWorkinghoursDao dao;

    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private SAPOutputServiceService sAPOutputServiceService;
    @Autowired
    private DictionaryItemService dictionaryItemService;

    @Autowired
    public void setDao(ReportSapWorkinghoursDao dao) {
        super.setBaseDao(dao);
        this.dao=dao;
    }
    
    
    @Override
	public void saveOrUpdate(ReportSapWorkinghours t) {
		super.saveOrUpdate(t);
		
		String switchValue = dictionaryItemService.findValueByThemeValueAndItemName("AutoReportSwitch", "WorkinghoursAuto");
        if(StringUtils.isNotBlank(switchValue) && switchValue.equalsIgnoreCase("true")) {
        	sendReportWord(new Long[] {t.getId()});
        }
	}

    @Override
    public void sendReportWord(Long[] ids) {
        List<ReportSapWorkinghours> list = new ArrayList<>();
        List<ReportSapWorkinghoursDto> dtolist = new ArrayList<>();
        for (Long id : ids){
            ReportSapWorkinghours reportSapWorkinghours = get(id);
            list.add(reportSapWorkinghours);

            ReportSapWorkinghoursDto dto = toEagerDto(reportSapWorkinghours);
            dtolist.add(dto);
        }

        String zuuid = CodeUtil.getUniqueCode();
        Date cDate = new Date();


        for(ReportSapWorkinghoursDto dto : dtolist){
            dto.setRecordId(zuuid);
            dto.setCnfDate(cDate);
        }

        //格式化工时报工数据
        JSONArray items = JSONArray.parseArray(JSONArray.toJSONString(dtolist));

        try {
            JSONObject jsonObject = sAPOutputServiceService.sendWorkinghoursToSap(items);
            JSONObject ES_BC_INFO = jsonObject.getJSONObject("ES_BC_INFO");
            String resultState = ES_BC_INFO.getString("TYPE");
            if(StringUtils.isNotBlank(resultState) && resultState.equals("E")){
                throw new RuntimeException(ES_BC_INFO.getString("MSG"));
            }
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }

        //构造发送数据格式
        for (ReportSapWorkinghours reportSapWorkinghours : list){
            reportSapWorkinghours.setRecordId(zuuid);
            reportSapWorkinghours.setStatus(true);
            reportSapWorkinghours.setCnfDate(cDate);
            saveOrUpdate(reportSapWorkinghours);
        }
    }

    public ReportSapWorkinghoursDto toEagerDto(ReportSapWorkinghours entity) {
        return modelMapper.map(entity, ReportSapWorkinghoursDto.class);
    }
}
