package com.sw.mes.sectionMes.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sw.mes.core.dto.sectionMes.FbInformationReceptionDto;
import com.sw.mes.core.dto.sectionMes.ReturnMaterialFeedBackDto;
import com.sw.mes.jip.entity.JipHeaderConfig;
import com.sw.mes.jip.enums.IFKeyWordEnums;
import com.sw.mes.jip.service.JipHeaderConfigService;
import com.sw.mes.sectionMes.entity.LogSectionMesInteractive;
import com.sw.mes.sectionMes.service.LogSectionMesInteractiveService;
import com.sw.mes.sectionMes.service.MESOutputService;
import com.sw.mes.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.util.Date;
import java.util.List;

/**
 * @author lf
 * @description
 * @date 2024/2/1 9:41
 */
@Service
public class MESOutputServiceImpl implements MESOutputService {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private JipHeaderConfigService jipHeaderConfigService;

    @Autowired
    private LogSectionMesInteractiveService logSectionMesInteractiveService;

    @Autowired
    private RestTemplate restTemplate;

    @Override
    public void fbInformationReception(List<FbInformationReceptionDto> data) {
        JipHeaderConfig jipHeaderConfig = jipHeaderConfigService.getByKeyword(IFKeyWordEnums.FBINFORMATIONRECEPTION.getKeyword());

        if (jipHeaderConfig == null) {
            throw new RuntimeException("MES中未配置方棒信息接收接口信息");
        }
        if (StringUtils.isBlank(jipHeaderConfig.getUrl())) {
            throw new RuntimeException("MES中未配置方棒信息接收接口URL请求地址");
        }
        String dataJson = JSON.toJSONString(data);

        //2.发送方棒信息接收到切片MES
        JSONObject result;
        try {
             result = restTemplate.postForObject(jipHeaderConfig.getUrl(), dataJson, JSONObject.class);
            //保存交互记录
            saveInteractiveLog("拉晶MES->切片MES", "方棒信息接收接口", "fbInformationReception", dataJson, result);
            System.out.println(objectMapper.writeValueAsString(result));
//            JSONObject ES_BC_INFO = result.getJSONObject("ES_BC_INFO");
//            if (!ES_BC_INFO.getString("TYPE").equals("S")) {
//                throw new RuntimeException(ES_BC_INFO.getString("MSG"));
//            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e.getMessage());
        }
    }

    @Override
    public void returnMaterialFeedBack(List<ReturnMaterialFeedBackDto> data) {
        JipHeaderConfig jipHeaderConfig = jipHeaderConfigService.getByKeyword(IFKeyWordEnums.FBRETURNMATERIALFEEDBACK.getKeyword());

        if (jipHeaderConfig == null) {
            throw new RuntimeException("MES中未配置方棒退料反馈接口信息");
        }
        if (StringUtils.isBlank(jipHeaderConfig.getUrl())) {
            throw new RuntimeException("MES中未配置方棒退料反馈接口URL请求地址");
        }
        String dataJson = JSON.toJSONString(data);

        //2.发送方棒退料反馈信息到切片MES
        JSONObject result;
        try {
            result = restTemplate.postForObject(jipHeaderConfig.getUrl(), dataJson, JSONObject.class);
            //保存交互记录
            saveInteractiveLog("拉晶MES->切片MES", "方棒退料反馈接口", "returnMaterialFeedBack", dataJson, result);
            System.out.println(objectMapper.writeValueAsString(result));
//            JSONObject ES_BC_INFO = result.getJSONObject("ES_BC_INFO");
//            if (!ES_BC_INFO.getString("TYPE").equals("S")) {
//                throw new RuntimeException(ES_BC_INFO.getString("MSG"));
//            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e.getMessage());
        }
    }

    // 保存原始记录到数据表
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void saveInteractiveLog(String direction, String controllerName, String oprMethod, Object data, Object result) {
        try {
            LogSectionMesInteractive log = new LogSectionMesInteractive();
            log.setDataDirection(direction);
            log.setControllerName(controllerName);
            log.setOprMethod(oprMethod);
            log.setContext(JSONObject.toJSONString(data));
            log.setReturnContext(JSONObject.toJSONString(result));
            log.setCreateTime(new Date());
            logSectionMesInteractiveService.saveOrUpdate(log);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
