package com.sw.mes.yb.alloy.service.impl;


import com.sw.mes.base.factory.material.entity.MaterialModel;
import com.sw.mes.base.factory.material.service.MaterialModelService;
import com.sw.mes.core.api.system.StoreFeignClients;
import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.store.WarehouseInputRecordBillDetailDto;
import com.sw.mes.core.response.Result;
import com.sw.mes.inspect.entity.YbInspectData;
import com.sw.mes.inspect.service.YbInspectDataService;
import com.sw.mes.production.entity.FormulaScheme;
import com.sw.mes.production.entity.FormulaWeightScheme;
import com.sw.mes.production.service.FormulaSchemeService;
import com.sw.mes.production.service.FormulaWeightSchemeService;
import com.sw.mes.yb.alloy.dto.*;
import com.sw.mes.yb.alloy.entity.FixLogic;
import com.sw.mes.yb.alloy.entity.RawMaterialLevel;
import com.sw.mes.yb.alloy.entity.ResistanceOfThisRod;
import com.sw.mes.yb.alloy.service.*;
import com.sw.mes.yb.plan.entity.ProductionSchedule;
import com.sw.mes.yb.plan.service.ProductionScheduleService;
import com.sw.mes.yb.production.entity.*;
import com.sw.mes.yb.production.service.*;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.sw.mes.yb.alloy.entity.AlloyMasterAlloy;
import com.sw.mes.yb.alloy.dao.AlloyMasterAlloyDao;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@Service
@Transactional
public class AlloyMasterAlloyServiceImpl extends BaseServiceImpl<AlloyMasterAlloy> implements AlloyMasterAlloyService {
    @Autowired
    private MaterialModelService materialModelService;

    @Autowired
    private AgainTaskService againTaskService;

    @Autowired
    private LoadingTaskService loadingTaskService;

    @Autowired
    private ResistanceOfThisRodService resistanceOfThisRodService;

    @Autowired
    private YbInspectDataService ybInspectDataService;

    @Autowired
    private FixLogicService fixLogicService;

    @Autowired
    private InputFurnaceDetailService inputFurnaceDetailService;

    @Autowired
    private BlastFurnaceDetailService blastFurnaceDetailService;

    @Autowired
    private RawMaterialLevelService rawMaterialLevelService;

    @Autowired
    private AgainTaskDetailService againTaskDetailService;

    @Autowired
    private ModelMapper modelMapper;

    @Autowired
    private CorrectingRecordService correctingRecordService;

    @Autowired
    private InspectionListService inspectionListService;

    @Autowired
    private ProductionFurnaceMaterialSplitService productionFurnaceMaterialSplitService;

    @Autowired
    private FormulaSchemeService formulaSchemeService;

    @Autowired
    private FormulaWeightSchemeService formulaWeightSchemeService;

    @Autowired
    private ProductionScheduleService productionScheduleService;

    @Autowired
    private StoreFeignClients storeFeignClients;

    AlloyMasterAlloyDao dao;

    @Autowired
    public void setDao(AlloyMasterAlloyDao dao) {
        super.setBaseDao(dao);
        this.dao = dao;
    }

    @Override
    public FurnaceBedDto findByCode(String furnaceCode, Integer isEnd) {
        FurnaceBedDto furnaceBedDto = new FurnaceBedDto();
//                复投重量
        BigDecimal againWeight = null;
//                 留锅量
        BigDecimal surplusWeight = null;
//        产品料号
        String materialCode;
//        复投次数
        Integer againNum;
        Long againTaskId = null;
//        工单号
        String sapNo = null;
        ProductionSchedule productionSchedule = productionScheduleService.findByDeviceNo(furnaceCode);
        Assert.notNull(productionSchedule, "当前炉台未开炉,请先开炉!!!");
        String lotNo = productionSchedule.getLotNo();
//        根据炉次号复投任务查如果没查到去配料任务查
        List<AgainTask> againTaskList = againTaskService.findSuccessByLotNo(lotNo);
        LoadingTask loadingTaskDb = loadingTaskService.findSuccessByLotNo(lotNo);
        if (CollectionUtils.isEmpty(againTaskList) && null == loadingTaskDb) {
            throw new RuntimeException("未查询到复投次数");
        }
        AgainTask againTaskDb = againTaskList.stream()
                .max(Comparator.comparing(AgainTask::getAgainNum))
                .orElse(null);
        if (null == againTaskDb) {
            materialCode = loadingTaskDb.getProductCode();
            againNum = 0;
        } else {
            materialCode = againTaskDb.getProductCode();
            againNum = againTaskDb.getAgainNum();
            againTaskId = againTaskDb.getId();
            sapNo = againTaskDb.getSapNo();
        }

        MaterialModel materialModel = materialModelService.findByCode(materialCode);
        Assert.notNull(materialModel, "物料主数据信息不存在!!!");
        BigDecimal resistanceOfThisRod = getResistanceOfThisRod(isEnd, againNum, furnaceCode);
        if (null == resistanceOfThisRod || resistanceOfThisRod.compareTo(BigDecimal.ZERO) == 0) {
            throw new RuntimeException("本棒电阻不存在");
        }
        buildFurnaceBedDto(furnaceCode, furnaceBedDto, againNum, sapNo, materialModel, resistanceOfThisRod, lotNo, isEnd);
//           计算二加上一棒重量和留过量、本棒电阻、上一棒电阻
        if (againNum == 2) {
//                初装量
            List<LoadingTask> loadingTaskList = loadingTaskService.findRealWeightByLotNo(lotNo);
            BigDecimal realWeight = loadingTaskList.stream()
                    .map(LoadingTask::getWeight)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            loadingTaskList.stream()
                    .max(Comparator.comparingLong(LoadingTask::getId))
                    .ifPresent(loadingTask -> furnaceBedDto.setHeatDimension(loadingTask.getDimension()));
            againTaskList.forEach(item -> {
                if (item.getAgainNum() == 1) {
                    furnaceBedDto.setUpBarWeight(realWeight.add(item.getAgainWeigth()));
                    furnaceBedDto.setRemainingPotCapacity(realWeight.add(item.getAgainWeigth()));
                }
            });
//                上一棒电阻
            ResistanceOfThisRod resistanceOfThisRodDb = resistanceOfThisRodService.findByFurnaceCodeAndAddTime(furnaceCode, 1);
            furnaceBedDto.setUpBarResistance(resistanceOfThisRodDb.getResistanceOfThisRod());
//                计算三加以上上一棒重量、留锅量、上一棒电阻
        } else if (againNum >= 3) {
            Integer againNum2 = againNum - 2;
            Integer againNum1 = againNum - 1;
            for (AgainTask againTask : againTaskList) {
                if (againNum2.equals(againTask.getAgainNum())) {
                    againWeight = againTask.getAgainWeigth();
                }
                if (againNum1.equals(againTask.getAgainNum())) {
                    surplusWeight = againTask.getSurplusWeigth();
                }
                if (null != againWeight && null != surplusWeight) {
                    furnaceBedDto.setUpBarWeight(surplusWeight.add(againWeight));
                    furnaceBedDto.setRemainingPotCapacity(surplusWeight.add(againWeight));
                }
            }
            getUpBarResistance(furnaceCode, furnaceBedDto, materialModel, againNum2, againNum1, lotNo);
        }

        getPolysiliconc(lotNo, furnaceBedDto, againNum, againTaskId);
        getRecycledMaterials(lotNo, furnaceBedDto, againNum, againTaskId);
        getInspectionListDto(lotNo, furnaceBedDto, againNum);
        return furnaceBedDto;
    }

    /**
     * 获取检验列表
     */
    private void getInspectionListDto(String lotNo, FurnaceBedDto furnaceBedDto, Integer againNum) {
        ArrayList<InspectionListDto> inspectionListDtoList = new ArrayList<>();
        List<YbInspectData> ybInspectDataList = ybInspectDataService.findByLotNoAndAgainNum(lotNo, againNum);
        if (!CollectionUtils.isEmpty(ybInspectDataList)) {
            for (YbInspectData ybInspectData : ybInspectDataList) {
                InspectionListDto inspectionListDto = new InspectionListDto();
                inspectionListDto.setCrystalNumber(ybInspectData.getSectionNo());
                inspectionListDto.setAddTime(ybInspectData.getAgainNum().toString());
                inspectionListDto.setHeadResistivity(ybInspectData.getTResistivity());
                inspectionListDto.setTailResistivity(ybInspectData.getWResistivity());
                inspectionListDto.setHeadLifeSpan(ybInspectData.getMeanCarrierLifetimeA());
                inspectionListDto.setTailLifeSpan(ybInspectData.getMeanCarrierLifetimeB());
                inspectionListDtoList.add(inspectionListDto);
            }
            furnaceBedDto.setInspectionListDtoList(inspectionListDtoList);
        }
    }

    @Override
    public Float calculationBaseAlloy(AlloyMasterAlloyDto dto) {
        if (null == dto.getUpBarWeight()) {
            dto.setUpBarWeight(new BigDecimal(0));
        }
        if (null == dto.getRemainingPotCapacity()) {
            dto.setRemainingPotCapacity(new BigDecimal(0));
        }
        if (null == dto.getPolysiliconResistance()) {
            dto.setPolysiliconResistance(new BigDecimal(0));
        }
        if (null == dto.getPolysiliconWeight()) {
            dto.setPolysiliconWeight(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialOneResistance()) {
            dto.setRecycledMaterialOneResistance(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialOneWeight()) {
            dto.setRecycledMaterialOneWeight(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialTwoResistance()) {
            dto.setRecycledMaterialTwoResistance(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialTwoWeight()) {
            dto.setRecycledMaterialTwoWeight(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialThreeResistance()) {
            dto.setRecycledMaterialThreeResistance(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialThreeWeight()) {
            dto.setRecycledMaterialThreeWeight(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialFourResistance()) {
            dto.setRecycledMaterialFourResistance(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialFourWeight()) {
            dto.setRecycledMaterialFourWeight(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialFiveResistance()) {
            dto.setRecycledMaterialFiveResistance(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialFiveWeight()) {
            dto.setRecycledMaterialFiveWeight(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialSixResistance()) {
            dto.setRecycledMaterialSixResistance(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialSixWeight()) {
            dto.setRecycledMaterialSixWeight(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialSevenResistance()) {
            dto.setRecycledMaterialSevenResistance(new BigDecimal(0));
        }
        if (null == dto.getRecycledMaterialSevenWeight()) {
            dto.setRecycledMaterialSevenWeight(new BigDecimal(0));
        }
        return dao.getComputedMotherAlloyWeight(dto.getUpBarWeight().floatValue(), dto.getRemainingPotCapacity().floatValue(),
                (dto.getUpBarResistance() != null) ? dto.getUpBarResistance().floatValue() : 1, dto.getPolysiliconResistance().floatValue(), dto.getPolysiliconWeight().floatValue(),
                dto.getRecycledMaterialOneResistance().floatValue(), dto.getRecycledMaterialOneWeight().floatValue(),
                dto.getRecycledMaterialTwoResistance().floatValue(), dto.getRecycledMaterialTwoWeight().floatValue(),
                dto.getRecycledMaterialThreeResistance().floatValue(), dto.getRecycledMaterialThreeWeight().floatValue(),
                dto.getRecycledMaterialFourResistance().floatValue(), dto.getRecycledMaterialFourWeight().floatValue(),
                dto.getRecycledMaterialFiveResistance().floatValue(), dto.getRecycledMaterialFiveWeight().floatValue(),
                dto.getRecycledMaterialSixResistance().floatValue(), dto.getRecycledMaterialSixWeight().floatValue(),
                dto.getRecycledMaterialSevenResistance().floatValue(), dto.getRecycledMaterialSevenWeight().floatValue(),
                dto.getResistanceOfThisRod().floatValue(), dto.getMotherAlloyGear().floatValue());
    }

    @Override
    public void save(AlloyMasterAlloyDto alloyMasterAlloyDto) {
        AlloyMasterAlloy alloyMasterAlloyDb = dao.findByLotNoAndAddTime(alloyMasterAlloyDto.getLotNo(), alloyMasterAlloyDto.getAddTime());
        Assert.isNull(alloyMasterAlloyDb,"该炉次号和复投次数已存在");
        List<AgainTask> againTaskList = againTaskService.findSuccessByLotNo(alloyMasterAlloyDto.getLotNo());
        if (!CollectionUtils.isEmpty(againTaskList)) {
            AgainTask againTaskDb = againTaskList.stream()
                    .max(Comparator.comparing(AgainTask::getAgainNum))
                    .orElse(null);
            if (null != againTaskDb) {
                alloyMasterAlloyDto.setEquipmentCode(againTaskDb.getEquipmentCode());
                MaterialModel materialModel = materialModelService.findByCode(againTaskDb.getProductCode());
                if (null != materialModel) {
                    alloyMasterAlloyDto.setProductCode(materialModel.getCode());
                    alloyMasterAlloyDto.setProductShortName(materialModel.getProductName());
                }
                List<ProductionFurnaceMaterialSplit> furnaceMaterialSplitList = productionFurnaceMaterialSplitService.findByLotNoAndAgainNum(againTaskDb.getLotNo(), againTaskDb.getAgainNum());
                if (!CollectionUtils.isEmpty(furnaceMaterialSplitList)) {
                    ProductionFurnaceMaterialSplit productionFurnaceMaterialSplit = furnaceMaterialSplitList.get(0);
                    FormulaScheme formulaScheme = formulaSchemeService.get(productionFurnaceMaterialSplit.getSchemeId());
                    FormulaWeightScheme formulaWeightScheme = formulaWeightSchemeService.get(productionFurnaceMaterialSplit.getWeightSchemeId());
                    if (null != formulaScheme) {
                        alloyMasterAlloyDto.setWeightSchemeName(formulaScheme.getName());
                        alloyMasterAlloyDto.setWeightSchemeVersion(formulaScheme.getVersion());
                    }
                    if (null != formulaWeightScheme) {
                        alloyMasterAlloyDto.setRecipeName(formulaWeightScheme.getName());
                        alloyMasterAlloyDto.setRecipeVersion(productionFurnaceMaterialSplit.getSchemeVersion());
                    }
                }
            }
            if (Integer.parseInt(alloyMasterAlloyDto.getAddTime()) > 0) {
                BigDecimal totalWeight = BigDecimal.ZERO;
                for (AgainTask againTask : againTaskList) {
                    if (againTask.getAgainWeigth() != null) {
                        totalWeight = totalWeight.add(againTask.getAgainWeigth());
                    }
                }
                alloyMasterAlloyDto.setFeedInWeight(totalWeight);
            }
        }

        if (Integer.parseInt(alloyMasterAlloyDto.getAddTime()) == 0) {
            LoadingTask loadingTaskDb = loadingTaskService.findSuccessByLotNo(alloyMasterAlloyDto.getLotNo());
            alloyMasterAlloyDto.setFeedInWeight(loadingTaskDb.getWeight());
        }
        AlloyMasterAlloy alloyMasterAlloy = fromDto(alloyMasterAlloyDto);
        alloyMasterAlloy.setCreateUser(BaseContextHandler.getUsername());
        alloyMasterAlloy.setCreateDate(new Date());
        super.saveOrUpdate(alloyMasterAlloy);
        if (!CollectionUtils.isEmpty(alloyMasterAlloyDto.getCorrectingRecordDtoList())) {
            correctingRecordService.save(alloyMasterAlloyDto.getCorrectingRecordDtoList(), alloyMasterAlloy.getId());
            alloyMasterAlloy.setIsCorrecting(0);
        } else {
            alloyMasterAlloy.setIsCorrecting(1);
        }
        if (!CollectionUtils.isEmpty(alloyMasterAlloyDto.getInspectionListDtoList())) {
            inspectionListService.save(alloyMasterAlloyDto.getInspectionListDtoList(), alloyMasterAlloy.getId());
        }
        super.saveOrUpdate(alloyMasterAlloy);
    }

    @Override
    public void deleteById(Integer id) {
        AlloyMasterAlloy alloyMasterAlloy = super.get(id);
        Assert.notNull(alloyMasterAlloy, "母合金信息不存在");
        super.delete(id);
        correctingRecordService.deleteByIds(id);
        inspectionListService.deleteByIds(id);

    }

    private AlloyMasterAlloy fromDto(AlloyMasterAlloyDto alloyMasterAlloyDto) {
        return modelMapper.map(alloyMasterAlloyDto, AlloyMasterAlloy.class);
    }

    /**
     * 计算循环料重量和电阻
     */
    private void getRecycledMaterials(String lotNo, FurnaceBedDto furnaceBedDto, Integer againNum, Long id) {
        String recycledMaterials = "循环料";
        ArrayList<RecycledMaterialDto> arrayList = new ArrayList<>();
        if (againNum == 0) {
            List<LoadingTask> loadingTaskList = loadingTaskService.findRealWeightByLotNo(lotNo);
            List<Long> loadingTaskIdList = loadingTaskList.stream()
                    .map(LoadingTask::getId)
                    .collect(Collectors.toList());
            List<BlastFurnaceDetail> blastFurnaceDetailList = blastFurnaceDetailService.findByLoadingTaskIds(loadingTaskIdList);

            List<String> productCodeList = blastFurnaceDetailList.stream()
                    .map(BlastFurnaceDetail::getProductCode)
                    .collect(Collectors.toList());
            List<MaterialModel> materialModelList = materialModelService.findMaterialModelExtra2ByCode(productCodeList, recycledMaterials);

            Map<String, List<BlastFurnaceDetail>> listMap = blastFurnaceDetailList.stream()
                    .collect(Collectors.groupingBy(BlastFurnaceDetail::getProductCode));

            for (MaterialModel model : materialModelList) {
                RecycledMaterialDto dto = new RecycledMaterialDto();
                List<BlastFurnaceDetail> blastFurnaceDetailList1 = listMap.get(model.getCode());
                BigDecimal recycledMaterialWeight = blastFurnaceDetailList1.stream()
                        .map(BlastFurnaceDetail::getWeight)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                RawMaterialLevel rawMaterialLevelDb = rawMaterialLevelService.findByMaterialCode(model.getCode());
                dto.setRecycledMaterialWeight(recycledMaterialWeight);
                if (null != rawMaterialLevelDb) {
                    dto.setRecycledMaterialResistance(rawMaterialLevelDb.getResistanceValue());
                }
                arrayList.add(dto);
            }
        } else {
            List<AgainTaskDetail> againTaskDetailList = againTaskDetailService.findByAgainTaskId(id);

            List<Long> againTaskDetailIds = againTaskDetailList.stream()
                    .map(AgainTaskDetail::getId)
                    .collect(Collectors.toList());
            List<InputFurnaceDetail> inputFurnaceDetailList = inputFurnaceDetailService.findByAgainTaskDetailIdList(againTaskDetailIds);

            List<String> productCodeList = inputFurnaceDetailList.stream()
                    .map(InputFurnaceDetail::getProductCode)
                    .collect(Collectors.toList());
            List<MaterialModel> materialModelList = materialModelService.findMaterialModelExtra2ByCode(productCodeList, recycledMaterials);
            Map<String, List<InputFurnaceDetail>> listMap = inputFurnaceDetailList.stream()
                    .collect(Collectors.groupingBy(InputFurnaceDetail::getProductCode));

            for (MaterialModel model : materialModelList) {
                RecycledMaterialDto dto = new RecycledMaterialDto();
                List<InputFurnaceDetail> inputFurnaceDetailList1 = listMap.get(model.getCode());
                BigDecimal recycledMaterialWeight = inputFurnaceDetailList1.stream()
                        .map(InputFurnaceDetail::getWeight)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                dto.setRecycledMaterialWeight(recycledMaterialWeight);
                for (InputFurnaceDetail inputFurnaceDetail : inputFurnaceDetailList1) {
                    Result<WarehouseInputRecordBillDetailDto> billDetail = storeFeignClients.getByMaterialCodeAndBatchNo(inputFurnaceDetail.getMaterialCode(), inputFurnaceDetail.getBatchNo());
                    Assert.isTrue(billDetail.getCode() == 200, "调用线边仓接口失败");
                    Assert.isTrue(billDetail.getData().getMaterialGear() != null, "线边仓中未维护物料档位，请先维护");
                    RawMaterialLevel rawMaterialLevelDb = rawMaterialLevelService.findByMaterialCodeAndResistanceGear(model.getCode(), billDetail.getData().getMaterialGear());
                    Assert.notNull(rawMaterialLevelDb, "未在原料档位中维护，请先维护");
                    dto.setRecycledMaterialResistance(rawMaterialLevelDb.getResistanceValue());
                }
                arrayList.add(dto);
            }
        }
//        furnaceBedDto.setRecycledMaterialDtoList(arrayList);
        int size = arrayList.size();
        if (size >= 1) {
            furnaceBedDto.setRecycledMaterialOneResistance(arrayList.get(0).getRecycledMaterialResistance());
            furnaceBedDto.setRecycledMaterialOneWeight(arrayList.get(0).getRecycledMaterialWeight());
        }
        if (size >= 2) {
            furnaceBedDto.setRecycledMaterialTwoResistance(arrayList.get(1).getRecycledMaterialResistance());
            furnaceBedDto.setRecycledMaterialTwoWeight(arrayList.get(1).getRecycledMaterialWeight());
        }
        if (size >= 3) {
            furnaceBedDto.setRecycledMaterialThreeResistance(arrayList.get(2).getRecycledMaterialResistance());
            furnaceBedDto.setRecycledMaterialThreeWeight(arrayList.get(2).getRecycledMaterialWeight());
        }
        if (size >= 4) {
            furnaceBedDto.setRecycledMaterialFourResistance(arrayList.get(3).getRecycledMaterialResistance());
            furnaceBedDto.setRecycledMaterialFourWeight(arrayList.get(3).getRecycledMaterialWeight());
        }
        if (size >= 5) {
            furnaceBedDto.setRecycledMaterialFiveResistance(arrayList.get(4).getRecycledMaterialResistance());
            furnaceBedDto.setRecycledMaterialFiveWeight(arrayList.get(4).getRecycledMaterialWeight());
        }
        if (size >= 6) {
            furnaceBedDto.setRecycledMaterialSixResistance(arrayList.get(5).getRecycledMaterialResistance());
            furnaceBedDto.setRecycledMaterialSixWeight(arrayList.get(5).getRecycledMaterialWeight());
        }
        if (size >= 7) {
            furnaceBedDto.setRecycledMaterialSevenResistance(arrayList.get(6).getRecycledMaterialResistance());
            furnaceBedDto.setRecycledMaterialSevenWeight(arrayList.get(6).getRecycledMaterialWeight());
        }
    }

    /**
     * 计算多晶硅重量和电阻
     */
    private void getPolysiliconc(String lotNo, FurnaceBedDto furnaceBedDto, Integer againNum, Long id) {
        //            多晶硅
        String polysiliconc = "多晶硅";
        if (againNum == 0) {
            List<LoadingTask> loadingTaskList = loadingTaskService.findRealWeightByLotNo(lotNo);
            List<Long> loadingTaskIdList = loadingTaskList.stream()
                    .map(LoadingTask::getId)
                    .collect(Collectors.toList());
            List<BlastFurnaceDetail> blastFurnaceDetailList = blastFurnaceDetailService.findByLoadingTaskIds(loadingTaskIdList);

            List<String> productCodeList = blastFurnaceDetailList.stream()
                    .map(BlastFurnaceDetail::getProductCode)
                    .collect(Collectors.toList());
            List<MaterialModel> materialModelList = materialModelService.findMaterialModelExtra2ByCode(productCodeList, polysiliconc);

            Map<String, List<BlastFurnaceDetail>> listMap = blastFurnaceDetailList.stream()
                    .collect(Collectors.groupingBy(BlastFurnaceDetail::getProductCode));

            for (MaterialModel model : materialModelList) {
                List<BlastFurnaceDetail> blastFurnaceDetailList1 = listMap.get(model.getCode());
                BigDecimal recycledMaterialWeight = blastFurnaceDetailList1.stream()
                        .map(BlastFurnaceDetail::getWeight)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                RawMaterialLevel rawMaterialLevelDb = rawMaterialLevelService.findByMaterialCode(model.getCode());
                furnaceBedDto.setPolysiliconWeight(recycledMaterialWeight);
                if (null != rawMaterialLevelDb) {
                    furnaceBedDto.setPolysiliconResistance(rawMaterialLevelDb.getResistanceValue());
                }
            }
        } else {
            List<AgainTaskDetail> againTaskDetailList = againTaskDetailService.findByAgainTaskId(id);

            List<Long> againTaskDetailIds = againTaskDetailList.stream()
                    .map(AgainTaskDetail::getId)
                    .collect(Collectors.toList());
            List<InputFurnaceDetail> inputFurnaceDetailList = inputFurnaceDetailService.findByAgainTaskDetailIdList(againTaskDetailIds);

            List<String> productCodeList = inputFurnaceDetailList.stream()
                    .map(InputFurnaceDetail::getProductCode)
                    .collect(Collectors.toList());
            List<MaterialModel> materialModelList = materialModelService.findMaterialModelExtra2ByCode(productCodeList, polysiliconc);
            Map<String, List<InputFurnaceDetail>> listMap = inputFurnaceDetailList.stream()
                    .collect(Collectors.groupingBy(InputFurnaceDetail::getProductCode));

            for (MaterialModel model : materialModelList) {
                List<InputFurnaceDetail> inputFurnaceDetailList1 = listMap.get(model.getCode());
//                计算多晶硅重量
                BigDecimal recycledMaterialWeight = inputFurnaceDetailList1.stream()
                        .map(InputFurnaceDetail::getWeight)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                furnaceBedDto.setPolysiliconWeight(recycledMaterialWeight);
                for (InputFurnaceDetail inputFurnaceDetail : inputFurnaceDetailList1) {
//            根据物料号和批次号查询电阻档位
                    Result<WarehouseInputRecordBillDetailDto> billDetail = storeFeignClients.getByMaterialCodeAndBatchNo(inputFurnaceDetail.getMaterialCode(), inputFurnaceDetail.getBatchNo());
                    Assert.isTrue(billDetail.getCode() == 200, "调用线边仓接口失败");
                    Assert.isTrue(billDetail.getData().getMaterialGear() != null, "线边仓中未维护物料档位，请先维护");
                    RawMaterialLevel rawMaterialLevelDb = rawMaterialLevelService.findByMaterialCodeAndResistanceGear(model.getCode(), billDetail.getData().getMaterialGear());
                    Assert.notNull(rawMaterialLevelDb, "未在原料档位中维护，请先维护");
                    furnaceBedDto.setPolysiliconResistance(rawMaterialLevelDb.getResistanceValue());
                }
            }
        }
    }

    /**
     * 计算三棒以上上一棒电阻
     */
    private void getUpBarResistance(String furnaceCode, FurnaceBedDto furnaceBedDto, MaterialModel materialModel, Integer againNum2, Integer againNum1, String lotNo) {
        ArrayList<CorrectingRecordDto> list = new ArrayList<>();
        CorrectingRecordDto dto = new CorrectingRecordDto();
        //                上一棒电阻
        List<YbInspectData> ybInspectDataList = ybInspectDataService.findByLotNoAndAgainNum(lotNo, againNum2);
        if (!CollectionUtils.isEmpty(ybInspectDataList)) {
            YbInspectData ybInspectData = Collections.max(ybInspectDataList, Comparator.comparing(YbInspectData::getId));
            if (null != ybInspectData) {
                BigDecimal tResistivity = ybInspectData.getTResistivity();
                furnaceBedDto.setCorrecting(tResistivity);
                dto.setCorrectingValue(tResistivity);
                dto.setAddTime(String.valueOf(againNum2 + 2));
                FixLogic fixLogicDb = fixLogicService.findByFixUpperLimit(tResistivity);
                if (null != fixLogicDb) {
                    ResistanceOfThisRod resistanceOfThisRodDb2 = resistanceOfThisRodService.findByFurnaceCodeAndAddTime(furnaceCode, againNum1);
                    furnaceBedDto.setUpBarResistance(resistanceOfThisRodDb2.getResistanceOfThisRod());
                } else {
                    List<LoadingTask> loadingTaskList = loadingTaskService.findRealWeightByLotNo(lotNo);
                    LoadingTask loadingTask = loadingTaskList.stream()
                            .max(Comparator.comparingLong(LoadingTask::getId)).orElse(null);
                    if (null != loadingTask && null != loadingTask.getDimension()) {
                        FixLogic fixLogic = fixLogicService.findByHeatFieldAndMaterialShortNameAndAddTime(loadingTask.getDimension(), materialModel.getProductName(), againNum2);
                        if (null != fixLogic) {
                            BigDecimal fixUpperLimit = fixLogic.getFixUpperLimit();
                            BigDecimal fixLowerLimit = fixLogic.getFixLowerLimit();
                            BigDecimal upperLimitGradient = fixLogic.getUpperLimitGradient();
                            BigDecimal lowerLimitGradient = fixLogic.getLowerLimitGradient();
                            BigDecimal fixUpperLimitConstant = fixLogic.getFixUpperLimitConstant();
                            BigDecimal fixLowerLimitConstant = fixLogic.getFixLowerLimitConstant();
//                            修正上限
                            BigDecimal upperLimit = tResistivity.add(tResistivity.subtract(fixUpperLimit.add(fixLowerLimit))
                                            .divide(new BigDecimal(2), 2, RoundingMode.HALF_UP))
                                    .divide(upperLimitGradient.multiply(fixUpperLimitConstant), 2, RoundingMode.HALF_UP);
//                            修正下限
                            BigDecimal lowerLimit = ((fixUpperLimit.add(fixLowerLimit)).divide(new BigDecimal(2), 2, RoundingMode.HALF_UP)
                                    .subtract(tResistivity))
                                    .divide(lowerLimitGradient.multiply(fixLowerLimitConstant), 2, RoundingMode.HALF_UP);
//                            实测电阻大于等于修正上限
                            if (tResistivity.compareTo(upperLimit) >= 0) {
                                furnaceBedDto.setUpBarResistance(upperLimit);
                                furnaceBedDto.setCorrectingAfterResistance(upperLimit);
                                dto.setCorrectingAfterResistance(upperLimit);
                            }
                            if (tResistivity.compareTo(lowerLimit) <= 0) {
                                furnaceBedDto.setUpBarResistance(lowerLimit);
                                furnaceBedDto.setCorrectingAfterResistance(lowerLimit);
                                dto.setCorrectingAfterResistance(lowerLimit);
                            }
                        }
                    }
                }
            }
        }
        list.add(dto);

        if (!CollectionUtils.isEmpty(list)) {
            furnaceBedDto.setCorrectingRecordDtoList(list);
        }
    }

    /**
     * 获取本棒电阻
     *
     * @param isEnd 是否完结
     */
    private BigDecimal getResistanceOfThisRod(Integer isEnd, Integer againNum, String furnaceCode) {
        if (isEnd == 0) {
            ResistanceOfThisRod resistanceOfThisRodDb = resistanceOfThisRodService.findByFurnaceCodeAndAddTime(furnaceCode, 99);
            Assert.notNull(resistanceOfThisRodDb, "本棒电阻中不存在复投次数为99的数据，请先配置!!!");
            return resistanceOfThisRodDb.getResistanceOfThisRod();
        } else {
            ResistanceOfThisRod resistanceOfThisRodDb = resistanceOfThisRodService.findByFurnaceCodeAndAddTime(furnaceCode, againNum);
            Assert.notNull(resistanceOfThisRodDb, "本棒电阻不存在,请先配置!!!");
            return resistanceOfThisRodDb.getResistanceOfThisRod();
        }
    }

    private static void buildFurnaceBedDto(String furnaceCode, FurnaceBedDto furnaceBedDto, Integer againNum,
                                           String sapNo, MaterialModel materialModel, BigDecimal resistanceOfThisRod,
                                           String lotNo, Integer isEnd) {
        furnaceBedDto.setWorkNumber(BaseContextHandler.getUserClassNo());
        furnaceBedDto.setUserName(BaseContextHandler.getUsername());
        furnaceBedDto.setFurnaceCode(furnaceCode);
        furnaceBedDto.setLotNo(lotNo);
        furnaceBedDto.setAddTime(againNum);
        if (null != sapNo) {
            furnaceBedDto.setWorkOrderNumber(sapNo);
        }
        furnaceBedDto.setInformationMainProduct(materialModel.getProductName());
        furnaceBedDto.setInformationMessage(lotNo + sapNo + againNum);
        furnaceBedDto.setIsEnd(isEnd);
        furnaceBedDto.setResistanceOfThisRod(resistanceOfThisRod);
    }
}
