package com.sw.mes.yb.alloy.service.impl;


import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.yb.alloy.dto.RawMaterialLevelDto;
import com.sw.mes.yb.alloy.dto.RawMaterialLevelImportDto;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.sw.mes.yb.alloy.entity.RawMaterialLevel;
import com.sw.mes.yb.alloy.dao.RawMaterialLevelDao;
import com.sw.mes.yb.alloy.service.RawMaterialLevelService;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@Service
@Transactional
public class RawMaterialLevelServiceImpl extends BaseServiceImpl<RawMaterialLevel> implements RawMaterialLevelService {
    RawMaterialLevelDao dao;

    @Autowired
    public void setDao(RawMaterialLevelDao dao) {
        super.setBaseDao(dao);
        this.dao = dao;
    }

    @Autowired
    private ModelMapper modelMapper;

    @Override
    public void importRawMaterialLevel(List<RawMaterialLevelImportDto> rawMaterialLevelImportDtoList) {
        if (CollectionUtils.isEmpty(rawMaterialLevelImportDtoList)) {
            throw new RuntimeException("原料导入信息不存在");
        }
        ArrayList<RawMaterialLevel> rawMaterialLevelArrayList = new ArrayList<>();
        for (RawMaterialLevelImportDto rawMaterialLevelDto : rawMaterialLevelImportDtoList) {
            RawMaterialLevel rawMaterialLevel = new RawMaterialLevel();
            RawMaterialLevel entityfromFto = fromDto(rawMaterialLevelDto);
            BeanUtils.copyProperties(entityfromFto, rawMaterialLevel, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            rawMaterialLevel.setCreateUser(BaseContextHandler.getUsername());
            rawMaterialLevel.setCreateDate(new Date());
            rawMaterialLevelArrayList.add(rawMaterialLevel);
        }
        dao.saveAll(rawMaterialLevelArrayList);
    }

    @Override
    public RawMaterialLevel findByMaterialCode(String code) {
        return dao.findByMaterialCode(code);
    }

    @Override
    public RawMaterialLevel findByMaterialCodeAndResistanceGear(String code, String materialGear) {
        return dao.findByMaterialCodeAndResistanceGear(code,materialGear);
    }

    private RawMaterialLevel fromDto(RawMaterialLevelImportDto dto) {
        RawMaterialLevel map = modelMapper.map(dto, RawMaterialLevel.class);
        if (null != dto.getResistanceGear()) {
            map.setResistanceGear(dto.getResistanceGear());

        }
        if (null != dto.getResistanceValue()) {
            map.setResistanceValue(new BigDecimal(dto.getResistanceValue()));
        }
        return map;
    }


}
