package com.sw.mes.yb.mcs.controller;

import com.sw.mes.base.factory.init.dto.SortingConfigDto;
import com.sw.mes.base.factory.init.dto.WarehouseDto;
import com.sw.mes.core.dto.RankResultDto;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.yb.mcs.dto.*;
import com.sw.mes.yb.mcs.service.YbMesToMcsService;
import com.sw.mes.yb.mcs.utils.ResultDto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * @Author: GaoXinJian
 * @Date: 2022-01-12 10:03
 */
@RestController
@RequestMapping("/mesToMcsController")
@Slf4j
@Api(description = "mes与mcs通信接口")
public class MesToMcsController {
    @Autowired
    YbMesToMcsService mesToMcsService;

    @ApiOperation(value = "圆棒托盘绑定接口")
    @PostMapping("/crystalTrayBindInfo")
    public Result crystalTrayBindInfo(@RequestBody CrystalTrayBindInfoDto crystalTrayBindInfoDto) {
        ResultDto resultDto = mesToMcsService.crystalTrayBindInfo(crystalTrayBindInfoDto);
        if (resultDto.getStatus() == 100) {
            return ResultGenerator.success(resultDto.getMessage(), resultDto.getData());
        } else {
            return ResultGenerator.message(Integer.valueOf(resultDto.getStatus()),resultDto.getMessage(),resultDto.getData());
        }
    }

     @ApiOperation(value = "圆棒托盘解绑接口")
    @PostMapping("/crystalTrayUnBindInfo")
    public Result crystalTrayUnBindInfo(@RequestBody CrystalTrayUnBindInfoDto crystalTrayUnBindInfoDto) {
        ResultDto resultDto = mesToMcsService.crystalTrayUnBindInfo(crystalTrayUnBindInfoDto);
        if (resultDto.getStatus() == 100) {
            return ResultGenerator.success(resultDto.getMessage(), resultDto.getData());
        } else {
            return ResultGenerator.message(Integer.valueOf(resultDto.getStatus()),resultDto.getMessage(),resultDto.getData());
        }
    }

    @ApiOperation(value = "圆棒评级结果下发")
    @PostMapping("/rankResult")
    public Result rankResult(@RequestBody RankResultDto rankResultDto) {
        ResultDto resultDto=mesToMcsService.rankResult(rankResultDto);
        if (resultDto.getStatus() == 100) {
            return ResultGenerator.success(resultDto.getMessage(), resultDto.getData());
        } else {
            return ResultGenerator.message(Integer.valueOf(resultDto.getStatus()),resultDto.getMessage(),resultDto.getData());
        }
    }

    @ApiOperation(value = "订单水位信息下发接口")
    @PostMapping("/orderConfig")
    public Result orderConfig(@RequestBody List<OrderConfigDto> orderConfigDtoList) {
        ResultDto resultDto=mesToMcsService.orderConfig(orderConfigDtoList);
        if (resultDto.getStatus() == 100) {
            return ResultGenerator.success(resultDto.getMessage(), resultDto.getData());
        } else {
            return ResultGenerator.message(Integer.valueOf(resultDto.getStatus()),resultDto.getMessage(),resultDto.getData());
        }
    }

    @ApiOperation(value = "拼棒绑定信息上传接口")
    @PostMapping("/spellBinding")
    public Result spellBinding(@RequestBody SpellBindingDto spellBindingDto) {
        ResultDto resultDto=mesToMcsService.spellBinding(spellBindingDto);
        if (resultDto.getStatus() == 100) {
            return ResultGenerator.success(resultDto.getMessage(), resultDto.getData());
        } else {
            return ResultGenerator.message(Integer.valueOf(resultDto.getStatus()),resultDto.getMessage(),resultDto.getData());
        }
    }

    @ApiOperation(value = "分选口规格配置信息下发接口")
    @PostMapping("/blankConfig")
    public Result blankConfig(@RequestBody List<BlankConfigDto> blankConfigDtoList) {
        ResultDto resultDto=mesToMcsService.blankConfig(blankConfigDtoList);
        if (resultDto.getStatus() == 100) {
            return ResultGenerator.success(resultDto.getMessage(), resultDto.getData());
        } else {
            return ResultGenerator.message(Integer.valueOf(resultDto.getStatus()),resultDto.getMessage(),resultDto.getData());
        }
    }


    @ApiOperation(value = "分选口更新前校验是否允许修改")
    @PostMapping("/blankConfigUpdateCheck")
    public ResultDto blankConfigUpdateCheck(@RequestBody SortingConfigDto sortingConfigDto) {
        ResultDto resultDto = mesToMcsService.blankConfigForUpdateCheck(sortingConfigDto);
        return resultDto;
    }



    @ApiOperation(value = "晶棒绑定-人工单晶上线体,调用MCS")
    @PostMapping("/pdaBing")
    public Result pdaBing(@RequestBody PdaBingDto pdaBingDto) {
        return mesToMcsService.pdaBing(pdaBingDto);
    }

    @ApiOperation(value = "晶棒绑定-人工单晶上线体,调用MCS-方棒")
    @PostMapping("/pdaBingFb")
    public Result pdaBingFb(@RequestBody PdaBingDto pdaBingDto) {
        return mesToMcsService.pdaBingFb(pdaBingDto);
    }

    @ApiOperation(value = "晶棒绑定-人工单晶上线体,调用MCS")
    @PostMapping("/unPdaBing")
    public Result unPdaBing(@RequestBody PdaBingDto pdaBingDto) {
        return mesToMcsService.unPdaBing(pdaBingDto);
    }

    @ApiOperation(value = "晶棒绑定-人工单晶上线体,调用MCS-方棒")
    @PostMapping("/unPdaBingFb")
    public Result unPdaBingFb(@RequestBody PdaBingDto pdaBingDto) {
        return mesToMcsService.unPdaBingFb(pdaBingDto);
    }


    @ApiOperation(value = "更新库位载具")
    @PostMapping("/updateWarehouse")
    public Result updateWarehouse(@RequestBody WarehouseDto warehouseDto) {
        return mesToMcsService.updateWarehouse(warehouseDto);
    }
}