package com.sw.mes.yb.mcs.service.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sw.mes.base.factory.inspect.entity.DefaultSample;
import com.sw.mes.base.factory.inspect.service.DefaultSampleService;
import com.sw.mes.base.system.entity.DictionaryItem;
import com.sw.mes.base.system.service.DictionaryItemService;
import com.sw.mes.base.workinprocess.dto.CurrentWipStorageStatusDto;
import com.sw.mes.base.workinprocess.dto.PlanProcessingOrderWipStorageDto;
import com.sw.mes.base.workinprocess.entity.PlanProcessingOrder;
import com.sw.mes.base.workinprocess.service.PlanProcessingOrderService;
import com.sw.mes.base.workinprocess.service.WipStorageService;
import com.sw.mes.core.api.system.FbFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.RankResultDto;
import com.sw.mes.core.dto.mes.pc.yb.ProductionBindingTrayDto;
import com.sw.mes.core.enums.EnumValuesFactory;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.inspect.dto.YbInspectDataCUDto;
import com.sw.mes.inspect.entity.YbInspectData;
import com.sw.mes.inspect.service.YbInspectDataService;
import com.sw.mes.production.service.TruncationReccutDetailsService;
import com.sw.mes.wipstorage.service.WipStorageYbService;
import com.sw.mes.yb.check.entity.PieceCheck;
import com.sw.mes.yb.check.service.PieceCheckService;
import com.sw.mes.yb.fls.dto.TrayAutoUnBinding;
import com.sw.mes.yb.fls.service.FlsToMesService;
import com.sw.mes.yb.mcs.dto.*;
import com.sw.mes.yb.mcs.entity.*;
import com.sw.mes.yb.mcs.service.*;
import com.sw.mes.yb.pad.dao.PieceDetailDao;
import com.sw.mes.yb.pad.entity.PieceDetail;
import com.sw.mes.yb.pad.entity.ProductionPaintingLineDetail;
import com.sw.mes.yb.pad.service.ProductionPaintingLineDetailService;
import com.sw.mes.yb.pad.service.ProductionPaintingLineService;
import com.sw.mes.yb.pc.service.HeadTailMaterialRuleService;
import com.sw.mes.yb.production.dao.LongCrystalDao;
import com.sw.mes.yb.production.entity.LongCrystal;
import com.sw.mes.yb.production.service.LongCrystalService;
import com.sw.mes.yb.turnstore.service.ProductionBindingTrayService;
import org.apache.commons.lang3.StringUtils;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.*;

/**
 * @Author: GaoXinJian
 * @Date: 2022-01-14 10:56
 */
@Service
@Transactional
public class YbMcsToMesServiceImpl implements YbMcsToMesService {
    @Autowired
    ModelMapper modelMapper;
    @Autowired
    private YbInspectDataService inspectDataBaseService;
    @Autowired
    private RecycledMaterialCaseService recycledMaterialCaseService;
    @Autowired
    private RecycledMaterialCaseRecordService recycledMaterialCaseRecordService;
    @Autowired
    @Lazy
    private FbFeignClients fbFeignClients;

    @Autowired
    private PlanProcessingOrderService planProcessingOrderService;
    /**
     * 托盘绑定service
     */
    @Autowired
    private ProductionBindingTrayService productionBindingTrayService;

    /**
     * 头尾料分类规则service
     */
    @Autowired
    HeadTailMaterialRuleService headTailMaterialRuleService;

    @Autowired
    private WipStorageService wipStorageService;

    @Autowired
    private SegmentInspectionPassPointInfoService segmentInspectionPassPointInfoService;

    @Autowired
    private SegmentInspectionPassPointRecordService segmentInspectionPassPointRecordService;

    @Autowired
    private WipStorageYbService wipStorageYbService;

    @Autowired
    private PieceCheckService pieceCheckService;

    @Autowired
    private PieceDetailDao pieceDetailDao;

    @Autowired
    private LongCrystalDao longCrystalDao;

    @Autowired
    private DefaultSampleService defaultSampleService;

    @Autowired
    private ProductionPaintingLineDetailService productionPaintingLineDetailService;

    @Autowired
    private FlsToMesService flsToMesService;
    /**
     * 截断返切记录service
     */
    @Autowired
    private TruncationReccutDetailsService truncationReccutDetailsService;

    @Autowired
    private DislocationInfoService dislocationInfoService;

    @Autowired
    private DislocationExceptionInfoService dislocationExceptionInfoService;

    @Autowired
    private ProductionPaintingLineService productionPaintingLineService;

    @Autowired
    private LongCrystalService longCrystalService;

    @Autowired
    private DictionaryItemService dictionaryItemService;

    @Autowired
    private McsSquareInventoryLogService mcsSquareInventoryLogService;

    @Autowired
    private SegmentBindInfoService segmentBindInfoService;

    @Override
    public Result segmentCheckInfo(SegmentCheckInfoDto segmentCheckInfoDto) {
        //托盘流转至检验区，MCS根据检验线点位将晶段上下检验信息发送给MES
        if (StringUtils.isBlank(segmentCheckInfoDto.getIdentification())) {
            //无上下线表示，暂不处理
            return ResultGenerator.success();
        }
        //下线
        if (segmentCheckInfoDto.getIdentification().equals("1")) {
            //删除上线信息
            List<SegmentInspectionPassPointInfo> segmentInspectionPassPointInfoList = segmentInspectionPassPointInfoService.findBySectionNo(segmentCheckInfoDto.getSectionNo());
            for (SegmentInspectionPassPointInfo segmentInspectionPassPointInfo : segmentInspectionPassPointInfoList) {
                segmentInspectionPassPointInfoService.delete(segmentInspectionPassPointInfo.getId());
            }

            //上线
        } else if (segmentCheckInfoDto.getIdentification().equals("0")) {
            //创建上线信息
            SegmentInspectionPassPointInfo segmentInspectionPassPointInfo = JSONObject.parseObject(JSONObject.toJSONString(segmentCheckInfoDto), SegmentInspectionPassPointInfo.class);
            segmentInspectionPassPointInfo.setCreateTime(new Date());
            segmentInspectionPassPointInfoService.saveOrUpdate(segmentInspectionPassPointInfo);
        } else {
            //除上下线点位外，暂不处理
            return ResultGenerator.success();
        }
        //创建上下限记录

        SegmentInspectionPassPointRecord segmentInspectionPassPointRecord = JSONObject.parseObject(JSONObject.toJSONString(segmentCheckInfoDto), SegmentInspectionPassPointRecord.class);
        segmentInspectionPassPointRecord.setCreateTime(new Date());
        segmentInspectionPassPointRecordService.saveOrUpdate(segmentInspectionPassPointRecord);


        return ResultGenerator.success();
    }

    @Override
    public Result ybCheckResult(YbCheckResultDto ybCheckResultDto) {
        YbInspectDataCUDto ybInspectDataCuDto = modelMapper.map(ybCheckResultDto, YbInspectDataCUDto.class);
        if (StringUtils.isBlank(ybInspectDataCuDto.getCheckReback())) {
            ybInspectDataCuDto.setCheckReback("无");
        }
        // 导电型号 -> 少子寿命  (sz * 1.801)+0.0658
        YbInspectDataCUDto coefficient = inspectDataBaseService.coefficient(ybInspectDataCuDto);
        inspectDataBaseService.saveYbInspectData(coefficient);
        return ResultGenerator.success();
    }

    @Override
    public Result skinCodingInfo(String segmentNo) {
        return null;
    }

    @Override
    public Result unbindInfo(UnbindInfoDto unbindInfoDto) {
        //解绑
        TrayAutoUnBinding trayAutoUnBinding = new TrayAutoUnBinding();
        trayAutoUnBinding.setPoint("截断上料");
        trayAutoUnBinding.setSectionNo(unbindInfoDto.getCrystalNo());
        trayAutoUnBinding.setReason("");
        flsToMesService.trayAutoUnBinding(trayAutoUnBinding, "mcs");
        //查询当前站点信息，判断是否满足进站条件
        PlanProcessingOrderWipStorageDto planProcessingOrderWipStorageDto = wipStorageService.selectWipStorage(unbindInfoDto.getCrystalNo(), EnumValuesFactory.WipStorageCode.圆棒截断.getKey(), false);
        List<String> processingOrderCodeArr = wipStorageYbService.getSectionNos(planProcessingOrderWipStorageDto.getPlanProcessingOrder().getMaterialType(), planProcessingOrderWipStorageDto.getPlanProcessingOrder().getCode());
        for (String processingOrderCode : processingOrderCodeArr) {
            if (!processingOrderCode.equals(planProcessingOrderWipStorageDto.getPlanProcessingOrder().getCode())) {
                wipStorageService.selectWipStorage(processingOrderCode, EnumValuesFactory.WipStorageCode.圆棒截断.getKey(), false);
            }
            Result result = wipStorageService.inStation(processingOrderCode, unbindInfoDto.getCutNo());
            if (result.getCode() != ResultCode.SUCCESS.getCode()) {
                throw new BusinessException(result.getMessage());
            }
        }
        //出截断站并生成子工单
        wipStorageYbService.execJd(planProcessingOrderWipStorageDto, true);
        return ResultGenerator.success();
    }

    @Override
    public HeadTailMaterialTypeDto getHeadTailMaterialType(String code) {
        HeadTailMaterialTypeDto headTailMaterialTypeDto = headTailMaterialRuleService.findHeadTailMaterialRuleByRecoveryNo(code);
        if (headTailMaterialTypeDto == null) {
            throw new RuntimeException("无匹配的头尾料分类规则");
        }

        return headTailMaterialTypeDto;
    }

    @Override
    public void headTailMaterialInfo(HeadTailMaterialInfoDto headTailMaterialInfoDto) {
        //查询并删除回收料箱信息
        List<RecycledMaterialCase> recycledMaterialCaseList = recycledMaterialCaseService.findByCaseNo(headTailMaterialInfoDto.getCaseNo());
        for (RecycledMaterialCase recycledMaterialCase : recycledMaterialCaseList) {
            recycledMaterialCaseService.delete(recycledMaterialCase.getId());
        }
        //创建回收料箱信息
        RecycledMaterialCase recycledMaterialCase = JSONObject.parseObject(JSONObject.toJSONString(headTailMaterialInfoDto), RecycledMaterialCase.class);
        recycledMaterialCaseService.saveOrUpdate(recycledMaterialCase);
        //创建回收料箱记录
        RecycledMaterialCaseRecord recycledMaterialCaseRecord = JSONObject.parseObject(JSONObject.toJSONString(headTailMaterialInfoDto), RecycledMaterialCaseRecord.class);
        recycledMaterialCaseRecordService.saveOrUpdate(recycledMaterialCaseRecord);

    }

    @Override
    public Result sectionWipStorageInfo(SkinCodingInfoDto skinCodingInfoDto) {
        String segmentNo = skinCodingInfoDto.getSegmentNo();
        if (null==segmentNo || StringUtils.isBlank(segmentNo)){
            return ResultGenerator.fail("晶段编号为空！");
        }
        PlanProcessingOrder byCode = planProcessingOrderService.findByCode(segmentNo);
        if (null == byCode) {
            return ResultGenerator.fail("加工工单表中无此编号： " + segmentNo);
        }
        if (null == byCode.getStatus() || byCode.getStatus() != 1) {
            return ResultGenerator.fail("此编号： " + segmentNo + " 不在加工中状态");
        }

//        if (!EnumValuesFactory.WipStorageCode.圆棒检验.getValue().equals(byCode.getWipStorage().getProcessName())) {
//            return ResultGenerator.fail("此编号： " + segmentNo + " 站点不在圆棒检验");
//        }
//        // 待进站和已进站通过
//        if (byCode.getWipStorage().getStatus() == EnumValuesFactory.WipStorageStatus.已出站.getKey()
//        ) {
//            return ResultGenerator.fail("此编号： " + segmentNo + " 已出站");
//        }
//        String toMcsInfo = byCode.getWipStorage().getProcessName() + EnumValuesFactory.WipStorageStatus.valueOf(byCode.getWipStorage().getProcessName());
        String toMcsInfo = byCode.getWipStorage().getProcessName();
        return ResultGenerator.success(toMcsInfo,null);
    }

    @Override
    public Result ybInspectStationIn(SkinCodingInfoDto skinCodingInfoDto) {
        //校验卡控：不是检验待进站状态不允许当前操作
        Result<CurrentWipStorageStatusDto> currentWipStorageStatusDtoResult = wipStorageService.selectWipStorageResult(skinCodingInfoDto.getSegmentNo());
        if (currentWipStorageStatusDtoResult.getCode() != ResultCode.SUCCESS.getCode()) {
            throw new BusinessException(currentWipStorageStatusDtoResult.getMessage());
        }
        CurrentWipStorageStatusDto currentWipStorageStatusDto = currentWipStorageStatusDtoResult.getData();
        if (currentWipStorageStatusDto == null || currentWipStorageStatusDto.getWipStorageCode() == null || currentWipStorageStatusDto.getWipStorageStatus() == null) {
            throw new BusinessException("站点信息查询失败，请检查在制品信息");
        }
        //如果不是检验待进站状态，报错
        boolean isStatus = currentWipStorageStatusDto.getWipStorageCode().equals(EnumValuesFactory.WipStorageCode.圆棒检验.getKey())
                && currentWipStorageStatusDto.getWipStorageStatus() == EnumValuesFactory.WipStorageStatus.待进站.getKey();
        if (!isStatus){
            return ResultGenerator.fail("晶棒:"+skinCodingInfoDto.getSegmentNo()+"不是检验待进站状态不允许当前操作！");
        }
        //操作过站
        Result result = wipStorageService.inStation(skinCodingInfoDto.getSegmentNo(), null, false);
        if (result.getCode() != ResultCode.SUCCESS.getCode()) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return result;
        }
        return ResultGenerator.success();
    }

    @Override
    public Result checkForAutoBind(PdaBingDto pdaBingDto) {

        //校验卡控：不是检验待进站状态不允许当前操作
        Result<CurrentWipStorageStatusDto> currentWipStorageStatusDtoResult = wipStorageService.selectWipStorageResult(pdaBingDto.getSectionNo());
        if (currentWipStorageStatusDtoResult.getCode() != ResultCode.SUCCESS.getCode()) {
            throw new BusinessException(currentWipStorageStatusDtoResult.getMessage());
        }
        CurrentWipStorageStatusDto currentWipStorageStatusDto = currentWipStorageStatusDtoResult.getData();
        if (currentWipStorageStatusDto == null || currentWipStorageStatusDto.getWipStorageCode() == null || currentWipStorageStatusDto.getWipStorageStatus() == null) {
            throw new BusinessException("站点信息查询失败，请检查在制品信息");
        }
        //如果不是检验待进站状态，报错
        boolean isStatus = currentWipStorageStatusDto.getWipStorageCode().equals(EnumValuesFactory.WipStorageCode.圆棒检验.getKey())
                && currentWipStorageStatusDto.getWipStorageStatus() == EnumValuesFactory.WipStorageStatus.待进站.getKey();
        if (!isStatus){
            return ResultGenerator.fail("晶棒:"+pdaBingDto.getSectionNo()+"不是检验待进站状态不允许当前操作！");
        }

        //校验是否存在绑定记录，存在则告警
        List<SegmentBindInfo> bindInfos = segmentBindInfoService.findBysegmentNo(pdaBingDto.getSectionNo());
        if (null!= bindInfos && !bindInfos.isEmpty()){
            return ResultGenerator.fail("晶棒:"+pdaBingDto.getSectionNo()+"已经执行过绑定操作不允许重复绑定！");
        }

        //绑定信息表中新增一条记录，用以预防重复绑定
        SegmentBindInfo segmentBindInfo = new SegmentBindInfo();
        segmentBindInfo.setSegmentNo(pdaBingDto.getSectionNo());
        segmentBindInfo.setDeviceNo(pdaBingDto.getCutNo());
        segmentBindInfo.setCreateTime(new Date());
        segmentBindInfo.setCreateUserName(BaseContextHandler.getUsername());
        segmentBindInfoService.saveOrUpdate(segmentBindInfo);

        // 中间段免检
        Map<String, String> map = productionPaintingLineDetailService.flowDirection(pdaBingDto.getSectionNo(), "截断下料");
        if(!map.isEmpty() && map.containsKey("index")){
            map.remove("index");
        }

        return ResultGenerator.success(map);
    }

    @Override
    public Result squareOrder(SquareOrderMesDto squareOrderDto) {
        SquareOrderMesResultDto squareOrderMesResultDto = new SquareOrderMesResultDto();
        //查询检验数据库
        YbInspectData inspectDataBase = inspectDataBaseService.findBySectionNo(squareOrderDto.getSectionNo());
        //匹配订单编号
        if (inspectDataBase != null) {
            RankResultDto rankResultDto = new RankResultDto();
            rankResultDto.setSectionNo(inspectDataBase.getSectionNo());
            rankResultDto.setProductRank(inspectDataBase.getProductLevel());
            rankResultDto.setTestRank(inspectDataBase.getProductLevelExperiment());
            rankResultDto.setType(inspectDataBase.getSectionType());
            rankResultDto.setGradeType(inspectDataBase.getGradeType());
            rankResultDto.setSpecs(inspectDataBase.getProductTypeSecond());
            rankResultDto.setConductiveType(inspectDataBase.getConductiveType());
            rankResultDto.setDopant(inspectDataBase.getDopant());
            String type = inspectDataBase.getSectionType();
            if (inspectDataBase.getSticksCount() != null && inspectDataBase.getSticksCount() > 1) {
                rankResultDto.setLength(inspectDataBase.getSpellBarLength());
                type = "拼棒";
            } else {
                rankResultDto.setLength(inspectDataBase.getActualLength());
            }
            if ("标棒".equals(rankResultDto.getType()) && "正常".equals(rankResultDto.getGradeType())) {
                Result<String> result = fbFeignClients.findProductionScheduleCodePro(rankResultDto);
                if (result.getCode() == ResultCode.SUCCESS.getCode() && result.getData() != null && StringUtils.isNotBlank(result.getData())) {
                    rankResultDto.setOrderNo(result.getData());
                }
            }

            if (inspectDataBase.getIsCutBack() != null && inspectDataBase.getIsCutBack() == 1) {
                rankResultDto.setType("反切");
            } else {
                rankResultDto.setType(type);
            }

            squareOrderMesResultDto = toLazyDto(rankResultDto);
        }

        return ResultGenerator.success(squareOrderMesResultDto);
    }

    @Override
    public Result sampleInfo(SampleCheckInfoDto sampleCheckInfoDto) {
        List<PieceCheck> pieceChecks = new ArrayList<>();
        List<PieceDetail> pieceMaterialDetails = this.pieceDetailDao.findByPieceNo(sampleCheckInfoDto.getPieceNo());
        if (pieceMaterialDetails == null || pieceMaterialDetails.size() <= 0) {
            return ResultGenerator.fail("样片编号不存在");
        }
        PieceDetail pieceDetail = pieceMaterialDetails.get(0);
        PieceCheck pieceCheck = pieceCheckService.getByPieceNo(sampleCheckInfoDto.getPieceNo());
        if (pieceCheck == null) {
            pieceCheck = new PieceCheck();
            pieceCheck.setPieceNo(sampleCheckInfoDto.getPieceNo());

            LongCrystal sectionDetail = this.longCrystalDao.findByPieceNo(sampleCheckInfoDto.getPieceNo());
            if (sectionDetail != null) {
                pieceCheck.setDeviceNo(sectionDetail.getEquipmentCode());
                pieceCheck.setLotNo(sectionDetail.getLotNo());
                pieceCheck.setMotherCrystalNo(sectionDetail.getCrystalNo());
            }
        }
        // 默认值
        // 截取样片倒数第二位,判断是头是尾
        String pieceNo = pieceDetail.getPieceNo();
        String TorW = pieceNo.substring(pieceNo.length() - 2, pieceNo.length() - 1);
        DefaultSample sample = this.defaultSampleService.findByDopantLikeAndSpecificationsLikeAndProductTypeLike(pieceDetail.getDopant(), pieceDetail.getSpecs(), pieceDetail.getProductType());
//        // 微缺陷
//        if (sampleCheckInfoDto.getMicroDefect() == null){
//            if (TorW.equals("T") && sample != null){
//                pieceCheck.setMicroDefect(sample.getHeadDefect());// 头微缺陷
//            }else if (TorW.equals("W") && sample != null){
//                pieceCheck.setMicroDefect(sample.getTailDefect());// 尾微缺陷
//            }else {
//                pieceCheck.setMicroDefect(null);
//            }
//        }else {
//            pieceCheck.setMicroDefect(sampleCheckInfoDto.getMicroDefect());// 微缺陷
//        }
//
//        // 位错
//        if (sampleCheckInfoDto.getLineDefect() == null){
//            if (TorW.equals("T") && sample != null){
//                pieceCheck.setLineDefect(sample.getHeadDislocation());// 头位错
//            }else if (TorW.equals("W") && sample != null){
//                pieceCheck.setLineDefect(sample.getTailDislocation());// 尾位错
//            }else {
//                pieceCheck.setLineDefect(null);
//            }
//        }else {
//            pieceCheck.setLineDefect(sampleCheckInfoDto.getLineDefect());// 位错
//        }
        BeanUtils.copyProperties(sampleCheckInfoDto, pieceCheck, BeanUtil.getIgnoreProperties(sampleCheckInfoDto, null, null));
        pieceCheck.setUserName1("mcs");
        pieceCheck.setCheckTime1(new Date());
        pieceCheck.setUserName2("mcs");
        pieceCheck.setCheckTime2(new Date());
        pieceCheck.setAnnealUser("mcs");
        pieceCheck.setAnnealTime(new Date());
        pieceChecks.add(pieceCheck);

        pieceCheckService.saveOrUpdateInBatch(pieceChecks, "样片检验-新增");
        this.pieceCheckService.saveInspectDataBaseInfo(sampleCheckInfoDto.getPieceNo());


        return ResultGenerator.success();
    }

    @Override
    public Result biddingRecycledMaterial(RecycledBindingInfoDto recycledBindingInfoDto) {
        //验证编号是否存在
        ProductionPaintingLineDetail bySegmentNo = productionPaintingLineDetailService.findBySegmentNo(recycledBindingInfoDto.getCode());
        if (bySegmentNo != null) {
            if (bySegmentNo.getType() == 0) {
                return ResultGenerator.fail("编号【" + recycledBindingInfoDto.getCode() + "】非样片、回收料！");
            }
        } else {
            String crystalNoBySectionNo = truncationReccutDetailsService.findCrystalNoBySectionNo(recycledBindingInfoDto.getCode());
            if (StringUtils.isBlank(crystalNoBySectionNo)) {
                throw new RuntimeException("编号【" + recycledBindingInfoDto.getCode() + "】非回收料！");
            }
        }

        //初始化绑定信息
        ProductionBindingTrayDto productionBindingTrayDto = new ProductionBindingTrayDto();
        productionBindingTrayDto.setCrystalNo(recycledBindingInfoDto.getCode());
        productionBindingTrayDto.setBindingPeople("mcs");
        productionBindingTrayDto.setBindingSite("截断自动下料");
        productionBindingTrayDto.setBindingTime(new Date());
        //自动绑定
        productionBindingTrayDto.setIsBinding(0);
        //只有一个工位，默认为空
        productionBindingTrayDto.setLocationNo("1");
        productionBindingTrayDto.setTrayNo(recycledBindingInfoDto.getTrayCode());
        productionBindingTrayDto.setCrystalDirection(recycledBindingInfoDto.getOrientation());
        productionBindingTrayDto.setType("回收料");
        productionBindingTrayDto.setPoint("JD");

        //绑定
        Result result = productionBindingTrayService.bindingCrystal(productionBindingTrayDto);

        //绑定结果，200成功，其它失败
        if (result.getCode() != 200) {
            return ResultGenerator.message(104, "绑定异常：" + result.getMessage(), null);
        }

        return ResultGenerator.message(200, "绑定成功", null);
    }

    @Override
    public Result staggerPositionInfo(DislocationInfoDto dislocationDto) {
        JSONArray detailJsonArray = JSONArray.parseArray(dislocationDto.getDetailJsonStr());
        DislocationInfo dislocationInfo = toLazyDto(dislocationDto);
        dislocationInfo.setCreateTime(new Date());
        dislocationInfoService.saveOrUpdate(dislocationInfo);
        List<DislocationExceptionInfoDto> list = JSONObject.parseArray(detailJsonArray.toJSONString(), DislocationExceptionInfoDto.class);
        for (DislocationExceptionInfoDto entity : list) {
            DislocationExceptionInfo dislocationExceptionInfo = toLazyDto(entity);
            dislocationExceptionInfo.setDislocationId(dislocationInfo.getId());
            dislocationExceptionInfoService.saveOrUpdate(dislocationExceptionInfo);
        }
        return ResultGenerator.message(200, "发送成功", null);
    }

    @Override
    public Result uploadInventory(List<McsFbInventoryDto> fbInventoryDto) {
        fbInventoryDto.stream().forEach(t->{
            McsSquareInventoryLog mcsSquareInventoryLog=new McsSquareInventoryLog();
            BeanUtil.copyProperties(mcsSquareInventoryLog,t);
            mcsSquareInventoryLog.setDelFlag(0);
            mcsSquareInventoryLog.setCreateTime(new Date());
            mcsSquareInventoryLog.setUpdateTime(new Date());
            mcsSquareInventoryLog.setCreateUser("mcs");
            mcsSquareInventoryLog.setInventoryType("out");
            mcsSquareInventoryLog.setLength(String.valueOf(t.getLength()));
            mcsSquareInventoryLog.setSpecification(t.getSpecs());
            mcsSquareInventoryLog.setConductionType(t.getConductiveType());
            mcsSquareInventoryLog.setProductLevelFb(t.getProductGrade());
            mcsSquareInventoryLog.setProductType(t.getProductType());
            //先删除在插入
            mcsSquareInventoryLogService.deleteByCrystalNoAndInventoryType(t.getCrystalNo(), "out");
            mcsSquareInventoryLogService.saveOrUpdate(mcsSquareInventoryLog);
        });
        try {
            List<String> sectionList = new ArrayList<>();
            for (McsFbInventoryDto mcsFbInventoryDto : fbInventoryDto) {
                sectionList.add(mcsFbInventoryDto.getCrystalNo());
            }
            fbFeignClients.create(sectionList);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResultGenerator.success();
    }

    @Override
    public CrystalTrayBindInfoResult findBindingInfoBySectionNo(SectionInfo info) {
        String sectionNo = info.getSectionNo();
        ProductionPaintingLineDetail lineDetail = productionPaintingLineDetailService.findBySegmentNo(sectionNo);
        if (lineDetail == null) {
            throw new RuntimeException("该晶棒" + sectionNo + "不存在");
        }

        LongCrystal longCrystal = longCrystalService.get(lineDetail.getCrystalId());
        if (longCrystal == null) {
            throw new RuntimeException("该晶棒" + sectionNo + "对应长晶棒信息不存在");
        }

        return buildCrystalTrayBindInfoResult(lineDetail, longCrystal);
    }

    private CrystalTrayBindInfoResult buildCrystalTrayBindInfoResult(ProductionPaintingLineDetail lineDetail, LongCrystal longCrystal) {
        CrystalTrayBindInfoResult result = new CrystalTrayBindInfoResult();
        result.setSectionNo(lineDetail.getNo());
        result.setLength(Convert.toStr(lineDetail.getLength(), null));
        DictionaryItem item = dictionaryItemService.findByThemeValueAndValueAndEnable("product_category", longCrystal.getProductType(), true);
        result.setProductType(Optional.ofNullable(item).map(DictionaryItem::getName).orElse(""));
        result.setDopant(longCrystal.getDopant());
        result.setConductiveType(longCrystal.getConductiveType());
        result.setType(longCrystal.getSpecs());
        result.setSegmentType(lineDetail.getType());
        result.setIsReCut(lineDetail.getIsCut());
//        result.setIsAvoidCheck();//Todo 后面预留

        return result;
    }

    public SquareOrderMesResultDto toLazyDto(RankResultDto entity) {
        return modelMapper.map(entity, SquareOrderMesResultDto.class);
    }

    public DislocationInfo toLazyDto(DislocationInfoDto entity) {
        return modelMapper.map(entity, DislocationInfo.class);
    }

    public DislocationExceptionInfo toLazyDto(DislocationExceptionInfoDto entity) {
        return modelMapper.map(entity, DislocationExceptionInfo.class);
    }
}
