package com.sw.mes.yb.plan.dao;

import com.sw.mes.core.base.BaseDao;
import com.sw.mes.yb.plan.entity.ProductionOrder;
import org.springframework.data.jpa.repository.Query;

import java.util.Date;
import java.util.List;

/**
 * @Author
 * @Description：
 * @Date:
 * @Modified By:
 */
public interface ProductionOrderDao extends BaseDao<ProductionOrder, Long> {

    @Query(value = "SELECT * FROM plan_production_order where aufnr=?1", nativeQuery = true)
    List<ProductionOrder> findByAufnr(String code);

    @Query(value = "SELECT * FROM plan_production_order ppo \n" +
            "WHERE ppo.matnr = ?1 AND ppo.gstrp < ?2  AND ppo.gltrp > ?2 AND ppo.status = 'REL' AND ppo.auart = ?3 AND ppo.field2 = ?4", nativeQuery = true)
    List<ProductionOrder> findByProductModelCode(String code, Date toDate, String orderType, String field2);

    @Query(value = "SELECT * FROM plan_production_order ppo \n" +
            " WHERE ppo.matnr = ?1 AND ppo.gstrp < ?2  AND ppo.gltrp > ?2 AND ppo.status = 'REL' AND ppo.auart = ?3 AND ppo.field2 <> ?4", nativeQuery = true)
    List<ProductionOrder> findByProductModelCodeV2(String code, Date toDate, String orderType, String field2);

    /**
     * 根据 物料类别id 查找订单
     * @param materialTypeId 物料类别id
     * @param toDate 时间
     * @param orderType 订单类型
     * @param field2 复检标识
     * @return
     */
    @Query(value = "SELECT ppo.*  FROM plan_production_order ppo " +
            " LEFT JOIN factory_product_model fpm ON ppo.product_model_id = fpm.id " +
            " WHERE fpm.material_type_id = ?1  AND ppo.gstrp < ?2  AND ppo.gltrp > ?2 AND ppo.status = 'REL' AND ppo.auart = ?3 AND ppo.field2 = ?4", nativeQuery = true)
    List<ProductionOrder> findByMaterialTypeId(Integer materialTypeId, Date toDate, String orderType, String field2);

    /**
     * 根据 物料类别id 查找订单
     * @param materialTypeId 物料类别id
     * @param toDate 时间
     * @param orderType 订单类型
     * @param field2 复检标识
     * @return
     */
    @Query(value = "SELECT ppo.*  FROM plan_production_order ppo " +
            " LEFT JOIN factory_product_model fpm ON ppo.product_model_id = fpm.id " +
            " WHERE fpm.material_type_id = ?1  AND ppo.gstrp < ?2  AND ppo.gltrp > ?2 AND ppo.status = 'REL' AND ppo.auart = ?3 AND ppo.field2 <> ?4", nativeQuery = true)
    List<ProductionOrder> findByMaterialTypeIdV2(Integer materialTypeId, Date toDate, String orderType, String field2);

    /**
     * 根据 设备加工能力 查找符合计划时间内的工单
     * @param machineCapacityId
     * @param status
     * @param productionStatus
     * @param gmtCreate
     * @return
     */
    @Query(value = "SELECT * FROM plan_production_order ppo \n" +
            "WHERE ppo.machine_capacity_id = ?1 AND ppo.`status` = ?2 AND ppo.production_status = ?3 AND ppo.gstrp <= ?4 AND ppo.gltrp >= ?4\n" +
            "ORDER BY ppo.id DESC", nativeQuery = true)
    List<ProductionOrder> findByMachineCapacityIdAndStatusAndProductionStatusAndGmtCreateTime(Integer machineCapacityId, String status, Boolean productionStatus, Date gmtCreate);

    @Query(value = "select * from plan_production_order ppo where ppo.production_status = ?1 and ppo.matnr = ?2 and (\t (ppo.gstrp > ?3 and ppo.gstrp < ?4)\n" +
            "\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tor (ppo.gltrp > ?3 and ppo.gltrp < ?4)\n" +
            "\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tor (ppo.gstrp <= ?3 and ppo.gltrp >= ?4)\t)", nativeQuery = true)
    List<ProductionOrder> findByStatusAndProductCodeAndTime(boolean b, String matnr, Date gstrp, Date gltrp);

    List<ProductionOrder> findByErpOrderNo(String code);

    List<ProductionOrder> findByFbOrderNo(String code);

	List<ProductionOrder> findByAufnrAndMatnr(String code, String matnr);
}
