package com.sw.mes.yb.production.controller;

import java.util.List;
import java.util.stream.Collectors;

import javax.servlet.ServletRequest;
import javax.validation.Valid;

import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.production.dto.BlastFurnaceDetail.BlastFurnaceAddDto;
import com.sw.mes.yb.production.dto.BlastFurnaceDetail.BlastFurnaceDetailDto;
import com.sw.mes.yb.production.dto.LoadingTask.LoadingTaskDto;
import com.sw.mes.yb.production.entity.BlastFurnaceDetail;
import com.sw.mes.yb.production.entity.LoadingTask;
import com.sw.mes.yb.production.service.BlastFurnaceDetailService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import springfox.documentation.annotations.ApiIgnore;


/**
* @Author 李儒
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/processLoadingTask/processBlastFurnaceDetail")
@Slf4j
@Api(description = "装料任务/装料(配料)明细表接口")
public class BlastFurnaceDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private BlastFurnaceDetailService blastFurnaceDetailService;



	@ApiOperation(value = "获取所有装料(配料)明细表")
	@ApiParam(name = "search_EQ_blastFurnaceId",value = "装料任务Id (LoadingTask.id)",type = "Long",required = true)
	@GetMapping("/listAll")
	public Result<List<BlastFurnaceDetailDto>> listAll(ServletRequest request) {
		Specification<BlastFurnaceDetail> specification = DynamicSpecifications.bySearchFilter(request, BlastFurnaceDetail.class);
		List<BlastFurnaceDetail> blastFurnaceDetails = blastFurnaceDetailService.findByExampleSort(specification, null);
		List<BlastFurnaceDetailDto> blastFurnaceDetailDtos = blastFurnaceDetails.stream()
					.map(blastFurnaceDetail -> toLazyDto(blastFurnaceDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(blastFurnaceDetailDtos);
	}

	@ApiOperation(value = "添加装料(配料)明细表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "装料(配料)明细表对象") BlastFurnaceAddDto addDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}

		try {
			addDto.getLoadingTaskDto().setCarrierNo(addDto.getVehicleCode());	// 将载具编号设置到任务中
			blastFurnaceDetailService.addBlastFurnace(addDto.getLoadingTaskDto(), addDto.getBlastFurnaceDetailDtos(),LoadingTaskDto.rawStatus_4,BaseContextHandler.getUsername());
			return ResultGenerator.success();
		}catch (Exception e) {
			return ResultGenerator.fail(e.getMessage());
		}
	}


	@ApiOperation(value = "删除装料(配料)明细表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				blastFurnaceDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	public BlastFurnaceDetailDto toLazyDto(BlastFurnaceDetail entity) {
		return modelMapper.map(entity, BlastFurnaceDetailDto.class);
	}

	public BlastFurnaceDetailDto toEagerDto(BlastFurnaceDetail entity) {
		BlastFurnaceDetailDto dto = modelMapper.map(entity, BlastFurnaceDetailDto.class);
		return dto;
	}

	public BlastFurnaceDetail fromDto(BlastFurnaceDetailDto dto) {
		return modelMapper.map(dto, BlastFurnaceDetail.class);
	}


}
