package com.sw.mes.yb.production.dao;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import com.sw.mes.core.base.BaseDao;
import com.sw.mes.yb.production.dto.AgainTaskDetail.AgainTaskDetailDto;
import com.sw.mes.yb.production.entity.AgainTaskDetail;

/**
 * @Author
 * @Description：
 * @Date:
 * @Modified By:
 */
public interface AgainTaskDetailDao extends BaseDao<AgainTaskDetail, Long> {

	List<AgainTaskDetail> findByLotNoAndPailNum(String lotNo, String pailNum);

	List<AgainTaskDetail> findByLotNoAndPailNumAndAgainNum(String lotNo, String pailNum, Integer againNum);

	@Query(nativeQuery = true, value = "select * from (select * from production_again_task_detail where raw_status<=1 "
			+ "and if(?1 is not null, equipment_code like CONCAT(?1,'%'),1=1) "
			+ "and series in (?2) "
			+ "group by again_task_id order by carrier_sort asc) t order by again_time desc",
			countQuery = "select count(*) from (select * from production_again_task_detail where raw_status<=1 "
					+ "and if(?1 is not null, equipment_code like CONCAT(?1,'%'),1=1) "
					+ "and series in (?2) "
					+ "group by again_task_id order by carrier_sort asc) t order by again_time desc")
	Page<AgainTaskDetail> findByPdaPage(String equipmentCode, List<String> series, Pageable page);

	@Query(nativeQuery = true, value = "select * from (select * from production_again_task_detail where raw_status<=1 "
			+ "and if(?1 is not null, equipment_code like CONCAT(?1,'%'),1=1) "
			+ "group by again_task_id order by carrier_sort asc) t order by again_time desc",
			countQuery = "select count(*) from (select * from production_again_task_detail where raw_status<=1 "
					+ "and if(?1 is not null, equipment_code like CONCAT(?1,'%'),1=1) "
					+ "group by again_task_id order by carrier_sort asc) t order by again_time desc")
	Page<AgainTaskDetail> findByPdaPage(String equipmentCode, Pageable page);


	@Query(nativeQuery = true, value = "select * from production_again_task_detail where equipment_code = ?1 and lot_no=?2 and (raw_status=3 or raw_status=4 or raw_status=7) ORDER BY id desc limit 1")
	AgainTaskDetail getLastSuccessTask(String equipmentCode, String lotNo);

	@Query(nativeQuery = true, value = "select sum(again_real_weight)guanliaoWeight from production_again_task_detail WHERE lot_no=?1 and (raw_status = 3 OR  raw_status = 4) group by lot_no")
	BigDecimal getTotaGuanLiaoWeightByLotNo(String lotNo);

	List<AgainTaskDetail> findByAgainTaskId(Long againTaskId);

	@Query(value = "SELECT \n" +
			"\ttd.id\n" +
			"\t,td.equipment_code equipmentCode  ##设备号 \n" +
			"\t,td.lot_no lotNo ##炉次号\n" +
			"\t,fb.vehicle_code vehicleCode ##载具编号\n" +
			"\t,CONCAT(td.again_num, '次复投') againNum ##任务类型\n" +
			"\t,td.product_code productCode ##产品料号\n" +
			"\t,\n" +
			"\t\tCASE\n" +
			"\t\t\tWHEN td.raw_status = 0 THEN '新建'\n" +
			"\t\t\tWHEN td.raw_status = 1 THEN '生产'\n" +
			"\t\t\tWHEN td.raw_status = 2 THEN '已取消'\n" +
			"\t\t\tWHEN td.raw_status = 3 THEN '自动完工'\n" +
			"\t\t\tWHEN td.raw_status = 4 THEN '人工完工'\n" +
			"\t\t\tWHEN td.raw_status = 5 THEN '退料'\n" +
			"\t\t\tWHEN td.raw_status = 6 THEN '等待'\n" +
			"\t\t\tWHEN td.raw_status = 7 THEN '满料入库送走'\n" +
            "\t\t\tELSE td.raw_status\n" +
            "\t\tEND rawStatus ##状态\n" +
            "\t,fb.pail_num pailNum ##灌料桶数\n" +
            "\t,fb.class_no classNo ##班次\n" +
            "\t,fb.team_group teamGroup ##班组\n" +
            "\t,fb.user_name userName1 ##灌料人\n" +
            "\t,td.start_time startTime ##灌料开始时间\n" +
            "\t,fb.end_time endTime ##灌料结束时间\n" +
            "\t,td.operation_cause operationCause ##操作原因\n" +
            "\t,td.user_name userName2 ##操作人\n" +
            "\t,td.operation_time operationTime ##操作时间\n" +
            "\t,fb.left_code leftCode ##左桶编号\n" +
            "\t,fb.right_code rightCode ##右桶编号\n" +
            "\t,fd.material_code materialCode ##料号\n" +
            "\t,fd.material_name materialName ##物料名称\n" +
            "\t,fd.batch_no batchNo ##批次号\n" +
            "\t,fd.supplier_name supplierName ##厂牌\n" +
            "\t,fd.weight weight ##重量\n" +
            "\t,fd.bucket_forty bucketForty ##箱数(40KG)\n" +
            "\t,fd.bucket_ten bucketTen ##箱数(10KG)\n" +
            "FROM production_again_task_detail td\n" +
            "LEFT JOIN production_input_furnace_burden fb ON fb.again_task_detail_id = td.id\n" +
            "LEFT JOIN production_input_furnace_detail fd ON fd.again_task_detail_id = td.id\n" +
            "WHERE td.raw_status not in (0,6,7)\n" +
            "\t\t\tAND IF(?1 is NOT NULL AND LENGTH(TRIM(?1)) > 0, td.start_time >= ?1, 1=1)\n" +
            "\t\t\tAND IF(?2 is NOT NULL AND LENGTH(TRIM(?2)) > 0, td.start_time <= ?2, 1=1)\n" +
            "\t\t\tAND IF(?3 is NOT NULL AND LENGTH(TRIM(?3)) > 0, td.raw_status = ?3, 1=1)\n" +
            "\t\t\tAND IF(?4 is NOT NULL AND LENGTH(TRIM(?4)) > 0, td.user_name LIKE CONCAT('%', ?4, '%'), 1=1)\n" +
            "\t\t\tAND IF(?5 is NOT NULL AND LENGTH(TRIM(?5)) > 0, td.lot_no = ?5, 1=1)\n" +
            "ORDER BY td.start_time DESC", nativeQuery = true)
    List<Map<String, Object>> findInputTask(String search_gte_startTime, String search_lte_startTime, String search_eq_rawStatus, String search_like_userName, String search_eq_lotNo);

	@Query(nativeQuery = true, value = "select id from production_again_task_detail where again_task_id = ?1 and raw_status != 5 and raw_status != 2 ")
	List<Long> findNotReturnByAgainTaskId(Long againTaskId);

	@Query(nativeQuery = true, value = "select * from production_again_task_detail where lot_no = ?1 and again_num= ?2 and raw_status !=2 and raw_status !=5")
	List<AgainTaskDetail> findByLotNoAndAgainNumAndStatus(String lotNo, Integer againNum);

	@Query(nativeQuery = true, value = "select * from production_again_task_detail where lot_no = ?1 and again_num= ?2 AND raw_status != 5 AND raw_status != 2")
	List<AgainTaskDetail> findByLotNoAndAgainNum(String lotNo, Integer againNum);

	@Query(nativeQuery = true, value = "select * from production_again_task_detail WHERE again_task_id in ?1 ")
	List<AgainTaskDetail> findByAgainTaskIdList(List<Long> idList);

	AgainTaskDetail findTopByEquipmentCodeOrderByIdDesc(String equipmentCode);

	@Query(nativeQuery = true, value = "select * from production_again_task_detail where again_task_id = ?1 and raw_status != 2 and raw_status != 5 ")
	List<AgainTaskDetail> findCancelTaskByAgainTaskId(Long id);

	@Query(value = "SELECT * FROM production_again_task_detail WHERE lot_no=?1 AND (raw_status=3 OR raw_status=4) AND confirm = 0 ORDER BY id DESC LIMIT 1 ", nativeQuery = true)
	AgainTaskDetail findUnconfirmedTask(String lotNo);
}
