package com.sw.mes.yb.production.dao;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import com.sw.mes.yb.production.entity.BlastFurnaceDetail;
import org.springframework.data.jpa.repository.Query;

import com.sw.mes.core.base.BaseDao;
import com.sw.mes.yb.production.entity.LoadingTask;

/**
 * @Author
 * @Description：
 * @Date:
 * @Modified By:
 */
public interface LoadingTaskDao extends BaseDao<LoadingTask, Long> {

    @Query(value = "select * from production_loading_task where equipment_code=?1 and lot_no = ?2 and (raw_status=3 or raw_status=4) ORDER BY id desc", nativeQuery = true)
    List<LoadingTask> getFinishTask(String equipmentCode, String lotNo);

    @Query(value = " SELECT SUM(pbfd.weight) as real_weight from production_loading_task plt LEFT JOIN production_blast_furnace_detail pbfd ON plt.id = pbfd.blast_furnace_id \n" +
            "\n" +
            "WHERE (plt.raw_status = 3 OR plt.raw_status = 4) AND pbfd.material_name not like '%坩埚%' and plt.lot_no = ?1 GROUP BY pbfd.blast_furnace_id", nativeQuery = true)
    BigDecimal getPeiLiaoTotalWeightByLotNo(String lotNo);

    List<LoadingTask> findByLotNo(String lotNo);

    List<LoadingTask> findByEquipmentCode(String equipmentCode);

    @Query(value = "select * from production_loading_task where equipment_code = ?1 and lot_no=?2 and raw_status=?3 ORDER BY id desc limit 1", nativeQuery = true)
    LoadingTask getByEquipmentAndStateLastTask(String equipmentCode, String lotNo, Integer rawstatus);

    @Query(value = "select * from production_loading_task where equipment_code = ?1 and lot_no=?2 and (raw_status=3 or raw_status=4) ORDER BY id desc limit 1", nativeQuery = true)
    LoadingTask getByEquipmentAndSuccessLastTask(String equipmentCode, String lotNo);

    @Query(value = "select crucible_height from production_loading_task where equipment_code = ?1 and (raw_status=3 or raw_status=4) ORDER BY id desc limit 1", nativeQuery = true)
    String getLastOneHeight(String equipmentCode);

    @Query(value = "SELECT plt.lot_no as lotNo, pbfd.batch_no as batchNo, pbfd.factory_brand as factoryBrand FROM production_blast_furnace_detail pbfd LEFT JOIN production_loading_task plt ON pbfd.blast_furnace_id = plt.id "+
            "\n" + "WHERE  plt.lot_no = ?1 AND pbfd.mark_crucible =1  AND ( plt.raw_status = 3 OR plt.raw_status = 4 )  Limit 1 ", nativeQuery = true)
    Map<String, Object>  findCrucibleInfoByLotNo(String lotNo);

    @Query(value = "select * from production_loading_task where lot_no in ?1",nativeQuery = true)
    List<LoadingTask> findByLotNoList(List<String> lotNo);

    @Query(value = "SELECT * FROM production_loading_task WHERE (raw_status=3 OR raw_status=4) AND confirm = 0 AND lot_no = ?1 ORDER BY id DESC LIMIT 1 ",nativeQuery = true)
    LoadingTask findUnconfirmedTask(String lotNo);

    @Query(value = "select * from production_loading_task where (raw_status=3 OR raw_status=4) and lot_no = ?1 ",nativeQuery = true)
    List<LoadingTask> findRealWeightByLotNo(String lotNo);

    @Query(value = "SELECT pbfd.* FROM production_loading_task plt LEFT JOIN production_blast_furnace_detail pbfd ON plt.id = pbfd.blast_furnace_id \n" +
            "WHERE ( plt.raw_status = 3 OR plt.raw_status = 4 ) AND plt.lot_no = ?1 AND plt.product_code = ?2 \n" +
            "ORDER BY plt.id DESC LIMIT 1",nativeQuery = true)
    List<BlastFurnaceDetail> findByLotNoAndProductCode(String lotNo, String code);

    @Query(value = "select * from production_loading_task where lot_no = ?1 and (raw_status=3 or raw_status=4) limit 1",nativeQuery = true)
    LoadingTask findSuccessByLotNo(String lotNo);
}
