package com.sw.mes.yb.production.service.impl;


import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.inspect.entity.YbInspectData;
import com.sw.mes.inspect.service.YbInspectDataService;
import com.sw.mes.yb.pad.entity.SectionNoDetail;
import com.sw.mes.yb.pad.service.SectionNoDetailService;
import com.sw.mes.yb.production.dao.CrystalBatchOperationDao;
import com.sw.mes.yb.production.dto.BatchOperationDto;
import com.sw.mes.yb.production.dto.CrystalBatchInfoDto;
import com.sw.mes.yb.production.dto.CrystalBatchOperationCreateDto;
import com.sw.mes.yb.production.dto.CrystalBatchOperationDto;
import com.sw.mes.yb.production.entity.CrystalBatchOperation;
import com.sw.mes.base.workinprocess.entity.WipStorage;
import com.sw.mes.base.workinprocess.service.WipStorageService;
import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.enums.EnumValuesFactory;
import com.sw.mes.core.utils.StringUtils;
import com.sw.mes.yb.production.entity.LongCrystal;
import com.sw.mes.yb.production.service.CrystalBatchOperationService;
import com.sw.mes.yb.production.service.LongCrystalService;
import org.apache.commons.collections.CollectionUtils;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@Service
@Transactional
public class CrystalBatchOperationServiceImpl extends BaseServiceImpl<CrystalBatchOperation> implements CrystalBatchOperationService {
    CrystalBatchOperationDao dao;

    @Autowired
    private WipStorageService wipStorageService;

    @Autowired
    private LongCrystalService longCrystalService;

    @Autowired
    private YbInspectDataService ybInspectDataService;

    @Autowired
    private ModelMapper modelMapper;

    @Autowired
    private SectionNoDetailService sectionNoDetailService;

	@Autowired
    public void setDao(CrystalBatchOperationDao dao) {
        super.setBaseDao(dao);
        this.dao=dao;
    }

    @Override
    public void batchOperateCrystalBatchOperation(BatchOperationDto batchOperationDto) {

	    if (null != batchOperationDto.getOperateStatus()){
            Date date = new Date();
            if (null == batchOperationDto.getLockTime()){
                batchOperationDto.setLockTime(date);
            }
            if (null == batchOperationDto.getUnlockTime()){
                batchOperationDto.setUnlockTime(date);
            }
            Optional.ofNullable(batchOperationDto.getIdList()).orElse(new ArrayList<>()).forEach((id)->{
                if (batchOperationDto.getOperateStatus() == 2){
                    this.delete(id);
                }else {
                    CrystalBatchOperation crystalBatchOperation = this.get(id);
                    if (null != crystalBatchOperation){
                        if (batchOperationDto.getOperateStatus() == 0){
                            crystalBatchOperation.setLockUser(batchOperationDto.getLockUser());
                            crystalBatchOperation.setLockTime(batchOperationDto.getLockTime());
                            crystalBatchOperation.setLockReason(batchOperationDto.getLockReason());
                            crystalBatchOperation.setLockRemark(batchOperationDto.getLockRemark());
                        }else if (batchOperationDto.getOperateStatus() == 1){
                            crystalBatchOperation.setUnlockUser(batchOperationDto.getUnlockUser());
                            crystalBatchOperation.setUnlockTime(batchOperationDto.getUnlockTime());
                            crystalBatchOperation.setUnlockReason(batchOperationDto.getUnlockReason());
                            crystalBatchOperation.setUnlockRemark(batchOperationDto.getUnlockRemark());
                        }
                        crystalBatchOperation.setStatus(batchOperationDto.getOperateStatus());
                        this.saveOrUpdate(crystalBatchOperation);
                    }
                }
            });
        }
    }

    @Override
    public void batchCreateCrystalBatchOperation(CrystalBatchOperationCreateDto crystalBatchOperationCreateDto) {

	    if (CollectionUtils.isEmpty(crystalBatchOperationCreateDto.getBatchInfoDtoList())){
	        throw new RuntimeException("批次操作单晶信息集合为空！");
        }

	    if (null == crystalBatchOperationCreateDto.getLockTime()){
            crystalBatchOperationCreateDto.setLockTime(new Date());
        }
	    Optional.ofNullable(crystalBatchOperationCreateDto.getBatchInfoDtoList()).orElse(new ArrayList<>()).forEach(crystalBatchInfoDto -> {
            CrystalBatchOperation crystalBatchOperation = modelMapper.map(crystalBatchInfoDto, CrystalBatchOperation.class);
            crystalBatchOperation.setLockReason(crystalBatchOperationCreateDto.getLockReason());
            crystalBatchOperation.setLockRemark(crystalBatchOperationCreateDto.getLockRemark());
            crystalBatchOperation.setLockUser(crystalBatchOperationCreateDto.getLockUser());
            crystalBatchOperation.setLockTime(crystalBatchOperationCreateDto.getLockTime());
            crystalBatchOperation.setStatus(0);
            this.saveOrUpdate(crystalBatchOperation);
        });
    }

    @Override
    public List<CrystalBatchInfoDto> getInfoBySectionNo(String sectionList) {

	    if (StringUtils.isBlank(sectionList)){
	        return null;
        }
	    List<CrystalBatchInfoDto> result = new ArrayList<>();
        String[] split = sectionList.split(",");
	    for(String section : split){
            List<WipStorage> byProcessingByOrderCode = wipStorageService.findByProcessingByOrderCode(section);
            if (CollectionUtils.isNotEmpty(byProcessingByOrderCode)){

                WipStorage wipStorage = byProcessingByOrderCode.get(0);
                String processName = wipStorage.getProcessName();
                YbInspectData ybInspectData = null;
                LongCrystal longCrystal = null;
                SectionNoDetail bySectionNo = sectionNoDetailService.findBySectionNo(section);
                if (null != bySectionNo){
                    ybInspectData = ybInspectDataService.findBySectionNo(section);
                    longCrystal = longCrystalService.findByCrystalNo(bySectionNo.getCrystalNo());
                }else {
                    longCrystal = longCrystalService.findByCrystalNo(section);
                }
                /*if (StringUtils.equals(EnumValuesFactory.WipStorageCode.方棒检验.getValue(),processName) || StringUtils.equals(EnumValuesFactory.WipStorageCode.圆棒检验.getValue(),processName)){
                    ybInspectData = ybInspectDataService.findBySectionNo(section);
                    longCrystal = longCrystalService.findByCrystalNo(section.substring(0,16));
                }else {
                    //去长晶棒获取信息
                    if (section.length() > 16){
                        longCrystal = longCrystalService.findByCrystalNo(section.substring(0,16));
                    }else {
                        longCrystal = longCrystalService.findByCrystalNo(section);
                    }
                }*/
                result.add(this.packageCrystalBatchOperationDto(longCrystal,ybInspectData,wipStorage));
            }
        }
        return result;
    }

    private CrystalBatchInfoDto packageCrystalBatchOperationDto(LongCrystal longCrystal,YbInspectData ybInspectData,WipStorage wipStorage){
        CrystalBatchInfoDto crystalBatchOperationDto = new CrystalBatchInfoDto();
        if (null != longCrystal){
            crystalBatchOperationDto.setCrystalNo(longCrystal.getCrystalNo());
            crystalBatchOperationDto.setSpecs(longCrystal.getSpecs());
            crystalBatchOperationDto.setMaterialCode(longCrystal.getProductCode());
            crystalBatchOperationDto.setMaterialName(longCrystal.getProductName());
            crystalBatchOperationDto.setAbbreviation(longCrystal.getSimpleName());
            crystalBatchOperationDto.setConductiveType(longCrystal.getConductiveType());
            crystalBatchOperationDto.setSpecs(longCrystal.getSpecs());
            crystalBatchOperationDto.setDopant(longCrystal.getDopant());
            crystalBatchOperationDto.setOrderNo(longCrystal.getOrderNo());
        }
        if (null != ybInspectData){
            crystalBatchOperationDto.setRecipeName(ybInspectData.getFormulation());
            crystalBatchOperationDto.setLevelCode(ybInspectData.getLevelCode());
            crystalBatchOperationDto.setProductLevel(ybInspectData.getProductLevel());
            crystalBatchOperationDto.setSectionNo(ybInspectData.getSectionNo());
        }
        if (null != wipStorage){
            crystalBatchOperationDto.setPosition(wipStorage.getProcessName());
            crystalBatchOperationDto.setSiteStatus(wipStorage.getStatus());
        }
        if (StringUtils.isBlank(crystalBatchOperationDto.getSectionNo()) && null != longCrystal){
            crystalBatchOperationDto.setSectionNo(longCrystal.getCrystalNo());
        }

        return crystalBatchOperationDto;
    }

    @Override
    public CrystalBatchOperation findByCrystalNoAndStatus(String crystalNo, Integer status) {
        if (StringUtils.isNotBlank(crystalNo)){
            return dao.findByCrystalNoAndStatus(crystalNo,status);
        }
        return null;
    }

    @Override
    public CrystalBatchOperation findBySectionNoAndStatus(String sectionNo, Integer status) {
        if (StringUtils.isNotBlank(sectionNo)){
            return dao.findBySectionNoAndStatus(sectionNo,status);
        }
        return null;
    }

    @Override
    public void judgeCrystalNoIsLock(String crystalNo, String sectionNo, Boolean isCrystalBar) {
        if (isCrystalBar){
            if (StringUtils.isNotBlank(crystalNo)){
                CrystalBatchOperation byCrystalNoAndStatus = this.findByCrystalNoAndStatus(crystalNo, EnumValuesFactory.BatchOperationStatus.LOCK_STATUS.getKey());
                if (null != byCrystalNoAndStatus){
                    throw new BusinessException("晶棒处于锁定状态，请先解锁！");
                }
            }
        }else {
            if (null != sectionNo){
                CrystalBatchOperation bySectionNoAndStatus = this.findBySectionNoAndStatus(sectionNo, EnumValuesFactory.BatchOperationStatus.LOCK_STATUS.getKey());
                if (null != bySectionNoAndStatus){
                    throw new BusinessException("晶段处于锁定状态，请先解锁！");
                }
                SectionNoDetail bySectionNo = sectionNoDetailService.findBySectionNo(sectionNo);
                if (null != bySectionNo && StringUtils.isNotBlank(bySectionNo.getCrystalNo())){
                    CrystalBatchOperation byCrystalNoAndStatus = this.findByCrystalNoAndStatus(bySectionNo.getCrystalNo(), EnumValuesFactory.BatchOperationStatus.LOCK_STATUS.getKey());
                    if (null != byCrystalNoAndStatus){
                        throw new BusinessException("晶棒处于锁定状态，请先解锁！");
                    }
                }
            }
        }
    }
}
