package com.sw.mes.yb.production.service.impl;

import java.math.RoundingMode;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.sw.mes.crucible.entity.CrucibleFixedCasting;
import com.sw.mes.crucible.service.CrucibleFixedCastingService;
import com.sw.mes.mqtt.service.NotificationService;
import com.sw.mes.production.entity.FormulaProductScheme;
import com.sw.mes.production.entity.FormulaWeightScheme;
import com.sw.mes.production.service.FormulaProductSchemeService;
import com.sw.mes.production.service.FormulaWeightSchemeService;
import com.sw.mes.yb.plan.entity.ProductionSchedule;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.sw.mes.core.constant.ScheduleConstants;
import com.sw.mes.base.system.entity.DictionaryItem;
import com.sw.mes.base.system.service.DictionaryItemService;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.mmcs.service.MMcsInterfaceService;
import com.sw.mes.quartz.entity.SysJob;
import com.sw.mes.quartz.service.SysJobService;
import com.sw.mes.quartz.util.CronUtils;
import com.sw.mes.yb.production.dto.AgainTask.AgainTaskDto;
import com.sw.mes.yb.production.dto.LoadingTask.LoadingTaskDto;
import com.sw.mes.yb.production.entity.AgainTask;
import com.sw.mes.yb.production.entity.LoadingTask;
import com.sw.mes.yb.production.entity.ProductionCircleRecipe;
import com.sw.mes.yb.production.entity.ProductionCircleWeight;
import com.sw.mes.yb.production.notification.AgainTaskMessage;
import com.sw.mes.yb.production.service.AgainTaskService;
import com.sw.mes.yb.production.service.EAPDockingService;
import com.sw.mes.yb.production.service.LoadingTaskService;
import com.sw.mes.yb.production.service.ProductionCircleRecipeService;
import com.sw.mes.yb.production.service.ProductionCircleWeightService;

/**
 * @author 李儒
 */
@Service
@Transactional
public class EAPDockingServiceImpl implements EAPDockingService {

	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private DictionaryItemService dictionaryItemService;

	@Autowired
	private ProductionCircleRecipeService productionCircleRecipeService;
	@Autowired
	private ProductionCircleWeightService productionCircleWeightService;

	@Autowired
	private LoadingTaskService loadingTaskService;
	@Autowired
	private AgainTaskService againTaskService;
	@Autowired
	private SysJobService sysJobService;
//	@Autowired
//	private TransportInspectTaskService transportInspectTaskService;
	@Autowired
	private NotificationService notificationService;

	@Autowired
	private MMcsInterfaceService mMcsInterfaceService;

	@Autowired
	private CrucibleFixedCastingService crucibleFixedCastingService;    // 热场尺寸

	@Autowired
	private FormulaProductSchemeService formulaProductSchemeService;	// 拆桶方案

	@Autowired
	private FormulaWeightSchemeService formulaWeightSchemeService;		// 重量方案


	/**装料信号必填项验证**/
	private static Map<String, String> requiredMapping = new HashMap<String, String>();
	/**复投信号必填项验证**/
	private static Map<String, String> againTaskRequiredMapping = new HashMap<String, String>();
	/**送验信号必填项验证**/
	private static Map<String, String> transportInspectTaskMapping = new HashMap<String, String>();



	static {
		//=========================装料必填项验证 ===============================
		requiredMapping.put("equipmentCode", "请设置设备号");
		requiredMapping.put("lotNo", "请设置炉次号");
		requiredMapping.put("series", "请设置所属系列");
		requiredMapping.put("area", "请设置所属模块");
		requiredMapping.put("furnaceType", "请设置炉台类型【0:默认；0-量产;1-在建;2-研发;】");
		requiredMapping.put("sapNo", "请设置SAP订单号");
		requiredMapping.put("productCode", "请设置料号");
//		requiredMapping.put("productGrade", "请设置产品等级");
		requiredMapping.put("specs", "请设置规格");
		requiredMapping.put("dopant", "请设置掺杂剂");
//		requiredMapping.put("productType", "请设置产品类别");


		//=========================复投必填项验证 ===============================
		againTaskRequiredMapping.put("equipmentCode", "请设置设备号");
		againTaskRequiredMapping.put("lotNo", "请设置炉次号");
		againTaskRequiredMapping.put("series", "请设置所属系列");
		againTaskRequiredMapping.put("area", "请设置所属模块");
		againTaskRequiredMapping.put("rawType", "请设置叫料类型【0:默认;1-初始复投;2-取段复投;】");
		againTaskRequiredMapping.put("furnaceType", "请设置炉台类型【0:默认；0-量产;1-在建;2-研发;】");
		againTaskRequiredMapping.put("sapNo", "请设置SAP订单号");
		againTaskRequiredMapping.put("productCode", "请设置料号");
//		againTaskRequiredMapping.put("productGrade", "请设置产品等级");
		againTaskRequiredMapping.put("specs", "请设置规格");
		againTaskRequiredMapping.put("dopant", "请设置掺杂剂");
//		againTaskRequiredMapping.put("productType", "请设置产品类别");


		//=========================复投必填项验证 ===============================
		transportInspectTaskMapping.put("equipmentCode", "请设置设备号");
		transportInspectTaskMapping.put("lotNo", "请设置炉次号");
		transportInspectTaskMapping.put("series", "请设置所属系列");
		transportInspectTaskMapping.put("area", "请设置所属模块");
		transportInspectTaskMapping.put("furnaceType", "请设置炉台类型【0:默认；0-量产;1-在建;2-研发;】");
		transportInspectTaskMapping.put("sapNo", "请设置SAP订单号");
		transportInspectTaskMapping.put("productCode", "请设置料号");
//		transportInspectTaskMapping.put("productGrade", "请设置产品等级");
		transportInspectTaskMapping.put("specs", "请设置规格");
		transportInspectTaskMapping.put("dopant", "请设置掺杂剂");
//		transportInspectTaskMapping.put("productType", "请设置产品类别");
	}

	@Override
	public void zlTaskCreate(LoadingTaskDto task) throws Exception {
		BeanUtil.requiredValidate(task, requiredMapping);
		task.setRawType(0);

		Date currentDate = new Date();
		task.setGmtCreate(currentDate);

		int ettime = getDicMinuteTime("FeedingTask","装料时间",900);

		Calendar calendar = Calendar.getInstance();
		calendar.setTime(currentDate);
		calendar.add(Calendar.MINUTE, ettime);
		task.setEtTime(calendar.getTime());

		// 获取热场坩埚尺寸
		CrucibleFixedCasting fixedCasting = crucibleFixedCastingService.findByEquipment(task.getEquipmentCode());
		if (fixedCasting != null){
			task.setDimensionId(fixedCasting.getId());
			task.setDimension(fixedCasting.getDimension());
		}

		// 设置配方初始装料
		FormulaProductScheme productScheme = formulaProductSchemeService.findByEquipmentCodeAndProductModeCode(
				task.getEquipmentCode(), task.getProductCode());
		if (productScheme == null) throw new RuntimeException("未查询到对应的拆桶方案");
		FormulaWeightScheme weightScheme = formulaWeightSchemeService.get(productScheme.getDecompositionSchemeId());
		if (weightScheme == null) throw new RuntimeException("未查询到对应的重量方案");
		// 设置初装重量
		task.setWeight(weightScheme.getInitialWeight().setScale(0, RoundingMode.UP));

		// 设置默认信息
		task.setUserName("EAP");
		task.setOperationTime(new Date());

		int delayedTime = getDicMinuteTime("DelayTime","初始装料延迟",0);

		if(delayedTime<=0) {
			task.setRawStatus(LoadingTaskDto.rawStatus_0);
			task.setDelayedStartTime(currentDate);
			LoadingTask model = fromDto(task);
			loadingTaskService.saveOrUpdate(model);

			//下发生产任务接口
			//mMcsInterfaceService.issueMoInterface(model);
		}else {
			task.setRawStatus(LoadingTaskDto.rawStatus_6);

			calendar.setTime(currentDate);
			calendar.add(Calendar.MINUTE, delayedTime);
			Date delayDate = calendar.getTime();
			task.setDelayedStartTime(delayDate);

			LoadingTask model = fromDto(task);
			loadingTaskService.saveOrUpdate(model);


			String cronExpression = CronUtils.getCronExpression(delayDate);

			SysJob job = new SysJob();
			job.setJobName("zlTask:"+model.getLotNo() + "-" + model.getId());
			job.setJobGroup("装料");
			job.setInvokeTarget("delayStartTask.zlTask('"+model.getLotNo()+"',"+model.getId()+"L)");
			job.setCronExpression(cronExpression);
			job.setExpireTime(delayDate);
			job.setCreateBy("EAP");
			job.setMisfirePolicy(ScheduleConstants.MISFIRE_DEFAULT);
			job.setConcurrent("0");
			job.setStatus(ScheduleConstants.Status.NORMAL.getValue());
			sysJobService.startJob(job);
		}
	}

	@Override
	public void ftTaskCreate(AgainTaskDto task) throws Exception {
		BeanUtil.requiredValidate(task, againTaskRequiredMapping);

		Date currentDate = new Date();
		task.setGmtCreate(currentDate);


		if(task.getRawType() == 1) {//初始复投
			Integer ettime = getDicMinuteTime("FeedingTask","初始复投时间",120);
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(currentDate);
			calendar.add(Calendar.MINUTE, ettime);
			task.setEtTime(calendar.getTime());

			// 改成桶桶分解的时候关联
			/*ProductionCircleRecipe circleRecipe = productionCircleRecipeService.getByProductCode(task.getProductCode());
			task.setRecipeId(circleRecipe == null ? null : circleRecipe.getId());
			ProductionCircleWeight circleWeight = productionCircleWeightService.getByProductCode(task.getProductCode());
			task.setWeigthId(circleWeight == null ? null : circleWeight.getId());*/

			// 设置默认信息
			task.setUserName("EAP");
//			task.setAgainTime(new Date());

			int delayedTime = getDicMinuteTime("DelayTime","初始复投延迟",0);

			if(delayedTime<=0) {
				task.setRawStatus(LoadingTaskDto.rawStatus_0);
				task.setDelayedStartTime(currentDate);
				AgainTask model = fromDto(task);
				againTaskService.saveOrUpdate(model);

//				//下发生产任务接口
//	    		mMcsInterfaceService.issueMoInterface(model);

				//发送MQTT通知
				AgainTaskMessage message = new AgainTaskMessage();
				message.setArea(model.getArea());
				message.setEquipmentCode(model.getEquipmentCode());
				message.setId(model.getId());
				message.setLotNo(model.getLotNo());
				message.setSeries(model.getSeries());
				notificationService.sendNotification(message);

			}else {
				task.setRawStatus(LoadingTaskDto.rawStatus_6);
				//设置延时时间
				calendar.setTime(currentDate);
				calendar.add(Calendar.MINUTE, delayedTime);
				Date delayDate = calendar.getTime();
				task.setDelayedStartTime(delayDate);

				AgainTask model = fromDto(task);
				againTaskService.saveOrUpdate(model);

				//下发生产任务接口
	    		//mMcsInterfaceService.issueMoInterface(model);

				String cronExpression = CronUtils.getCronExpression(delayDate);

				SysJob job = new SysJob();
				job.setJobName("ftTask:"+task.getLotNo() +"-"+model.getId());
				job.setJobGroup("装料");
				job.setInvokeTarget("delayStartTask.ftTask('"+model.getLotNo()+"',"+model.getId()+"L)");
				job.setCronExpression(cronExpression);
				job.setExpireTime(delayDate);
				job.setCreateBy("EAP");
				job.setMisfirePolicy(ScheduleConstants.MISFIRE_DEFAULT);
				job.setConcurrent("0");
				job.setStatus(ScheduleConstants.Status.NORMAL.getValue());
				sysJobService.startJob(job);
			}


		}else if(task.getRawType() == 2){//取段复投
			Integer ettime = getDicMinuteTime("FeedingTask","取段复投时间",0);
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(currentDate);
			calendar.add(Calendar.MINUTE, ettime);
			task.setEtTime(calendar.getTime());
			task.setRawStatus(LoadingTaskDto.rawStatus_0);

			// 改成桶桶分解的时候关联
			/*ProductionCircleRecipe circleRecipe = productionCircleRecipeService.getByProductCode(task.getProductCode());
			task.setRecipeId(circleRecipe == null ? null : circleRecipe.getId());
			ProductionCircleWeight circleWeight = productionCircleWeightService.getByProductCode(task.getProductCode());
			task.setWeigthId(circleWeight == null ? null : circleWeight.getId());*/

			// 设置默认信息
			task.setUserName("EAP");
//			task.setAgainTime(new Date());


			int delayedTime = getDicMinuteTime("DelayTime","取段复投延迟",0);

			if(delayedTime<=0) {
				task.setRawStatus(LoadingTaskDto.rawStatus_0);
				task.setDelayedStartTime(currentDate);
				AgainTask model = fromDto(task);
				againTaskService.saveOrUpdate(model);

				//下发生产任务接口
	    		//mMcsInterfaceService.issueMoInterface(model);


				//发送MQTT通知
				AgainTaskMessage message = new AgainTaskMessage();
				message.setArea(model.getArea());
				message.setEquipmentCode(model.getEquipmentCode());
				message.setId(model.getId());
				message.setLotNo(model.getLotNo());
				message.setSeries(model.getSeries());
				notificationService.sendNotification(message);

			}else {
				task.setRawStatus(LoadingTaskDto.rawStatus_6);

				calendar.setTime(currentDate);
				calendar.add(Calendar.MINUTE, delayedTime);
				Date delayDate = calendar.getTime();
				task.setDelayedStartTime(delayDate);

				AgainTask model = fromDto(task);
				againTaskService.saveOrUpdate(model);

				//下发生产任务接口
	    		//mMcsInterfaceService.issueMoInterface(model);

				String cronExpression = CronUtils.getCronExpression(delayDate);

				SysJob job = new SysJob();
				job.setJobName("ftTask:"+task.getLotNo() +"-"+model.getId());
				job.setJobGroup("装料");
				job.setInvokeTarget("delayStartTask.ftTask('"+model.getLotNo()+"',"+model.getId()+"L)");
				job.setCronExpression(cronExpression);
				job.setExpireTime(delayDate);
				job.setCreateBy("EAP");
				job.setMisfirePolicy(ScheduleConstants.MISFIRE_DEFAULT);
				job.setConcurrent("0");
				job.setStatus(ScheduleConstants.Status.NORMAL.getValue());
				sysJobService.startJob(job);
			}

		}else {
			throw new RuntimeException("叫料类型传入异常【1-初始复投;2-取段复投;】");
		}
	}

//	@Override
//	public void syTaskCreate(TransportInspectTaskDto task) throws Exception{
//		BeanUtil.requiredValidate(task, transportInspectTaskMapping);
//
//		if(task.getLength().floatValue()>=1500) {
//			task.setCreateTime(new Date());
//			task.setSendTask(true);
//			task.setCrystalStatus(TransportInspectTaskDto.crystalStatus_0);
//			TransportInspectTask model = fromDto(task);
//			transportInspectTaskService.saveOrUpdate(model);
//
//			//发送MQTT通知
//			SongYanMessage message = new SongYanMessage();
//			message.setArea(model.getArea());
//			message.setEquipmentCode(model.getEquipmentCode());
//			message.setId(model.getId());
//			message.setLotNo(model.getLotNo());
//			message.setSeries(model.getSeries());
//			notificationService.sendNotification(message);
//		}else {
//			task.setCreateTime(new Date());
//			task.setSendTask(false);
//			task.setCrystalStatus(TransportInspectTaskDto.crystalStatus_0);
//			TransportInspectTask model = fromDto(task);
//			transportInspectTaskService.saveOrUpdate(model);
//		}
//	}



	private Integer getDicMinuteTime(String themeValue,String name , int defValue) {
		int time = 0;
		try {
			//设置预计初始复投时间
			DictionaryItem dic = dictionaryItemService.findByThemeValueAndNameAndEnable(themeValue, name, true);
			if(dic!=null) {
				time = Integer.valueOf(dic.getValue());
			}else {
				time = defValue;
			}
		}catch (Exception e) {
			time = defValue;
		}
		return time;
	}


	private LoadingTask fromDto(LoadingTaskDto dto) {
		return modelMapper.map(dto, LoadingTask.class);
	}

	private AgainTask fromDto(AgainTaskDto dto) {
		return modelMapper.map(dto, AgainTask.class);
	}

//	private TransportInspectTask fromDto(TransportInspectTaskDto dto) {
//		return modelMapper.map(dto, TransportInspectTask.class);
//	}
}
