package	com.sw.mes.yb.production.service.impl;


import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;
import com.sw.mes.base.factory.material.entity.MaterialModel;
import com.sw.mes.base.factory.material.service.MaterialModelService;
import com.sw.mes.base.system.entity.DictionaryItem;
import com.sw.mes.base.system.service.DictionaryItemService;
import com.sw.mes.core.api.system.StoreFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.WarehouseModelDto;
import com.sw.mes.core.dto.store.StoreOperatorDetailDto;
import com.sw.mes.core.dto.store.StoreOperatorDto;
import com.sw.mes.core.dto.store.WarehouseOutputBillBo;
import com.sw.mes.core.dto.store.WarehouseOutputBillDetailBo;
import com.sw.mes.core.enums.StoreBusinessTypeEnum;
import com.sw.mes.core.enums.StoreOperatorTypeEnum;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.redis.server.RedisUtils;
import com.sw.mes.sap.entity.ReportSapGoodsmvt;
import com.sw.mes.sap.service.ReportSapGoodsmvtService;
import com.sw.mes.utils.StringUtils;
import com.sw.mes.yb.production.entity.*;
import com.sw.mes.yb.production.service.*;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.mmcs.service.MMcsInterfaceService;
import com.sw.mes.yb.production.dao.InputFurnaceBurdenDao;
import com.sw.mes.yb.production.dto.InputFurnaceDetailDto;
import com.sw.mes.yb.production.dto.AgainTask.AgainTaskDto;
import com.sw.mes.yb.production.dto.AgainTaskDetail.AgainTaskDetailDto;
import com.sw.mes.yb.production.dto.InputFurnaceBurden.InputFurnaceBurdenDto;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@Service
@Transactional
public class InputFurnaceBurdenServiceImpl extends BaseServiceImpl<InputFurnaceBurden> implements InputFurnaceBurdenService {

	private final static String BurdenKey = "six_burden_task_";

	@Autowired
	ModelMapper modelMapper;

	InputFurnaceBurdenDao dao;
	@Autowired
    public void setDao(InputFurnaceBurdenDao dao) {
        super.setBaseDao(dao);
        this.dao=dao;
    }

	@Autowired
	private AgainTaskDetailService againTaskDetailService;
	@Autowired
	private AgainTaskService againTaskService;
	@Autowired
	private InputFurnaceDetailService inputFurnaceDetailService;

	@Autowired
	@Lazy
	private MMcsInterfaceService mMcsInterfaceService;

	@Autowired
	private RedisUtils redisUtils;
	@Autowired
	private ProductionCircleRecipeDetailService productionCircleRecipeDetailService;
	@Autowired
	private StoreFeignClients storeFeignClients;

	@Autowired
	private DictionaryItemService dictionaryItemService;

	@Autowired
	private MaterialModelService materialModelService;

	@Autowired
	private ReportSapGoodsmvtService reportSapGoodsmvtService;



	@Override
	public InputFurnaceBurdenDto getStartInfo(Long againTaskDetailId) {
		try {
			InputFurnaceBurdenDto data = (InputFurnaceBurdenDto) redisUtils.get(BurdenKey+""+againTaskDetailId);
			return data;
		}catch (Exception e) {
			return null;
		}
	}


	@Override
	public void startInputFurnace(InputFurnaceBurdenDto inputFurnaceBurdenDto) {
		//桶任务 - 灌料任务
		AgainTaskDetail againTaskDetail = againTaskDetailService.get(inputFurnaceBurdenDto.getAgainTaskDetailId());
//		AgainTask againTask = againTaskService.get(againTaskDetail.getAgainTaskId());
		if(againTaskDetail.getRawStatus().equals(AgainTaskDetailDto.rawStatus_0)) {
			againTaskDetail.setRawStatus(AgainTaskDetailDto.rawStatus_1);
		}else {
			throw new RuntimeException("当前任务未处在可执行状态");
		}

		List<InputFurnaceDetailDto> detailDto = inputFurnaceBurdenDto.getDetails();
		List<String> batchNos = new ArrayList<>();
		for (InputFurnaceDetailDto inputFurnaceDetailDto : detailDto) {
			String batchNo = inputFurnaceDetailDto.getBatchNo();
			BigDecimal weightD = inputFurnaceDetailDto.getWeight();
			if(weightD == null || weightD.compareTo(BigDecimal.ZERO) <= 0 ){
				throw new RuntimeException("重量不允许为0或者负数");
			}
			if(batchNos.contains(batchNo)){
				throw new RuntimeException("物料有重复，请检查！");
			}
			batchNos.add(batchNo);
		}

		//校验物料类型是否在方案内
//		List<InputFurnaceDetailDto> detailDtos = inputFurnaceBurdenDto.getDetails();
//		List<String> materialTypeNames = detailDtos.stream().map(temp->temp.getMaterialTypeName()).distinct().collect(Collectors.toList());
//		if(null == materialTypeNames||materialTypeNames.isEmpty()){
//			throw new RuntimeException("未查询到物料");
//		}
//		Long recipeId = againTaskDetail.getRecipeId();
//		Integer againNum = againTaskDetail.getAgainNum();
//		List <String> recipeMaterialTypeNames;
//		//是不是完结复投
//		if(againTaskDetail.getIsDone() == 1){
//			recipeMaterialTypeNames = productionCircleRecipeDetailService.getRecipeMaterialTypeNamesByParamAccurate(recipeId,"0");
//		}else {
//			recipeMaterialTypeNames = productionCircleRecipeDetailService.getRecipeMaterialTypeNamesByParam(recipeId, "%" + againNum.toString() + "%");
//		}
//		if(null == materialTypeNames||materialTypeNames.isEmpty()){
//			throw new RuntimeException("未查询到物料");
//		}
//		if(!recipeMaterialTypeNames.containsAll(materialTypeNames)){
//			throw new RuntimeException("投放的物料类型不在物料方案内，请检查");
//		}

		//校验箱数总量是否等于总重量
//		Integer bucketFortyCount = detailDtos.stream().mapToInt(temp->Optional.ofNullable(temp.getBucketForty()).orElse(0)).sum();
//		Integer bucketTenCount = detailDtos.stream().mapToInt(temp->Optional.ofNullable(temp.getBucketTen()).orElse(0)).sum();
//		Integer sum = bucketFortyCount*40 + bucketTenCount*10;
//		if(new BigDecimal(sum).compareTo(inputFurnaceBurdenDto.getWeigth()) != 0){
//			throw new RuntimeException("投放的总量与需要投放的总量不相等，请检查");
//		}
		againTaskDetail.setStartTime(new Date());

		againTaskDetailService.saveOrUpdate(againTaskDetail);

		//复投任务

//		Integer pailNum = againTask.getPailNum();
//		pailNum = pailNum==null?0:pailNum;
//		againTask.setPailNum((int)(pailNum+againTaskDetail.getNum()));//当前桶数
//
//		againTaskService.saveOrUpdate(againTask);

		//下发生产任务接口
		//mMcsInterfaceService.issueMoInterface(againTask);

		//开始灌料缓存参数
		redisUtils.set(BurdenKey+""+inputFurnaceBurdenDto.getAgainTaskDetailId(), inputFurnaceBurdenDto);
	}


	@Override
	public void cancelInputFurnace(Long againTaskDetailId) {
		//桶任务 - 灌料任务
		AgainTaskDetail againTaskDetail = againTaskDetailService.get(againTaskDetailId);
		if(againTaskDetail.getRawStatus().equals(AgainTaskDetailDto.rawStatus_1)) {
			againTaskDetail.setRawStatus(AgainTaskDetailDto.rawStatus_0);
		}else {
			throw new RuntimeException("当前任务未处在生产状态");
		}


		againTaskDetailService.saveOrUpdate(againTaskDetail);

		//复投任务
		AgainTask againTask = againTaskService.get(againTaskDetail.getAgainTaskId());
		Integer pailNum = againTask.getPailNum();
		pailNum = pailNum==null?0:pailNum;
		againTask.setPailNum((int)(againTaskDetail.getNum()-pailNum));//当前桶数
		againTaskService.saveOrUpdate(againTask);

		//下发生产任务接口
		//mMcsInterfaceService.issueMoInterface(againTask);

		//删除redis中填写数据信息
		redisUtils.remove(BurdenKey+""+againTaskDetailId);
	}


	@Override
	public void back(InputFurnaceBurdenDto inputFurnaceBurdenDto) {
		//开始灌料缓存参数
		redisUtils.set(BurdenKey+""+inputFurnaceBurdenDto.getAgainTaskDetailId(), inputFurnaceBurdenDto);
	}



	@Override
	@Transactional(rollbackFor = Exception.class)
	public void endInputFurnace(InputFurnaceBurdenDto inputFurnaceBurdenDto) {
		//桶任务 - 灌料任务
		AgainTaskDetail againTaskDetail = againTaskDetailService.get(inputFurnaceBurdenDto.getAgainTaskDetailId());
		if(inputFurnaceBurdenDto.getUserName() == null){
			inputFurnaceBurdenDto.setUserName(BaseContextHandler.getUsername());
		}
		if(againTaskDetail.getRawStatus().equals(AgainTaskDetailDto.rawStatus_1)||againTaskDetail.getRawStatus().equals(AgainTaskDetailDto.rawStatus_0)) {
			againTaskDetail.setRawStatus(AgainTaskDetailDto.rawStatus_4);
		}else {
			throw new RuntimeException("当前任务未处在可执行状态");
		}

		// 晶科删除溯源卡控-WXL标记-待删除
		//检验溯源标识
		// checkFromMark(againTaskDetail,inputFurnaceBurdenDto);

		List<InputFurnaceDetailDto> detailDto = inputFurnaceBurdenDto.getDetails();

		//复投任务
		AgainTask againTask = againTaskService.get(againTaskDetail.getAgainTaskId());


		//已灌料桶数
		int alreadyPailNum = againTask.getAlreadyPailNum()==null?0:againTask.getAlreadyPailNum();
		alreadyPailNum += inputFurnaceBurdenDto.getPailNum();

		againTaskDetail.setCarrierNo(inputFurnaceBurdenDto.getVehicleCode());
		againTaskDetail.setAlreadyPailNum(alreadyPailNum);
		againTaskDetail.setStartTime(inputFurnaceBurdenDto.getStartTime());
		againTaskDetail.setEndTime(inputFurnaceBurdenDto.getEndTime());
		againTaskDetail.setUserName(inputFurnaceBurdenDto.getUserName());
		againTaskDetail.setAgainRealWeight(inputFurnaceBurdenDto.getAgainRealWeight());
		againTaskDetailService.saveOrUpdate(againTaskDetail);

		againTask.setAlreadyPailNum(alreadyPailNum);
		//完成任务 - 复投任务
		if(againTask.getAgainPailNum()==null || againTask.getAlreadyPailNum() >= againTask.getAgainPailNum()) {
			if(againTask.getRawStatus().equals(AgainTaskDto.rawStatus_1)) {
				againTask.setRawStatus(AgainTaskDto.rawStatus_4);
			}else {
				throw new RuntimeException("当前复投任务未处在可执行状态");
			}
		}
		Integer pailNum = againTask.getPailNum();
		pailNum = pailNum==null?0:pailNum;
		againTask.setPailNum(pailNum+againTaskDetail.getNum());//当前桶数
		againTask.setAlloyBrand(inputFurnaceBurdenDto.getAlloyBrand());
		againTask.setAlloyType(inputFurnaceBurdenDto.getAlloyType());
		againTask.setAlloyWeight(inputFurnaceBurdenDto.getAlloyWeight());
		againTask.setRemark(inputFurnaceBurdenDto.getRemark());
		againTask.setTrialType(inputFurnaceBurdenDto.getTrialType());
		againTaskService.saveOrUpdate(againTask);

		//完善灌料开始数据
		InputFurnaceBurden inputFurnaceBurden = fromDto(inputFurnaceBurdenDto);
		initStartInputFurnaceBurdenData(againTaskDetail, inputFurnaceBurden);
		saveOrUpdate(inputFurnaceBurden);

		List<String> batchNos = new ArrayList<>();
		for (InputFurnaceDetailDto inputFurnaceDetailDto : detailDto) {
			if(inputFurnaceDetailDto.getWeight() == null
					|| inputFurnaceDetailDto.getWeight().compareTo(BigDecimal.ZERO) <= 0 ){
				throw new RuntimeException("重量不允许为0或者负数");
			}
			if(batchNos.contains(inputFurnaceDetailDto.getBatchNo())){
				throw new RuntimeException("物料有重复，请检查！");
			}
			if(inputFurnaceDetailDto.getMaterialName().contains("合金")){
				//如果无合金牌号和类型，提示必填
				if(StringUtils.isAnyBlank(inputFurnaceBurdenDto.getAlloyBrand(),inputFurnaceBurdenDto.getAlloyType())){
					throw new RuntimeException("无合金牌号或者类型");
				}
				inputFurnaceDetailDto.setAlloyBrand(inputFurnaceBurdenDto.getAlloyBrand());
				inputFurnaceDetailDto.setAlloyType(inputFurnaceBurdenDto.getAlloyType());
			}

			InputFurnaceDetail detail = fromDto(inputFurnaceDetailDto);
			detail.setInputFurnaceId(inputFurnaceBurden.getId());
			detail.setAgainTaskDetailId(againTaskDetail.getId());
			inputFurnaceDetailService.saveOrUpdate(detail);

			batchNos.add(inputFurnaceDetailDto.getBatchNo());
		}

		//删除redis中填写数据信息
		redisUtils.remove(BurdenKey+""+inputFurnaceBurdenDto.getAgainTaskDetailId());
	}

	private void checkFromMark(AgainTaskDetail againTaskDetail, InputFurnaceBurdenDto inputFurnaceBurdenDto) {
		String productType = againTaskDetail.getProductType();
		Date startDate ;
		DictionaryItem item = dictionaryItemService.findByThemeValueAndNameAndEnable("traceabilityTime","startTime",true);
		if(item == null){
			throw new RuntimeException("溯源卡控时间未维护！");
		} else{
			String startTime = item.getValue();
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			try {
				startDate = sdf.parse(startTime);
			} catch (ParseException e) {
				throw new RuntimeException("溯源卡控时间格式不正确！");
			}
		}
		List<String> fromMarks = new ArrayList<>();
		if(productType.length() > 1 && startDate != null && startDate.compareTo(new Date()) < 0) {
			String productCategory = productType.substring(0, 1);
			if (productCategory != null) {
				fromMarks = dictionaryItemService.findValuesByThemeValueAndItemName("fillMarkMap", productCategory);
				if (CollectionUtil.isEmpty(fromMarks)) {
					throw new RuntimeException(productCategory+"的溯源卡控标识对应关系未维护！");
				}
			}
			List<InputFurnaceDetailDto> details = inputFurnaceBurdenDto.getDetails();
			for (InputFurnaceDetailDto dto : details) {
				String batchNo = dto.getBatchNo();
				String fromMark = batchNo.substring(0,1);
				if(!fromMarks.contains(fromMark)){
					throw new RuntimeException("批次号"+batchNo+"溯源卡控不通过！工单溯源标识为"+productCategory);
				}

			}
		}

	}

	@Override
	public List<InputFurnaceBurden> findByAgainTaskDetailId(Long ainTaskDetailId) {
		return dao.findByAgainTaskDetailId(ainTaskDetailId);
	}



	private void initStartInputFurnaceBurdenData(AgainTaskDetail againTaskDetail, InputFurnaceBurden inputFurnaceBurden) {
        inputFurnaceBurden.setId(null);
        inputFurnaceBurden.setAgainNum(againTaskDetail.getAgainNum());
        inputFurnaceBurden.setEquipmentCode(againTaskDetail.getEquipmentCode());
        inputFurnaceBurden.setLotNo(againTaskDetail.getLotNo());
        inputFurnaceBurden.setSeries(againTaskDetail.getSeries());
        inputFurnaceBurden.setArea(againTaskDetail.getArea());
        inputFurnaceBurden.setRawType(againTaskDetail.getRawType());
        if (inputFurnaceBurden.getPailNum() == null) {
            inputFurnaceBurden.setPailNum(againTaskDetail.getNum());
        }
        inputFurnaceBurden.setWeigth(againTaskDetail.getAgainWeigth());
        inputFurnaceBurden.setEndTime(new Date());
        inputFurnaceBurden.setFurnaceType(againTaskDetail.getFurnaceType());
        inputFurnaceBurden.setSapNo(againTaskDetail.getSapNo());
        inputFurnaceBurden.setProductCode(againTaskDetail.getProductCode());
        inputFurnaceBurden.setProductGrade(againTaskDetail.getProductGrade());
        inputFurnaceBurden.setSpecs(againTaskDetail.getSpecs());
        inputFurnaceBurden.setDopant(againTaskDetail.getDopant());
        inputFurnaceBurden.setProductType(againTaskDetail.getProductType());
        inputFurnaceBurden.setAgainTaskDetailId(againTaskDetail.getId());
        //下次复投是否完结【0：不是; 1 是;】
        inputFurnaceBurden.setIsNextEnd(againTaskDetail.getIsNextEnd());
    }


	public InputFurnaceBurden fromDto(InputFurnaceBurdenDto dto) {
		return modelMapper.map(dto, InputFurnaceBurden.class);
	}


	public InputFurnaceDetail fromDto(InputFurnaceDetailDto dto) {
		InputFurnaceDetail inputFurnaceDetail = modelMapper.map(dto, InputFurnaceDetail.class);
		inputFurnaceDetail.setId(null);
		return inputFurnaceDetail;
	}

	@Override
	public List<InputFurnaceBurden> findByStartTime(String startTime, String endTime, String series) {
		return dao.findByStartTime(startTime, endTime, series);
	}

	@Override
	public List<InputFurnaceBurden> findUnconfirmedTask(String lotNo) {
		return dao.findUnconfirmedTask(lotNo);
	}

	@Override
	public void confirmMaterials(Long id) {
		InputFurnaceBurden burden = dao.getOne(id);
		Date date = new Date();
		burden.setConfirm(1);
		burden.setConfirmTime(date);
		dao.save(burden);
		AgainTaskDetail taskDetail = againTaskDetailService.get(burden.getAgainTaskDetailId());
		taskDetail.setConfirm(1);
		taskDetail.setConfirmTime(date);
		againTaskDetailService.saveOrUpdate(taskDetail);

		// 线边仓出库
		List<InputFurnaceDetail> detailList = inputFurnaceDetailService.findByInputFurnaceId(id);
		WarehouseOutputBillBo outputBillBo = new WarehouseOutputBillBo();
		outputBillBo.setType("生产领用");
		outputBillBo.setAutoAudit(true);
		outputBillBo.setDestination("车间");
		List<WarehouseOutputBillDetailBo> boList = new ArrayList<>();
		for (InputFurnaceDetail detail : detailList) {
			MaterialModel model = materialModelService.findByCode(detail.getMaterialCode());
			WarehouseOutputBillDetailBo bo = new WarehouseOutputBillDetailBo();
			bo.setMaterialModelId(model.getId().longValue());
			bo.setMaterialModelCode(model.getCode());
			bo.setMaterialModelName(model.getName());
			bo.setBatchNo(detail.getBatchNo());
			bo.setMaterialTypeName(model.getMaterialType().getName());
			bo.setSupplier(model.getSupplier());
			bo.setSpecification(model.getSpecification());
			bo.setMaterialTh(model.getNumberOfDrawings());
			bo.setRequireCount(detail.getWeight());
			bo.setUnitName(detail.getUnit());
			bo.setWarehouseId(detail.getStoreLocationId().longValue());
			bo.setWarehouseName(detail.getStoreLocationName());

			boList.add(bo);
		}
		outputBillBo.setOutputBillDetails(boList);
		Result<String> result = storeFeignClients.outputApply(outputBillBo);
		if (result.getCode() != ResultCode.SUCCESS.getCode()){
			throw new RuntimeException("线边仓出库失败：" + result.getMessage());
		}

		// SAP报消耗
		for (InputFurnaceDetail detail : detailList) {
			ReportSapGoodsmvt reportSap = new ReportSapGoodsmvt();
			reportSap.setBwart("261");		// 移动类型
			reportSap.setKostl(null);		// 成本中心
			reportSap.setProcName("拉晶");	// 工序
			reportSap.setAufnr(taskDetail.getSapNo());		// SAP生产订单
			reportSap.setMatnr(detail.getMaterialCode());	// 物料编码
			reportSap.setCharg(detail.getBatchNo());		// 批次号
			reportSap.setMenge(detail.getWeight().toString());	// 数量
			reportSap.setMeins(detail.getUnit());		// 单位
			Result<WarehouseModelDto> result1 = storeFeignClients.getById(detail.getStoreLocationId().longValue());
			if (result.getCode() == ResultCode.SUCCESS.getCode()){
				WarehouseModelDto dto = result1.getData();
				if (dto != null){
					reportSap.setLgort(dto.getCode());	// 库存地点
				}
			}

			reportSapGoodsmvtService.saveOrUpdate(reportSap);
		}

	}

}
