package com.sw.mes.yb.production.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.google.gson.Gson;
import com.sw.mes.base.erp.entity.ErpOrder;
import com.sw.mes.base.erp.service.ErpOrderService;
import com.sw.mes.base.factory.inspect.entity.ReturnRecord;
import com.sw.mes.base.factory.material.entity.MaterialModel;
import com.sw.mes.base.factory.material.service.MaterialModelService;
import com.sw.mes.base.system.entity.DictionaryItem;
import com.sw.mes.base.system.entity.SysLog;
import com.sw.mes.base.system.service.DictionaryItemService;
import com.sw.mes.base.system.service.SysLogService;
import com.sw.mes.core.annotation.Jinko;
import com.sw.mes.core.api.materialclean.MaterialCleanFeignClients;
import com.sw.mes.core.api.system.FbFeignClients;
import com.sw.mes.core.api.system.StoreFeignClients;
import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.DictionaryItemDto;
import com.sw.mes.core.dto.MaterialModelDto;
import com.sw.mes.core.dto.ProductionOrderDto;
import com.sw.mes.core.dto.erp.ErpAddResultDto;
import com.sw.mes.core.dto.erp.ErpRequestDto;
import com.sw.mes.core.dto.materialCleanDto.MaterialStoreSignBoxDto;
import com.sw.mes.core.dto.mes.pc.fb.FbProductionOrderDto;
import com.sw.mes.core.dto.sap.ReportSapChargItemDto;
import com.sw.mes.core.dto.sap.ReportSapGoodsmvtDto;
import com.sw.mes.core.dto.store.WarehouseInputBillBo;
import com.sw.mes.core.dto.store.WarehouseInputBillDetailBo;
import com.sw.mes.core.dto.wms.*;
import com.sw.mes.core.entity.MesToErpCcpEntity;
import com.sw.mes.core.entity.SearchFilter;
import com.sw.mes.core.enums.EnumValues;
import com.sw.mes.core.enums.ErpPypeEnum;
import com.sw.mes.core.enums.ItemCleanEnums;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.response.ResultWms;
import com.sw.mes.core.utils.*;
import com.sw.mes.enumvalue.StockStateEnum;
import com.sw.mes.rabbitmq.constants.RoutingKey;
import com.sw.mes.rabbitmq.enums.ExchangeEnum;
import com.sw.mes.rabbitmq.enums.QueueEnum;
import com.sw.mes.rabbitmq.service.RabbitMqService;
import com.sw.mes.sap.entity.ReportSapGoodsmvt;
import com.sw.mes.sap.entity.ReportSapWorkinghours;
import com.sw.mes.sap.service.ReportSapGoodsmvtService;
import com.sw.mes.sap.service.ReportSapWorkinghoursService;
import com.sw.mes.third.utils.ErpRequestUtil;
import com.sw.mes.utils.ExcelUtil;
import com.sw.mes.wms.service.WMSOutputService;
import com.sw.mes.yb.accessory.dto.EquipmentAccessoryExportDto;
import com.sw.mes.yb.accessory.dto.RecycledMaterialInDto;
import com.sw.mes.yb.plan.entity.ProductionOrder;
import com.sw.mes.yb.plan.entity.ProductionSchedule;
import com.sw.mes.yb.plan.service.ProductionOrderService;
import com.sw.mes.yb.plan.service.ProductionScheduleService;
import com.sw.mes.yb.production.dao.RecycledMaterialManageDao;
import com.sw.mes.yb.production.dto.RecycledMaterialManageDetailExportDto;
import com.sw.mes.yb.production.dto.RecycledMaterialManageDto;
import com.sw.mes.yb.production.entity.LongCrystal;
import com.sw.mes.yb.production.entity.RecycledMaterialManage;
import com.sw.mes.yb.production.entity.RecycledMaterialManageDetail;
import com.sw.mes.yb.production.service.LongCrystalService;
import com.sw.mes.yb.production.service.RecycledMaterialManageDetailService;
import com.sw.mes.yb.production.service.RecycledMaterialManageService;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: xushu
 * @Date: 2022/08/17/16:24
 * @Description:
 */
@Service
@Transactional
public class RecycledMaterialManageServiceImpl extends BaseServiceImpl<RecycledMaterialManage> implements RecycledMaterialManageService {
    RecycledMaterialManageDao dao;
    @Autowired
    public void setDao(RecycledMaterialManageDao dao) {
        super.setBaseDao(dao);
        this.dao=dao;
    }
    @Autowired
    RecycledMaterialManageDetailService recycledMaterialManageDetailService;
    @Autowired
    ModelMapper modelMapper;
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private RabbitMqService rabbitMqService;
    @Autowired
    private DictionaryItemService dictionaryItemService;
    @Autowired
    private SysLogService sysLogService;

    @Autowired
    private LongCrystalService longCrystalService;

    @Autowired
    private MaterialModelService materialModelService;
    @Autowired
    private ProductionScheduleService productionScheduleService;

    @Autowired
    private MaterialCleanFeignClients materialCleanFeignClients;

    @Autowired
    private ErpOrderService erpOrderService;
    @Value("${spring.erpServiceUrl}")
    private String erpServiceUrl;
    @Autowired
    private ErpRequestUtil erpRequestUtil;
    @Autowired
    private ProductionOrderService productionOrderService;
    @Autowired
    private FbFeignClients fbFeignClients;

    @Autowired
    private StoreFeignClients storeFeignClients;

    @Autowired
    private ReportSapGoodsmvtService reportSapGoodsmvtService;

    @Autowired
    private WMSOutputService wmsInputService;

    @Value("${spring.profiles.active}")
    private String active;

    //TODO 设置wms api
    @Value("${spring.wmsServiceUrl}")
    private String wmsServiceUrl;

    private final String recycleToWms = "/api/InboundRecycledMaterial";


    @Override
    public void softDeleteById(Long id) {
        dao.softDeleteById(id);
    }


    @Override
    public void exportExcel(HttpServletRequest request, HttpServletResponse resp) throws  Exception{
        SearchFilter filter = new SearchFilter("del", SearchFilter.Operator.EQ,0);//查询未删除的列表
        Specification<RecycledMaterialManage> specification = DynamicSpecifications.bySearchFilter(request, RecycledMaterialManage.class,filter);
        //主表
        List<RecycledMaterialManage> recycledMaterialManages = findByExampleSort(specification, Sort.by(Sort.Direction.ASC,  "id"));
        List<DictionaryItem> productCategoryItems = dictionaryItemService.findByThemeValueAndEnable("product_category",true);
//        List<DictionaryItem> factoryCateItems = dictionaryItemService.findByThemeValueAndEnable("FactoryCate",true);
//        List<DictionaryItem> gearItems = dictionaryItemService.findByThemeValueAndEnable("Gear",true);
        //附表
        List<RecycledMaterialManageDetailExportDto> recycledMaterialManageDetailExportDtos = new ArrayList<>();
        for(RecycledMaterialManage recycledMaterialManage : recycledMaterialManages){
            List<RecycledMaterialManageDetail> recycledMaterialManageDetails = recycledMaterialManageDetailService.getDetailByManageId(recycledMaterialManage.getId());
            if(null == recycledMaterialManageDetails || recycledMaterialManageDetails.isEmpty()){
                RecycledMaterialManageDetailExportDto recycledMaterialManageDetailExportDto = modelMapper.map(recycledMaterialManage,RecycledMaterialManageDetailExportDto.class);
                List<String> productionTypeNames = productCategoryItems.stream().filter(temp->temp.getValue().equals(recycledMaterialManageDetailExportDto.getProductionCategory())).map(DictionaryItem::getName).collect(Collectors.toList());
                if(productionTypeNames.size()>0){
                    recycledMaterialManageDetailExportDto.setProductionCategory(productionTypeNames.get(0));
                }
                recycledMaterialManageDetailExportDtos.add(recycledMaterialManageDetailExportDto);
                continue;
            }
            for(RecycledMaterialManageDetail recycledMaterialManageDetail : recycledMaterialManageDetails){
                RecycledMaterialManageDetailExportDto recycledMaterialManageDetailExportDto = modelMapper.map(recycledMaterialManage,RecycledMaterialManageDetailExportDto.class);
                List<String> productionTypeNames = productCategoryItems.stream().filter(temp->temp.getValue().equals(recycledMaterialManageDetailExportDto.getProductionCategory())).map(DictionaryItem::getName).collect(Collectors.toList());
                if(productionTypeNames.size() >0 ){
                    recycledMaterialManageDetailExportDto.setProductionCategory(productionTypeNames.get(0));
                    recycledMaterialManageDetailExportDto.setProductionType(productionTypeNames.get(0));
                }
//                recycledMaterialManageDetailExportDtos.add(recycledMaterialManageDetailExportDto);
                recycledMaterialManageDetailExportDto.setRecycleNo(recycledMaterialManageDetail.getRecycleNo());
                recycledMaterialManageDetailExportDto.setProductLevel(recycledMaterialManageDetail.getProductLevel());
                recycledMaterialManageDetailExportDto.setStatus(StockStateEnum.getDescByCode(recycledMaterialManage.getStatus()));
                recycledMaterialManageDetailExportDto.setErpOrder(recycledMaterialManage.getErpOrder());
                recycledMaterialManageDetailExportDtos.add(recycledMaterialManageDetailExportDto);

            }
        }
        if(recycledMaterialManageDetailExportDtos.isEmpty()){
            throw new  BusinessException("无匹配数据");
        }
        try {
            EasyExcelUtils.write(resp, RecycledMaterialManageDetailExportDto.class, "回收料管理", "回收料管理", recycledMaterialManageDetailExportDtos);
        } catch (Exception e) {
            e.printStackTrace();
            throw new  BusinessException("导出异常");
        }
    }

    @Override
    public Integer countTimeByParam(String key) {
        return dao.countTimeByParam(key);
    }

    @Override
    public BigDecimal findTiZhaGaiByLotNo(String lotNo) {
        return dao.findTiZhaGaiByLotNo(lotNo);
    }

    @Override
    public BigDecimal findTiJianLiaoByLotNo(String lotNo) {
        return dao.findTiJianLiaoByLotNo(lotNo);
    }

    @Override
    public BigDecimal findGuoDiLiaoByLotNo(String lotNo) {
        return dao.findDunGuoLiaoByLotNo(lotNo);
    }

    @Override
    public BigDecimal findDunGuoLiaoByLotNo(String lotNo) {
        return dao.findDunGuoLiaoByLotNo(lotNo);
    }

    @Override
    public void recycledMaterialIn(RecycledMaterialManageDto recycledMaterialManageDto) {

        //回收料管理的数据应存到待清洗表中
        MaterialStoreSignBoxDto materialStoreSignBoxDto = new MaterialStoreSignBoxDto();
        materialStoreSignBoxDto.setWeight(recycledMaterialManageDto.getWeight());
        materialStoreSignBoxDto.setMaterialCode(recycledMaterialManageDto.getMaterialCode());
        materialStoreSignBoxDto.setMaterialName(recycledMaterialManageDto.getMaterialName());
        materialStoreSignBoxDto.setMaterialLevel(recycledMaterialManageDto.getGrade());
        materialStoreSignBoxDto.setMaterialModel(recycledMaterialManageDto.getSpecification());
        materialStoreSignBoxDto.setMaterialGear(recycledMaterialManageDto.getGear());
        materialStoreSignBoxDto.setCreateUser(recycledMaterialManageDto.getCreateBy());
        materialStoreSignBoxDto.setCreateTime(Instant.ofEpochMilli(recycledMaterialManageDto.getCreateTime().getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime());
        materialStoreSignBoxDto.setUpdateUser(recycledMaterialManageDto.getUpdateBy());
        materialStoreSignBoxDto.setUpdateTime(Instant.ofEpochMilli(recycledMaterialManageDto.getUpdateTime().getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime());
        materialStoreSignBoxDto.setAufnr(recycledMaterialManageDto.getErpOrder());
        materialStoreSignBoxDto.setBoxNo(recycledMaterialManageDto.getBoxNo());
        materialStoreSignBoxDto.setPickingStatus(0);
        materialStoreSignBoxDto.setMaterialType(2);
        materialStoreSignBoxDto.setErpWarehouseCode(recycledMaterialManageDto.getErpWarehouseCode());
        //新增托号，托号用回收料管理模块中的托号
        materialStoreSignBoxDto.setTrayNo(recycledMaterialManageDto.getTrayNo());
        //批次号用回收料管理模块中的批次号
        materialStoreSignBoxDto.setProductBatchNo(recycledMaterialManageDto.getProduceBatchNo());

        Result result = materialCleanFeignClients.create(materialStoreSignBoxDto);
        if (result.getCode() != ResultCode.SUCCESS.getCode()) {
            throw new BusinessException("调用清洗原料接收:[materialStoreSignBox/create]失败: " + result.getMessage());
        }
        RecycledMaterialInDto recycledMaterialInDto;
        String url = wmsServiceUrl + recycleToWms;
        //不可回收料，入库
        if (recycledMaterialManageDto.getRecycledMaterialType().equals("不可用回收料")) {
            recycledMaterialInDto = buildNonRecycledMaterialInDto(recycledMaterialManageDto);
            if ("prod".equals(active)) {

                ResultWms resultJson = requestRecycleMaterialToWMS(url, recycledMaterialInDto, "不可用回收料入库到WMS");
                if (null != resultJson) {
                    if (!resultJson.getCode().equals("1")) {
                        throw new BusinessException("入库到wms失败,请求接口失败");
                    }
                } else {
                    throw new BusinessException("入库到wms失败,请求接口异常");
                }
            }
            //可回收料,重量>0，入库
        } else if (recycledMaterialManageDto.getRecycledMaterialType().equals("可用回收料")) {
            if (null != recycledMaterialManageDto.getWeight() && recycledMaterialManageDto.getWeight().compareTo(BigDecimal.ZERO) > 0) {
                recycledMaterialInDto = buildRecycledMaterialInDto(recycledMaterialManageDto);
                if ("prod".equals(active)) {

                    ResultWms resultJson = requestRecycleMaterialToWMS(url, recycledMaterialInDto, "可用回收料过账到WMS,发起到清洗原料车间");
                    if (null != resultJson) {
                        if (!resultJson.getCode().equals("1")) {
                            throw new BusinessException("入库到wms失败,请求接口失败");
                        }
                    } else {
                        throw new BusinessException("入库到wms失败,请求接口异常");
                    }
                }
            }

        } else {
            //todo:循环料现在未过回收料
        }
//        String url = wmsServiceUrl+recycleToWms;
//        ResultWms resultJson = requestRecycleMaterialToWMS(url,recycledMaterialInDto,"可回收料入库到WMS");
//        if(null != resultJson) {
//            if (!resultJson.getCode().equals("1")) {
//                throw new BusinessException("入库到wms失败,请求接口失败");
//            }
//        }else{
//            throw new BusinessException("入库到wms失败,请求接口异常");
//        }

    }

    @Override
    public void recycledMaterialIn(RecycledMaterialManageDto recycledMaterialManageDto, String hasGone) {

        if ("WMS".equals(hasGone)) {

            RecycledMaterialInDto recycledMaterialInDto;
            String url = wmsServiceUrl + recycleToWms;
            //不可回收料，入库
            if (recycledMaterialManageDto.getRecycledMaterialType().equals("不可用回收料")) {
                recycledMaterialInDto = buildNonRecycledMaterialInDto(recycledMaterialManageDto);
                if ("prod".equals(active)) {

                    ResultWms resultJson = requestRecycleMaterialToWMS(url, recycledMaterialInDto, "不可用回收料入库到WMS");
                    if (null != resultJson) {
                        if (!resultJson.getCode().equals("1")) {
                            throw new BusinessException("入库到wms失败,请求接口失败");
                        }
                    } else {
                        throw new BusinessException("入库到wms失败,请求接口异常");
                    }
                }
                //可回收料,重量>0，入库
            } else if (recycledMaterialManageDto.getRecycledMaterialType().equals("可用回收料")) {
                if (null != recycledMaterialManageDto.getWeight() && recycledMaterialManageDto.getWeight().compareTo(BigDecimal.ZERO) > 0) {
                    recycledMaterialInDto = buildRecycledMaterialInDto(recycledMaterialManageDto);
                    if ("prod".equals(active)) {

                        ResultWms resultJson = requestRecycleMaterialToWMS(url, recycledMaterialInDto, "可用回收料过账到WMS,发起到清洗原料车间");
                        if (null != resultJson) {
                            if (!resultJson.getCode().equals("1")) {
                                throw new BusinessException("入库到wms失败,请求接口失败");
                            }
                        } else {
                            throw new BusinessException("入库到wms失败,请求接口异常");
                        }
                    }
                }

            } else {
                //todo:循环料现在未过回收料
            }
        } else {
            //回收料管理的数据应存到待清洗表中
            MaterialStoreSignBoxDto materialStoreSignBoxDto = new MaterialStoreSignBoxDto();
            materialStoreSignBoxDto.setWeight(recycledMaterialManageDto.getWeight());
            materialStoreSignBoxDto.setMaterialCode(recycledMaterialManageDto.getMaterialCode());
            materialStoreSignBoxDto.setMaterialName(recycledMaterialManageDto.getMaterialName());
            materialStoreSignBoxDto.setMaterialLevel(recycledMaterialManageDto.getGrade());
            materialStoreSignBoxDto.setMaterialModel(recycledMaterialManageDto.getSpecification());
            materialStoreSignBoxDto.setMaterialGear(recycledMaterialManageDto.getGear());
            materialStoreSignBoxDto.setCreateUser(recycledMaterialManageDto.getCreateBy());
            materialStoreSignBoxDto.setCreateTime(Instant.ofEpochMilli(recycledMaterialManageDto.getCreateTime().getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime());
            materialStoreSignBoxDto.setUpdateUser(recycledMaterialManageDto.getUpdateBy());
            materialStoreSignBoxDto.setUpdateTime(Instant.ofEpochMilli(recycledMaterialManageDto.getUpdateTime().getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime());
            materialStoreSignBoxDto.setAufnr(recycledMaterialManageDto.getErpOrder());
            materialStoreSignBoxDto.setBoxNo(recycledMaterialManageDto.getBoxNo());
            materialStoreSignBoxDto.setPickingStatus(0);
            materialStoreSignBoxDto.setMaterialType(2);
            materialStoreSignBoxDto.setErpWarehouseCode(recycledMaterialManageDto.getErpWarehouseCode());
            //新增托号，托号用回收料管理模块中的托号
            materialStoreSignBoxDto.setTrayNo(recycledMaterialManageDto.getTrayNo());
            //批次号用回收料管理模块中的批次号
            materialStoreSignBoxDto.setProductBatchNo(recycledMaterialManageDto.getProduceBatchNo());

            Result result = materialCleanFeignClients.create(materialStoreSignBoxDto);
            if (result.getCode() != ResultCode.SUCCESS.getCode()) {
                throw new BusinessException("调用清洗原料接收:[materialStoreSignBox/create]失败: " + result.getMessage());
            }
        }
    }

    /**
     * 余料发起入库到线边仓
     *
     * @param recycledMaterialManage 回收料数据
     * @param reason                 入库原因
     * @return 入库单号
     */
    @Override
    public String startSurplusWarehousing(RecycledMaterialManage recycledMaterialManage, String reason) {
        try {
            //通过计划工单中的片区field4找到字典中的成品仓
            String warehouseCode = getWareHouseCodeByWorkOrderNo(recycledMaterialManage.getWorkOrderNo());
            //调线边仓
            WarehouseInputBillBo warehouseInputBillBo = newInputBill(recycledMaterialManage, reason ,warehouseCode);
            Result<String> result = storeFeignClients.automaticInputOut(warehouseInputBillBo);
            Integer code = result.getCode();
            String data = result.getData();
            if (code != 200 || StringUtils.isBlank(data)) {
                throw new BusinessException("余料入库失败，状态码：" + code + "，反馈信息：" + result.getMessage());
            } else {
                //1 回收料产出报工SAP
                String finalWarehouseCode = warehouseCode;
//                ThreadManager.getThreadPollProxy().execute(() -> {
                    try {
                        this.handleReportSapGoodsmvtDto(recycledMaterialManage.getWorkOrderNo(), recycledMaterialManage.getMaterialCode(), recycledMaterialManage.getProduceBatchNo(), recycledMaterialManage.getWeight(), recycledMaterialManage.getUnitName(), finalWarehouseCode);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
//                });
            }
            return data;
        } catch (Exception e) {
            throw new BusinessException("余料入库异常，" + e.getMessage());
        }
    }

    /**
     * 通过工单号获取成品仓库位
     *
     * @param workOrderNo 计划工单
     * @return
     */
    @Jinko(author = "yyl", value = "通过工单号获取库位")
    private String getWareHouseCodeByWorkOrderNo(String workOrderNo) {
        //通过计划工单中的片区field4找到字典中的成品仓
        ProductionOrder productionOrder = productionOrderService.findByAufnr(workOrderNo);
        String field4 = "";
        String warehouseCode = "";
        if (null != productionOrder) {
            field4 = productionOrder.getField4();
        }
        if (null != field4) {
            DictionaryItem warehouseLocation = dictionaryItemService.findNameByThemeValueAndItemValue("ByProductWarehouseLocation", field4);
            if (null != warehouseLocation) {
                if (null != warehouseLocation) {
                    warehouseCode = warehouseLocation.getExtendValue1();
                }
            }else {
                throw new BusinessException("关键字[ByProductWarehouseLocation]中的[工作中心编码]: " + field4 + " 未配置");
            }
        }
        return warehouseCode;
    }


    @Override
    public RecycledMaterialManage getByProduceBatchNo(String produceBatchNo) {
        return dao.getByProduceBatchNo(produceBatchNo);
    }

    /**
     * 废料入库到线边仓(并线边仓退料到wms)
     *
     * @param recycledMaterialManage
     * @param reason
     * @return
     */
    @Override
    public String startSurplusToWMS(RecycledMaterialManage recycledMaterialManage, String reason) {
        try {
            //通过计划工单中的片区field4找到字典中的成品仓
            String warehouseCode = getWareHouseCodeByWorkOrderNo(recycledMaterialManage.getWorkOrderNo());
            //调线边仓,废料成品入库(线边仓退料到wms)
            WarehouseInputBillBo warehouseInputBillBo = newFlInputBill(recycledMaterialManage, reason, warehouseCode);
            Result<String> result = storeFeignClients.inputApply(warehouseInputBillBo);
            Integer code = result.getCode();
            String data = result.getData();
            if (code != 200 || StringUtils.isBlank(data)) {
                throw new BusinessException("废料入库失败，状态码：" + code + "，反馈信息：" + result.getMessage());
            }
            return data;
        } catch (Exception e) {
            throw new BusinessException("废料入库到线边仓并由线边仓退库到wms异常，" + e.getMessage());
        }
    }


    @SneakyThrows
    @Override
    public String importExcel(MultipartFile file) {
        Map<String, List<Map<String, Object>>> excelMap = null;
        try {
            excelMap = ExcelUtil.getExcelInfo(file);
        } catch (Exception e) {
            throw new RuntimeException("文件导入异常！" + e.getMessage());
        }
        StringBuilder stringBuilder=new StringBuilder();
        if (!excelMap.isEmpty()) {
            Collection<List<Map<String, Object>>> collection = excelMap.values();
            Iterator<List<Map<String, Object>>> iterator = collection.iterator();
            while (iterator.hasNext()) {
                List<Map<String, Object>> list = iterator.next();
                //将读取到的list做去空处理，解决因为表格编辑软件、版本不同造成的问题
                list.removeIf(map -> "null".equals(String.valueOf(map.get("序号"))) || "".equals(String.valueOf(map.get("序号"))));

                for (Map<String, Object> map : list) {
                    RecycledMaterialManage recycledMaterialManage = new RecycledMaterialManage();
                    String row = String.valueOf(map.get("序号"));
                    //当行号为空跳过这行
                    if ("null".equals(row) || "".equals(row)) {
                        stringBuilder.append("第"+ row +"行序号不能为空！");
//                        throw new RuntimeException("第"+ row +"行序号不能为空！");
                    }
                    if(StringUtils.isEmpty(String.valueOf(map.get("回收料类型")).trim())){
                        stringBuilder.append("第"+ row +"行 回收料类型不能为空！");
//                        throw new RuntimeException("第"+ row +"行 回收料类型不能为空！");
                    }
                    recycledMaterialManage.setRecycledMaterialType(String.valueOf(map.get("回收料类型")).trim());
                    if(StringUtils.isEmpty(String.valueOf(map.get("来源")).trim())){
                        stringBuilder.append("第"+ row +"行 来源不能为空！");
//                        throw new RuntimeException("第"+ row +"行 来源不能为空！");
                    }
                    recycledMaterialManage.setSource(String.valueOf(map.get("来源")).trim());
                    if(StringUtils.isEmpty(String.valueOf(map.get("产品类型")).trim())){
                        stringBuilder.append("第"+ row +"行 产品类型不能为空！");
//                        throw new RuntimeException("第"+ row +"行 产品类型不能为空！");
                    }
                    recycledMaterialManage.setProductionCategory(String.valueOf(map.get("产品类型")).trim());
                    if(StringUtils.isEmpty(String.valueOf(map.get("等级")).trim())){
                        stringBuilder.append("第"+ row +"行 等级不能为空！");
//                        throw new RuntimeException("第"+ row +"行 等级不能为空！");
                    }
                    recycledMaterialManage.setGrade(String.valueOf(map.get("等级")).trim());
                    if(StringUtils.isEmpty(String.valueOf(map.get("档位")).trim())){
                        stringBuilder.append("第"+ row +"行 档位不能为空！");
//                        throw new RuntimeException("第"+ row +"行 档位不能为空！");
                    }
                    recycledMaterialManage.setGear(String.valueOf(map.get("档位")).trim());
                    if(StringUtils.isEmpty(String.valueOf(map.get("厂区")).trim())){
                        stringBuilder.append("第"+ row +"行 厂区不能为空！");
//                        throw new RuntimeException("第"+ row +"行 厂区不能为空！");
                    }
                    recycledMaterialManage.setFactory(String.valueOf(map.get("厂区")).trim());
                    recycledMaterialManage.setStatus(1);
                    recycledMaterialManage.setCreateBy(BaseContextHandler.getUsername());
                    if(StringUtils.isEmpty(String.valueOf(map.get("组拖时间")).trim())){
                        stringBuilder.append("第"+ row +"行 组拖时间不能为空！");
//                        throw new RuntimeException("第"+ row +"行 组拖时间不能为空！");
                    }
                    try {
                        Date checkTime1 = DateUtil.toDate(String.valueOf(map.get("组拖时间")).trim(), "yyyy-MM-dd HH:mm:ss");
                        recycledMaterialManage.setCreateTime(new Date());
                    } catch (Exception e) {
                        stringBuilder.append("第"+ row +"行  时间格式错误！");
//                        throw new Exception("第"+ row +"行  时间格式错误！");
                    }
                    String lotNo = String.valueOf(map.get("炉次号")).trim();
                    if(StringUtils.isEmpty(lotNo)){
                       // stringBuilder.append("第"+ row +"行 炉次号不能为空！");
//                        throw new RuntimeException("第"+ row +"行 炉次号不能为空！");
                    }
                    //导入的炉次号，需要查询圆棒工单（【圆棒生产域—订单管理】生产工单(排程表) ）表炉次是否存在，不存在提示错误出来
                    ProductionSchedule longCrystal=productionScheduleService.findDataByLotNo(lotNo);
                    if(null==longCrystal){
                      //  stringBuilder.append("第"+ row +"行 炉次号"+ lotNo +"在圆棒工单中不存在！");
//                        throw new RuntimeException("第"+ row +"行 炉次号"+ lotNo +"在长晶棒检验中不存在！");
                    }
                    recycledMaterialManage.setLotNo(String.valueOf(map.get("炉次号")).trim());
                    if(StringUtils.isEmpty(String.valueOf(map.get("物料名称")).trim())){
                        stringBuilder.append("第"+ row +"行 物料名称不能为空！");
//                        throw new RuntimeException("第"+ row +"行 物料名称不能为空！");
                    }
                    recycledMaterialManage.setMaterialName(String.valueOf(map.get("物料名称")).trim());
                    String code = materialModelService.findCodeByName(String.valueOf(map.get("物料名称")).trim());
                    if (StringUtils.isEmpty(code)) {
                        stringBuilder.append("第" + row + "行 物料名称对应编号不存在！");
//                        throw new RuntimeException("第" + row + "行 物料名称对应编号不存在！");
                    }
                    recycledMaterialManage.setMaterialCode(code);
                    if (StringUtils.isEmpty(String.valueOf(map.get("重量")).trim())) {
                        stringBuilder.append("第" + row + "行 重量不能为空！");
//                        throw new RuntimeException("第" + row + "行 重量不能为空！");
                    }
                    try {
                        recycledMaterialManage.setWeight(new BigDecimal(String.valueOf(map.get("重量")).trim()).setScale(2, BigDecimal.ROUND_HALF_UP));
                    } catch (Exception e) {
                        e.printStackTrace();
                        stringBuilder.append("第" + row + "行 重量字段格式异常:"+String.valueOf(map.get("重量")).trim());
                    }
//                    recycledMaterialManage.setReceiver(String.valueOf(map.get("接收人")).trim());
//                    recycledMaterialManage.setReceiveTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(String.valueOf(map.get("接收时间")).trim()));
                    String sheetStock = String.valueOf(map.get("片料")).trim();
                    if (StringUtils.isEmpty(sheetStock) || "null".equalsIgnoreCase(sheetStock)) {
                        sheetStock = "";
                    } else if (!"片料".equals(sheetStock)) {
                        stringBuilder.append("不允许填写除片料和空的其他类型!");
//                        throw new BusinessException("不允许填写除片料和空的其他类型!");
                    }
                    recycledMaterialManage.setSheetStock(sheetStock);

//                    String boxNo = String.valueOf(map.get("料箱号")).trim();
//                    recycledMaterialManage.setBoxNo(boxNo);
//                    if (StringUtils.isEmpty(boxNo)) {
//                        stringBuilder.append("第" + row + "行 对应料箱号不存在！");
////                        throw new RuntimeException("第" + row + "行 物料名称对应编号不存在！");
//                    }
                    //生成托号
                    String trayNo = creatTrayNo(recycledMaterialManage, sheetStock);
                    RecycledMaterialManage entity = dao.findByTrayNo(trayNo);
                    if (null != entity) {
                        stringBuilder.append("第" + row + "行托号已存在");
//                        throw new RuntimeException("第" + row + "行托号已存在");
                    }
                    recycledMaterialManage.setTrayNo(trayNo);
                    if (StringUtils.isEmpty(String.valueOf(map.get("ERP工单")).trim())) {
                        stringBuilder.append("第" + row + "行 ERP工单不能为空！");
                    }
                    recycledMaterialManage.setErpOrder(String.valueOf(map.get("ERP工单")).trim());

                    if(StringUtils.isNotBlank(stringBuilder.toString())){
                        return stringBuilder.toString();
                    }
                    this.saveOrUpdate(recycledMaterialManage);
                }
            }
        }
        if(StringUtils.isNotBlank(stringBuilder.toString())){
            throw new RuntimeException(stringBuilder.toString());
        }
        return null;
    }

    @Override
    public void updateRecycledManageWeight(String materialBoxNo, BigDecimal weight) {
        dao.updateRecycledManageWeight(materialBoxNo,weight);
    }

    @Override
    public RecycledMaterialManage getByBoxNoAndStatus(String materialBoxNo) {
        return dao.getByBoxNoAndStatus(materialBoxNo);
    }


    @Override
    public RecycledMaterialManage getByBoxNo(String boxNo) {
        return dao.getByBoxNo(boxNo);
    }

    @Override
    public Result reportedErp(RecycledMaterialManageDto recycledMaterialManageDto) {
        Date date = new Date();
        RecycledMaterialManage recycledMaterialManage = get(recycledMaterialManageDto.getId());
        if (null == recycledMaterialManage) {
            return ResultGenerator.fail("回收料原信息不存在");
        }
//        // 已接收
//        if(StockStateEnum.RECEIVED.getCode() != recycledMaterialManage.getStatus()){
//            return ResultGenerator.fail(recycledMaterialManage.getSource() + "回收料当前不处于已接收状态,不能进行上报ERP操作：" + recycledMaterialManage.getTrayNo());
//        }
        // 待出库
//        if(!StockStateEnum.TOBEOUT.getCode().equals(recycledMaterialManage.getStatus())){
//            return ResultGenerator.fail(recycledMaterialManage.getSource() + "回收料当前不处于待出库状态,不能进行上报ERP操作：" + recycledMaterialManage.getTrayNo());
//        }
        if(null == recycledMaterialManage.getReportedState() || EnumValues.ReportedErp.未上报.getKey() != recycledMaterialManage.getReportedState()){
            return ResultGenerator.fail(recycledMaterialManage.getSource() + "回收料当前不处于未上报状态,不能进行上报ERP操作：" + recycledMaterialManage.getTrayNo());
        }
        if(recycledMaterialManage.getWeight().doubleValue() < 0){
            return ResultGenerator.fail(recycledMaterialManage.getSource() + "回收料当前重量小于0,不能进行上报ERP操作：" + recycledMaterialManage.getTrayNo());
        }
        // todo 上报成功后，更改上报状态

        MesToErpCcpEntity mesToErpCcpEntity = new MesToErpCcpEntity();

        // "cbatch": "(2)2023-8-16/129BP(A)正泰045", --批号
        mesToErpCcpEntity.setCbatch(recycledMaterialManage.getProduceBatchNo());
//        mesToErpCcpEntity.setCbatch(recycledMaterialManage.getMaterialCode() + "HSL");
        // "iquantity": "1.0000000000", --数量	同种订单产品合并汇总后的数量
        mesToErpCcpEntity.setIquantity(String.valueOf(recycledMaterialManage.getWeight()));
        // "cinvcode": "100100180103",  回收料中的物料编号
        mesToErpCcpEntity.setCinvcode(recycledMaterialManage.getMaterialCode());
        if("单晶".equals(recycledMaterialManageDto.getSource())) {
            mesToErpCcpEntity.setErpType(ErpPypeEnum.单晶回收料.getKey());
            // "cwhcode": "315", --仓库编码	从订单，常规：315，研发：316
            mesToErpCcpEntity.setCwhcode("029");
            recycledMaterialManage.setErpWarehouseCode("029");
            // "crdcode": "10216", --出库类别/收发类别	常规：10216，研发：10201
            mesToErpCcpEntity.setCrdcode("10220");

            // "cwhname": "回收原料库", --仓库名称  todo 无
            mesToErpCcpEntity.setCwhname("回收料入库单库位");

            ProductionOrder productionOrder = productionOrderService.findByAufnr(recycledMaterialManageDto.getErpOrder());
            if(null != productionOrder){
                // "iproorderid": "1000009317", --生产订单id	从订单获取
                mesToErpCcpEntity.setIproorderid(productionOrder.getTid());

                // "cmpocode": "2308011408-12吋(BNE01A1)(正泰045)", --订单号     合并汇总用的订单：erpOrderNo
                mesToErpCcpEntity.setCmpocode(productionOrder.getAufnr());
                if(productionOrder.getField1().equals("一车间")){
                    mesToErpCcpEntity.setCdepcode("1201");
                } else if (productionOrder.getField1().equals("二车间")){
                    mesToErpCcpEntity.setCdepcode("1206");
                }
//                if(StringUtils.isEmpty(productionOrder.getField1())){
//                    String substring = productionOrder.getAufnr().substring(10, 11);
//                    if("5".equals(substring)){
//                        // "cdepname": "开方五车间",--部门名称	从订单获取   todo
//                        mesToErpCcpEntity.setCdepname("单晶五车间（三期）");
//                        // "cdepcode": "1413", --部门编码	从订单获取
//                        mesToErpCcpEntity.setCdepcode("1413");
//                    } else {
//                        mesToErpCcpEntity.setCdepname("单晶六车间（三期）");
//                        // "cdepcode": "1413", --部门编码	从订单获取
//                        mesToErpCcpEntity.setCdepcode("1417");
//                    }
//                } else {
//                    if("五车间".equals(productionOrder.getField1())){
//                        // "cdepname": "开方五车间",--部门名称	从订单获取   todo
//                        mesToErpCcpEntity.setCdepname("单晶五车间（三期）");
//                        // "cdepcode": "1413", --部门编码	从订单获取
//                        mesToErpCcpEntity.setCdepcode("1413");
//                    } else {
//                        mesToErpCcpEntity.setCdepname("单晶六车间（三期）");
//                        // "cdepcode": "1413", --部门编码	从订单获取
//                        mesToErpCcpEntity.setCdepcode("1417");
//                    }
//                }
                List<MaterialModel> materialModelByCode = materialModelService.findMaterialModelByCode(productionOrder.getMatnr());
                if(!materialModelByCode.isEmpty()){
                    MaterialModel materialModel = materialModelByCode.get(0);
                    // 材料名称  产成品使用母建，回收料使用子健 新增  cinvname
                    mesToErpCcpEntity.setCinvname(materialModel.getName());
                    //  单位 产成品使用母建，回收料使用子健 新增   cinvm_unit
                    mesToErpCcpEntity.setCinvm_unit(materialModel.getUnit().getCode());
                }
                List<ErpOrder> byModIdAndInvCode1 = erpOrderService.findByModIdAndInvCode1(productionOrder.getModid(), productionOrder.getAufnr());
                if(!byModIdAndInvCode1.isEmpty()){
                    ErpOrder erpOrder = byModIdAndInvCode1.get(0);
                    // 生产订单行号,目前填写1 产成品使用母建，回收料使用子健 新增  imoseq
                    mesToErpCcpEntity.setImoseq(NumberUtil.object2String(erpOrder.getImoseq(),null));
                    // 子建行号,目前填写1 产成品使用母建，回收料使用子健 新增    irowno
                    mesToErpCcpEntity.setIrowno(NumberUtil.object2String(erpOrder.getIrowno(),null));
                }
                // 规格型号 产成品使用母建，回收料使用子健 新增 cinsted
//            mesToErpCcpEntity.setCinsted(fbProductionOrder.get);

                // 应入库数量，取订单数量 产成品使用母建，回收料使用子健 新增 inquantity todo 不确定
                mesToErpCcpEntity.setInquantity(NumberUtil.object2String(productionOrder.getMenge(),null));

                // 入库记录ID 先不进行填写进行调试 产成品使用母建，回收料使用子健 新增 impoids todo
//            mesToErpCcpEntity.setImpoids();

                // 是否关联副产品 先不进行填写进行调试 产成品使用母建，回收料使用子健 新增    brelated
//            mesToErpCcpEntity.setBre  lated();

                // 生产订单号 产成品使用母建，回收料使用子健 新增     cmocode
                mesToErpCcpEntity.setCmocode(productionOrder.getAufnr());
            }
        }   else if("截断及开方".equals(recycledMaterialManageDto.getSource())) {
            mesToErpCcpEntity.setErpType(ErpPypeEnum.机加回收料.getKey());
            FbProductionOrderDto fbProductionOrderDto = new FbProductionOrderDto();
            fbProductionOrderDto.setAufnr(recycledMaterialManageDto.getErpOrder());
            Result<FbProductionOrderDto> result = fbFeignClients.findByAufnr(fbProductionOrderDto);
            FbProductionOrderDto fbProductionOrderDto1 = null;
            if(null != result && ResultCode.SUCCESS.getCode() == result.getCode() && null != result.getData()){
                fbProductionOrderDto1 = result.getData();
            }
            // "cwhcode": "315", --仓库编码	从订单，常规：315，研发：316
            mesToErpCcpEntity.setCwhcode("013");
            recycledMaterialManage.setErpWarehouseCode("013");
            // "crdcode": "10216", --出库类别/收发类别	常规：10216，研发：10201
            mesToErpCcpEntity.setCrdcode("10204");
            // "cdepcode": "1413", --部门编码	从订单获取
            mesToErpCcpEntity.setCdepcode("1202");
            // "cwhname": "回收原料库", --仓库名称
            mesToErpCcpEntity.setCwhname("开方产成品库（三期）");
            // "cdepname": "开方五车间",--部门名称	从订单获取
            mesToErpCcpEntity.setCdepname("开方一车间");
            if(null != fbProductionOrderDto1){
                // "iproorderid": "1000009317", --生产订单id	从订单获取 todo
//                mesToErpCcpEntity.setIproorderid(fbProductionOrderDto1.gett);
                // "cmpocode": "2308011408-12吋(BNE01A1)(正泰045)", --订单号     合并汇总用的订单：erpOrderNo
                mesToErpCcpEntity.setCmpocode(fbProductionOrderDto1.getAufnr());


                List<MaterialModel> materialModelByCode = materialModelService.findMaterialModelByCode(fbProductionOrderDto1.getMatnr());
                if(!materialModelByCode.isEmpty()){
                    MaterialModel materialModel = materialModelByCode.get(0);
                    // 材料名称  产成品使用母建，回收料使用子健 新增  cinvname
                    mesToErpCcpEntity.setCinvname(materialModel.getName());
                    //  单位 产成品使用母建，回收料使用子健 新增   cinvm_unit
                    mesToErpCcpEntity.setCinvm_unit(materialModel.getUnit().getCode());
                }
                List<ErpOrder> byModIdAndInvCode1 = erpOrderService.findByModIdAndInvCode1(fbProductionOrderDto1.getModid(), fbProductionOrderDto1.getAufnr());
                if(!byModIdAndInvCode1.isEmpty()){
                    ErpOrder erpOrder = byModIdAndInvCode1.get(0);
                    // 生产订单行号,目前填写1 产成品使用母建，回收料使用子健 新增  imoseq
                    mesToErpCcpEntity.setImoseq(NumberUtil.object2String(erpOrder.getImoseq(),null));
                    // 子建行号,目前填写1 产成品使用母建，回收料使用子健 新增    irowno
                    mesToErpCcpEntity.setIrowno(NumberUtil.object2String(erpOrder.getIrowno(),null));
                    // todo 过滤子件料号
                }
                // 规格型号 产成品使用母建，回收料使用子健 新增 cinsted
//            mesToErpCcpEntity.setCinsted(fbProductionOrder.get);

                // 应入库数量，取订单数量 产成品使用母建，回收料使用子健 新增 inquantity todo 不确定
                mesToErpCcpEntity.setInquantity(NumberUtil.object2String(fbProductionOrderDto1.getMenge(),null));

                // 入库记录ID 先不进行填写进行调试 产成品使用母建，回收料使用子健 新增 impoids todo
//            mesToErpCcpEntity.setImpoids();

                // 是否关联副产品 先不进行填写进行调试 产成品使用母建，回收料使用子健 新增    brelated
//            mesToErpCcpEntity.setBrelated();

                // 生产订单号 产成品使用母建，回收料使用子健 新增     cmocode
                mesToErpCcpEntity.setCmocode(fbProductionOrderDto1.getAufnr());
            }
        }
        mesToErpCcpEntity.setDocTime(recycledMaterialManage.getUpdateTime());
        ErpRequestDto erpRequestDto = new ErpRequestDto();
        erpRequestDto.setMesToErpCcpEntity(mesToErpCcpEntity);
        erpRequestDto.setUrl(erpServiceUrl);
        ErpAddResultDto erpAddResultDto = erpRequestUtil.postToErp(erpRequestDto);
        if(null == erpAddResultDto || !erpAddResultDto.getErrcode().equals(String.valueOf(ResultCode.SUCCESS.getCode()))){
            throw new RuntimeException("回收料上报失败：" + JSONObject.toJSONString(erpAddResultDto.getErrmsg()));
        }

        recycledMaterialManage.setReportedState(EnumValues.ReportedErp.已上报.getKey());
        recycledMaterialManage.setErpOrder(erpAddResultDto.getCcode());
        recycledMaterialManage.setErpTime(date);
        saveOrUpdate(recycledMaterialManage);
        return ResultGenerator.success();
    }

    /**
     * 生成托号
     *
     * @param recycledMaterialManageDto
     * @param sheetStock                片料
     * @return
     */
    private String creatTrayNo(RecycledMaterialManage recycledMaterialManageDto, String sheetStock) {
        //厂区车间
        //String factory = recycledMaterialManageDto.getFactory();
        //todo:三期暂时写死 3
        String factory = "3";
        if (com.sw.mes.core.utils.StringUtils.isNotBlank(recycledMaterialManageDto.getProductionCategory())) {
            factory = factory + "-" + recycledMaterialManageDto.getProductionCategory();
        }
        String today = LocalDate.now().toString();
        String key = "(" + factory + ")" + today;
        Integer times = this.countTimeByParam(key) + 1;
        String materialName = recycledMaterialManageDto.getMaterialName();
        String grade = recycledMaterialManageDto.getGrade();
        if (com.sw.mes.core.utils.StringUtils.isNotBlank(recycledMaterialManageDto.getGear())) {
            grade = grade + "-" + recycledMaterialManageDto.getGear();
        }
        String materialTypeValue = null;
        if (com.sw.mes.core.utils.StringUtils.isBlank(materialName)) {
            throw new BusinessException("请输入物料名称!");
        } else {
            materialTypeValue = dictionaryItemService.findValueByThemeValueAndItemName("ReduceTypeName", materialName);
        }
        String trayNo = key + "/" + times + materialTypeValue + "(" + grade + ")" + sheetStock;

        return trayNo;
    }

    private RecycledMaterialInDto buildNonRecycledMaterialInDto(RecycledMaterialManageDto recycledMaterialManageDto) {
        RecycledMaterialInDto recycledMaterialInDto = new RecycledMaterialInDto();
        recycledMaterialInDto.setBatchNo(recycledMaterialManageDto.getTrayNo());
        recycledMaterialInDto.setInQty(recycledMaterialManageDto.getWeight());
        recycledMaterialInDto.setLevelID(recycledMaterialManageDto.getGrade());
        recycledMaterialInDto.setMaterialClassID("2");
        recycledMaterialInDto.setMaterialNo(recycledMaterialManageDto.getMaterialCode());
        recycledMaterialInDto.setOperateDT(new SimpleDateFormat("yyyy-MM-dd").format(new Date()));
        recycledMaterialInDto.setOperator(BaseContextHandler.getUsername());
        recycledMaterialInDto.setOutQty(new BigDecimal("0"));
        recycledMaterialInDto.setPlanNo(recycledMaterialManageDto.getTrayNo());
        recycledMaterialInDto.setRemark(null);
        recycledMaterialInDto.setSpecification(recycledMaterialManageDto.getSpecification());
        recycledMaterialInDto.setUnit(recycledMaterialManageDto.getUnitName());
        return recycledMaterialInDto;
    }

    private RecycledMaterialInDto buildRecycledMaterialInDto(RecycledMaterialManageDto recycledMaterialManageDto) {
        RecycledMaterialInDto recycledMaterialInDto = new RecycledMaterialInDto();
        recycledMaterialInDto.setBatchNo(recycledMaterialManageDto.getTrayNo());
        recycledMaterialInDto.setInQty(recycledMaterialManageDto.getWeight());
        recycledMaterialInDto.setLevelID(recycledMaterialManageDto.getGrade());
        if(recycledMaterialManageDto.getRecycledMaterialType().equals("可用回收料")){
            recycledMaterialInDto.setMaterialClassID("3");
        }else{
            recycledMaterialInDto.setMaterialClassID("1");
        }
        recycledMaterialInDto.setMaterialNo(recycledMaterialManageDto.getMaterialCode());
        recycledMaterialInDto.setOperateDT(new SimpleDateFormat("yyyy-MM-dd").format(new Date()));
        recycledMaterialInDto.setOperator(BaseContextHandler.getUsername());
        recycledMaterialInDto.setOutQty(recycledMaterialManageDto.getWeight());
        recycledMaterialInDto.setPlanNo(recycledMaterialManageDto.getTrayNo());
        recycledMaterialInDto.setRemark(null);
        recycledMaterialInDto.setSpecification(recycledMaterialManageDto.getSpecification());
        recycledMaterialInDto.setUnit(recycledMaterialManageDto.getUnitName());
        return recycledMaterialInDto;
    }

    private ResultWms requestRecycleMaterialToWMS(String url, RecycledMaterialInDto recycledMaterialInDto, String requestName) {
        String resultJson;
        boolean isErr = false;
        try {
            HttpHeaders headers = new HttpHeaders();
            MediaType mediaType = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(mediaType);
            headers.add("Accept", MediaType.APPLICATION_JSON.toString());
            List<RecycledMaterialInDto> recycledMaterialInDtos =new ArrayList<>();
            recycledMaterialInDtos.add(recycledMaterialInDto);
            HttpEntity<List<RecycledMaterialInDto>> httpEntity = new HttpEntity<>(recycledMaterialInDtos, headers);
            ResponseEntity<String> responseEntity = restTemplate.postForEntity(url, httpEntity, String.class);
            resultJson = responseEntity.getBody();
        } catch (Exception e){
            e.printStackTrace();
            resultJson = e.getMessage();
            isErr = true;
        }
        try{
            SysLog syslog = new SysLog();
            syslog.setCreateTime(new Date());
            syslog.setType("info");
            syslog.setCreateUserId(BaseContextHandler.getUserID());
            syslog.setCreateUserName("SYSTEM");
            syslog.setContent(JSON.toJSONString(recycledMaterialInDto));
            syslog.setResult(resultJson);
            syslog.setMethodName("requestRecycleMaterialToWMS()");
            syslog.setStatus(1);
            syslog.setName(requestName);
            syslog.setServerAddr(url);
            syslog.setReqType("HTTP");
            syslog.setReqIp(IpUtils.getLocalIP());
            sysLogService.save(syslog);
            //rabbitMqService.sendMessage(JSONObject.toJSONString(syslog), ExchangeEnum.SYS_LOG_EXCHANGE, QueueEnum.SYS_LOG_QUEUE, RoutingKey.SYS_LOG_KEY);
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (isErr){
            return null;
        }else {
            return JSON.parseObject(resultJson, new TypeReference<ResultWms>(){});
        }


    }


    /**
     * 封装 回收料入库信息
     *
     * @param manage 回收料信息
     * @param reason 入库原因
     * @param warehouseCode 库位编码
     * @return
     */
    private WarehouseInputBillBo newInputBill(RecycledMaterialManage manage, String reason,String warehouseCode) {
        WarehouseInputBillBo inputBill = new WarehouseInputBillBo();
        inputBill.setType(StockStateEnum.余料入库.getDesc());
        inputBill.setRemark(reason);
        inputBill.setWarehouseCode(warehouseCode);
        inputBill.setAutoAudit(true);
        inputBill.setAutomaticOut(false);
        //入库物料明细
        List<WarehouseInputBillDetailBo> inputBillDetails = new ArrayList<>();
        WarehouseInputBillDetailBo dto = new WarehouseInputBillDetailBo();

        //组装物料明细数据
        dto.setBatchNo(manage.getProduceBatchNo());
        dto.setSupplier("太原晶科");
        dto.setSupplierCode("TYJK");
        dto.setCount(manage.getWeight());
        //字段需要与物料主数据对齐
        List<MaterialModel> materialModelList = materialModelService.findMaterialModelByCode(manage.getMaterialCode());
        if (null == materialModelList || materialModelList.size() == 0) {
            throw new BusinessException("该料号不存在，请检查");
        }
        MaterialModel materialModel = materialModelList.get(0);

        dto.setMaterialModelId(Long.valueOf(materialModel.getId()));
        dto.setMaterialModelCode(materialModel.getCode());
        dto.setMaterialModelName(materialModel.getName());
        dto.setMaterialTypeName(materialModel.getMaterialType().getName());
        dto.setMaterialTh(materialModel.getNumberOfDrawings());
        dto.setUnitName(manage.getUnitName());
        dto.setSpecification(materialModel.getSpecification());
        dto.setBoxNo(manage.getBoxNo());
        dto.setWeight(manage.getWeight());
        dto.setMaterialGear(manage.getGear());
        dto.setMaterialModel(manage.getConductionType());
        dto.setWorkOrderNo(manage.getWorkOrderNo());
        dto.setWarehouseCode(warehouseCode);
        inputBillDetails.add(dto);
        inputBill.setInputBillDetails(inputBillDetails);
        return inputBill;
    }


    /**
     * 废料成品入库(线边仓退料到wms)
     *
     * @param entity
     * @param reason
     * @param warehouseCode
     * @return
     */
    private WarehouseInputBillBo newFlInputBill(RecycledMaterialManage entity, String reason, String warehouseCode) {
        WarehouseInputBillBo inputBill = new WarehouseInputBillBo();
        inputBill.setType(StockStateEnum.废料入库.getDesc());
        inputBill.setRemark(reason);
        inputBill.setAutoAudit(true);
        inputBill.setAutomaticOut(false);
        inputBill.setWarehouseCode(warehouseCode);
        inputBill.setCreateUserId(BaseContextHandler.getUserID());
        inputBill.setCreateUserName(BaseContextHandler.getUserClassNo());
        //入库物料明细
        List<WarehouseInputBillDetailBo> inputBillDetails = new ArrayList<>();
        WarehouseInputBillDetailBo dto = new WarehouseInputBillDetailBo();

        //组装物料明细数据
        //暂时使用固定编码,待后期业务完善
        dto.setSupplier("太原晶科");
        dto.setSupplierCode("TYJK");
        dto.setBatchNo(entity.getProduceBatchNo());
        dto.setWarehouseCode(warehouseCode);
        dto.setCount(entity.getWeight());
        //字段需要与物料主数据对齐
        //字段需要与物料主数据对齐
        MaterialModel materialModel = materialModelService.findByCode(entity.getMaterialCode());
        if (materialModel == null) {
            throw new BusinessException("【" + entity.getMaterialCode() + "】物料主数据不存在，请检查数据！");
        }
        dto.setMaterialModelId(Long.valueOf(materialModel.getId()));
        dto.setMaterialModelCode(materialModel.getCode());
        dto.setMaterialModelName(materialModel.getName());
        dto.setMaterialTypeName(materialModel.getMaterialType().getName());
        dto.setMaterialTh(materialModel.getNumberOfDrawings());
        dto.setUnitName(materialModel.getUnit().getName());
        dto.setSpecification(materialModel.getSpecification());
        dto.setWorkOrderNo(entity.getWorkOrderNo());
        dto.setMaterialGear(entity.getGear());
        dto.setMaterialLevel(entity.getGrade());
        dto.setSpecification(entity.getSpecification());
        dto.setTraceid(entity.getTrayNo());
        dto.setSerialNo(entity.getTrayNo());
        //todo:主数据中无等级,待下发后再判断 AB : N,CD:Y;
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Date currentTime = new Date();
        String currentTimeString = formatter.format(currentTime);
        dto.setLotatt01(currentTimeString);
        dto.setLotatt08("N");
        dto.setLotatt12(entity.getProduceBatchNo());
        dto.setLotatt19("00");
        inputBillDetails.add(dto);
        inputBill.setAutomaticOut(false);
        inputBill.setInputBillDetails(inputBillDetails);
        return inputBill;
    }

    /**
     * 余料收集,入库成功后副产品报内部系统,供上报sap使用
     *
     * @param workOrderNo    工单
     * @param materialCode   物料编码
     * @param produceBatchNo 产品批次
     * @param weight         数量(重量)
     * @param unitName       单位(KG,PC)
     * @param warehouseCode       库存地点编码
     */
    @Jinko(author = "yyl", value = "余料收集,入库成功后副产品报内部系统,供上报sap使用")
    private void handleReportSapGoodsmvtDto(String workOrderNo, String materialCode, String produceBatchNo, BigDecimal weight, String unitName,String warehouseCode) {
        ReportSapGoodsmvtDto reportSapGoodsmvtDto = new ReportSapGoodsmvtDto();
        List<ReportSapChargItemDto> reportSapChargItemDtos = new ArrayList<>();
        ReportSapChargItemDto reportSapChargItemDto = new ReportSapChargItemDto();
//        reportSapChargItemDto.setAtnam("P01");
//        reportSapChargItemDto.setAtwrt("JK1");
        reportSapChargItemDtos.add(reportSapChargItemDto);
        reportSapGoodsmvtDto.setChargItems(reportSapChargItemDtos);
        reportSapGoodsmvtDto.setAufnr(workOrderNo);
        reportSapGoodsmvtDto.setCharg(produceBatchNo);
        reportSapGoodsmvtDto.setBwart("531");
        reportSapGoodsmvtDto.setWerks("5041");
        reportSapGoodsmvtDto.setMatnr(materialCode);
        reportSapGoodsmvtDto.setMenge(weight == null ? "0" : weight.toString());
        reportSapGoodsmvtDto.setMeins(unitName);
        reportSapGoodsmvtDto.setLgort(warehouseCode);
        reportSapGoodsmvtDto.setZfcpFlag("X");
        ReportSapGoodsmvt reportSapGoodsmvt = fromDto(reportSapGoodsmvtDto);
        reportSapGoodsmvt.setStatus(false);
        reportSapGoodsmvt.setProcName("余料");
        reportSapGoodsmvtService.saveOrUpdate(reportSapGoodsmvt);
    }

    public ReportSapGoodsmvt fromDto(ReportSapGoodsmvtDto dto) {
        ReportSapGoodsmvt reportSapGoodsmvt = modelMapper.map(dto, ReportSapGoodsmvt.class);
        reportSapGoodsmvt.setChargItem(new Gson().toJson(dto.getChargItems()));
        return reportSapGoodsmvt;
    }
}
