package com.sw.mes.yb.production.service.impl;


import com.sw.mes.core.base.BaseServiceImpl;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.yb.plan.entity.ProductionOrder;
import com.sw.mes.yb.plan.entity.ProductionSchedule;
import com.sw.mes.yb.plan.service.ProductionScheduleService;
import com.sw.mes.yb.production.dao.SurplusInfoDao;
import com.sw.mes.yb.production.dto.SurplusInfoDto;
import com.sw.mes.yb.production.entity.SurplusInfo;
import com.sw.mes.yb.production.service.SurplusInfoService;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@Service
@Transactional
public class SurplusInfoServiceImpl extends BaseServiceImpl<SurplusInfo> implements SurplusInfoService {
    SurplusInfoDao dao;

    @Autowired
    public void setDao(SurplusInfoDao dao) {
        super.setBaseDao(dao);
        this.dao = dao;
    }

    @Autowired
    ModelMapper modelMapper;

    @Autowired
    ProductionScheduleService productionScheduleService;

    @Override
    public SurplusInfo queryLimitInfoByDeviceNo(String deviceNo) {
        return dao.queryLimitInfoByDeviceNo(deviceNo);
    }

    @Override
    public SurplusInfo queryLimitInfoBySurplusCode(String surplusCode) {
        return dao.queryLimitInfoBySurplusCode(surplusCode);
    }

    @Override
    public SurplusInfo queryLimitInfoByLotNo(String lotNo) {
        return dao.queryLimitInfoByLotNo(lotNo);
    }

    @Override
    public void saveData(SurplusInfoDto surplusInfoDto) {
        String deviceNo = surplusInfoDto.getDeviceNo();
        ProductionSchedule productionSchedule = productionScheduleService.findByEquipmentCode(deviceNo);
        if (productionSchedule == null) {
            throw new BusinessException("该炉台当前无排程工单!");
        }
        ProductionOrder order = productionSchedule.getProductionOrder();
        String lotNo =productionSchedule.getLotNo();
        surplusInfoDto.setLotNo(lotNo);
        surplusInfoDto.setWorkOrderNo(order.getAufnr());
        Integer againNum = surplusInfoDto.getFeedTimes();
        String surplusType = surplusInfoDto.getSurplusType();
        String workOrderNo = surplusInfoDto.getWorkOrderNo();
        String surplusCode = lotNo + againNum + surplusType + workOrderNo;
        surplusInfoDto.setSurplusCode(surplusCode);
        surplusInfoDto.setStatus(1);
        surplusInfoDto.setCreateBy(BaseContextHandler.getUsername());
        surplusInfoDto.setCreateTime(new Date());
        surplusInfoDto.setDataType(2);
        surplusInfoDto.setDel(0);
        surplusInfoDto.setArea(order.getField4());
        SurplusInfo surplusInfo = fromDto(surplusInfoDto);
        dao.save(surplusInfo);
    }


    public SurplusInfo fromDto(SurplusInfoDto dto) {
        return modelMapper.map(dto, SurplusInfo.class);
    }
}
