package com.sw.mes.yb.seed.api.service.consumer.impl;

import com.sw.mes.base.factory.material.service.UnitService;
import com.sw.mes.base.factory.product.entity.ProductModel;
import com.sw.mes.base.factory.product.service.ProductModelService;
import com.sw.mes.core.api.system.StoreFeignClients;
import com.sw.mes.core.api.system.YbFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.wms.MesasnDto;
import com.sw.mes.core.dto.wms.MesasnOrderDto;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.yb.seed.api.service.consumer.SeedConsumerService;
import com.sw.mes.yb.seed.entity.SeedGroupBracket;
import com.sw.mes.yb.seed.entity.SeedGroupBracketDetails;
import com.sw.mes.yb.seed.enums.InputBillStatusEnum;
import com.sw.mes.yb.seed.service.SeedGroupBracketDetailsService;
import com.sw.mes.yb.seed.service.SeedGroupBracketService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class SeedConsumerServiceImpl implements SeedConsumerService {
    @Autowired
    private SeedGroupBracketService seedGroupBracketService;
    @Autowired
    private SeedGroupBracketDetailsService seedGroupBracketDetailsService;
    @Autowired
    private ProductModelService productModelService;
    @Autowired
    private UnitService unitService;
    @Autowired
    private StoreFeignClients storeFeignClients;
    @Autowired
    private YbFeignClients ybFeignClients;
    @Override
    public Result wmsInitiate(List<Long> ids) {
        SimpleDateFormat sdf = new SimpleDateFormat();
        //查询组托数据
        List<SeedGroupBracket> seedGroupBrackets = seedGroupBracketService.findByIds(ids);
        List<MesasnDto> mesasnDtos = new ArrayList<>();
        seedGroupBrackets.stream().forEach(seedGroupBracket->{
            if(!ObjectUtils.isEmpty(seedGroupBracket)&&seedGroupBracket.getStatus().equals("已入库")){
                //查询明细数据
                List<SeedGroupBracketDetails> seedGroupBracketDetails = seedGroupBracketDetailsService.findParentId(seedGroupBracket.getId());
                //查询对应的主数据
                ProductModel productModel = productModelService.findByCode(seedGroupBracket.getMaterialCode());
                //组装对应数据
                MesasnDto mesasnDto = new MesasnDto();
                mesasnDto.setOrganizationid("JINKO");
                mesasnDto.setWarehouseid("5041");
                mesasnDto.setCustomerid("5041");
                mesasnDto.setDocno(seedGroupBracket.getInputBillCode());
                mesasnDto.setAsntype("籽晶入库");
                mesasnDto.setAsnstatus("OK");
//                mesasnDto.setAsnreferenceb();//车间编号
                mesasnDto.setAsnreferencec("P501");
                mesasnDto.setAsncreationtime(sdf.format(seedGroupBracket.getCreateDate()));
                mesasnDto.setTraceid(seedGroupBracket.getCode());//托盘号
                mesasnDto.setSerialno(seedGroupBracket.getCode());//箱号
                mesasnDto.setExpectedqty(String.valueOf(seedGroupBracketDetails.size()));//组件数量
                mesasnDto.setLotatt01(sdf.format(new Date()));
                mesasnDto.setLotatt04(seedGroupBracketDetails.get(0).getBatchNo());//入库批次
                mesasnDto.setLotatt08("OK");//质量状态
//                mesasnDto.setLotatt09();//厂家
                mesasnDto.setLotatt12(seedGroupBracketDetails.get(0).getBatchNo());//生产批次
                if(!ObjectUtils.isEmpty(productModel)){
                    mesasnDto.setLotatt19(productModel.getResistivity());//电阻率
                    mesasnDto.setLotatt20(productModel.getProductGrade());//等级
                }
                mesasnDto.setSku(seedGroupBracket.getMaterialCode());
                mesasnDto.setCreatesource("MES");
                List<MesasnOrderDto> mesasnOrderDtos = new ArrayList<>();
                seedGroupBracketDetails.stream().forEach(detais->{
                    MesasnOrderDto mesasnOrderDto = new MesasnOrderDto();
                    mesasnOrderDto.setSku(seedGroupBracket.getInputBillCode());
                    mesasnOrderDto.setTraceid(detais.getSeedBatchNo());
                    mesasnOrderDto.setSerialno(detais.getSeedBatchNo());
                    mesasnOrderDto.setExpectedqty(String.valueOf(detais.getTheWeight()));//箱重量
                    mesasnOrderDto.setLotatt01(sdf.format(new Date()));
                    mesasnOrderDto.setLotatt04(detais.getSeedBatchNo());//入库批次
                    mesasnOrderDto.setLotatt08("OK");//质量状态
//                    mesasnOrderDto.setLotatt09();//厂家
                    mesasnOrderDto.setLotatt12(detais.getBatchNo());//生产批次
                    mesasnOrderDto.setLotatt19(productModel.getResistivity());//电阻率
                    mesasnOrderDto.setLotatt20(productModel.getProductGrade());//等级
                    mesasnOrderDto.setDedi07(String.valueOf(detais.getTotalLength()));//长度,整箱长度
                    mesasnOrderDto.setDedi08(detais.getSeedBatchNo());//批号,MES下线批
                    mesasnOrderDtos.add(mesasnOrderDto);
                });
                mesasnDto.setOrderList(mesasnOrderDtos);
                mesasnDtos.add(mesasnDto);
            }
        });
        //发起WMS入库
        Result result = ybFeignClients.mesasn(mesasnDtos);
        //根据返回状态修改组托状态
        if(!result.getCode().equals(200)){
            return ResultGenerator.fail("发起入库失败!");
        }else {
            seedGroupBrackets.stream().forEach(seedGroupBracket -> {
                seedGroupBracket.setInputBillCode(result.getData().toString());
                seedGroupBracket.setLaunchBy(BaseContextHandler.getUsername());
                seedGroupBracket.setLaunchDate(new Date());
                seedGroupBracket.setStatus(InputBillStatusEnum.待审核.getValue());
                seedGroupBracketService.saveOrUpdate(seedGroupBracket);
            });
        }
        return ResultGenerator.success();
    }
}
