package com.sw.mes.yb.seed.controller;

import com.alibaba.fastjson.JSONObject;
import com.sw.mes.base.factory.material.entity.MaterialType;
import com.sw.mes.base.factory.material.service.MaterialTypeService;
import com.sw.mes.base.factory.product.entity.ProductModel;
import com.sw.mes.core.api.system.FbFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.inspect.entity.YbInspectData;
import com.sw.mes.inspect.service.YbInspectDataService;
import com.sw.mes.yb.plan.entity.ProductionOrder;
import com.sw.mes.yb.plan.service.ProductionOrderService;
import com.sw.mes.yb.printer.dto.FbCheckOutDataDto;
import com.sw.mes.yb.seed.dto.SeedBatchesDto;
import com.sw.mes.yb.seed.dto.SeedDataDto;
import com.sw.mes.yb.seed.entity.SeedData;
import com.sw.mes.yb.seed.entity.SeedPlanOrder;
import com.sw.mes.yb.seed.enums.SeedMetaEnum;
import com.sw.mes.yb.seed.service.SeedPlanOrderService;
import com.sw.mes.yb.seed.service.SeedService;
import com.sw.mes.yb.seed.utils.SeedCodeGenerator;
import io.swagger.annotations.*;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 籽晶页面接口
 *
 * @Author：wanjia
 * @Date：2024/1/9 16:57
 */
@RestController
@RequestMapping("/seed")
@Api(description = "籽晶对象接口")
@Transactional
public class SeedDataController {

    @Autowired
    private SeedService seedService;
    @Autowired
    private SeedCodeGenerator seedCodeGenerator;
    @Autowired
    ModelMapper modelMapper;
    @Autowired
    private YbInspectDataService ybInspectDataService;
    @Autowired
    private FbFeignClients fbFeignClients;
    @Autowired
    private ProductionOrderService productionOrderService;
    @Autowired
    private MaterialTypeService materialTypeService;

    @ApiOperation(value = "分页查询籽晶信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<SeedDataDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<SeedData> specification = DynamicSpecifications.bySearchFilter(request, SeedData.class);
        Page<SeedData> seedData = seedService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<SeedDataDto> dtos = seedData.getContent().stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, seedData.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @GetMapping("/listAll")
    @ApiOperation(value = "查询籽晶信息列表")
    public Result<List<SeedDataDto>> listAll(ServletRequest request) {
        Specification<SeedData> specification = DynamicSpecifications.bySearchFilter(request, SeedData.class);
        List<SeedData> seedData = seedService.findByExampleSort(specification,null);
        List<SeedDataDto> dtos = seedData.stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        return ResultGenerator.success(dtos);
    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询籽晶详情")
    public Result<SeedDataDto> getById(@PathVariable @ApiParam(value = "id") Long id) {
        SeedData seedData = seedService.get(id);
        if(ObjectUtils.isEmpty(seedData)){
            return ResultGenerator.fail("未查到相关数据!");
        }
        return ResultGenerator.success(toEagerDto(seedData));
    }

    @PostMapping("/seedCode")
    @ApiOperation(value = "通过籽晶编码查询籽晶详情")
    public Result<SeedDataDto> getSeedCode(@PathVariable @ApiParam(value = "seedCode") String seedCode) {
        SeedData seedData = seedService.findByCode(seedCode);
        if(ObjectUtils.isEmpty(seedData)){
            return ResultGenerator.fail("未查到相关数据!");
        }
        return ResultGenerator.success(toEagerDto(seedData));
    }

    @PostMapping("/batchNo")
    @ApiOperation(value = "通过籽晶序列号查询籽晶详情")
    public Result<SeedDataDto> getBatchNo(@PathVariable @ApiParam(value = "batchNo") String batchNo) {
        SeedData seedData = seedService.findByBatchNo(batchNo);
        if(ObjectUtils.isEmpty(seedData)){
            return ResultGenerator.fail("未查到相关数据!");
        }
        return ResultGenerator.success(toEagerDto(seedData));
    }
    @ApiOperation(value = "创建籽晶对象")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "SeedDataDto的对象")SeedDataDto seedDataDto
            , BindingResult bindingResult){
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        SeedData seedDataCode = seedService.findByCode(seedDataDto.getSeedCode());
        if(!ObjectUtils.isEmpty(seedDataCode)){
            return ResultGenerator.fail("该籽晶对象已存在!");
        }
        SeedData seedBatchNo = seedService.findByBatchNo(seedDataDto.getSeedBatchNo());
        if(!ObjectUtils.isEmpty(seedBatchNo)){
            return ResultGenerator.fail("该籽晶对象已存在!");
        }
        //查询是否来源于圆棒
        Result dataByFbSectionNo = fbFeignClients.findDataByFbSectionNo(seedDataDto.getBatchNo());
        YbInspectData bySectionNo = new YbInspectData();
        FbCheckOutDataDto  fbCheckOutDataDto = new FbCheckOutDataDto();
        if(ObjectUtils.isEmpty(dataByFbSectionNo.getData())){
            bySectionNo = ybInspectDataService.findBySectionNo(seedDataDto.getBatchNo());
        }else {
            fbCheckOutDataDto = JSONObject.parseObject(JSONObject.toJSONString(dataByFbSectionNo.getData()), FbCheckOutDataDto.class);
        }
        SeedData seedData = fromDto(seedDataDto);
        //如果方棒没有查询到数据，其余一概默认来源圆棒
        if(ObjectUtils.isEmpty(fbCheckOutDataDto)){
            if(!ObjectUtils.isEmpty(bySectionNo)){
                seedData.setDopant(bySectionNo.getDopant());
            }
            seedData.setSource(SeedMetaEnum.SEED_INGOT.getOriginDesc());
        }else {
            seedData.setDopant(fbCheckOutDataDto.getDopant());
            seedData.setSource(SeedMetaEnum.SEED_BRICK.getOriginDesc());
        }
        seedData.setSeedCode(seedCodeGenerator.generator(seedData));
        seedData.setSeedBatchNo(seedCodeGenerator.batchNoGenerator(seedData));
        seedData.setCreateBy(BaseContextHandler.getUsername());
        seedData.setCreateDate(new Date());
        seedService.saveOrUpdate(seedData);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "籽晶分批接口")
    @PostMapping("/batches")
    public Result batches(@RequestBody @Valid @ApiParam(value = "SeedBatchesDto的对象") SeedBatchesDto seedBatchesDto
            , BindingResult bindingResult){
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if(ObjectUtils.isEmpty(seedBatchesDto)){
            return ResultGenerator.fail("请传入相关参数!");
        }
        if(seedBatchesDto.getCount()<1){
            return ResultGenerator.fail("分批数量不能小于1!");
        }
        SeedData byBatch = seedService.findByBatch(seedBatchesDto.getBatchNo());
        if(!ObjectUtils.isEmpty(byBatch)){
            return ResultGenerator.fail("该批次已分批,请勿再次分批!");
        }
        //查询是否来源于圆棒
        Result dataByFbSectionNo = fbFeignClients.findDataByFbSectionNo(seedBatchesDto.getBatchNo());
        //获取订单
        ProductionOrder productionOrder = productionOrderService.findByAufnr(seedBatchesDto.getPlanCode());
        YbInspectData bySectionNo = new YbInspectData();
        FbCheckOutDataDto  fbCheckOutDataDto = new FbCheckOutDataDto();
        if(ObjectUtils.isEmpty(dataByFbSectionNo.getData())){
            bySectionNo = ybInspectDataService.findBySectionNo(seedBatchesDto.getBatchNo());
        }else {
            fbCheckOutDataDto = JSONObject.parseObject(JSONObject.toJSONString(dataByFbSectionNo.getData()), FbCheckOutDataDto.class);
        }
        List<SeedDataDto> seedDataDtoList = new ArrayList<>();
        for(int i = 0; i < seedBatchesDto.getCount(); i++){
            SeedData seedData = new SeedData();
            SeedDataDto seedDataDto = new SeedDataDto();
            seedData.setBatchNo(seedBatchesDto.getBatchNo());
            seedData.setPlanCode(seedBatchesDto.getPlanCode());
            seedData.setWorkShopCode(seedBatchesDto.getWorkShopCode());
            seedData.setSeedCoringDevice(seedBatchesDto.getSeedCoringDevice());
            seedData.setSeedGrindingDevice(seedBatchesDto.getSeedGrindingDevice());
            seedData.setProcessLength(seedBatchesDto.getProcessLength());
            //如果方棒没有查询到数据，其余一概默认来源圆棒
            if(ObjectUtils.isEmpty(fbCheckOutDataDto)){
                if(!ObjectUtils.isEmpty(bySectionNo)){
                    seedData.setDopant(bySectionNo.getDopant());
                }
                seedData.setSource(SeedMetaEnum.SEED_INGOT.getOriginDesc());
            }else {
                seedData.setDopant(fbCheckOutDataDto.getDopant());
                seedData.setSource(SeedMetaEnum.SEED_BRICK.getOriginDesc());
            }
            if(!ObjectUtils.isEmpty(productionOrder)&&!ObjectUtils.isEmpty(productionOrder.getProductModel())){
                ProductModel productModel = productionOrder.getProductModel();
                MaterialType materialType = materialTypeService.get(productModel.getId());
                seedData.setMaterialCode(productModel.getCode());
                seedData.setMaterialId(productModel.getId());
                seedData.setSpec(productModel.getSpecs());
                seedData.setMaterialModel(productModel.getModelNumber());
                if(!ObjectUtils.isEmpty(materialType)){
                    seedData.setMaterialType(materialType.getName());
                }
                seedData.setMaterialName(productModel.getName());
            }
            seedData.setSeedCode(seedCodeGenerator.generator(seedData));
            seedData.setSeedBatchNo(seedCodeGenerator.batchNoGenerator(seedData));
            seedData.setCreateBy(BaseContextHandler.getUsername());
            seedData.setCreateDate(new Date());
            BeanUtils.copyProperties(seedData,seedDataDto,"id");
            seedDataDtoList.add(seedDataDto);
        }
        return ResultGenerator.success(seedDataDtoList);
    }
    @ApiOperation(value = "籽晶对象分批提交接口")
    @PostMapping("/submit")
    public Result submit(@RequestBody @Valid @ApiParam(value = "SeedDataDto的对象")List<SeedDataDto> seedDataDto
            , BindingResult bindingResult){
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        seedDataDto.stream().forEach(entity->{
            SeedData seedData = new SeedData();
            BeanUtils.copyProperties(entity,seedData,"id");
            seedService.saveOrUpdate(seedData);
        });
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新籽晶对象信息")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "SeedDataDto的对象")SeedDataDto seedDataDto
            , BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        SeedData seedDataCode = seedService.findByCode(seedDataDto.getSeedCode());
        SeedData seedBatchNo = seedService.findByBatchNo(seedDataDto.getSeedBatchNo());
        if(ObjectUtils.isEmpty(seedDataCode)&&ObjectUtils.isEmpty(seedBatchNo)){
            return ResultGenerator.fail("该籽晶对象不存在!");
        }
        SeedData seedData = fromDto(seedDataDto);
        if(!ObjectUtils.isEmpty(seedDataCode)){
            seedData.setId(seedDataCode.getId());
        }
        if(!ObjectUtils.isEmpty(seedBatchNo)){
            seedData.setId(seedBatchNo.getId());
        }
        seedData.setUpdateBy(BaseContextHandler.getUsername());
        seedData.setUpdateDate(new Date());
        seedService.saveOrUpdate(seedData);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "删除籽晶对象")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
        for (Long id : ids) {
            seedService.delete(id);
        }
        return ResultGenerator.success();
    }

    public SeedDataDto toLazyDto(SeedData entity) {
        return modelMapper.map(entity, SeedDataDto.class);
    }

    public SeedDataDto toEagerDto(SeedData entity) {
        SeedDataDto dto = modelMapper.map(entity, SeedDataDto.class);
        return dto;
    }

    public SeedData fromDto(SeedDataDto dto) {
        return modelMapper.map(dto, SeedData.class);
    }
    public SeedData fromBatchDto(SeedBatchesDto dto) {
        return modelMapper.map(dto, SeedData.class);
    }

}
