package com.sw.mes.yb.seed.controller;

import com.sw.mes.base.factory.material.entity.Unit;
import com.sw.mes.base.factory.material.service.UnitService;
import com.sw.mes.base.factory.product.entity.ProductModel;
import com.sw.mes.base.factory.product.service.ProductModelService;
import com.sw.mes.core.api.system.StoreFeignClients;
import com.sw.mes.core.api.system.YbFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.store.WarehouseInputBillBo;
import com.sw.mes.core.dto.store.WarehouseInputBillDetailBo;
import com.sw.mes.core.dto.wms.MesasnDto;
import com.sw.mes.core.dto.wms.MesasnOrderDto;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.seed.dto.SeedGroupBracketDetailsDto;
import com.sw.mes.yb.seed.dto.SeedGroupBracketDto;
import com.sw.mes.yb.seed.entity.SeedData;
import com.sw.mes.yb.seed.entity.SeedGroupBracket;
import com.sw.mes.yb.seed.entity.SeedGroupBracketDetails;
import com.sw.mes.yb.seed.enums.InputBillStatusEnum;
import com.sw.mes.yb.seed.service.SeedGroupBracketDetailsService;
import com.sw.mes.yb.seed.service.SeedGroupBracketService;
import com.sw.mes.yb.seed.service.SeedService;
import com.sw.mes.yb.seed.utils.CodeSender;
import io.swagger.annotations.*;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/seed/bracket")
@Api(description = "籽晶组托主表接口")
@Transactional
public class SeedGroupBracketController {
    @Autowired
    ModelMapper modelMapper;
    @Autowired
    private SeedGroupBracketService seedGroupBracketService;
    @Autowired
    private SeedGroupBracketDetailsService seedGroupBracketDetailsService;
    @Autowired
    private SeedService seedService;
    @Autowired
    private ProductModelService productModelService;
    @Autowired
    private UnitService unitService;
    @Autowired
    private StoreFeignClients storeFeignClients;
    @Autowired
    private YbFeignClients ybFeignClients;
    @ApiOperation(value = "分页查询籽晶组托列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<SeedGroupBracketDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<SeedGroupBracket> specification = DynamicSpecifications.bySearchFilter(request, SeedGroupBracket.class);
        Page<SeedGroupBracket> seedGroupBrackets = seedGroupBracketService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<SeedGroupBracketDto> dtos = seedGroupBrackets.getContent().stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, seedGroupBrackets.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @GetMapping("/listAll")
    @ApiOperation(value = "查询籽晶组托列表")
    public Result<List<SeedGroupBracketDto>> listAll(ServletRequest request) {
        Specification<SeedGroupBracket> specification = DynamicSpecifications.bySearchFilter(request, SeedGroupBracket.class);
        List<SeedGroupBracket> seedGroupBrackets = seedGroupBracketService.findByExampleSort(specification,null);
        List<SeedGroupBracketDto> dtos = seedGroupBrackets.stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        return ResultGenerator.success(dtos);
    }
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询籽晶组托详情")
    public Result<SeedGroupBracketDto> getById(@PathVariable @ApiParam(value = "id") Long id) {
        SeedGroupBracket seedGroupBracket = seedGroupBracketService.get(id);
        if(ObjectUtils.isEmpty(seedGroupBracket)){
            return ResultGenerator.fail("未查到相关数据!");
        }
        List<SeedGroupBracketDetails> seedGroupBracketDetails = seedGroupBracketDetailsService.findParentId(id);
        List<SeedGroupBracketDetailsDto> seedGroupBracketDetailsDtos = new ArrayList<>();
        seedGroupBracketDetails.stream().forEach(entity->{
            SeedGroupBracketDetailsDto seedGroupBracketDetailsDto = new SeedGroupBracketDetailsDto();
            BeanUtils.copyProperties(entity,seedGroupBracketDetailsDto);
            seedGroupBracketDetailsDto.setId(entity.getId());
            seedGroupBracketDetailsDtos.add(seedGroupBracketDetailsDto) ;
        });
        SeedGroupBracketDto seedGroupBracketDto = toEagerDto(seedGroupBracket);
        seedGroupBracketDto.setDetailsDtos(seedGroupBracketDetailsDtos);
        return ResultGenerator.success(seedGroupBracketDto);
    }

    @ApiOperation(value = "创建籽晶组托")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "SeedGroupBracketDto的对象")SeedGroupBracketDto seedGroupBracketDto
            , BindingResult bindingResult){
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        SeedGroupBracket seedGroupBracketCode = seedGroupBracketService.findCode(seedGroupBracketDto.getCode());
        if(!ObjectUtils.isEmpty(seedGroupBracketCode)){
            return ResultGenerator.fail("该数据已存在!");
        }
        List<SeedGroupBracketDetails> batchNo = seedGroupBracketDetailsService.findBatchNo(seedGroupBracketDto.getBatchNo());
        if(!ObjectUtils.isEmpty(batchNo)){
            return ResultGenerator.fail("该批次数据已存在!");
        }
        SeedData seedData = seedService.findByBatch(seedGroupBracketDto.getBatchNo());
        SeedGroupBracket seedGroupBracket = fromDto(seedGroupBracketDto);
        seedGroupBracket.setCode(CodeSender.getNumber(4));
        seedGroupBracket.setCount(seedGroupBracketDto.getDetailsDtos().size());
        seedGroupBracket.setStatus(InputBillStatusEnum.待入库.getValue());
        seedGroupBracket.setCreateBy(BaseContextHandler.getUsername());
        seedGroupBracket.setCreateDate(new Date());
        if(!ObjectUtils.isEmpty(seedData)){
            seedGroupBracket.setDopant(seedData.getDopant());
            seedGroupBracket.setMaterialCode(seedData.getMaterialCode());
            seedGroupBracket.setMaterialId(seedData.getMaterialId());
            seedGroupBracket.setMaterialModel(seedData.getMaterialModel());
            seedGroupBracket.setMaterialType(seedData.getMaterialType());
            seedGroupBracket.setMaterialName(seedData.getMaterialName());
        }
        seedGroupBracketService.saveOrUpdate(seedGroupBracket);
        seedGroupBracketDto.getDetailsDtos().stream().forEach(dto->{
            SeedGroupBracketDetails seedGroupBracketDetails = new SeedGroupBracketDetails();
            BeanUtils.copyProperties(dto,seedGroupBracketDetails,"id");
            seedGroupBracketDetails.setParentId(seedGroupBracket.getId());
            seedGroupBracketDetailsService.saveOrUpdate(seedGroupBracketDetails);
        });
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新籽晶组托")
    @PostMapping("/update")
    @Transactional(rollbackFor = Exception.class)
    public Result update(@RequestBody @Valid @ApiParam(value = "SeedGroupBracketDto的对象")SeedGroupBracketDto seedGroupBracketDto){
        SeedGroupBracket seedGroupBracketCode = seedGroupBracketService.findCode(seedGroupBracketDto.getCode());
        if(ObjectUtils.isEmpty(seedGroupBracketCode)){
            return ResultGenerator.fail("该数据不存在!");
        }
        if(seedGroupBracketCode.getStatus().equals("已入库")||seedGroupBracketCode.getStatus().equals("待审核")){
            return ResultGenerator.fail("该数据状态不可编辑!");
        }
        SeedGroupBracket seedGroupBracket = fromDto(seedGroupBracketDto);
        seedGroupBracket.setCount(seedGroupBracketDto.getDetailsDtos().size());
        seedGroupBracket.setCreateBy(BaseContextHandler.getUsername());
        seedGroupBracket.setCreateDate(new Date());
        seedGroupBracket.setId(seedGroupBracketDto.getId());
        seedGroupBracket.setStatus("待入库");
        seedGroupBracketService.saveOrUpdate(seedGroupBracket);
        //先删除之前数据
        List<SeedGroupBracketDetails> details = seedGroupBracketDetailsService.findParentId(seedGroupBracketDto.getId());
        details.stream().forEach(entity->{
            seedGroupBracketDetailsService.delete(entity.getId());
        });
        seedGroupBracketDto.getDetailsDtos().stream().forEach(dto->{
            SeedGroupBracketDetails seedGroupBracketDetails = new SeedGroupBracketDetails();
            BeanUtils.copyProperties(dto,seedGroupBracketDetails,"id");
            seedGroupBracketDetails.setParentId(seedGroupBracket.getId());
            seedGroupBracketDetailsService.saveOrUpdate(seedGroupBracketDetails);
        });
        return ResultGenerator.success();
    }

    /**
     * 更新籽晶组托(接收线边仓)
     * @param billCode
     * @param status
     * @return
     */
    @ApiOperation(value = "更新籽晶组托(接收线边仓)")
    @GetMapping("/bracketUpdate")
    public Result bracketUpdate(@RequestParam String billCode,@RequestParam String status){
        SeedGroupBracket serviceBillCode = seedGroupBracketService.findBillCode(billCode);
        serviceBillCode.setStatus(status);
        seedGroupBracketService.saveOrUpdate(serviceBillCode);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "发起入库")
    @GetMapping ("/initiate")
    public Result initiate(@RequestParam @ApiParam(value = "id") Long id){
        SeedGroupBracket seedGroupBrackets = seedGroupBracketService.get(id);
        if(ObjectUtils.isEmpty(seedGroupBrackets)){
            return ResultGenerator.fail("未查到相关数据!");
        }
        if(seedGroupBrackets.getStatus().equals("已入库")||seedGroupBrackets.getStatus().equals("待审核")){
            return ResultGenerator.fail("该数据状态不可发起入库!");
        }
        List<SeedGroupBracketDetails> seedGroupBracketDetails = seedGroupBracketDetailsService.findParentId(id);
        List<SeedGroupBracketDetailsDto> seedGroupBracketDetailsDtos = new ArrayList<>();
        seedGroupBracketDetails.stream().forEach(entity->{
            SeedGroupBracketDetailsDto seedGroupBracketDetailsDto = new SeedGroupBracketDetailsDto();
            BeanUtils.copyProperties(entity,seedGroupBracketDetailsDto);
            seedGroupBracketDetailsDto.setId(entity.getId());
            seedGroupBracketDetailsDtos.add(seedGroupBracketDetailsDto) ;
        });
        SeedGroupBracketDto seedGroupBracketDto = toEagerDto(seedGroupBrackets);
        seedGroupBracketDto.setDetailsDtos(seedGroupBracketDetailsDtos);
        WarehouseInputBillBo warehouseInputBillBo = new WarehouseInputBillBo();
        warehouseInputBillBo.setCreateUserName(BaseContextHandler.getUsername());
        warehouseInputBillBo.setCreateUserId(BaseContextHandler.getUserID());
        warehouseInputBillBo.setType(InputBillStatusEnum.籽晶入库.getValue());
        warehouseInputBillBo.setAutomaticOut(false);
        warehouseInputBillBo.setAutoAudit(false);
        ProductModel productModel = productModelService.findByCode(seedGroupBracketDto.getMaterialCode());
        if(ObjectUtils.isEmpty(productModel)){
            return ResultGenerator.fail("该主数据不存在!");
        }
        Unit unit = unitService.get(productModel.getUnit().getId());
        List<WarehouseInputBillDetailBo> inputBillDetails = new ArrayList<>();
        seedGroupBracketDto.getDetailsDtos().forEach(dto->{
            WarehouseInputBillDetailBo warehouseInputBillDetailBo = new WarehouseInputBillDetailBo();
            warehouseInputBillDetailBo.setCount(new BigDecimal(1));
            warehouseInputBillDetailBo.setMaterialModelId(Long.valueOf(String.valueOf(seedGroupBracketDto.getMaterialId())));
            warehouseInputBillDetailBo.setMaterialModelCode(seedGroupBracketDto.getMaterialCode());
            warehouseInputBillDetailBo.setMaterialModelName(seedGroupBracketDto.getMaterialName());
            warehouseInputBillDetailBo.setMaterialTypeName(seedGroupBracketDto.getMaterialType());
            warehouseInputBillDetailBo.setBatchNo(dto.getSeedBatchNo());
            if(!ObjectUtils.isEmpty(unit)){
                warehouseInputBillDetailBo.setUnitName(unit.getName());
            }
            warehouseInputBillDetailBo.setSupplier(productModel.getSupplier());
            warehouseInputBillDetailBo.setSpecification(productModel.getSpecification());
            warehouseInputBillDetailBo.setMaterialTh(productModel.getNumberOfDrawings());
            inputBillDetails.add(warehouseInputBillDetailBo);
        });
        warehouseInputBillBo.setInputBillDetails(inputBillDetails);
        Result result = storeFeignClients.inputApply(warehouseInputBillBo);
        if(!result.getCode().equals(200)){
            return ResultGenerator.fail("发起入库失败!");
        }else {
            seedGroupBrackets.setInputBillCode(result.getData().toString());
            seedGroupBrackets.setLaunchBy(BaseContextHandler.getUsername());
            seedGroupBrackets.setLaunchDate(new Date());
            seedGroupBrackets.setStatus(InputBillStatusEnum.待审核.getValue());
            seedGroupBracketService.saveOrUpdate(seedGroupBrackets);
        }
        return ResultGenerator.success(result.getData());
    }

    @ApiOperation(value = "删除籽晶组托")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
        for (Long id : ids) {
            seedGroupBracketService.delete(id);
            seedGroupBracketDetailsService.delete(id);
        }
        return ResultGenerator.success();
    }


    public SeedGroupBracketDto toLazyDto(SeedGroupBracket entity) {
        return modelMapper.map(entity, SeedGroupBracketDto.class);
    }

    public SeedGroupBracketDto toEagerDto(SeedGroupBracket entity) {
        SeedGroupBracketDto dto = modelMapper.map(entity, SeedGroupBracketDto.class);
        return dto;
    }

    public SeedGroupBracket fromDto(SeedGroupBracketDto dto) {
        return modelMapper.map(dto, SeedGroupBracket.class);
    }

    public SeedGroupBracketDetails fromDetailsDto(SeedGroupBracketDetailsDto dto) {
        return modelMapper.map(dto, SeedGroupBracketDetails.class);
    }
}
