package com.sw.mes.yb.seed.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.seed.dto.SeedDataDto;
import com.sw.mes.yb.seed.dto.SeedIQCInspectionDto;
import com.sw.mes.yb.seed.dto.SeedSelfInspectionDto;
import com.sw.mes.yb.seed.dto.SelfIQCInspectionDto;
import com.sw.mes.yb.seed.entity.SeedData;
import com.sw.mes.yb.seed.entity.SeedIQCInspection;
import com.sw.mes.yb.seed.entity.SeedSelfInspection;
import com.sw.mes.yb.seed.service.SeedIQCInspectionService;
import com.sw.mes.yb.seed.service.SeedSelfInspectionService;
import com.sw.mes.yb.seed.service.SeedService;
import io.swagger.annotations.*;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/seed/iqc")
@Api(description = "籽晶iqc检测接口")
@Transactional
public class SeedIQCInspectionController {
    @Autowired
    ModelMapper modelMapper;
    @Autowired
    private SeedIQCInspectionService seedIQCInspectionService;
    @Autowired
    private SeedSelfInspectionService seedSelfInspectionService;
    @Autowired
    private SeedService seedService;
    @ApiOperation(value = "分页查询籽晶信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<SeedIQCInspectionDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<SeedIQCInspection> specification = DynamicSpecifications.bySearchFilter(request, SeedIQCInspection.class);
        Page<SeedIQCInspection> selfInspections = seedIQCInspectionService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<SeedIQCInspectionDto> dtos = selfInspections.getContent().stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, selfInspections.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @GetMapping("/qualified/listAll")
    @ApiOperation(value = "查询籽晶IQC检验合格/不合格的数据")
    public Result<List<SeedIQCInspectionDto>> qualified(Boolean isQualified) {
        List<SeedIQCInspection> selfInspections = seedIQCInspectionService.getQualified(isQualified);
        List<SeedIQCInspectionDto> dtos = selfInspections.stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        return ResultGenerator.success(dtos);
    }

    @GetMapping("/listAll")
    @ApiOperation(value = "查询籽晶IQC检验信息列表")
    public Result<List<SeedIQCInspectionDto>> listAll(ServletRequest request) {
        Specification<SeedIQCInspection> specification = DynamicSpecifications.bySearchFilter(request, SeedIQCInspection.class);
        List<SeedIQCInspection> selfInspections = seedIQCInspectionService.findByExampleSort(specification,null);
        List<SeedIQCInspectionDto> dtos = selfInspections.stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        return ResultGenerator.success(dtos);
    }
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询籽晶IQC检验信息")
    public Result<SeedIQCInspectionDto> getById(@PathVariable @ApiParam(value = "id") Long id) {
        SeedIQCInspection SeedIQCInspection = seedIQCInspectionService.get(id);
        if(ObjectUtils.isEmpty(SeedIQCInspection)){
            return ResultGenerator.fail("未查到相关数据!");
        }
        return ResultGenerator.success(toEagerDto(SeedIQCInspection));
    }
    @ApiOperation(value = "创建籽晶IQC检验信息")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "SeedIQCInspectionDto的对象")SeedIQCInspectionDto seedIQCInspectionDto
            , BindingResult bindingResult){
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        return seedIQCInspectionService.create(seedIQCInspectionDto);
    }

    @ApiOperation(value = "修改籽晶IQC检验信息")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "SeedIQCInspectionDto的对象")SeedIQCInspectionDto SeedIQCInspectionDto
            , BindingResult bindingResult){
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        SeedIQCInspection seedCode = seedIQCInspectionService.getBySeedCode(SeedIQCInspectionDto.getSeedCode());
        if(ObjectUtils.isEmpty(seedCode)){
            return ResultGenerator.fail("该数据不存在!");
        }
        SeedIQCInspection SeedIQCInspection = fromDto(SeedIQCInspectionDto);
        SeedIQCInspection.setId(seedCode.getId());
        SeedIQCInspection.setUpdateBy(BaseContextHandler.getUsername());
        SeedIQCInspection.setUpdateDate(new Date());
        seedIQCInspectionService.saveOrUpdate(SeedIQCInspection);
        return ResultGenerator.success();
    }
    @ApiOperation(value = "批量修改籽晶IQC检验信息")
    @PostMapping("/batchUpdate")
    public Result batchUpdate(@RequestBody @Valid @ApiParam(value = "SeedIQCInspectionDto的对象")List<SeedIQCInspectionDto> SeedIQCInspectionDtos
            , BindingResult bindingResult){
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        SeedIQCInspectionDtos.stream().forEach(seedIQCInspectionDto -> {
            SeedIQCInspection SeedIQCInspection = fromDto(seedIQCInspectionDto);
            SeedIQCInspection.setId(seedIQCInspectionDto.getId());
            SeedIQCInspection.setUpdateBy(BaseContextHandler.getUsername());
            SeedIQCInspection.setUpdateDate(new Date());
            seedIQCInspectionService.saveOrUpdate(SeedIQCInspection);
        });
        return ResultGenerator.success();
    }
    @ApiOperation(value = "删除籽晶IQC检验信息")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
        for (Long id : ids) {
            seedIQCInspectionService.delete(id);
        }
        return ResultGenerator.success();
    }

    @GetMapping("/seedCode")
    @ApiOperation(value = "通过籽晶编码查询籽晶IQC检验信息")
    public Result<SeedIQCInspectionDto> getSeedCode(@RequestParam @ApiParam(value = "seedCode") String seedCode) {
        SeedSelfInspection seedSelfInspection = seedSelfInspectionService.getBySeedCode(seedCode);
        if(seedSelfInspection.getIsQualified()==false){
            return ResultGenerator.fail("该籽晶自检不合格!");
        }
        SeedIQCInspection seedData = seedIQCInspectionService.getBySeedCode(seedCode);
        return ResultGenerator.success(toEagerDto(seedData));
    }

    @GetMapping("/seedBatchNo")
    @ApiOperation(value = "通过籽晶序列号查询籽晶IQC检验信息")
    public Result<SeedIQCInspectionDto> getSeedBatchNo(@RequestParam @ApiParam(value = "seedBatchNo") String seedBatchNo) {
        SeedIQCInspection seedData = seedIQCInspectionService.getBySeedBatchNo(seedBatchNo);
        return ResultGenerator.success(toEagerDto(seedData));
    }

    @GetMapping("/planCode")
    @ApiOperation(value = "通过籽晶计划单号查询籽晶IQC检验信息")
    public Result<List<SeedIQCInspection> > getByPlanCode(@RequestParam @ApiParam(value = "planCode") String planCode) {
        List<SeedIQCInspection> selfInspections = seedIQCInspectionService.getByPlanCode(planCode);
        return ResultGenerator.success(selfInspections);
    }

    @GetMapping("/batchNo")
    @ApiOperation(value = "通过序列号查询IQC合格/不合格检验信息")
    public Result<List<SeedIQCInspection>> getBatchNo(@RequestParam @ApiParam(value = "batchNo") String batchNo
            ,@RequestParam @ApiParam(value = "isQualified") Boolean isQualified) {
        List<SeedIQCInspection> inspections = seedIQCInspectionService.getByBatchNo(batchNo,isQualified);
        return ResultGenerator.success(inspections);
    }

    @ApiOperation(value = "分页查询籽晶自检andIQC检测信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listSelfIQCPage")
    public Result<ResponsePage<SelfIQCInspectionDto>> listSelfIQCPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<SeedIQCInspection> seedIQCInspectionSpecification = DynamicSpecifications.bySearchFilter(request, SeedIQCInspection.class);
        Specification<SeedSelfInspection> seedSelfInspectionSpecification = DynamicSpecifications.bySearchFilter(request, SeedSelfInspection.class);
        Specification<SeedData> specification = DynamicSpecifications.bySearchFilter(request, SeedData.class);
        Page<SeedIQCInspection> seedIQCInspections = seedIQCInspectionService.findByExample(seedIQCInspectionSpecification, PageUtil.createPageable(requestPage));
        Page<SeedSelfInspection> seedSelfInspections = seedSelfInspectionService.findByExample(seedSelfInspectionSpecification, PageUtil.createPageable(requestPage));
        Page<SeedData> seedDataPage = seedService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<SeedIQCInspectionDto> seedIQCInspectionDtos = seedIQCInspections.getContent().stream().map(entity ->
                toLazyDto(entity)).collect(Collectors.toList());
        List<SeedSelfInspectionDto> seedSelfInspectionDtos = seedSelfInspections.getContent().stream().map(entity ->
                toLazySelfDto(entity)).collect(Collectors.toList());
        List<SelfIQCInspectionDto> dtos = new ArrayList<>();
        List<SeedData> seedAll = seedDataPage.getContent();
        seedAll.stream().forEach(seedData -> {
            SelfIQCInspectionDto selfIQCInspectionDto = new SelfIQCInspectionDto();
            selfIQCInspectionDto.setSeedCode(seedData.getSeedCode());
            SeedDataDto lazySeedDto = toLazySeedDto(seedData);
            selfIQCInspectionDto.setSeedDataDto(lazySeedDto);
            Optional<SeedIQCInspectionDto> iqcInspectionDto = seedIQCInspectionDtos.stream().filter(entity ->
                    entity.getSeedCode().equals(seedData.getSeedCode())).findFirst();
            if(iqcInspectionDto.isPresent()){
                selfIQCInspectionDto.setSeedIQCInspection(iqcInspectionDto.get());
            }
            Optional<SeedSelfInspectionDto> selfInspectionDto = seedSelfInspectionDtos.stream().filter(entity ->
                    entity.getSeedCode().equals(seedData.getSeedCode())).findFirst();
            if(selfInspectionDto.isPresent()){
                selfIQCInspectionDto.setSeedSelfInspection(selfInspectionDto.get());
            }
            dtos.add(selfIQCInspectionDto);
        });
        ResponsePage responsePage = new ResponsePage(dtos, seedDataPage.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    public SeedIQCInspectionDto toLazyDto(SeedIQCInspection entity) {
        if(ObjectUtils.isEmpty(entity)){
            return null;
        }
        return modelMapper.map(entity, SeedIQCInspectionDto.class);
    }

    public SeedIQCInspectionDto toEagerDto(SeedIQCInspection entity) {
        if(ObjectUtils.isEmpty(entity)){
            return null;
        }
        SeedIQCInspectionDto dto = modelMapper.map(entity, SeedIQCInspectionDto.class);
        return dto;
    }

    public SeedDataDto toLazySeedDto(SeedData entity) {
        if(ObjectUtils.isEmpty(entity)){
            return null;
        }
        return modelMapper.map(entity, SeedDataDto.class);
    }
    public SeedSelfInspectionDto toLazySelfDto(SeedSelfInspection entity) {
        return modelMapper.map(entity, SeedSelfInspectionDto.class);
    }
    public SeedIQCInspection fromDto(SeedIQCInspectionDto dto) {
        return modelMapper.map(dto, SeedIQCInspection.class);
    }
}
