package com.sw.mes.store.api.interfaces.consumer;

import com.sw.mes.core.api.system.AccessoryFeignClients;
import com.sw.mes.core.api.system.YbFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.ApplyAccessoryData;
import com.sw.mes.core.dto.sap.ReportSapChargItemDto;
import com.sw.mes.core.dto.sap.ReportSapGoodsmvtDto;
import com.sw.mes.core.dto.wms.*;
import com.sw.mes.core.enums.InputBillTypeEnum;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.StringUtils;
import com.sw.mes.store.dto.WarehouseAllotBillDto;
import com.sw.mes.store.entity.*;
import com.sw.mes.store.enums.AllotBillTypeEnum;
import com.sw.mes.store.enums.InputBillStatusEnum;
import com.sw.mes.store.enums.OutputBillTypeEnum;
import com.sw.mes.store.enums.WmsInputStatusEnum;
import com.sw.mes.store.service.WarehouseInputBillDetailService;
import com.sw.mes.store.service.WarehouseInputBillService;
import com.sw.mes.store.service.WarehouseModelService;
import org.apache.xmlbeans.GDate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.SimpleFormatter;

/**
 * @Description 需要外部系统提供接口，由线边仓调用
 * @Author wupeng
 * @Date 2023/12/28 14:56
 **/
@Service
@Transactional(rollbackFor = Exception.class)
public class WarehouseConsumerServiceImpl implements WarehouseConsumerService {

    @Autowired
    private AccessoryFeignClients accessoryFeignClients;
    @Autowired
    private YbFeignClients ybFeignClients;
    @Autowired
    private WarehouseInputBillDetailService detailService;

    @Autowired
    private WarehouseModelService warehouseModelService;
    @Override
    public String genInputBillNo() {
        return null;
    }

    @Override
    public String genOutputBillNo() {
        return null;
    }

    @Override
    public void afterAuditInput(WarehouseInputBill inputBill) {
        this.inputFeedback(inputBill);
    }

    @Override
    public void afterAutoInput(WarehouseInputBill inputBill) {
        this.inputFeedback(inputBill);
    }

    @Override
    public void afterInputReject(WarehouseInputBill inputBill) {
        this.inputFeedback(inputBill);
    }

    @Override
    public void afterInputCancel(WarehouseInputBill inputBill) {

    }

    @Override
    public void afterAuditOutput(WarehouseOutputBill outputBill) {
        this.outputFeedback(outputBill, 0);
    }

    @Override
    public void afterAutoOutput(WarehouseOutputBill outputBill) {

    }

    @Override
    public void afterOutputReject(WarehouseOutputBill outputBill) {
        this.outputFeedback(outputBill, 1);
    }

    @Override
    public void afterAuditAllot(WarehouseAllotBillDto warehouseAllotBillDto) {
        this.allotFeedback(warehouseAllotBillDto);
    }

    @Override
    public void afterOutputCancel(WarehouseOutputBill outputBill) {

    }

    private void inputFeedback(WarehouseInputBill inputBill) {
        String type = inputBill.getType();
        Integer status = null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //反馈 辅料退库 审核状态
        if (InputBillTypeEnum.辅料退库.getValue().equals(type)) {
            if(InputBillStatusEnum.已取消.getValue().equals(inputBill.getStatus())){
                status = 2;
            }
            if (InputBillStatusEnum.拒收.getValue().equals(inputBill.getStatus())) {
                status = 1;
            }
            if (InputBillStatusEnum.已入库.getValue().equals(inputBill.getStatus())) {
                status = 0;
            }
            if (status != null) {
                try {
                    Result<Void> voidResult = accessoryFeignClients.stockReturnApplyFeedback(inputBill.getBillNo(), status, BaseContextHandler.getUsername());
                    if (voidResult.getCode() != 200) {
                        throw new BusinessException("辅料退库反馈状态失败，状态码" + voidResult.getCode() + "，反馈信息" + voidResult.getMessage());
                    }
                } catch (Exception e) {
                    throw new BusinessException("反馈【辅料退库】审核状态失败，失败原因：" + e.getMessage());
                }
            }
        }
        if(InputBillTypeEnum.籽晶入库.getValue().equals(type)){
            String zjStatus = new String();
            if (InputBillStatusEnum.拒收.getValue().equals(inputBill.getStatus())) {
                zjStatus = InputBillStatusEnum.拒收.getValue();
            }else if(InputBillStatusEnum.已取消.getValue().equals(inputBill.getStatus())) {
                zjStatus = InputBillStatusEnum.已取消.getValue();
            }else if (InputBillStatusEnum.已入库.getValue().equals(inputBill.getStatus())) {
                zjStatus = InputBillStatusEnum.已入库.getValue();
            }
            if (!StringUtils.isEmpty(zjStatus)) {
                try {
                    Result voidResult = ybFeignClients.bracketUpdate(inputBill.getBillNo(), zjStatus);
                    if (voidResult.getCode() != 200) {
                        throw new BusinessException("辅料退库反馈状态失败，状态码" + voidResult.getCode() + "，反馈信息" + voidResult.getMessage());
                    }
                } catch (Exception e) {
                    throw new BusinessException("反馈【辅料退库】审核状态失败，失败原因：" + e.getMessage());
                }
            }
        }

        if(InputBillTypeEnum.原辅料入库.getValue().equals(type)){
            String wmsStatus;
            if (InputBillStatusEnum.拒收.getValue().equals(inputBill.getStatus())) {
                wmsStatus = WmsInputStatusEnum.拒收.getValue();
            }else if (InputBillStatusEnum.已入库.getValue().equals(inputBill.getStatus())) {
                wmsStatus = WmsInputStatusEnum.接收.getValue();
            }else {
                wmsStatus="";
            }
            if (!StringUtils.isEmpty(wmsStatus)) {
                try {
                    List<WarehouseInputBillDetail> details = detailService.getByBillId(inputBill.getId());
                    List<RemesstaDto> rematDetailsDto = new ArrayList<RemesstaDto>();
                    if(!ObjectUtils.isEmpty(details)){
                        details.stream().forEach(entity->{
                            RemesstaDto remesstaDto = new RemesstaDto();
                            remesstaDto.setOrganizationid(entity.getCustomer());
                            remesstaDto.setWarehouseid(entity.getWarehouseCode());
                            remesstaDto.setCustomerid(entity.getCustomer());
                            remesstaDto.setDocno(inputBill.getBillNo());
                            remesstaDto.setAufn(inputBill.getBillNo());
                            remesstaDto.setTraceid(entity.getTraceid());
                            remesstaDto.setSerialno(entity.getTraceid());
                            remesstaDto.setBillresult(wmsStatus);
                            remesstaDto.setNotes(inputBill.getRemark());
                            rematDetailsDto.add(remesstaDto);
                        });
                    }
                    Result voidResult = ybFeignClients.remessta(rematDetailsDto);
                    if (voidResult.getCode() != 200) {
                        throw new BusinessException("辅料退库反馈状态失败，状态码" + voidResult.getCode() + "，反馈信息" + voidResult.getMessage());
                    }
                } catch (Exception e) {
                    throw new BusinessException("反馈【辅料退库】审核状态失败，失败原因：" + e.getMessage());
                }
            }
        }
        if(InputBillTypeEnum.废料入库.getValue().equals(type)){
            try {
                List<MesasnDto> mesasnDtos = new ArrayList<>();
                //组装对应数据
                MesasnDto mesasnDto = new MesasnDto();
                mesasnDto.setOrganizationid("JINKO");
                mesasnDto.setWarehouseid("5041");
                mesasnDto.setCustomerid("5041");
                mesasnDto.setDocno(inputBill.getBillNo());
                mesasnDto.setAsntype("废料入库");
                mesasnDto.setAsnstatus("OK");
//                mesasnDto.setAsnreferenceb();//车间编号
                mesasnDto.setAsnreferencec("P501");
                mesasnDto.setAsncreationtime(sdf.format(inputBill.getCreateTime()));
                mesasnDto.setTraceid(inputBill.getInputBillDetails().get(0).getTraceid());//托盘号
                mesasnDto.setSerialno(inputBill.getInputBillDetails().get(0).getTraceid());//箱号
                mesasnDto.setExpectedqty(String.valueOf(inputBill.getInputBillDetails().size()));//组件数量
                mesasnDto.setLotatt01(sdf.format(new Date()));
                mesasnDto.setLotatt04(inputBill.getInputBillDetails().get(0).getBatchNo());//入库批次
                mesasnDto.setLotatt08("Y");//质量状态
//                mesasnDto.setLotatt09();//厂家
                mesasnDto.setLotatt12(inputBill.getInputBillDetails().get(0).getBatchNo());//生产批次
                mesasnDto.setLotatt19(inputBill.getInputBillDetails().get(0).getLotatt20());//电阻率
                mesasnDto.setLotatt20(inputBill.getInputBillDetails().get(0).getMaterialLevel());//等级
                mesasnDto.setSku(inputBill.getInputBillDetails().get(0).getMaterialModelCode());
                mesasnDto.setCreatesource("MES");
                List<MesasnOrderDto> mesasnOrderDtos = new ArrayList<>();
                inputBill.getInputBillDetails().stream().forEach(detais->{
                    MesasnOrderDto mesasnOrderDto = new MesasnOrderDto();
                    mesasnOrderDto.setSku(detais.getMaterialModelCode());
                    mesasnOrderDto.setTraceid(detais.getTraceid());
                    mesasnOrderDto.setSerialno(detais.getSerialNo());
                    mesasnOrderDto.setExpectedqty(String.valueOf(detais.getCount()));//箱重量
                    mesasnOrderDto.setLotatt01(sdf.format(new Date()));
                    mesasnOrderDto.setLotatt04(detais.getBatchNo());//入库批次
//                    mesasnOrderDto.setLotatt09();//厂家
                    mesasnOrderDto.setLotatt12(detais.getBatchNo());//生产批次
                    mesasnOrderDto.setLotatt19(detais.getLotatt19());//电阻率
                    mesasnOrderDto.setLotatt20(detais.getMaterialLevel());//等级
                    mesasnOrderDto.setDedi07(String.valueOf(detais.getCount()));//长度,整箱长度
                    mesasnOrderDto.setDedi08(detais.getBatchNo());//批号,MES下线批
                    mesasnOrderDto.setLotatt08("Y");
                    mesasnOrderDtos.add(mesasnOrderDto);
                });
                mesasnDto.setOrderList(mesasnOrderDtos);
                mesasnDtos.add(mesasnDto);
                //发起WMS入库
                Result result = ybFeignClients.mesasn(mesasnDtos);
                if(result.getCode()!=200){
                    throw new BusinessException("车间退料入库接口，状态码" + result.getCode() + "，反馈信息" + result.getMessage());
                }
            }catch (Exception ex){
                throw new BusinessException("出库失败->" + ex.getMessage());
            }
        }
    }

    private void outputFeedback(WarehouseOutputBill outputBill, int status) {
        if (OutputBillTypeEnum.辅料出库.getValue().equals(outputBill.getType())) {
            try {
                ApplyAccessoryData applyAccessoryData = new ApplyAccessoryData();
                applyAccessoryData.setCode(outputBill.getBillNo());
                applyAccessoryData.setStatus(status);
                applyAccessoryData.setUserConfirm(BaseContextHandler.getUsername());
                applyAccessoryData.setAccessoryDatas(new ArrayList<>());
                if (status == 0) {
                    for (WarehouseOutputBillDetail warehouseOutputBillDetail : outputBill.getOutputBillDetails()) {
                        ApplyAccessoryData.AccessoryData accessoryData = new ApplyAccessoryData.AccessoryData();
                        accessoryData.setBatchCode(warehouseOutputBillDetail.getBatchNo());
                        accessoryData.setUniqueCode(warehouseOutputBillDetail.getUniqueCode());
                        accessoryData.setSupplier(warehouseOutputBillDetail.getSupplier());
                        applyAccessoryData.getAccessoryDatas().add(accessoryData);
                    }
                }
                Result<Void> voidResult = accessoryFeignClients.applyAccessoryFeedback(applyAccessoryData);
                if (voidResult.getCode() != 200) {
                    throw new BusinessException("辅料出库反馈状态失败，状态码" + voidResult.getCode() + "，反馈信息" + voidResult.getMessage());
                }
            } catch (Exception e) {
                throw new BusinessException("出库失败->" + e.getMessage());
            }
        }
        if(OutputBillTypeEnum.WMS退料.getValue().equals(outputBill.getType())){
            try {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                RematDto data = new RematDto();
                RematHeaderDto header = new RematHeaderDto();
                List<RematDetailsDto> details = new ArrayList<RematDetailsDto>();
                header.setCustomerid(outputBill.getOutputBillDetails().get(0).getCustomer());
                header.setDocno(outputBill.getBillNo());
                header.setAsntype(outputBill.getType());
                header.setCreationtime(sdf.format(new Date()));
                header.setHedi01(String.valueOf(outputBill.getCreateUserId()));
                header.setHedi02(outputBill.getCreateUserName());
                header.setHedi05(outputBill.getOutputBillDetails().get(0).getWarehouseCode());
                data.setHeader(header);
                outputBill.getOutputBillDetails().stream().forEach(entity->{
                    RematDetailsDto dto = new RematDetailsDto();
                    dto.setLineno(String.valueOf(entity.getId()));
                    dto.setCustomerid(entity.getCustomer());
                    dto.setSku(entity.getMaterialModelCode());
                    dto.setQty(String.valueOf(entity.getOutputCount()));
                    dto.setLotatt08("Y");
                    dto.setTraceid(entity.getTraceid());
                    dto.setSerialno(entity.getTraceid());
                    details.add(dto);
                });
                data.setDetails(details);
                Result remat = ybFeignClients.remat(data);
                if(remat.getCode()!=200){
                    throw new BusinessException("车间退料入库接口，状态码" + remat.getCode() + "，反馈信息" + remat.getMessage());
                }
            }catch (Exception ex){
                throw new BusinessException("出库失败->" + ex.getMessage());
            }
        }
    }

    private void allotFeedback(WarehouseAllotBillDto warehouseAllotBillDto) {
        if (AllotBillTypeEnum.手动新建.getValue().equals(warehouseAllotBillDto.getAllotType())) {
            try {
                warehouseAllotBillDto.getAllotBillDetailDtos().stream().forEach(details->{
                    ReportSapGoodsmvtDto reportSapGoodsmvtDto = new ReportSapGoodsmvtDto();
                    List<ReportSapChargItemDto> reportSapChargItemDtos = new ArrayList<>();
                    ReportSapChargItemDto reportSapChargItemDto = new ReportSapChargItemDto();
//        reportSapChargItemDto.setAtnam("P01");
//        reportSapChargItemDto.setAtwrt("JK1");
                    reportSapChargItemDtos.add(reportSapChargItemDto);
                    reportSapGoodsmvtDto.setChargItems(reportSapChargItemDtos);
                    reportSapGoodsmvtDto.setAufnr(details.getAllotBillNo());
                    reportSapGoodsmvtDto.setCharg(details.getBatchNo());
                    reportSapGoodsmvtDto.setBwart("Z41");
                    reportSapGoodsmvtDto.setWerks("5041");
                    reportSapGoodsmvtDto.setUmcha(details.getBatchNo());
                    reportSapGoodsmvtDto.setUmwrk("5041");
                    reportSapGoodsmvtDto.setMatnr(details.getMaterialModelCode());
                    reportSapGoodsmvtDto.setMenge(details.getCount() == null ? "0" : details.getCount().toString());
                    reportSapGoodsmvtDto.setMeins(details.getUnitName());
                    reportSapGoodsmvtDto.setZfcpFlag("X");
                    reportSapGoodsmvtDto.setLgort(details.getWarehouseCode());
                    reportSapGoodsmvtDto.setUmlgo(details.getAllotWarehouseCode());
                    reportSapGoodsmvtDto.setProcName("线边仓审核调拨");
                    Result result = ybFeignClients.sendSapGoodsmvt(reportSapGoodsmvtDto);
                    if(!result.getCode().equals(200)){
                        throw new BusinessException("报工接口失败，状态码" + result.getCode() + "，反馈信息" + result.getMessage());
                    }
                });
            } catch (Exception e) {
                throw new BusinessException("调拨失败->" + e.getMessage());
            }
        }

    }
}
