package com.sw.mes.store.api.interfaces.supplier;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sw.mes.core.api.materialclean.MaterialCleanFeignClients;
import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.dto.materialCleanDto.MaterialStoreSignBoxDto;
import com.sw.mes.core.dto.store.WarehouseInputBillBo;
import com.sw.mes.core.dto.store.WarehouseMaterialBo;
import com.sw.mes.core.dto.store.WarehouseOutputBillBo;
import com.sw.mes.core.enums.InputBillTypeEnum;
import com.sw.mes.core.exception.BusinessException;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultCode;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.redis.server.RedissonUtils;
import com.sw.mes.store.constants.WarehouseConstant;
import com.sw.mes.store.dto.*;
import com.sw.mes.store.entity.*;
import com.sw.mes.store.enums.AllotBillStatusEnum;
import com.sw.mes.store.enums.AllotBillTypeEnum;
import com.sw.mes.store.enums.OutputBillTypeEnum;
import com.sw.mes.store.enums.OutputDestinationEnum;
import com.sw.mes.store.mapper.WarehouseModelMapper;
import com.sw.mes.store.service.*;
import com.sw.mes.store.utils.CodeGenUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.modelmapper.ModelMapper;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 外部调用线边仓的接口
 * @Author wupeng
 * @Date 2023/12/22 16:45
 **/
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class WarehouseSupplierServiceImpl implements WarehouseSupplierService {

    private static final Logger logger = LoggerFactory.getLogger(WarehouseSupplierServiceImpl.class);
    @Autowired
    private ModelMapper modelMapper;

    @Autowired
    private WarehouseInputBillService warehouseInputBillService;

    @Autowired
    private WarehouseOutputBillService warehouseOutputBillService;

    @Autowired
    private WarehouseInventoryService inventoryService;

    @Autowired
    private WarehouseAllotBillService warehouseAllotBillService;

    @Autowired
    private WarehouseAllotBillDetailService warehouseAllotBillDetailService;

    @Autowired
    private WarehouseModelMapper warehouseModelMapper;

    @Autowired
    private MaterialCleanFeignClients materialCleanFeignClients;

    @Autowired
    private WarehouseInventoryService warehouseInventoryService;

    @Autowired
    private WarehouseAllotRecordBillService warehouseAllotRecordBillService;

    @Autowired
    private WarehouseAllotRecordBillDetailService warehouseAllotRecordBillDetailService;

    @Autowired
    private WarehouseModelService warehouseModelService;

    @Autowired
    private WarehouseMaterialService warehouseMaterialService;

    @Autowired
    private RedissonClient redissonClient;

    @Override
    public String input(WarehouseInputBillBo inputBillBo) {
        if (inputBillBo == null || CollectionUtils.isEmpty(inputBillBo.getInputBillDetails())) {
            throw new BusinessException("入库数据为空");
        }
        WarehouseInputBillDto warehouseInputBill = new WarehouseInputBillDto();
        List<WarehouseInputBillDetailDto> warehouseInputBillDetailDtos= new ArrayList<>();
        BeanUtils.copyProperties(inputBillBo,warehouseInputBill,"inputBillDetails");
        inputBillBo.getInputBillDetails().stream().forEach(entity->{
            WarehouseInputBillDetailDto warehouseInputBillDetailDto = new WarehouseInputBillDetailDto();
            BeanUtils.copyProperties(entity,warehouseInputBillDetailDto,"id");
            warehouseInputBillDetailDtos.add(warehouseInputBillDetailDto);
        });
        warehouseInputBill.setInputBillDetails(warehouseInputBillDetailDtos);
        warehouseInputBillService.saveInputBill(warehouseInputBill);
        return warehouseInputBill.getBillNo();
    }

    @Override
    public WarehouseInputReturnDto inputData(WarehouseInputBillBo inputBillBo) {
        if (inputBillBo == null || CollectionUtils.isEmpty(inputBillBo.getInputBillDetails())) {
            throw new BusinessException("入库数据为空");
        }
        WarehouseInputBillDto warehouseInputBill = new WarehouseInputBillDto();
        List<WarehouseInputBillDetailDto> warehouseInputBillDetailDtos= new ArrayList<>();
        BeanUtils.copyProperties(inputBillBo,warehouseInputBill,"inputBillDetails");
        inputBillBo.getInputBillDetails().stream().forEach(entity->{
            WarehouseInputBillDetailDto warehouseInputBillDetailDto = new WarehouseInputBillDetailDto();
            BeanUtils.copyProperties(entity,warehouseInputBillDetailDto,"id");
            warehouseInputBillDetailDtos.add(warehouseInputBillDetailDto);
        });
        warehouseInputBill.setInputBillDetails(warehouseInputBillDetailDtos);
        warehouseInputBillService.saveInputBill(warehouseInputBill);
        WarehouseInputReturnDto warehouseInputReturnDto = new WarehouseInputReturnDto();
        warehouseInputReturnDto.setBillNo(warehouseInputBill.getBillNo());
        warehouseInputReturnDto.setWarehouseCode(warehouseInputBill.getInputBillDetails().get(0).getWarehouseCode());
        warehouseInputReturnDto.setCustomer(warehouseInputBill.getInputBillDetails().get(0).getCustomer());
        return warehouseInputReturnDto;
    }

    @Override
    public String output(WarehouseOutputBillBo outputBillBo) {
        if (outputBillBo == null || CollectionUtils.isEmpty(outputBillBo.getOutputBillDetails())) {
            throw new BusinessException("出库数据为空");
        }
        WarehouseOutputBill warehouseOutputBill = modelMapper.map(outputBillBo, WarehouseOutputBill.class);
        warehouseOutputBillService.saveOutputBill(warehouseOutputBill);
        return warehouseOutputBill.getBillNo();
    }

    @Override
    public WarehouseInputReturnDto outputData(WarehouseOutputBillBo outputBillBo) {
        if (outputBillBo == null || CollectionUtils.isEmpty(outputBillBo.getOutputBillDetails())) {
            throw new BusinessException("出库数据为空");
        }
        WarehouseOutputBill warehouseOutputBill = modelMapper.map(outputBillBo, WarehouseOutputBill.class);
        warehouseOutputBillService.saveOutputBill(warehouseOutputBill);
        WarehouseInputReturnDto warehouseInputReturnDto = new WarehouseInputReturnDto();
        warehouseInputReturnDto.setBillNo(warehouseOutputBill.getBillNo());
        warehouseInputReturnDto.setWarehouseCode(warehouseOutputBill.getOutputBillDetails().get(0).getWarehouseCode());
        warehouseInputReturnDto.setCustomer(warehouseOutputBill.getOutputBillDetails().get(0).getCustomer());
        return warehouseInputReturnDto;
    }

    @Override
    public void cancelInput(String billNo) {
        warehouseInputBillService.cancelApply(billNo);
    }

    @Override
    public void cancelOutput(String billNo) {
        warehouseOutputBillService.cancelApply(billNo);
    }

    @Override
    public List<WarehouseInventory> findByBatchNo(String batchNo) {
        List<WarehouseInventory> warehouseInventories = inventoryService.getByBatchNo(batchNo);
        return warehouseInventories;
    }

    /**
     * 对外创建调拨单接口
     *
     * @param warehouseAllotBillDto
     * @return
     */
    @Override
    public Boolean createAllotBill(WarehouseAllotBillDto warehouseAllotBillDto) {
        List<WarehouseAllotBillDetailDto> details = warehouseAllotBillDto.getAllotBillDetailDtos();
        if (CollectionUtils.isEmpty(details)) {
            throw new BusinessException("调拨明细为空，请检查数据！");
        }
        //调拨单号
        String billNo = StringUtils.isNotBlank(warehouseAllotBillDto.getBillNo()) ? warehouseAllotBillDto.getBillNo() : CodeGenUtil.genAllotBillNo();
        WarehouseAllotBill oldAllotBill = warehouseAllotBillService.getByBillNo(billNo);
        if (ObjectUtils.isNotEmpty(oldAllotBill)) {
            throw new BusinessException("调拨单号已存在，请检查数据！");
        }
        WarehouseModel warehouseModel = warehouseModelService.getCode(warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getAllotWarehouseCode());
        if (ObjectUtils.isEmpty(warehouseModel)) {
            throw new BusinessException("调拨库位" + warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getAllotWarehouseCode() + "不存在，请维护！");
        }
        WarehouseAllotBill allotBill = new WarehouseAllotBill();
        BeanUtils.copyProperties(warehouseAllotBillDto, allotBill);
        allotBill.setBillNo(billNo);
        if (StringUtils.isEmpty(allotBill.getAllotType())) {
            allotBill.setAllotType(AllotBillTypeEnum.手动新建.getValue());
        }
        allotBill.setCreateTime(new Date());
        allotBill.setCreateUserId(BaseContextHandler.getUserID());
        allotBill.setCreateUserName(BaseContextHandler.getUsername());
        allotBill.setStatus(AllotBillStatusEnum.待审核.getValue());
        allotBill.setAllotWarehouseId(warehouseModel.getId());
        allotBill.setAllotWarehouseName(warehouseModel.getName());
        //保存入库单
        warehouseAllotBillService.saveOrUpdate(allotBill);
        //保存调拨单记录
        WarehouseAllotRecordBill recordBill = new WarehouseAllotRecordBill();
        List<WarehouseAllotRecordBillDetail> allotRecordBillDetails = new ArrayList<>();
        BeanUtils.copyProperties(allotBill, recordBill, "id");
        recordBill.setAllotBillNo(allotBill.getBillNo());
        recordBill.setAllotBillId(allotBill.getId());
        recordBill.setRecordBillNo(allotBill.getBillNo());
        warehouseAllotRecordBillService.save(recordBill);
        List<WarehouseAllotBillDetail> detaiList = new ArrayList<>(details.size());
        //当开启了唯一码，用于判断是否存在相同的唯一码
        Set<String> uniqueCodeSet = new HashSet<>();
        for (WarehouseAllotBillDetailDto allotBillDetail : details) {
            String uniqueCode = allotBillDetail.getUniqueCode();
            if (StringUtils.isNotBlank(uniqueCode)) {
                //校验调拨明细中唯一码是否重复
                boolean add = uniqueCodeSet.add(uniqueCode);
                //查询是否已存在重复唯一码
                if (!add) {
                    throw new BusinessException("唯一码【" + uniqueCode + "】重复，请检查数据重新输入！");
                } else {
                    BigDecimal count = allotBillDetail.getCount();
                    //唯一码物料库存只能是 1
                    if (count.compareTo(new BigDecimal(1)) != 0) {
                        throw new BusinessException("唯一码【" + uniqueCode + "】物料调拨数量只能为1，请检查数据重新输入！");
                    }
                }
            }
            WarehouseInventory warehouseInventory = new WarehouseInventory();
            if (null != warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getWarehouseId()) {
                List<WarehouseInventory> warehouseInventories = inventoryService.getByMaterialAndBatchNoAndWarehouseId( warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getBatchNo(),allotBillDetail.getMaterialModelCode(),
                        warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getWarehouseId());
                if(warehouseInventories.size() > 0){
                    warehouseInventory = warehouseInventories.get(0);
                }
            }
            if (StringUtils.isNotBlank(warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getWarehouseCode()) && null == warehouseInventory) {
                List<WarehouseInventory> warehouseInventories = inventoryService.getByMaterialAndBatchNoAndWarehouseCode(warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getBatchNo(),
                        allotBillDetail.getMaterialModelCode(), warehouseAllotBillDto.getAllotBillDetailDtos().get(0).getWarehouseCode());
                if(warehouseInventories.size() > 0){
                    warehouseInventory = warehouseInventories.get(0);
                }
            }
            WarehouseAllotBillDetail detail = new WarehouseAllotBillDetail();
            BeanUtils.copyProperties(allotBillDetail, detail);
            detail.setRemark(allotBill.getRemark());
            detail.setStatus(AllotBillStatusEnum.待审核.getValue());
            detail.setAllotBillId(allotBill.getId());
            detail.setAllotBillNo(allotBill.getBillNo());
            detail.setCreateTime(allotBill.getCreateTime());
            detail.setCreateUserId(allotBill.getCreateUserId());
            detail.setCreateUserName(allotBill.getCreateUserName());
            detail.setAllotWarehouseId(warehouseModel.getId());
            detail.setAllotWarehouseName(warehouseModel.getName());
            if (!ObjectUtils.isEmpty(warehouseInventory)) {
                detail.setInventoryId(warehouseInventory.getId());
            }
            detaiList.add(detail);
            WarehouseAllotRecordBillDetail warehouseAllotRecordBillDetail = new WarehouseAllotRecordBillDetail();
            BeanUtils.copyProperties(detail, warehouseAllotRecordBillDetail, "id");
            warehouseAllotRecordBillDetail.setRecordBillId(recordBill.getId());
            warehouseAllotRecordBillDetail.setRecordBillNo(recordBill.getRecordBillNo());
            warehouseAllotRecordBillDetail.setAllotBillNo(detail.getAllotBillNo());
            warehouseAllotRecordBillDetail.setAllotBillId(detail.getAllotBillId());
            warehouseAllotRecordBillDetail.setCreateUserId(detail.getCreateUserId());
            warehouseAllotRecordBillDetail.setCreateUserName(detail.getCreateUserName());
            warehouseAllotRecordBillDetail.setCreateTime(detail.getCreateTime());
            warehouseAllotRecordBillDetail.setAllotWarehouseId(Integer.valueOf(String.valueOf(warehouseModel.getId())));
            warehouseAllotRecordBillDetail.setAllotWarehouseName(warehouseModel.getName());
            warehouseAllotRecordBillDetail.setMaterialModelId(Integer.valueOf(String.valueOf(detail.getMaterialModelId())));
            allotRecordBillDetails.add(warehouseAllotRecordBillDetail);
        }
        warehouseAllotBillDetailService.saveBatch(detaiList);
        warehouseAllotRecordBillDetailService.saveBatch(allotRecordBillDetails);

        //如果是自动审核，直接入库
        if(allotBill.getAutoAudit()) {
        BeanUtil.copyBeanPropWithOutNullValue(warehouseAllotBillDto, allotBill);
        warehouseAllotBillService.autoReceiveAtomic(warehouseAllotBillDto);
        }
        return true;
    }

    /**
     * 自动出入库对外接口
     *
     * @param inputBillBo
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result automaticInputOut(WarehouseInputBillBo inputBillBo) {
        if (inputBillBo == null || CollectionUtils.isEmpty(inputBillBo.getInputBillDetails())) {
            throw new BusinessException("入库数据为空");
        }
        //入库单号
        String inputBillNo = inputBillBo.getBillNo();
        inputBillNo = StringUtils.isNotBlank(inputBillNo) ? inputBillNo : CodeGenUtil.genInputBillNo();
        WarehouseInputBill byBillNo = warehouseInputBillService.getByBillNo(inputBillNo);
        if (byBillNo != null) {
            throw new BusinessException("入库单号【" + inputBillNo + "】已存在，请检查数据！");
        }
        WarehouseInputBillDto warehouseInputBill = modelMapper.map(inputBillBo, WarehouseInputBillDto.class);
        warehouseInputBill.setCreateTime(new Date());
        warehouseInputBill.setBillNo(inputBillNo);
        warehouseInputBill.setCreateUserId(BaseContextHandler.getUserID());
        warehouseInputBill.setCreateUserName(BaseContextHandler.getUsername());
        warehouseInputBill.setAutoAudit(true);
        warehouseInputBill.getInputBillDetails().stream().forEach(details -> {
            if (null == details.getWarehouseId() && null == details.getWarehouseCode()) {
                details.setWarehouseId(46L);
                details.setWarehouseName("虚拟库位");
                details.setWarehouseCode("xnkw");
            }
        });
        Result result = null;
        List<MaterialStoreSignBoxDto> storeSignBoxDtos = new ArrayList<>();
        if (inputBillBo.getType().equals(InputBillTypeEnum.余料入库.getValue())) {
            inputBillBo.getInputBillDetails().stream().forEach(details -> {
                //拼装原料接收对象
                MaterialStoreSignBoxDto storeSignBoxDto = new MaterialStoreSignBoxDto();
                storeSignBoxDto.setProductBatchNo(details.getBatchNo());
                storeSignBoxDto.setMaterialCode(details.getMaterialModelCode());
                storeSignBoxDto.setMaterialName(details.getMaterialModelName());
                storeSignBoxDto.setCreateUser(BaseContextHandler.getUsername());
                storeSignBoxDto.setUpdateUser(BaseContextHandler.getUsername());
                storeSignBoxDto.setWeight(details.getWeight());
                storeSignBoxDto.setMaterialBoxNo(details.getMaterialBoxNo());
                storeSignBoxDto.setMaterialLevel(details.getMaterialLevel());
                storeSignBoxDto.setMaterialModel(details.getMaterialModel());
                storeSignBoxDto.setMaterialGear(details.getMaterialGear());
                storeSignBoxDto.setBoxNo(details.getBoxNo());
                storeSignBoxDto.setPickingStatus(0);
                storeSignBoxDto.setMaterialType(2);
                storeSignBoxDto.setSupplier(details.getSupplier());
                storeSignBoxDto.setSupplierCode(details.getSupplierCode());
                storeSignBoxDto.setAufnr(details.getWorkOrderNo());
                storeSignBoxDtos.add(storeSignBoxDto);
            });
        }
        if (inputBillBo.getAutomaticOut()) {
            WarehouseOutputBill warehouseOutputBill = new WarehouseOutputBill();
            warehouseOutputBill.setType(inputBillBo.getType().equals(InputBillTypeEnum.余料入库.getValue())
                    ? OutputBillTypeEnum.余料出库.getValue() : inputBillBo.getType());
            warehouseOutputBill.setRemark(inputBillBo.getRemark());
            warehouseOutputBill.setDestination(OutputDestinationEnum.车间.getValue());
            warehouseOutputBill.setAutoAudit(true);
            warehouseOutputBill.setCreateTime(new Date());
            warehouseOutputBill.setCreateUserName(BaseContextHandler.getUsername());
            warehouseOutputBill.setStatus("0");
            warehouseOutputBill.setInputBillCode(inputBillNo);
            inputBillBo.getInputBillDetails().stream().forEach(details -> {
                WarehouseOutputBillDetail detail = new WarehouseOutputBillDetail();
                BeanUtils.copyProperties(details, detail);
                detail.setOutputCount(details.getCount());
                detail.setRequireCount(details.getCount());
                detail.setQuantity(details.getCount());
                detail.setCreateUserId(BaseContextHandler.getUserID());
                detail.setCreateUserName(BaseContextHandler.getUsername());
                detail.setCreateTime(new Date());
                detail.setType(inputBillBo.getType().equals(InputBillTypeEnum.余料入库.getValue())
                        ? OutputBillTypeEnum.余料出库.getValue() : inputBillBo.getType());
                detail.setStatus("已出库");
                detail.setDestination(OutputDestinationEnum.车间.getValue());
                warehouseOutputBill.getOutputBillDetails().add(detail);
            });
            String outputBillNos = warehouseOutputBillService.saveOutputBills(warehouseOutputBill);
            if (ObjectUtils.isEmpty(outputBillNos)) {
                return ResultGenerator.fail("生成出库单失败");
            }
        }
        if (!inputBillBo.getType().equals(InputBillTypeEnum.余料入库.getValue())) {
            //生成入库单及明细
            inputBillNo = warehouseInputBillService.saveInputBills(warehouseInputBill);
            if (!ObjectUtils.isEmpty(inputBillNo)) {
                return ResultGenerator.success("成功", inputBillNo);
            }else {
                return ResultGenerator.fail(result.getMessage());
            }
        }else{
            //生成入库单及明细
            inputBillNo = warehouseInputBillService.saveInputBills(warehouseInputBill);
            if (!ObjectUtils.isEmpty(inputBillNo)) {
                //调用原料接收接口
                result = materialCleanFeignClients.create(storeSignBoxDtos.get(0));
                if (result.getCode() != ResultCode.SUCCESS.getCode()) {
                    throw new BusinessException("调用清洗硅料创建:[materialStoreSignBox/create]失败: " + result.getMessage());
                }
            }else {
                return ResultGenerator.fail(result.getMessage());
            }
        }
        return ResultGenerator.success("成功", inputBillNo);
    }

    @Override
    public WarehouseInventoryDto getByBatchWarehouseId(String batchNo, Long warehouseId) {
        WarehouseInventory byBatchWarehouseId = warehouseInventoryService.getByBatchWarehouseId(batchNo, warehouseId);
        WarehouseInventoryDto warehouseInventoryDto = new WarehouseInventoryDto();
        BeanUtils.copyProperties(byBatchWarehouseId, warehouseInventoryDto);
        return warehouseInventoryDto;
    }

    /**
     *
     * @param storeDto 线边仓对象
     * @return
     */
    @Override
    @Transactional
    public Result createInputAndAllotAndOut(StoreDto storeDto) {
        try{
            if(ObjectUtils.isNotEmpty(storeDto)){
                long start = System.currentTimeMillis();
                //执行入库逻辑
                if(ObjectUtils.isNotEmpty(storeDto.getInputBillBo())){
                    input(storeDto.getInputBillBo());
                }
                //执行调拨逻辑
                if(ObjectUtils.isNotEmpty(storeDto.getWarehouseAllotBillDto())){
                    createAllotBill(storeDto.getWarehouseAllotBillDto());
                }
                long end = System.currentTimeMillis();
                System.out.println("调拨执行时间"+(end-start));
                //执行出库逻辑
                if(ObjectUtils.isNotEmpty(storeDto.getOutputBillBo())){
                    output(storeDto.getOutputBillBo());
                }
            }
        }catch (Exception e){
            throw new BusinessException("调用创建线边仓流程单据接口失败:"+e.getMessage());
        }
        return ResultGenerator.success();
    }

    /**
     * 同步物料主数据
     *
     * @param warehouseMaterialBos
     */
    @Override
    @Transactional
    public Result syncMaterial(List<WarehouseMaterialBo> warehouseMaterialBos) {
        //log.error("receive data from mes, code:{}", warehouseMaterialBos.get(0).getMaterialCode());
        for (WarehouseMaterialBo warehouseMaterialBo : warehouseMaterialBos) {
            boolean result;
            WarehouseMaterial warehouseMaterial = warehouseMaterialService.getMaterialByMaterialCode(warehouseMaterialBo.getMaterialCode());
            if (warehouseMaterial == null) {
                warehouseMaterial = new WarehouseMaterial();
                warehouseMaterial.setMaterialId(warehouseMaterialBo.getMaterialId());
                warehouseMaterial.setMaterialCode(warehouseMaterialBo.getMaterialCode());
                warehouseMaterial.setMaterialName(warehouseMaterialBo.getMaterialName());
                warehouseMaterial.setCreateTime(new Date());
                warehouseMaterial.setCreateUserId(BaseContextHandler.getUserID());
                warehouseMaterial.setCreateUserName(BaseContextHandler.getUsername());
                warehouseMaterial.setLowerLimit(new BigDecimal(1));
                warehouseMaterial.setUpperLimit(new BigDecimal(1000000));
                warehouseMaterial.setDefaultLocation("A001");
                warehouseMaterial.setBatch(true);
                warehouseMaterial.setUniqueCode(false);
                warehouseMaterial.setSafetyStock("1-100");
                result = warehouseMaterialService.save(warehouseMaterial);
            } else {
                warehouseMaterial.setMaterialId(warehouseMaterialBo.getMaterialId());
                warehouseMaterial.setMaterialCode(warehouseMaterialBo.getMaterialCode());
                warehouseMaterial.setMaterialName(warehouseMaterialBo.getMaterialName());
                result = warehouseMaterialService.updateById(warehouseMaterial);
            }
            if (!result) {
                log.error("insert or update data error, code:{}", warehouseMaterial.getMaterialCode());
            }
            log.error("id:{} 处理结果：{}", warehouseMaterialBo.getMaterialCode(), result);
        }
        return ResultGenerator.success();
    }

    @Override
    public WarehouseModel getById(Long id) {
        return warehouseModelMapper.findById(id);
    }

    @Override
    public List<WarehouseModelDto> getChildLocationIds(Long parentId) {
        List<WarehouseModelDto> storeLocations = new ArrayList<WarehouseModelDto>();
        addChild(storeLocations, parentId);
        return storeLocations;
    }

    /**
     * 递归查询库位子节点
     *
     * @param storeLocations
     * @param parentId
     */
    private void addChild(List<WarehouseModelDto> storeLocations, Long parentId) {
        List<WarehouseModel> children = warehouseModelMapper.findByParentId(parentId);
        if (CollectionUtils.isEmpty(children)) {
            return;
        }
        for (WarehouseModel warehouseModel : children) {
            WarehouseModelDto warehouseModelDto = modelMapper.map(warehouseModel, WarehouseModelDto.class);
            //查询所属厂区
            Long factoryId = warehouseModel.getFactoryId();
            WarehouseModel factory = warehouseModelMapper.findById(factoryId);
            if (factory != null) {
                warehouseModelDto.setFactoryName(factory.getName());
            }
            //查询所属仓库
            Long warehouseId = warehouseModel.getWarehouseId();
            WarehouseModel warehouse = warehouseModelMapper.findById(warehouseId);
            if (warehouse != null) {
                warehouseModelDto.setWarehouseName(warehouse.getName());
            }
            //查询所属库区
            Long areaId = warehouseModel.getAreaId();
            WarehouseModel area = warehouseModelMapper.findById(areaId);
            if (area != null) {
                warehouseModelDto.setAreaName(area.getName());
            }
            storeLocations.add(warehouseModelDto);
            addChild(storeLocations, warehouseModel.getId());
        }
    }
}
